 			//FMM.C file
#include "Lowest.h" 
// #include "int3.h"
#include "mpole.h"
#include "FMM.h"
#include "Systm.h"
#include "cell.h"
#include <iomanip> // needed for setw
#include <fstream> // needed for ostream?
extern Systm *My;
class cell;

//moved from cell.cpp
//added by DMAf
// was short operator<(const int3 &left, const int3 &right)
// bool operator<(const int3 left, const int3 right) fails too, & other combinations
short operator<(const int3 &left, const int3 &right) //needed for map operations // was short op.. const int3
{
  for (short i=0; i<3; i++)
    if (left.values[i] < right.values[i])//if wrong then left>=right
      return 1;
    else
      if (left.values[i] > right.values[i]) //if left > right return 0
        return 0;
    return 0;//if left==right
}

FMM::FMM()//constructor for FMM, called at the file scope when MyFMM is created
{
  // cout << "enter constructor for FMM"<<endl;
  CubeKer = NULL;
  //use default values:
  // don't bother, init. later: MaxAngle2 = 0.5;  MaxOrder = 1;   MaxOrder1 = MaxOrder + 1;
  NoInterRectKernels = 0; 
  // cout<<"finishing constructor...\n"<<endl;
}

FMM::~FMM()  //deallocates memory
{
  cout<<"Destructor for FMM"<<endl;
  cout<<"deallocating memory for arrays... ";
  delete[] NumBelow;
  delete[] RecurseAxis;
  delete[] RecurseFactor;
  delete[] RecurseMpole;

//specific for a system of uniformly magnetized cells

  for (short i = 0; i < NumBelow[MaxOrder]; i++) // this was NumComps, but local to setup()
    delete[] ConvSum[i];
  delete[] ConvSum;
  delete[] IndArr;
  short MaxOrderD1 = MaxOrder+ExtraOrderD+1; // must be same as in setup()
  for (short i = 0; i < MaxOrderD1; i++)
  {
    for (short j = 0; j < MaxOrderD1; j++)
      delete[] index[i][j];
    delete[] index[i];
  }
  delete[] index;
  cout<<"deallocated"<<endl;
}

//This function calculates some FMM arrays.
//This can't be done before, in Read_SetSizes, because AvgCoeff depends on
//the smallest cell size, determined in ParseBuild
void FMM::setup() 
{  // orders: constructor read in UsedOrder (for main calculation) and MaxOrder (for possible debugging).
   // However, the REAL maximum order is higher (MaxOrderD) because "Deriv" requires higher order 
   // than everything else.  So some things need to be higher order than others.
   ExtraOrderD=1; // could conceivably change this 1 to 2?
   MaxOrderD = MaxOrder + ExtraOrderD; 
   // For array sizes, define also:
	//short MaxOrder1 = MaxOrder + 1; // now same as MaxOrderD, but don't want to hard-wire this
   short MaxOrderD1 = MaxOrderD + 1; 

            //we need to set up NumBelow array, which will be used to allocate memory for
            //multipole moment when cells are created.
            NumBelow = new short[MaxOrderD1];  //the size of these arrays is already known

            for (short i = 0; i < MaxOrderD1; i++)   //initialization
            NumBelow [i] = (i+1)*(i+2)*(i+3)/6;  // no. components w/order <= argument;
                                             // used to predict ConvTerms

#ifdef DEBUG2
    cout<<"starting FMM::setup, MaxOrder = "<<MaxOrder
      <<", MaxOrderD = "<<MaxOrderD<<endl;
#endif  //  
 
  short NumComps = NumBelow[MaxOrder]; //total number of components, will be used for
                                //allocating arrays in setup()
  short NumCompsD = NumBelow[MaxOrderD]; // NOT NumComps+1!
  scratchUpM = new mpole(UsedOrder);  
  if(!scratchUpM) cout<<"failed to allocate scratchUpM, is UsedOrder "
     <<UsedOrder<<" too big?"<<endl;
    //three levels of hierarchy in function calls(??)
  mpole* scratchMpole1 = new mpole(MaxOrderD); // local
 // hopefully these are automatically de-allocated when setup() finishes
 
#ifndef H_ONLY
  Vshell = new mpole();  //added by DMApf
  Vext = new mpole(); // defaults to UsedOrder?
  VextNew = new mpole();

  // reducedSize is used only in readAndSetupRootImg below; 
  // don't calculate it if H_ONLY:

//initialize reducedSize:  //DMApf
  //find geometrical average:
  averageSize = 1.;
  //short numOfPerDir = 0; //number of periodic directions
  for (short i=0; i<3; i++)
    if (My->periodic[i])
    {
      // numOfPerDir++;
      averageSize*=My->PeriodVec[i];  //temporary storage for "volume"
    }
  
  averageSize = pow(averageSize, 1./My->numOfPerDir); // was powf, but doesn't exist? pow is double precision 
      // this gave error, chgd averageSize to float
  for (short i=0; i<3; i++)
    if (My->periodic[i]) 
      reducedSize[i] = My->PeriodVec[i]/averageSize; 
  // reduced size multiplies to 1; depends on aspect ratio only
  if(My->numOfPerDir) cout<<"reducedSize is "<<reducedSize[0]<<" "<<reducedSize[1]
    <<" "<<reducedSize[2]<<endl; // undefined for non-periodic directions??
#endif  //  H_ONLY

  RecurseMpole = new short[NumCompsD];  //used for recursive computations
  RecurseAxis = new short[NumCompsD];  //0 for x, 1 for y, 2 for z
  RecurseFactor = new real[NumCompsD];  //1/n[RecurseAxis]
#ifdef _DEBUG // this is system-defined, off for production version & timing tests
  for (short i = 0; i < NumCompsD; i++)   //to catch possible errors
    RecurseFactor[i] = RecurseAxis[i] = RecurseMpole[i] = -1;
#endif // DEBUG_
  IndArr = new int3[NumCompsD];     //inverts index[nx][ny][nz]

  ConvSum = new short*[NumComps]; // need figure out how big this really is, in ~FMM
    // the arguments of ConvSum are small (not D)

  //allocating memory for index ... - sequential index i_n in paper
  index = new short**[MaxOrderD1];
  for (short i = 0; i < MaxOrderD1; i++)
  {
    index[i] = new short*[MaxOrderD1];
    for (short j = 0; j < MaxOrderD1; j++)
      index[i][j] = new short [MaxOrderD1];
  }

//initializing index, IndArr
  short nxi = 0, in;//dummies
  short i = 0; // indexes 3-tuple n 
  for (short order = 0; order < MaxOrderD1; order++)// loop order 
    {
    for (short nzi = 0; nzi <= order; nzi++) 
    // loop index multiplicities, to add to order
      for (short nyi = 0; nyi <= order - nzi; nyi++)
      {
        nxi = order - nzi - nyi;
        IndArr[i][0] = nxi;    // store this index set 
        IndArr[i][1] = nyi;
        IndArr[i][2] = nzi;
        index[nxi][nyi][nzi] = i;
        // Now figure out how to recursively calculate things.
        // from a lower index, if possible from (nxi-1,nyi,nzi):
        if (nxi) { RecurseMpole[i] = index[nxi-1][nyi][nzi];
           RecurseAxis[i] = 0; RecurseFactor[i] = 1.0/nxi;}
             //for products c^p, see paper
        else if(nyi) { RecurseMpole[i] = index[nxi][nyi-1][nzi];
           RecurseAxis[i] = 1; RecurseFactor[i] = 1.0/nyi;}
        else if(nzi) { RecurseMpole[i] = index[nxi][nyi][nzi-1];
           RecurseAxis[i] = 2; RecurseFactor[i] = 1.0/nzi;}
#ifdef DEBUG2
          cout<<"i = "<<i<<" n_i = ( "<<IndArr[i][0]<<" "<<IndArr[i][1]
            <<" "<<IndArr[i][2]<<" )"<<endl;
#endif  //  
        i++;
      } // end for nyi,nzi
     assert(NumBelow[order]==i);
  } // end for order
  assert(NumBelow[MaxOrderD]==i);

   //initialize ConvSum
  for (short i = 0; i <NumComps; i++)
  {
    int3 ni(IndArr[i]);  //copy-constructor
    short n = ni.sum();  //order of ni
	short OrderP = MaxOrderD - n; // p+n is limited to MaxOrderD
	if (OrderP>MaxOrder) OrderP = MaxOrder; // can't use p beyond MaxOrder
    short NumP = NumBelow[OrderP];
	 
    ConvSum[i] = new short [NumP];

    for (short ip = 0; ip < NumP; ip++)
    { //int jj=IndArr[ip][0];debug
      int3 tot(ni + IndArr[ip]);
      assert(tot.sum()<=MaxOrderD);
      ConvSum [i][ip] = index[tot[0]][tot[1]][tot[2]];
    }//end for ip
  }  //end for i

//now compute Deriv. index arrays
  DerivTerms = new short[NumCompsD];
  DerivPower = new short*[NumCompsD];
  DerivPower [0] = new short[1];
  DerivCoef = new real*[NumCompsD];
  DerivCoef [0] = new real[1];
  DerivTerms[0] = 1;  //only one p
  DerivPower[0][0] = 0;
  DerivCoef[0][0] = 1/(4.*M_PI);
  for (short i = 1; i<NumCompsD; i++) // i <-> n+x in paper 
  {  // (if RecurseAxis=0 -- assume that for simplicity)
    in = RecurseMpole[i];  // in <-> n in paper
    short n = (IndArr[in]).sum();
	assert((n+1)==IndArr[i].sum()); // in should be one lower order
#ifdef DEBUG2
    cout<<"Setup DerivCoef["<<i<<"], DerivTerms[in="<<in<<"] = "<<DerivTerms[in]<<endl;
#endif //DEBUG2
   scratchMpole1->SetToZero();//this will be F_i=F_n+x
      // so size must be NumCompsD, not just NumComps
    assert(scratchMpole1->order==MaxOrderD);

    for (short j = 0; j < DerivTerms[in]; j++) // j<->p in paper
    { // more precisely, j is the index in a list of p's for which F_n[p]!=0
      real term = DerivCoef[in][j];  //F_n[p] in paper
      int3 pVec = IndArr[DerivPower[in][j]]; // p in paper
 	  // delete: if(i==87)
		//  cout<<"Term j="<<j<<" of "<<DerivTerms[in]<<" <-> p="<<pVec<<", n="<<
		//    IndArr[in]<<", F_n[p]="<<term<<endl;
      real term1 = -term * pVec[RecurseAxis[i]];//like -F_n*p_x in paper
      pVec[RecurseAxis[i]]+=1;  //now pVec is p+x in paper
      assert(index[pVec[0]][pVec[1]][pVec[2]]<NumCompsD);
      scratchMpole1->AddCarefully(pVec,term1+term*(2*n+1)); // chged sign of term1
	  //if(i==87) cout<<pVec<<scratchMpole1->values[index[pVec[0]][pVec[1]][pVec[2]]]<<endl;
      pVec[RecurseAxis[i]]-=1;  //restores the original pVec
      if (pVec[RecurseAxis[i]])
      {   //no contribution if pVec[RecurseAxis] == 0
        pVec[RecurseAxis[i]]-=1; 
        for (short axis = 0; axis < 3; axis++)
        {
          if(axis!=RecurseAxis[i])
          //need to loop only the other components, see paper
          {
            pVec[axis]+=2;
			scratchMpole1->AddCarefully(pVec,term1);
            // scratchMpole1->values[index[pVec[0]][pVec[1]][pVec[2]]]-=term1;
		    //if(i==87) cout<<"& "<<pVec<<scratchMpole1->values[index[pVec[0]][pVec[1]][pVec[2]]]<<endl;
            pVec[axis]-=2; //restore the old value
          }
        }
      }//end for if(pVec)
    }//end for j

//Now extract nonzero terms from scratchMpole1 
    short nterms = 0;  //count nonzero terms to allocate array
    for (short j = 0; j < NumCompsD; j++)
      if (fabs(scratchMpole1->values[j])) nterms++;
    DerivCoef[i] = new real [nterms];
    DerivPower[i] = new short [nterms];
    DerivTerms[i] = nterms;
//loop again to fill arrays
    nterms = 0;
    for (short j = 0; j < NumCompsD; j++) // j is p' in paper, argument of F_i
    {
      //if (fabs(scratchMpole1->values[j])>0.0001) failed, somehow was smaller!
	  if(scratchMpole1->values[j]!=0.0) //if F_i[p'] nonzero
      {
        DerivCoef[i][nterms] = scratchMpole1->values[j]; // F_i[p']
        DerivPower[i][nterms] = j; // p'
#ifdef DEBUG2
        cout<<"Create "<<nterms;print("-th term inF_", IndArr[i]);
        print("[p'", IndArr[DerivPower[i][nterms]]);
        cout<<"], F_i[p'] = "<<DerivCoef[i][nterms]<<endl;
#endif  //DEBUG2
        nterms++;
      }
    } //end for j
	assert(nterms==DerivTerms[i]);
  } //end for i
  //if (My->Psetup) cout<<"finishing FMM::setup"<<endl;
} //end of setup

//read Kernels from file KernelsFile and store them in Kernels
void FMM::ReadKernels(string FName,std::map<int3,Kernel>* Ker)
{
  char ch;
  std::pair<int3, Kernel> Ktemp;
  char junk[MAXLENFMM+1];
  fstream KernelsFile;
  KernelsFile.open(FName.c_str(), ios::in); // needs c string
      if (!KernelsFile){cout<<"can't open kernel file "
		  <<FName<<endl; MYEXIT(31);}
  while((ch=(char)KernelsFile.peek())!='!')
  {
    switch(ch)
    {
      case '#':
        KernelsFile.getline(junk, MAXLENFMM, '\n');
        break;
      case 'k':
		{
        KernelsFile>>junk>>Ktemp.first.values[0]>>Ktemp.first.values[1]
          >>Ktemp.first.values[2]>>Ktemp.second.Kxx>>Ktemp.second.Kxy
          >>Ktemp.second.Kxz>>Ktemp.second.Kyy>>Ktemp.second.Kyz
          >>Ktemp.second.Kzz;
//This line is commented out because M is the magnetization
//      Ktemp.second/=My->smallest->size[0]*My->smallest->size[1]*My->smallest->size[2];
        KernelsFile.getline(junk, MAXLENFMM, '\n');
		//bool  insertedNew=(Ker->insert(Ktemp)).second;
        //if (!insertedNew)
        if (!((Ker->insert(Ktemp)).second))
          cout<<"\n\t\tERROR WHILE READING KERNELS, key = "<<Ktemp.first<<endl;
		}
        break;
      default:
        cout<<"ERROR READING KERNELS, 1st char ="<<ch<<endl;
        MYEXIT(33);
    } //end of switch
  } //end of while

  KernelsFile.getline(junk, MAXLENFMM, '\n');
  KernelsFile.close();
  cout<<endl<<" Kernels have been read"<<endl;
}//end of ReadKernels

mpole** FMM::MakeAvCoef(MVector half, short order)
{  cout<<"Enter MakeAvCoef, order="<<order<<", half="<<half<<endl;
// Note that AC is the coef of cell dipole moment (=M*vol) in the mpole moment.
// See p.  21.4. Note that Eq. 26 of JMMM paper is WRONG by a factor of 2.
   mpole** AC = new mpole*[3]; // to return
     //these coefficients are used in cell::Average()
   //initialize them:
  for (short axis=0; axis<3; axis++)
  { AC[axis] = new mpole(order); 
    AC[axis]->SetToPowers(half); // was half*0.5! 
    for (short iVal=0; iVal < NumBelow[order]; iVal++)
    { // iVal is linear index of "n" in paper
      //for convenience:
      short n0=IndArr[iVal][axis], // ie. nx if axis=x
          n1=IndArr[iVal][(axis+1)%3],
          n2=IndArr[iVal][(axis+2)%3];

      if (n0%2 && !(n1%2) && !(n2%2))  //non-zero if n0 is odd, n1, n2 are even
        AC[axis]->values[iVal]/=half[axis]*(n1 + 1)*(n2 + 1);
        // pre-12/13/2011, was AvCoef[axis]->values[iVal]/=-CellSize[axis]*0.5*(n1 + 1)*(n2 + 1);
	    // PBV removed - sign as test, didn't help,restored, then removed 2014. 
	    // Also changed sign in calc average over cell.
      else
        AC[axis]->values[iVal]=0.0;  //zero otherwise
    } //end for iVal 
  } //end for axis	

// print it out nicely (dma)
//AC[0]->printMP("AC[0]=",cout);

  return AC;
} // end MakeAvCoef

void Kernel::SetToZero() {Kxx=Kxy=Kxz=Kyy=Kyz=Kzz=0.0;}
real Kernel::Square() {return Kxx*Kxx+Kxy*Kxy+Kxz*Kxz+Kyy*Kyy+Kyz*Kyz+Kzz*Kzz;}
void Kernel::SetToSelfInteract()
{
  Kxx = -1./3.;
  Kyy = Kxx;
  Kzz = Kxx;
  Kxy = 0.;
  Kxz = 0.;
  Kyz = 0.;
}

MVector Kernel::Dot(MVector V)
{ MVector Res;
  Res[0] = Kxx*V[0]+Kxy*V[1]+ Kxz*V[2];
  Res[1] = Kxy*V[0]+Kyy*V[1]+ Kyz*V[2];
  Res[2] = Kxz*V[0]+Kyz*V[1]+ Kzz*V[2];
  return Res;
}  // end Dot

/*
void Kernel::SetToMonopoleField(const int3 &key)
{
  mpole *D = new mpole(1);
  MVector dist (key);
  dist*=-0.5;  //0.5 is due to double space key
  D->SetToDeriv (dist);
  
  Kx = D->GetValue(1,0,0);
  Ky = D->GetValue(0,1,0);
  Kz = D->GetValue(0,0,1);
}
*/

Kernel Kernel::operator+(const Kernel& Kright) const
{
  Kernel Kresult;
  Kresult.Kxx = Kxx + Kright.Kxx;
  Kresult.Kxy = Kxy + Kright.Kxy;
  Kresult.Kxz = Kxz + Kright.Kxz;
  Kresult.Kyy = Kyy + Kright.Kyy;
  Kresult.Kyz = Kyz + Kright.Kyz;
  Kresult.Kzz = Kzz + Kright.Kzz;
  return Kresult;
}

void Kernel::operator+=(const Kernel& Kright)
{
  *this = *this + Kright;
}

Kernel Kernel::operator- (const Kernel & Kright) const
{
  Kernel Kresult;
  Kresult.Kxx = Kxx - Kright.Kxx;
  Kresult.Kxy = Kxy - Kright.Kxy;
  Kresult.Kxz = Kxz - Kright.Kxz;
  Kresult.Kyy = Kyy - Kright.Kyy;
  Kresult.Kyz = Kyz - Kright.Kyz;
  Kresult.Kzz = Kzz - Kright.Kzz;
  return Kresult;
}

void Kernel::operator-=(const Kernel& Kright)
{
  *this = *this - Kright;
}

Kernel Kernel::operator/ (const float &right) const
{
  Kernel Kresult;
  Kresult.Kxx = Kxx/right;
  Kresult.Kxy = Kxy/right;
  Kresult.Kxz = Kxz/right;
  Kresult.Kyy = Kyy/right;
  Kresult.Kyz = Kyz/right;
  Kresult.Kzz = Kzz/right;
  return Kresult;
}

void Kernel::operator/= (const float &right)
{
  *this = *this/right;
}


//added by DMAk
void Kernel::operator*= (const float number)
{
  Kxx*=number;
  Kxy*=number;
  Kxz*=number;
  Kyy*=number;
  Kyz*=number;
  Kzz*=number;
}

NearPartner::NearPartner(LLGcell *C, Kernel *KerPtr)
{
  cel = C;
  K = KerPtr;
}

// the following should overload << for derived class ofstream as well?
ostream& operator<<(ostream &out, const Kernel &k) //overloaded operator<< for Kernel class
{
  out<<" ("<<k.Kxx<<' '<<k.Kxy<<' '<<k.Kxz<<' '<<k.Kyy<<' '<<k.Kyz<<' '<<k.Kzz<<") ";
  return out;
}

#ifndef H_ONLY
void FMM::print(char *title, const short array[], const short size)
{
  cout <<title<<" = ";

  for (short i = 0; i < size; i++)
    cout << array[i] <<' ';
  cout<<flush;
}

//output the valarray to console: "title = (1, 2, 3)" NO ENDLINE AT THE END
void FMM::print(char *title, const int3 &array)
{
  cout <<title<<" = ( ";

  for (short i = 0; i < 3; i++)
    cout << array[i] <<' ';
  cout<<")"<<flush;
}

//calculate newKernels, put it to Kernels and return the maximum (DMA)- see any version < 5/2013


//This function reads the FMM parameters and calculates the array size
//(NumBelow) that is needed by ParseBuild to build the cells
void FMM::ReadScriptFMM (ifstream &inputFile)
{
	        char junk[MAXLENFMM+1];
            inputFile>>junk>>MaxAngle2>>UsedOrder; // reads to white space
			MaxOrder = UsedOrder; // unless DBG increases it below
#ifdef DEBUG2
			cout<<"fmm params: MaxAngle2="<<MaxAngle2
				<<"MaxOrder="<<MaxOrder<<endl;
#endif  //  

/* GONE:
#ifdef DBG_ // debug Green's fns
  short Gdir1; // direction of source moment
  inputFile>>MaxOrder>>Gdir1; // was on end of line
#endif // DBG_
   */
		setup(); // set up lots of arrays, depend on MaxOrder.

	 	 if(!inputFile) {cout<<"Error in fmm line, near:"
		    <<junk<<endl; MYEXIT(32);}
/* REMOVE:
#ifdef DBG_ // can't set up mpoles til after setup
  My->DBG = new DebugGreen(Gdir1); // pass direction
#endif //DBG_
  */
// moved kernel-reading away from here to ReadScript:'G'
} // end ReadScriptFMM
//added by DMAd (for debug)
void FMM::printMap (char *title, const std::map<int3, Kernel> & mapToPrint)
{
  cout<<"From FMM::printMap: "<<title<<endl;
  if (mapToPrint.empty() )
    cout<<"Map is empty";
  else {
    std::map<int3, Kernel>::const_iterator j;
    for (j = mapToPrint.begin(); j!=mapToPrint.end(); j++)
      cout<<j->first<<"   Kernel   "<<j->second<<endl;
  }
  cout<<endl;
}

//reads all images of the root cell and puts them to root->Partners list
void FMM::readAndSetupRootImg () 
{  // must come AFTER geometry, i.e. not in fmm block
  cout<<"reading images and Vext from file "<<endl;
  char ch, junk[MAXLENFMM+1];
  short imgDisp[3];
  real reducedSizeIn[3];  //to check reducedSize
  assert(ImageFile);
  while ((ch = (char)ImageFile.peek()) != '!')
  {
    switch (ch)
    {
    case '#':
      ImageFile.getline(junk, MAXLENFMM, '\n');
      break;
    case 'S':   //reduced Size of the system 
		// (i.e., scaled so volume = 1; sensitive to aspect ratio but not size
		// cored-lattice potential Vext should depend only on .
      ImageFile>>junk>>reducedSizeIn[0]>>reducedSizeIn[1]>>reducedSizeIn[2];
      ImageFile.getline(junk, MAXLENFMM, '\n');
      cout<<"reducedSizeIn = "<<MVector (reducedSizeIn)<<endl;
      cout<<"reducedSize = "<<MVector (reducedSize)<<endl;

      for (short i=0; i<3; i++)  //compare the sizes
        assert (My->almost_same(reducedSizeIn[i], reducedSize[i]));
      break;
    case 'R':
      ImageFile>>junk>>imgDisp[0]>>imgDisp[1]>>imgDisp[2];
      ImageFile.getline(junk, MAXLENFMM, '\n');
      cout<<"RootImage = "<<imgDisp[0]<<"  "<<imgDisp[1]<<"  "<<imgDisp[2]<<endl;
      My->root->Partners.push_back(DispCell(My->root, imgDisp));
      break;
    case 'V':
      ImageFile>>junk>>*Vext;
      cout<<"Vext is "<<*Vext<<endl;
      Vext->rescale (averageSize);
      ImageFile.getline(junk, MAXLENFMM, '\n');
      break;
    default:
        cout<<"ERROR WITH Image, first character is "<<ch<<endl;
        MYEXIT(35);
    }
  }
} // end readAndSetupRootImg


//to write calculated Kernels
void FMM::WriteKernels()  //added by DMAf
{
 //write the Kernels only if they have been calculated
  //assert (CalcCubicKernels); 
  KernelsFile.open((My->ExeDir+OutName).c_str(), ios::out);   //open it for output
  if (!KernelsFile){
     cout<<"can't open kernel file "<<My->ExeDir+OutName<<endl; // dunno where cerr goes!
     MYEXIT(31);}
 
  map<int3, Kernel>::iterator iterMap;


  KernelsFile<<"# Kernels"<<endl;
   KernelsFile<<"#"<<setw(11)<<"key"<<setw(18)<<"Kxx"<<setw(10)<<"Kxy"<<setw(10)
            <<"Kxz"<<setw(10)<<"Kyy"<<setw(10)<<"Kyz"<<setw(10)<<"Kzz"<<endl;

// OR, maybe it's in a rectangle's map RectKer?????
  for (iterMap=CubeKer->begin(); iterMap!=CubeKer->end(); iterMap++)
  {
    CleanUpSmall(iterMap->second.Kxx); // sets to 0 if small
    CleanUpSmall(iterMap->second.Kxy); CleanUpSmall(iterMap->second.Kxz); 
    CleanUpSmall(iterMap->second.Kyy); CleanUpSmall(iterMap->second.Kyz); 
    CleanUpSmall(iterMap->second.Kzz); 
 //This line is commented out because M is the magnetization (not moment)
//    iterMap->second*=My->smallest->size[0]*My->smallest->size[1]*My->smallest->size[2];
    KernelsFile<<setw(3)<<iterMap->first.values[0]<<' '
      <<setw(3)<<iterMap->first.values[1]<<' '<<setw(3)
      <<iterMap->first.values[2]<<' '
      <<setw(10)<<iterMap->second.Kxx<<' '<<setw(10)<<iterMap->second.Kxy
      <<' '<<setw(10)<<iterMap->second.Kxz<<' '<<setw(10)
      <<iterMap->second.Kyy<<' '<<setw(10)<<iterMap->second.Kyz<<' '
      <<setw(10)<<iterMap->second.Kzz<<endl;
  }
  KernelsFile<<"! end of Kernels file"<<flush;
}

//used for periodic FMM (DMApf):
//returns 1 if all three values are even
short FMM::coarseCharacter (const int3 &key) const
{
  return (!(key.values[0] % 2) && !(key.values[1] % 2) && !(key.values[2] % 2));
}

//used for periodic FMM
//recursively calculate Vext, find images of the root cell and write everything
//to ImageFile
// Note that this follows the 2003 paper, except instead of regarding image as near 
// if it is closer than some R, the FMM opening-angle criterion is used.
void FMM::calcVextAndWriteImages ()
{
  assert (My->periodic[0] || My->periodic[1] ||My->periodic[2]); // at least one periodic
  cout<<"Begin recursive calculation of exterior potential"<<endl;
   //need to calculate Acoef's
  calcVshellAndWriteImages ();

  //we need to setup a recursive procedure for calculation of Vext
  short counter = 0;  //to prevent infinite recursion
  real delta = 0.;
  while (((delta = calcExtOneIter()) > 0.000005) && (counter < 30))
    counter++;
  if (counter==30)
  {
    cerr<<"ERROR: iterative calculation of Vext, delta = "<<delta<<endl;
    MYEXIT(34);
  }
//write down the external potential mpole (calculated assuming root cell size =1)
//it needs to be rescaled before use
  ImageFile<<"Size: "<<reducedSize[0]<<"  "<<reducedSize[1]<<"  "<<reducedSize[2]<<endl;
  ImageFile<<"Vext: "<<*Vext<<endl;
  ImageFile<<"! end of the file"<<endl;
  cout<<"Recursive calculation has been finished, error = "<<delta<<endl;
}

//used for periodic FMM (DMApf):
//Vshell is calculated and images of the root cell are written to the ImageFile
void FMM::calcVshellAndWriteImages() // as described in 2003 paper
{
  real Acoef, sum;
// We will loop over p, an integer vector describing 
// the position of an image of the root cell, in units of root cell size.
// First determine keyMax, the largest possible value for p we need to worry about.
// This will occur in the direction of the smallest dimension of the system,
// so we find this direction:
  short iSmallest=0;
  for (short i=0; i<3; i++)
    if ((My->PeriodVec[i] < My->PeriodVec[iSmallest]) && My->periodic[i])
      iSmallest = i;

  int3 keyMax; // only one component is actually used, for now
  // Increase keyMax until it's in the exterior. 
  keyMax[iSmallest]=2;   //start from 2
  
  do {keyMax[iSmallest]++;}
  while (My->root->Near(DispCell(My->root, keyMax)));

  // cout<<"keyMax = "<<keyMax<<endl;

  keyMax[iSmallest]*=2;  //the radius of the sphere is 2R (see notes)
  mpole *Dshell = new mpole(); //utility mpole, it is used to set the pot'l of the shell

  int3 p, t; // see paper; t is the small displacement (+-1 or 0) of the
  // coarse lattice to construct the fine lattice.  It is nonzero only in
  // the periodic directions, so the loop is trivial (0 to 0) if not periodic.
  // If periodic, loop is from -keyMax to +keyMax.
  // note (L ? a:b) is a if L is true, b if L is false.
  // This nested loop over p and t calculates the shell charge q^shell (see 2003 paper)
  // (Acoef) from Eq. ...

  // PBV 2009: This seems awkward -- could use p0,p1,p2! 

  for (p.values[0] = (My->periodic[0] ? -keyMax[iSmallest] : 0);
      p.values[0] <= (My->periodic[0] ? keyMax[iSmallest] : 0); p.values[0]++)
    for (p.values[1] = (My->periodic[1] ? -keyMax[iSmallest] : 0);
        p.values[1] <= (My->periodic[1] ? keyMax[iSmallest] : 0); p.values[1]++)
      for (p.values[2] = (My->periodic[2] ? -keyMax[iSmallest] : 0);
          p.values[2] <= (My->periodic[2] ? keyMax[iSmallest] : 0); p.values[2]++)
      {
        sum = .0;
		// now loop displacements t (nonzero only in periodic directions)
        for (t.values [0]=(My->periodic[0] ? -1 : 0);
            t.values [0]<= (My->periodic[0] ? 1 :0); t.values[0]++)
          for (t.values [1]=(My->periodic[1] ? -1 : 0);
              t.values [1]<= (My->periodic[1] ? 1 :0); t.values[1]++)
            for (t.values [2]=(My->periodic[2] ? -1 : 0);
                t.values [2]<= (My->periodic[2] ? 1 :0); t.values[2]++)
              if (coarseCharacter (p-t)) //so (p-t)/2 is an integer
			  // seems odd to loop all p-t and then use only even ones? 
			  // Alternative is to loop all t & use only 0,+-1! Worse.
                if (!My->root->Near(DispCell(My->root, (p-t)/2)))
                  sum+=calculateSymFactor(t); 
		          // ..SymFactor is w(t) of paper=1,1/2,1/4, or 1/8:
        // end of t loop -- we're still in p loop        

        Acoef = !My->root->Near(DispCell(My->root, p)) - sum; // shell charge at p

    //now we know shell charge, we need to know its Taylor expansion around origin:
        if (!My->almost_same(Acoef, 0.))  //needed to eliminate p=(0,0,0) case
        { mpole* scrat2=new mpole();
          Dshell->SetToDeriv (MVector(p[0]*reducedSize[0], p[1]*reducedSize[1],
                              p[2]*reducedSize[2]), scrat2);
          *Vshell += (*Dshell)*Acoef;
        }
#ifdef DEBUG2
		if(Acoef>0.)
		cout<<"p="<<p.values[0]<<' '<<p.values[1]<<' '<<p.values[2]<<"; shell chg="<<Acoef<<endl;
#endif  //  
    //writing down the images of the root cell
        if (My->root->Near(DispCell(My->root, p)) && !p.isZero())
        {
          ImageFile<<"RootImage "<<p.values[0]<<' '<<p.values[1]<<' '<<p.values[2]<<endl;
          assert (My->almost_same(Acoef, 0.0)); // should vanish for Near images
        }
      } // go back for new p
  delete Dshell;
}  //end of the function



//used for periodic FMM (DMApf):
real FMM::calcExtOneIter()
{ // Assumes an estimate of Vext, the potential of a cored array of point charges,
	// is known and calculates a better one.
	// Uses Vext 
  mpole *tempMpole = new mpole(),
        *VextAdd = new mpole();
  tempMpole->SetToPowerSym(My->periodic); // prepare to convolute potential with w(t)
  VextAdd->AddDownConv (Vext, tempMpole,0); // set VextAdd to the convolution
//divide VextAdd by 2^(n+1)
  VextAdd->rescale (2.);

  *VextNew = *Vshell + *VextAdd;  // add shell to get new estimate for Vext
// we need to put the divergent values to zero.
// 
  short periodicity = My->periodic[0]+My->periodic[1]+My->periodic[2];
  // for periodicity = 1    2    3
  // divergent orders  0    0    2
  // vanishing orders  1    1    3
  // so the following works if you check each case:
  for (short i=0; i<NumBelow[periodicity-1]; i++)
    VextNew->values[i]=0.0;

  real delta = Vext->compare (VextNew);  //maximum difference between mpoles
  // cout<<"delta = "<<delta<<endl;

  *Vext = *VextNew;   //put VextNew to Vext

  delete tempMpole;
  delete VextAdd;
  return delta;
}

//used for periodic FMM (DMApf):
real FMM::calculateSymFactor (const int3 &key) // the function w(t) in the writeup
{
  real W = 1.;
  for (short i=0; i<3; i++)
    if (key.values[i]!=0)
      W/=2.;
  return W;
}
#endif  //  H_ONLY