//FMM.h classes needed for FMM

#ifndef FMM_H
#define FMM_H

#include <map> // leave this here for now, someday replace by class map
   // (tricky because it's a template)
#include <fstream> // for ofstream
// #include <ostream> // 
//#include "Lowest.h"

class mpole;
class int3;
class face;
class MVector; //added by DMAk
class cell;
class LLGcell;
 
//using namespace std;
const short MAXLENFMM = 100;
short operator<(const int3 &left, const int3 &right); // not in a class

class Kernel 
{
  friend std::ostream &operator<< (std::ostream&, const Kernel&);
public:
//  data member:
  real Kxx, Kxy, Kxz, Kyy, Kyz, Kzz;

// functions:
  Kernel () { Kxx=Kxy=Kxz=Kyy=Kyz=Kzz=0.;} // constructor
  void SetToZero();
  real Square();
  void SetToSelfInteract();  //sets this Kernel to Kernel for self-interaction (H_stat = -M/3)
  MVector Dot(MVector V);
//operations:
  Kernel operator+ (const Kernel &) const; //Kernel addition
  void operator+= (const Kernel &);
  Kernel operator- (const Kernel &) const; //subtraction
  void operator-= (const Kernel &);
  Kernel operator/ (const real &) const; //division by a real number
  void operator/= (const real &);
  void operator*= (const real number);  //multiplication by number
};

class FMM // has functions for fast multipole method
{  // This FMM class is ALMOST always used. 
	// Exception is for "layered" system, infinite in y & z.
   // Even then, the root cell needs mpole so we have total moment of system?? for now.

	// Also used for periodic FMM--note tho that writing image file is independent of 
	// the system of cells; could be separate program, but uses FMM functions.

public:
  FMM();// initilize some parameters to default values
  ~FMM();//deallocate memory occupied by the arrays
  void setup();//allocate and initialize all arrays
//added by DMAk
  // real calcKernels();  //calculate cubic Kernels and return delta - maximum difference
                            //between the old Kernels and the new ones
  // real compareAndReplaceKernels();  //compare newKernels with Kernels, returns difference
  void ReadKernels(string FName,std::map<int3,Kernel>*);
  mpole** MakeAvCoef(MVector half, short order);

//********************** data members: *************************
  fstream KernelsFile; //file to input or output (CalcCubicKernels=1) cubic Kernels 
  // These are now local to setup():
  //mpole *scratchMpole1, *scratchMpole2;  //scratch mpoles (to increase performance)
  mpole *scratchUpM; // used by updateMoment and FFM::setup
  //short NumComps; local to setup() now             //total number of components, see paper
   //the arrays needed for mpole:
  short *RecurseAxis;          //these arrays are needed for recursive calculations of
  real *RecurseFactor;      //Deriv. arrays
  short *RecurseMpole;         //the "previous multipole"
  short ***index;              //opposite to IndArr, gives i_n from vector n
  int3 *IndArr;//opposite to index, gives vector index n from i_n
  short *NumBelow;             //total number of components with order <=
  short **ConvSum;             //give i_{n+m} from i_n and i_m
  short *DerivTerms;           //give the number of p's for a certain order
  short **DerivPower;          //give all p's from i_n
  real **DerivCoef;         //F_n[p] in paper
  std::map<int3, Kernel> *CubeKer; //Kernels for calculation of the potential from
                             //the NearPartners - close smallest cells
  real MaxAngle2;           // square of max opening angle
//added by DMAk:
  bool NoInterRectKernels;  // 1 makes it use multipole field of other rectangles
  short UsedOrder, MaxOrder;  // maximum order of multipole moments
  short ExtraOrderD,MaxOrderD; // extra order for D, usually 1
#ifndef H_ONLY
  void WriteKernels();
  void print(char *, const short [], const short);//to output the array of integers
  void print(char *, const int3&);//to output the valarray
  void printMap (char *title, const std::map<int3, Kernel> & mapToPrint);
  void ReadScriptFMM(ifstream &);  //to read parameters from the script file
           //and calculate the array size NumBelow that is needed by ParseBuild

//the following functions are used in periodic FMM (DMApf)
    // see IEEE Trans. Mag. v. 39, p. 3478-80 (2003)
  void calcVextAndWriteImages (); //calculate Vext, find images of the root cell
  real calculateSymFactor (const int3& key);  //calculate W(key)=1/(2^nkey)
                               //where nkey is the number of nonzero key.values 
  void readAndSetupRootImg (); //read in the images and put them to root->partners

  mpole *Vext;               //mpole needed for periodic FMM (DMApf)
  real reducedSize[3];   //reduced size of the system (DMApf),can it be MVector? 
  double averageSize; //geometrical average of the size along periodic directions

  string OutName;
  //char OutName[MAXLENFMM+1]; // name for output
  fstream ImageFile;  //file to input (calculateExterior=0) or output (=1) images and Vext

  std::map<int3, Kernel> *newKernels;  //temporary storage for calculated Kernels
  void setupCenterFaces (cell *); //find the center faces recursively
  face *centerFaces[3][4];  //center faces, 3 - # of different NormaLs
                    //the 2nd index is to distinguish 4 faces with the same NormaL
            // These are used ONLY for calc kernels; 2010, may not work
  bool CalcCubicKernels;  //=1 if we calculate Kernels,


private:                 //(utility functions)
  void calcVshellAndWriteImages ();       //calculate Vshell and write down the images
  real calcExtOneIter (); //make one iteration for Exterior mpole calculation
  short coarseCharacter (const int3 &) const;  //coarse lattice character. function

//data members
  mpole *VextNew, *Vshell;
#endif  //  H_ONLY
}; // end class FMM


//specific to a system of uniformly magnetized cells:
class NearPartner //used to store NearPartner for smallest cells
{
public:
  LLGcell *cel;//cell that produces the potential
  Kernel *K; //Kernel for brute force calculations for smallest cells

  NearPartner(LLGcell *, Kernel*);  //constructor for M-based FMM
};

#endif  //   //FMM_H

