//this file contains lowest-level headers, to include in *.C 
#ifndef LOWEST_H
#define LOWEST_H

//#define _BETA // adds beta modules, TEMPORARY? where else?
#define _GROUPS
#define _NORMAL
#define MOREKERNELS
// #define WINDOW2
//#define ORBITS
//#define SEAPLUG

// #define COIN // Uncomment one of these 2 lines, OR do it as preprocessor command
// #define OPENGL

// note that _OPENMP is defined if OMP support is turned on in project 
// properties (Visual C++). Turn off there if desired?

#define DEBUG1 // comment this to get rid of M'static field calc prints; see also PD1
//#define DEBUG2 // comment this to get rid of (FMM,...) setup prints
//#define DEBUG3 // comment this to get rid of prints while evolving
#define DEBUG4 // prints while debugging Green function
#define DBG_ // "DeBug Green's fn", comment out line if not
#ifdef OPENGL
#define GRAPHICS
#endif  //  

#ifdef COIN
#define GRAPHICS
#endif  //    // so GRAPHICS is defined if EITHER of others is

// H_ONLY is defined in makefile (*.sln), NOT here, so we don't have to change Lowest.h to switch to H_ONLY.
// we have to change the makefile anyway.
#ifdef H_ONLY // and make sure other stuff is off
#undef _GROUPS
#undef _NORMAL
#undef COIN
#undef GRAPHICS // overrides above!
#undef OPENGL
#endif  //  H_ONLY

#include <iostream> // oddly, this must PRECEDE "using namespace.."
using namespace std; // needed for iostream,list,string,...
// could #include <list>,<string> here, but not needed in all *.cpp
#ifdef WIN32
  #include <process.h> // for _cexit()
#define MYEXIT(n) {cout<<"Press enter in this window to exit("<<n<<")"; cin.get(); exit(n);} // was _cexit() 
  // "exit() closes window unreadably" -- not with cin.get!!
#define EXIT1(ss) {cout<<"Error: "<<ss<<endl; cin.get(); exit(1);}
   // tho _cexit is a BAD solution, triggers many system errors
#else
#define MYEXIT(n) {cout<<"Exiting at MYEXIT("<<"n<<endl; exit(1);}
#endif

  #include <cassert>
#include <cstddef> // for g++
typedef float real; //or typedef double real;
#ifdef OPENGL // this is temporary, until include OpenGL header
typedef int GLuint;
#endif  //   OPENGL

// Now define types used in code
/* below works in VStudio 2010, not 2008 (no cstdint)
#include <cstdint> // this gives fastest with at least 8 or 16 bits(uint_fast16_t)
typedef uint_fast16_t CellIDType;
#define MaxCellID UINT_FAST16_MAX // 65535 if really 16 bits; or more
typedef uint_fast8_t RotType; // really need only 4 but 8 is smallest in cstdint
so instead revert to: */
#ifdef _OPENMP
typedef int CellIDType; // must be a signed int for OpenMP 2.0 to parallelize for loops. TODO: remove this if higher version becomes available
#define MaxCellID 2147483647 //2^31-1
#else
// typedef int CellIDType; // un-comment 2 lines if huge system w/0 OPENMP
// #define MaxCellID 2147483647 //2^31-1
typedef unsigned short CellIDType; // un-comment for systems under 65535 cells
#define MaxCellID 65535 // 2^16-1
#endif  //   _OPENMP
#define DIGITS 4
// in either cluster or windows
  #include <cmath>
  #define M_PI 3.14159265
#define d_ ','   // delimiter, used in << overloads and file output, to give .csv file
#define EOD '|' // end of data

#ifndef H_ONLY

#define WRITECHLINE {cout<<"ch="<<ch<<"=#"<<(short)ch; getline(My->paramFile,junk); cout<<", line="<<junk<<endl;}
	// My->paramFile.getline(junk, 80, '\n'); cout<<", line="<<junk<<endl;}
   // WRITECHLINE can only be used where ch and junk are both defined.
   // 2011: I don't know why 2 functions stream::getline and getline(stream,..)
   // usually we use getline().

typedef unsigned short RotType; // 16 bits; need only 4 for space rotations if no 3-fold rotation
// was unsigned char, 8 bits, but that prints funny! And probably not faster, 
// or even much more concise

// definitions of required constants.
#define mu_0 (4*M_PI/10)          // permeability in J'/nm/(uA)^2 (Arko)
#define kB (1.38e4)             // Boltzmann constant in J'/K (J' = 1e-18 joules)
// #define KOe_to_kA_per_meter (1e3/4/M_PI)
#define decibel 1.258925412
#define cuberoothalf 0.7937005259
#endif // !H_ONLY
#endif  //  LOWEST_H
