#ifndef _MVECTOR_HH_
#define _MVECTOR_HH_

#include "Lowest.h"
//#include "int3.h"
#ifdef COIN // for constructing MVector from SbVec3f only
#include <Inventor/nodes/SoTransform.h>
#endif  //  COIN
#define VDIMENSION 3

// using namespace std;

#include <fstream>
class int3 // create int3 first, is used by MVector functions
{
  friend ostream &operator<<(ostream&, const int3&);
public:
  short values[3]; //the displacement of the source cell. Must be integer to be
                 //used as a key for an STL map

//****************** member functions *******************
  int3(short x=0, short y=0, short z=0) //constructor
    {values[0]=x; values[1]=y; values[2]=z;}
  int3 (const int3& right)  //copy constructor
    {values[0]=right.values[0]; values[1]=right.values[1];
     values[2]=right.values[2];
  }
  void orient ();  //DMAk - orient the key so that 0<=x<=y<=z
  bool isZero () const;  //DMAk - returns 1 if it is (0,0,0)
  void SetToZero() {values[0]=0; values[1]=0; values[2]=0;}
  short sum() const { return values[0]+values[1]+values[2];}
  int Square() const { return values[0]*values[0]
        +values[1]*values[1]+values[2]*values[2];}
//******************* operations ************************
  int3 operator- (const int3&) const;  //subtraction operator
  int3 operator+ (const int3&) const;  //addition operator
  int3 operator/ (const short&) const;  //division operator
  const int3 & operator= (const int3&); //assignment
  short& operator[](short i) { assert ((i<3) && (i>=0)); return values[i];}
  const short & operator[](short i) const {assert ((i<3) && (i>=0)); return values[i];}
}; /// end class int3

class MVector{
public:

  // LIFECYCLE
  MVector(){
    for(short i=0 ; i<VDIMENSION;i++)
      val[i] = 0.0;
  }

#ifdef COIN
MVector(SbVec3f vec){
    for(short i=0;i<VDIMENSION;i++) val[i] = vec.getValue()[i];
  }
#endif  //  COIN

  MVector(const real* x){
    for(short i=0 ; i<VDIMENSION;i++)
      val[i] = x[i];
  }
  MVector (const short* n) {       //added by DMA, used for mpole::GetValue
    for (short i=0; i<VDIMENSION; i++)
      val[i] = n[i];
  }

  MVector(real x0, real x1, real x2){
    val[0] = x0;
    val[1] = x1;
    val[2] = x2;
  }
  MVector(real xyz){
    for(short i=0 ; i<VDIMENSION;i++)
      val[i] = xyz;
  }
  MVector(const MVector& v){
    for(short i=0 ; i<VDIMENSION;i++)
      val[i] = v.val[i];
  }
  MVector (const int3& key) { //(DMApf)
    for (short i=0; i<3; i++)
    val[i] = key.values[i];
}

  //
  ~MVector(){;}

  //added 05/16/00 by Xuebing
  inline void SetToArray(real *c);

  //added 05/16/00 by Xuebing
  inline void SetToZero(); 
  //added 7/21/04 by PBV 
  inline bool IsZero() const;
  inline bool Is111() const;
  // OPERATORS

  // MVector sum. ->out
  inline MVector operator+(const MVector& v) const;
  // ->in + return
  inline void   operator+=(const MVector& v);

  // MVector subtraction ->out
  inline MVector operator-(const MVector& v) const ;
  // ->in + return
  inline void operator-=(const MVector& v);
  
  // MVector times a real number ->out
  inline MVector operator*(real) const;
  // ->in  + return
  inline void operator*=(real);
  
  // MVector divided by a real number ->out
  inline MVector operator/(real) const;
  // ->in + return
  inline void operator/=(real);

  // MVector dot product ->out
  inline real operator*(const MVector& v) const;
  
  // negate MVector ->out
  inline MVector operator-(void) const;
  
  // MVector comparison ->out
  inline bool operator==(const MVector& v) const; 
  inline bool operator!=(const MVector& v) const;
  inline bool operator<=(const MVector& v) const;


  real Square() const {
    return (val[0]*val[0]+val[1]*val[1]+val[2]*val[2]);
  }

  inline real& operator[](short i){
    return val[i];
  }
  inline const real& operator[](short i) const{
    return val[i];
  }

  // OPERATIONS

  // MVector magnitude  ->out
  inline real Magnitude(void) const;
  
  /* dlete
  // random MVector
  // was static 
  void set_to_random_unit_MVector(void);
  void set_to_random(const MVector& );
  */

  //added by Xuebing, Rotate a MVector by 90 degrees 
  // need declare here; definition outside class
  //inline MVector Rot90(); now gone
 
  // inline void put_back_in_unit_box(); unused
  
  inline short largest(void) const;
  inline real get_largest(void) const;
  inline real get_smallest(void) const;
  //inline MVector square() const;
  inline MVector square_root() const;
  inline real get_componant(short) const;


  
  // print MVector
  void print(const char *) const;
  //void print_MVector(ofstream& stream) const;
  void read_MVector(ifstream&) const;
  void print(ofstream& fstrm, char *title) const; //added 06/09/00 by Xuebing

  // azimuthal angle
  real azimuthal() const;
  
  // angle theta
  real theta() const;
  
  friend inline MVector min_comp(const MVector&, const MVector&);
  friend inline MVector max_comp(const MVector&, const MVector&);

  friend ostream& operator<< (ostream&, const MVector&) ; 
  friend istream& operator>> (istream&, MVector&) ; 
  friend MVector operator * (const real, const MVector&);
  // ACCESS
  
  const real* get_val() const { return val; }
  real its_val(short i) const{ 
    assert (i>=0);
    assert (i<VDIMENSION);
    return val[i]; 
  }
  
  // INQUIRY

  void CopyTo(real& q1, real& q2, real& q3){
    assert( VDIMENSION > 2  );
    q1=val[0];  
    q2=val[1];  
    q3=val[2];
  }

  MVector units(const MVector& L);
  void dyadic_diagonal(const MVector& n);
  // delete static MVector RandomMVector(void); 
  MVector normalize(void); // defined inline below
  void normalizeTo(real);
  // MVector cross product. Be Very Careful: 
   // ^ has LOW precedence, so A+B^C is interpreted as (A+B)^C!! 
  MVector operator^(const MVector& v) const;
  
  // void print(FILE *, char *); // hope this isn't used

  ostream& iv_print(ostream& fout) const;
  
  real x() const {return val[0];}
  real y() const {return val[1];}
  real z() const {return val[2];}

  void write_to_array(real* a) const {
    a[0]=val[0]; a[1]=val[1]; a[2]=val[2];
  }
  
  void read_from_array(const real* a){
    val[0]=a[0]; val[1]=a[1]; val[2]=a[2];
  }
 
  //MVector converts
  real* pointer_to_array(){
   return val;
  }

      //added by DMA - returns the integer displacement of the cell,
      //needed for map
int3 ToInt (MVector quant)
{
  int3 temp;
  for (short i=0; i<3; i++)
  {temp.values[i] = (short)(val[i]/quant[i] + 9999.5) - 9999; // rounds to nearest integer
  if(abs(val[i]-temp.values[i]*quant[i])>0.001)
     {cout<<"Error, is everything a multiple of quantum?"<<endl; MYEXIT(1);
     }
  }
  return temp;
}

int3 ToInt (real quant)
{
  int3 temp;
  for (short i=0; i<3; i++)
  {temp.values[i] = (short)(val[i]/quant + 9999.5) - 9999; // rounds to nearest integer
  if(abs(val[i]-temp.values[i]*quant)>0.001)
     {cout<<"Error, is everything a multiple of quantum?"<<endl; MYEXIT(1);
     }
  }
  return temp;
}

protected:
private:
  // should be unsigned int
  real val[VDIMENSION];
}; // end class MVector

//
// INLINE METHODS
//
// MVector sum.
inline
MVector 
MVector::operator+(const MVector& v) const
{
  real temp[VDIMENSION];
  for(short i=0 ; i<VDIMENSION;i++)
    temp[i] = val[i] + v.val[i];
  return MVector(temp);
}

inline
void
MVector::operator+=(const MVector& v)
{
  for(short i=0 ; i<VDIMENSION;i++)
    val[i] += v.val[i];
}


// MVector subtraction

inline
MVector
MVector::operator-(const MVector& v) const{
  return MVector(val[0] - v.val[0],val[1] - v.val[1],val[2] - v.val[2]);
}

inline
void 
MVector::operator-=(const MVector& v){
    val[0] -= v.val[0];
    val[1] -= v.val[1];
    val[2] -= v.val[2];

}
  
// MVector times a real number.

inline
MVector 
MVector::operator*(real aux) const{
  return MVector(val[0]*aux,val[1]*aux,val[2]*aux);
}

inline
void 
MVector::operator*=(real aux)
{
  for(short i=0 ; i<VDIMENSION;i++)
    val[i] *= aux;
}

// MVector divided by a real number

inline
MVector 
MVector::operator/(real aux) const
{
  real temp[VDIMENSION];
  assert ( aux != 0.0 );
  for(short i=0 ; i<VDIMENSION;i++)
    temp[i] = val[i] / aux;
  return MVector(temp);
}

inline
void 
MVector::operator/=(real aux)
{
  assert ( aux != 0.0 );
  for(short i=0 ; i<VDIMENSION;i++)
    val[i] /= aux;
}
 

inline 
real 
MVector::operator*(const MVector& v) const
{
  real temp=0.0;
  for(short i=0 ; i<VDIMENSION;i++)
    temp += val[i]*v.val[i];
  //return val[0]*v.val[0]+val[1]*v.val[1]+val[2]*v.val[2];
  //cout.precision(16);
  //cout<<*this<<"   "<<v<<endl;
  //cout<<"dot prod"<<temp<<endl;
  return temp;
}


// negate MVector;

inline
MVector 
MVector::operator-(void) const
{
  real temp[VDIMENSION];
  for(short i=0 ; i<VDIMENSION;i++)
    temp[i] = -val[i];
  return MVector(temp);
}
  
// MVector comparison.

inline
bool
MVector::operator==(const MVector& v) const
{
  bool test = ( val[0] == v.val[0] )  ;
  for(short i=1 ; i<VDIMENSION;i++)
    test = test && ( val[i] == v.val[i] ) ;
  return test;
}

inline bool MVector::operator<=(const MVector& v) const // means ALL components <=
{ for(short i=0 ; i<VDIMENSION;i++)
{   //cout<<"Am in loop with i="<<i<<", val[i]="<<val[i]<<", v.val="<<v.val[i];
    if( val[i] > v.val[i]) return 0; // false
	//cout<<"did not return 0 becuase not >"<<endl;
}
  return 1; // true
}

inline
bool
MVector::operator!=(const MVector& v) const
{
  bool test = ( val[0] != v.val[0] ) ;
  for(short i=1 ; i<VDIMENSION;i++)
    test = test || ( val[i] != v.val[i] ) ;
  return test;
}

// MVector magnitude

inline
real 
MVector::Magnitude(void) const
{
  real res = val[0] * val[0] ;
  for(short i=1 ; i<VDIMENSION;i++)
    res += ( val[i] * val[i] ) ;
  return (sqrt(res));
}

//inline MVector MVector::Rot90() // ccwise about z axis
//{  return MVector(-val[1], val[0], val[2]);}

/* this is unused
inline 
void 
MVector::put_back_in_unit_box()
{
  for(short i=0 ; i<VDIMENSION;i++)// was i=1!!!
    {
      if( val[i] >   0.5) val[i] -= 1.0;
      if( val[i] <  -0.5) val[i] += 1.0;
    }
}
*/
inline 
short
MVector::largest(void) const
{ 
  short Direction=0;
  for(short i=1 ; i<VDIMENSION;i++)
    if( fabs(val[i]) > fabs(val[Direction])) 
      Direction = i;
  return Direction;
}

inline 
real
MVector::get_largest(void) const
{ 
  short Direction=0;
  for(short i=1 ; i<VDIMENSION;i++)
    if( fabs(val[i]) > fabs(val[Direction])) 
      Direction = i;
  return val[Direction];
}

inline 
real
MVector::get_smallest(void) const
{ 
  short Direction=0;
  for(short i=1 ; i<VDIMENSION;i++)
    if( fabs(val[i]) < fabs(val[Direction])) 
      Direction = i;
  return val[Direction];
}

//  inline 
//  MVector 
//  MVector::square() const
//  {
//    real temp[VDIMENSION];
//    for(short i=0 ; i<VDIMENSION ; i++)
//      temp[i] = val[i]*val[i];
//    return MVector(temp);
//  }

inline 
MVector 
MVector::square_root() const
{
  real temp[VDIMENSION];
  for(short i=0 ; i<VDIMENSION ; i++)
    {
      assert( val[i] >= 0 );
      temp[i] = sqrt(val[i]);
    }
  return MVector(temp);
}

inline
real 
MVector::get_componant(short i) const 
{
  assert ( i < VDIMENSION );
  assert ( i >= 0 );
  return val[i]; 
}

// azimuthal angle

inline 
real 
MVector::azimuthal() const
{
  assert ( VDIMENSION > 2 );
//Here the azimuthal angle phi calculated between 0 and pi.
  if      (val[0] < 0)                   return M_PI+atan(val[1]/val[0]);
  else if (val[0] > 0 && val[1] <  0)    return 2*M_PI+atan(val[1]/val[0]);
  else if (val[0] > 0 && val[1] >= 0)    return atan(val[1]/val[0]);
  else if (val[1] > 0)                   return M_PI/2;  // x=0
  else if (val[1] < 0)                   return 3*M_PI/2;// x=0
  else return 0.0;                                     // x=0, y=0
}

// angle theta
inline 
real 
MVector::theta() const
{
  assert ( VDIMENSION > 2 );
  return acos(val[2]/Magnitude());
}



inline MVector MVector::units(const MVector& L)
{
  return MVector( short(val[0]/L.val[0]), short(val[1]/L.val[1]), short(val[2]/L.val[2]) );
}

inline void MVector::dyadic_diagonal(const MVector& n)
{
  val[0] *= n.val[0]; val[1] *= n.val[1]; val[2] *= n.val[2];
}

inline void MVector::SetToArray(real* c)
{
  val[0]=c[0]; val[1]=c[1]; val[2]=c[2];
}

inline void MVector::SetToZero() 
{
  val[0] = 0.; val[1] = 0.; val[2] = 0.; 
} 

inline bool MVector::IsZero() const
{
  return (val[0]==0.0)&&(val[1]==0.0)&&(val[2]==0.0);
}
 
inline bool MVector::Is111() const {return ((val[0]==val[1]) && (val[1]==val[2]));}

inline MVector MVector::normalize(void)
{
  //cout.precision(16);
  //cout<<"before norm"<<*this<<endl;
  // could test mag=0, but takes time and is done often. May have to track down error.
  *this /= this->Magnitude();
  //cout<<"after norm"<<*this<<endl;
  return *this; 
}

inline void MVector::normalizeTo(real Mag)
{
  // could test mag=0, but takes time and is done often. May have to track down error.
  *this *=(Mag/Magnitude());
  //cout<<"after norm"<<*this<<endl;
}

inline MVector MVector::operator^(const MVector& v) const
{
  return MVector(val[1]*v.val[2] - val[2]*v.val[1],
		val[2]*v.val[0] - val[0]*v.val[2],
		val[0]*v.val[1] - val[1]*v.val[0]);
}

//inline void MVector::print(FILE *fp, char *title)
//{
//  fprintf(fp, "%s (%10.6f, %10.6f, %10.6f)", title, val[0], val[1], val[2]);
//}

// #define MVector_zero MVector(0.,0.,0.)
#define MVector_x    MVector(1.,0.,0.)
#define MVector_y    MVector(0.,1.,0.)
#define MVector_z    MVector(0.,0.,1.)
#define MVector_xyz  MVector(1.,1.,1.)
#define MVector_reference MVector_y

#endif  //    // _MVECTOR_HH_
