// AlaMag -- Copyright(C) 1999-2009 Pieter B. Visscher, see main.cpp for notices
// This is NorModes.cpp
#include "Lowest.h"
#include "cell.h"
#include "Systm.h"
#include "NorModes.h"
#ifdef _GROUPS
#include "_Groups.h"
#endif

extern Systm* My;
// This assumes system has 4-fold symmetry about z axis, or some subgroup.

NorModes::NorModes()
{ // Initializing Normodes
  //SeedMode=NULL;
  DriverMode=NULL;
  PoPt=0;
  PodPt=0;
  PodPt_prev=0;
} 

ostream& operator<< (ostream& fout, const ModeCx& Mo) 
{  fout<<"NorMode "<<Mo.ModeNest->NesName<<" "<<Mo.ModeGroup<<" "<<Mo.Period<<" "<<Mo.JCrit<<endl;
   fout<<Mo.ModeRe<<'!'<<endl; // mark end of config
   fout<<Mo.ModeIm<<'!'<<endl;
   return fout;
}

istream& operator>> (istream& fin, ModeCx& Mo) 
{  string junk,Nes;
   fin>>junk>>Nes>>Mo.ModeGroup>>Mo.Period>>Mo.JCrit>>Mo.ModeRe;
   //cout<<"Reading ModeCx, header="<<junk<<", nesting="<<Nes<<endl;
   My->CheckScript(junk,"NorMode");
   Mo.ModeNest=My->NH->NesNamed(Nes);
   //cout<<"peek="<<(char)fin.peek()<<", #="<<fin.peek()<<endl;
   //getline(fin,junk); 
   //cout<<"junk="<<junk<<", peek="<<(char)fin.peek()<<", #="<<fin.peek()<<endl;
   //My->CheckScript(junk,"!");
   //cout<<"junk="<<junk<<", peek="<<(char)fin.peek()<<", #="<<fin.peek()<<endl;
   fin>>Mo.ModeIm; // may be at EOF! if(!fin.good()) EXIT1("Error reading ModeCx");
   return fin;
}

void ModeCx::WriteHeader(ostream& Fil) // all but configs
{  Fil<<"ModeCx name="<<ModeNest->NesName<<", Gp="<<ModeGroup<<
    ", Per="<<Period<<", JCrit="<<JCrit<<endl;
}
void NorModes::setNormalModeFromScript(ifstream& Fil) // initialize normal mode and read calculation info from paramFile
{   //read Driver mode, perturbation mode, symmetry info, group num 
	string junk, DriverModeS;
    // reading from param file
	Fil>>junk>>DriverModeS;
	// FInd driver mode
	DriverMode=My->Modes[Mode::ModeNamed(DriverModeS)];
	if(DriverMode->ConstrainIndex<0) EXIT1("To calculate normal modes, need constrain driver mode first");
	//was if(My->Modes[DriverMode]->ConstrainIndex<0) EXIT1("To calculate normal modes, need constrain driver mode first");
	assert(My->cons);
    // Earlier in script file, should have created constraint on driver mode
	// constructor set MaxField, but perhaps too large for normal mode calc:
	My->cons->MaxField *=0.5;
	cout<<"Decreasing MaxField to "<<My->cons->MaxField<<endl;
	junk=My->runID+".mIn";
	ModesInF.open((My->ScriptDir+junk).c_str());
	ModesOutF.open((My->ScriptDir+My->runID+".mOut").c_str());
	if(!ModesOutF.good()) EXIT1("can't open modes output file "+My->runID+".mOut");
    if(ModesInF.good()) 
		{cout<<"Reading file "<<junk<<" from "<<My->ScriptDir<<"\\"<<endl;
	     //ModesInF>>junk; cout<<"read junk="<<junk<<endl;
	     ModesInF>>Meq; // read equilibrium config.
		 //getline(ModesInF,junk); now done inside >>
		 //if(junk[0]!='!') EXIT1("Error reading equilib. config.");
		 //EXIT1("Haven't finished code here");
	    }
	else
	{  cout<<"Didn't find file "<<junk<<" in "<<My->ScriptDir<<", starting from present config"<<endl;
	   	Meq.saveM(); // Get current magnetization from all the cells
	}
	ModesOutF<<Meq<<'!'<<endl; // whether read in or not; ! is delimiter, alone on line
}

  void NorModes::StartNewModeCx(ifstream& Fil)
  {// read info on new mode
	string junk, SeedModeS,nestS;
	Mode* SeedMode;
	short Gp1; real perturbation;
	// reading from param file
	Fil>>junk>>nestS>>Gp1>>SeedModeS>>perturbation;
	My->CheckScript(junk,"nextNorm");
	CurModeCx = new ModeCx;	// start new complex mode
	if(ModesInF.good() && ModesInF.peek()==(int)'N') // can read this mode from file
	{  
		cout<<"Trying to read new ModeCx from file"<<endl;
		ModesInF>>*CurModeCx; //FAILS to COMPILE if omit "*"
	   if(ModesInF.bad()) EXIT1("Error reading mode file");
	   // bad!=!good! (bad is not !good)
/*	was:	ModesInF>>junk>>nestS1>>Gp1>>SeedMode1;
	   My->CheckScript(junk,"NorMode");
*/
	   if(CurModeCx->ModeNest->NesName!=nestS) EXIT1("Nests don't match");
	   if(Gp1!=CurModeCx->ModeGroup) EXIT1("groups don't match");
/*
       if(SeedMode1!=SeedModeS) EXIT1("Groups don't match");
	   ModesInF>>CurModeCx->ModeRe>>CurModeCx->ModeIm; 
	   getline(ModesInF,junk);
	   if(junk[0]!='!') EXIT1("Error reading ModeCx");
*/
	   cout<<"Read "; CurModeCx->WriteHeader(cout); 

	   	ModeListCx.push_back(*CurModeCx);// put CurModeCx to back of the list
		  // (no easy way to combine this with push_back after creating from scratch)
		assert(ModesOutF.good());
		ModesOutF<<*CurModeCx; // write to file

	   My->CBMode=My->READ_COMMAND;	// ready for next mode.
	   return;
	}
	// if we get here, we need to create this mode from scratch
	cout<<"Didn't find ModeCx in file, creating from scratch"<<endl;
    SeedMode=My->Modes[Mode::ModeNamed(SeedModeS)]; 
	CurModeCx->ModeNest=My->NH->NesNamed(nestS);
	if(CurModeCx->ModeNest==My->NH->Nestings.end()) EXIT1("Normal Modes calculation setup: Could not find nesting");
	CurModeCx->ModeGroup=Gp1;
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++){// turn the seed mode to a perturbation config file
		LLGcell* C1=My->CellFromID[i]; // including proper scale factor "perturbation"
		My->CalcShapeFCellJustNormalMode(C1, SeedMode);
		Po.ConfigArr[i] = SeedMode->ShapeF*(C1->rect->Ms*perturbation/C1->volume());
	}
	Pmag=Po*Po; // calculate Po^2
	//add perturbation to the current magnetization and save
	Mnow=Meq+Po; // (note (Meq+Po).Restore.. is shorter, but allocating config may be slow)
	Mnow.restoreAnd(); // restores AND renormalizes & CalcAllFromM
    assert(My->cons); // set if the Driver mode contraint has been set 
	My->cons->SetmWantToPresent(); // use present value of mz; control with "perturbation"
// got too small ampl. if old seed norm used.  Now reset seed ampl for each new modeCx.    
	My->cons->InitDms();
	ProjectOutAndEnforceSymmetry(); // initially, & also after every cycle
	UpdateNormMode(); // done at every step, monitor PoPt
	//set up all other variables
	Pt_prev_cycle.ConfigArr=Pt.ConfigArr;
	Pt_prev.ConfigArr=Pt.ConfigArr;
	//set time and counter 
    t_prev=My->t;
	counter=0;
	  
	 //set CBMode to run with criterion being CYCLE_OVER
	 My->CBMode=My->RUN;
	 My->TermCrit=My->CYCLE_OVER;

	 cout<<"New ModeCx set up; there are "<<ModeListCx.size()<<" previous ones."<<endl;
	 My->Pausing=true; My->Repeat=false; // let user absorb this news, then hit right arrow 
  }

void NorModes::UpdateNormMode(){ // get component along Po, to test for cycle end 
	//Update PoPt PodPt
	PodPt_prev=PodPt; // save previous time step podpt
	Mnow.saveM();
	Pt=Mnow-Meq;
	PoPt=Po*Pt;
	Mtnow.saveDelM();
	dPt=Mtnow;
	PodPt=Po*dPt;
}


void NorModes::CheckForCycleEnd()
{  
	double a, b, t_cross_0, steps;
	Config del_p;

	UpdateNormMode();
	 
	 // if PodPt crosses teh x axis going from pos to neg, renormalize normal mode and update all info
	 if(PodPt_prev>0 && PodPt<0 && (My->t > (t_prev+0.005))){

		// adjusting dt now by linear interpolation PodPt=a*t +b
	    a=(PodPt-PodPt_prev)/(My->dt);
	    b=PodPt-a*My->t;
	    t_cross_0=-b/a; // exact time when PodPt crossed zero 
	    steps=(My->t-t_prev)/(My->dt);
	    My->dt=(t_prev+2*(t_cross_0-t_prev)-My->t)/steps;// dt adjusted 
		My->SetEvolParams(); My->Changed_dt=true; 
		CurModeCx->Period=t_cross_0-t_prev; // set the period in the CurModeCx class
	    t_prev=t_cross_0; // T-prev will be used again in next cycle 

		 
	  
		Config A;
		//extrapolate P(t) when it exactly crosses t_cross_0
	    A =(1.0/My->dt)*(Pt - Pt_prev);
        Pt = Pt + (t_cross_0-My->t)*A; 

		

		//Check for cocnvergance 
	    if(!My->cons->Calc_dm()){	 //check in delM is not too big 
			
			cout<<"M_St="<<DriverMode->MVal<<" is close to mWant="<<My->cons->mWant[DriverMode->ConstrainIndex]<<". Checking for cycle convergence.";
			del_p=Pt_prev_cycle-Pt;

            if(((del_p*del_p)/Pmag)<(My->RotTol*My->RotTol)){
                 counter=counter+1; // count number of consecutive converged configs
				 cout<<"Counter ="<<counter <<", error^2="<< (del_p*del_p)/Pmag<<", want < "<<(My->RotTol*My->RotTol)<<endl;
			}
		} else {
			counter=0;
		}

		cout<<endl;
		//just for debugging purpose
		cout<<"Counter ="<<counter <<", error^2="<< (del_p*del_p)/Pmag<<", want < "<<(My->RotTol*My->RotTol)<<endl;
			
        // if not converged, stary new cycle 
        if(counter==3){
             cout<<"M_St is close to mWant and del_p has converged for the last 3 cycle. "<<endl;
			 My->TermCrit=My->QUARTER_CYCLE;
			 My->tstop=My->t+CurModeCx->Period/4; 
			 My->DisplayTime=true; // force display
			 My->Pausing=true; My->Repeat=false; // let user absorb this news, then hit right arrow 

			// CurModeCx->ModeRe.print();
		} else {
		StartNewCycle();
		 // do symmetry projection 
		ProjectOutAndEnforceSymmetry();
		}
		 	
		//save p(t) for next cycle
		Pt_prev_cycle= Pt;

		if(My->Strobe) My->DisplayTime=true;

	 }

	 Pt_prev= Pt;	 
}

void NorModes::ProjectOutAndEnforceSymmetry(){
			
	    // Project out Curmode against all the previous modes saved in ModeListCx
        std::list<ModeCx>::iterator PrevModeCx;
		CurModeCx->ModeRe.saveM();
		CurModeCx->ModeRe=CurModeCx->ModeRe-Meq;
	    for(PrevModeCx=ModeListCx.begin();PrevModeCx!=ModeListCx.end();PrevModeCx++){
		    if(CurModeCx->ModeNest==PrevModeCx->ModeNest){
			 CurModeCx->ModeRe.ProjectOut(PrevModeCx->ModeRe);
			 CurModeCx->ModeRe.ProjectOut(PrevModeCx->ModeIm);
		    }
	    }
		
		CurModeCx->ModeRe=CurModeCx->ModeNest->Project(CurModeCx->ModeRe,CurModeCx->ModeGroup);
		
		Config M1=CurModeCx->ModeRe+Meq;
		M1.restoreAnd(); 
/*
		ofstream CFsave;
			CFsave.open("project1.config");
			CurModeCx->ModeRe.ExporttoFile(CFsave);
	        CFsave.close();
		if(ModeListCx.size()>0){
			CurModeCx->ModeRe.ProjectOut(PrevModeCx->ModeRe);
			CFsave.open("project2.config");
	        CurModeCx->ModeRe.ExporttoFile(CFsave);
	        CFsave.close();

		}
		*/

}


void NorModes::StartNewCycle(){
	   // adjust field before starting new cycle 
	//if(ModeListCx.size()<1){
	   if(!(My->cons->AdjustFields())) EXIT1("AdjustFields failed, quitting");
       My->cons->InitDms(); // initialize derivatives, to integrate over cycle.  
//	}
}

// If you have reached a quater of a cycle after the convergance, save Imaginery part of the normal mode, set critical current and push 
// CurModeCx to the MOdelistCx
// Then set new CurModeCx unless there is enough MOdeCx in the list or the critical current is too high
void NorModes::QuarterEnd()
{
	CurModeCx->ModeIm.saveM();
        CurModeCx->ModeIm-=Meq; // saving imaginary part 
		CurModeCx->JCrit = DriverMode->HVal;  // set critical current 
		// was CurModeCx->JCrit = My->Modes[DriverMode]->HVal;  // set critical current 
		ModeListCx.push_back(*CurModeCx);// put CurModeCx to back of the list
		cout<<"Done with ModeCx: period="<<CurModeCx->Period<<", JCrit="<<CurModeCx->JCrit<<endl;
		assert(ModesOutF.good());
		ModesOutF<<*CurModeCx; // write to file

		// now each script line asks for one mode; done with this one, read new
		My->CBMode=My->READ_COMMAND; 

/* previous code:
        if((abs(CurModeCx->JCrit) > Max_critical_current) || ( ModeListCx.size()>=3)){
			// Go back to reading script if the critical current limit has been passed or enough normal modes saved in teh list
			My->CBMode=My->READ_COMMAND; 
		} else{
			// Or else start new normal mode calculation with information from the previous one
			StartNewModeCx();
		}
*/
}
