// AlaMag -- Copyright(C) 1999-2009 Pieter B. Visscher, see main.cpp for notices
// This is Systm.cpp
#include "Lowest.h"
#include "cell.h"
#include "Systm.h"
#include "FMM.h"
#include "mpole.h"

#ifndef H_ONLY
#include "Outputs.h"
// #include "pulse.h" // is in system.h
#include "acfield.h"
#include "display.h"
#endif  //  H_ONLY

#include <sstream>
#include <string>

#ifdef _GROUPS
#include "_Groups.h"
#endif  //  

#ifdef ORBITS
   #include "Orbits.h"
#endif  //  ORBITS

#ifdef CONTOURS
   #include "Contours.h"
#endif  //  

#ifdef _NORMAL
#include "NorModes.h"
#endif  //  

#ifdef H_ONLY
#include "SeaPlug.h"
#endif  //  

#ifdef COIN
  static void mySensorCallback (void *data, SoSensor *)
  {  // not part of a class!
	  //cout<<"Entering mySensorCallback"<<endl;
    if (((Systm*)data)->HandleCallback())
    return; //if returns 1 display scene & repeat
    else
      MYEXIT(60);
  } // "Step OVer" here fails, use F5 run
  extern void myKeyPressCB(void*, SoEventCallback*);
#endif  //  COIN
#ifdef OPENGL
  static void GL_Callback() 
  {  // counterpart of mySensorCallback for COIN
     // replace by standard freeGLUT name?
	 cout<<"GL_Callback() not implemented"<<endl;
	 if(My->HandleCallback()) return; //if returns 1 display scene & repeat
     else MYEXIT(66);
  }
#endif  //  OPENGL

#ifdef DEBUG4
  bool Systm::PD4=false; // controls UpdateStat error messages
#endif  //  

#ifndef H_ONLY
extern unsigned long randint; //defined in Random.cpp
extern const string HDispLabel[NDisp+1];

unsigned long getSeed(); // defined in Random.cpp 
void setSeed(unsigned long); // defined in Random.cpp

#define FileLen 160 
// maximum length of init. cond. file name, runID, and movie file name
  // Following functions are not part of a class:

void ChangeInc(real& x, bool Up) 
{
  
  //if(!My->Repeat) 
	{if(Up)cout<<"In"; else cout<<"De"; 
    cout<<"crease(in ChangeInc) "<<x;}
  real mant=x;
  if(x==0) {cout<<"Error: initial increment=0?"<<endl; MYEXIT(37);}
  while (mant>=9.9) mant*=0.1; // sometimes 9.99999!
  while (mant<1.0) mant*=10; // now 1<=mantissa<10
  real pow10=x/mant;
  if(Up)
	  if(mant>4) mant=10; // usually was 5
	  else if(mant>2) mant=5;
	  else mant=2.5;
  else // down
	  if(mant<2) mant=0.5;
	  else if (mant<4) mant=1.0;
	  else mant=2.5;
  x=mant*pow10;
  // if(!My->Repeat) 
  cout<<" to "<<x<<endl;
} 
real ReadReal() // allows it to be INFINITY
{ My->paramFile>>ws; // get rid of white space
  if(My->paramFile.peek()=='I') 
    {string junk; My->paramFile>>junk; return My->Infty;}
  else {real x; My->paramFile>>x; return x;}
}

void Systm::HandleError()
{ if(IgnoreError) return;
  cout<<"press 'i' in this window & enter to ignore further errors, 'p' to ";
  cout<<"continue pausing on error"<<endl;
  char ch;
  cin.get(ch);
  switch(ch)
  {
  case 'i': IgnoreError=true; cout<<"NO LONGER PAUSING ON ERRORS"<<endl; break;
  case 'p': break;
  default: cout<<"incorrect key "<<ch<<" pressed, quit program"<<endl;
  }
}

#endif  //  H_ONLY

void CleanUpSmall(real x) {if(fabs(x)<2*My->MachineAcc) x=0.0;}

short IntRatio(real x, real y) // returns ratio, or 0 if not integer
{  real rr=x/y; short r=(short)rr; real remain=rr-r; // assumes y!=0!
   if (abs(remain)>0.01) return 0; else return r;
} 

#ifdef GRAPHICS
void Read12() // for debuggin
{  char* char12;
   char12= new char[12];
   My->movIn.read(char12,12);
   cout<<"Read12:"<<endl;
   for (int i=0;i<12;i++) 
   {  cout<<(int)char12[i];
      if(char12[i]>'A') cout<<"="<<char12[i];
	  cout<<endl;
   }
   delete char12;
}
#endif  //  

Systm::Systm(){
  num_threads=1; //will be increased/decreased as necessary to inform recursive functions how many threads they have
  NRects=0;
  // cout<<"MaxCellID="<<MaxCellID<<endl; 
  // use with <cstdint>: cout<<"sizeof(uint_fast16_t)="<<
    //sizeof(uint_fast16_t)<<",UINT_FAST16_MAX="<<UINT_FAST16_MAX<<endl; 
  if(MaxCellID!=(CellIDType) (pow((double)2,(int)(sizeof(CellIDType)*8) )-1) )
    {cout<<"Inconsistent with size of CellIDType, "<<sizeof(CellIDType)<<
	 " 8bit bytes"<<endl; cout<<". <cstdint> gives this error"<<endl; }
#ifndef _OPENMP // is only case where we use short int
  if(MaxCellID!=(65535+NRects)) // +NRects=+0 is just to avoid constant-expression warning
  {  cout<<"MaxCellID="<<MaxCellID<<", but for unsigned short int, should be 65535"
     <<endl; MYEXIT(79);
  }
#endif !_OPENMP
#ifdef COIN
   rootPhong=rootBase=rootReal=NULL;
   mySensor=NULL;
   //TimeText=ExchText=IntEnergyText=EnergyText=RotText=NULL;
   UpVector=NULL;
#endif  //  COIN
   
#ifdef GRAPHICS // initialize some stuff needed at top
    Pausing=false; // default, can change in script
	Strobe=false;
	ShiftIsDown=ControlIsDown=AltIsDown=0; // no one wd. start program w/ down, tho no guarantee
	for (short i=0;i<MaxLocalColor;i++) cell::HDispNumb[i]=NULL; // was HDispText
    //TimeFormat[0]='\0'; // empty string, can't just set TimeFormat="t=..."
    //strcat_s(TimeFormat,"t=%3.1f ns"); // default format to use before dt set
	FieldLimit=1.0; // just so nonzero
	gameMode=false; 

    TermStrings[0]="TIME"; // must agree with order in systm.h
	TermStrings[1]="CYCLE_OVER";
	TermStrings[2]="QUARTER_CYCLE";
    TermStrings[3]="MODEVAL";
    TermStrings[4]="MX";
    TermStrings[5]="LOOP_ORD";
    TermStrings[6]="CALC_GREEN";
    TermStrings[7]="NONE";



//added by DMAm & DMAs
  Movname=""; // used as flag that there isn't one
  this_frame=0; //current frame
  frame_offset=frame_size=0;
  numberSkipFrames = 1; //number of frames to skip
  Repeat = 1; // if not reset by P command, don't pause after each frame
  //ShowWatchedMStatic=ShowWatchedHk=ShowWatchedExch=0; // now just use MStaticField etc.
  //ShowWatchedTot=0; 
  externalField=MStaticField=ExchField=AnisField=TotField=WatchedM=NULL;//used as init. flags 

#else %GRAPHICS // no graphics
	  cout<<"compiler directive GRAPHICS is off, no graphics used."<<endl;
	  // cin.get(ch); // remove this for real batch work.
#endif  //   |GRAPHICS
#ifdef DBG_
     DBG=NULL; // to test if(DBG)
#endif //DBG_

  MyFMM=NULL; // may be necessary to create successfully????
  WatchedCell=NULL; WatLLGPart=NULL; 
  PeriodVec.SetToZero(); // may be changed later, but be sure it's initialized
#ifndef H_ONLY
  Version=1.96;
  nt = 0;
  t=0.0; tDisp=0.0;
  Infty=1.0/t; // infinity; also used later as uninitialized flag:
  WritingMovie=false;
  //cout<<"Infty address="<<&Infty<<"<Infty="<<Infty<<endl; // prints as 1.#INF (was #IO??)
  // can use in test: Failure cout<<"yes"<<endl; else cout<<"No"<<endl;
  Ext_Field1=NULL; Ext_Field2=NULL; MyMedField=NULL;
  StepMode=0;
  NudgeMode=0; //codes for nonexistence, index starts at 1
  Increment=100; // field
  ErrDM=99; DMTol=0.002; hTol=0.1; MaxIters=1000; IgnoreError=false; // defaults, usually read in too
  MaxDM=9999.; // was Infty; //was 9e9;// updated in evolve
  Changed_dt=false;
  fieldScale=-9999999.; // initialize for debugging
      // would be safer to set =Float.NaN (or define real.NaN?)
  OrdFile=NULL;
  CBMode=RUN; // default
  TermCrit=NONE; // termination criterion
  movieVersion = 2.0;   //version that this program can read and write, version 2.0   01 jul 2013
  NModes=0; for(int i=0;i<=MaxModes; i++) Modes[i]=NULL;
  // MyOrbs=NULL; 
  cons=NULL; MyInt=NULL; // constraint, integral classes
  GridInf1=NULL;


#ifdef _GROUPS

  NH=NULL; // NestHolder
  SymH=NULL; // rotation holder
#endif  //  _GROUPS

#ifdef _NORMAL
  My_norm=NULL;
#endif  //  

#ifdef ORBITS
   MyOrbs=NULL;
#endif  //  ORBITS

  //StartConfig=LineStart=NULL;
  //StepPtr=Step2Ptr=NULL;  // for stepping
  XportInt=31000; // max short integer?
  WriteFileInt=0; XportConsole=0;
  Xpor2ing=0;// Xport=NULL; if pointer, but that screws up.
  randint=314159265; // random number seed
  CompAvg=0; 
  Proj=1; // this is very old, affects unused update_q only
  // movie=0; // usually don't write movie frames
  use_q=0; // use dm/dt instead of dq/dt
  Heun=1; // use Heun algorithm unless demonstrating Euler
  // restore this if use avgs!  
  // for (short i=0; i<Max_Avgs; i++) Avgs[i]=WatchedAvgs[i]=NULL;
  ACExt_FieldPtr = NULL;
  Layered=false;
  H_bias.SetToZero();
//MyMR=NULL;// magnetoresistance; assume you want to output it

#endif  //  H_ONLY

  } // end constructor

Systm::~Systm() {
	// if(Xport) delete Xport; 
    }

// Systm *My = new Systm(); // errors if this is in main()???

void Systm::CreatePartners ()
{
  root->Partners.push_back(DispCell(root, int3(0,0,0)));
  assert(MyFMM);
#ifndef H_ONLY
  if (!(MyFMM->CalcCubicKernels) && MyFMM->ImageFile
     && ((periodic[0]) || (periodic[1]) || (periodic[2]))) //DMApf
    MyFMM->readAndSetupRootImg(); // AFTER geometry, pushes onto root's partner list
#endif  //  H_ONLY

  cout<<"Calling CullPartners:"<<endl;
  #ifdef _DEBUG
  double begin = omp_get_wtime(); // remember to turn of _DEBUG & all 
    // other debugging directives in Lowest.h, for timing tests
  root->CullPartners(); //recursively generates all partner lists
  double end = omp_get_wtime();
  cout<<"CullPartners took "<<end-begin<<" seconds."<<endl;
  #else
  root->CullPartners(); //recursively generates all partner lists
  #endif  //  
} // end CreatePartners

MVector Systm::Displace(MVector V, const int3 disp1) //short* disp1)  
// displaces vector by "disp1";
{
  //assert(PeriodVec[0]);
  V[0]+=disp1[0]*PeriodVec[0];
  V[1]+=disp1[1]*PeriodVec[1];
  V[2]+=disp1[2]*PeriodVec[2];
  return V;
}

bool Systm::almost_same(real x, real y)//added by DMApf to compare two real numbers
{
  if ((fabs(x) < 0.000001) && (fabs(y) < 0.000001))
     return 1;  //both are zero
  else
  if (fabs(x/y - 1.) < 0.00001) // seems ok if y=0? gives infinity, no error?
    return 1;   //both are non-zero but equal
  else
    return 0;
} // end almost_same()

bool Systm::almost_same(MVector M1, MVector M2)
{  for (short i=0;i<3;i++) if(!almost_same(M1[i],M2[i])) return false;
   return true;
}

#ifndef H_ONLY

void Systm::ListCells(ostream& fstrm)
{   fstrm<<"List of cells:"<<endl;
for (CellIDType i=0;i<LLGcell::NextSmCellID;i++)
		fstrm<<"#"<<i<<":"<<CellFromID[i]<<endl;
}

real Systm::MinCos() // returns cosine of largest angle
 { real cos1, cos = 1.0; // want min cos for max angle
   for(CellIDType i=0;i<LLGcell::NextSmCellID;i++)
   {LLGcell* c1=CellFromID[i]; assert(!c1->child0); // sanity check
    for (short faceN=0; faceN<6; faceN++) 
      { //loop faces to get neighbors
       //need to find neighbor cell on the other side of this face
         cell *nbr;
		 //cout<<c1->cellfacelist[0].myFace->numFace<<endl;
         nbr = c1->cellfacelist[faceN].myFace->cellList[c1->cellfacelist[faceN].out].cel;
         if(nbr) // see if nbr is in different rectangle & listed as Partner (meaning discontinuity allowed)
		 { assert(typeid(nbr)==typeid(LLGcell));
           LLGcell* LL=static_cast<LLGcell*>(nbr);
		   if(nbr->rect->index!=c1->rect->IntPartner[faceN]) // check angle
		     {cos1 = (c1->M*LL->M)/(c1->rect->Ms*nbr->rect->Ms);
		     if (cos1<cos) cos = cos1;
		     }
		  }
      } //end for faceN
   } // end for i
   return cos;
} // end MinCos

void Systm::prepareExit()  //output some information, close files, etc
{
#ifdef GRAPHICS
	if (movIn) //we have been reading a movie
    movIn.close();
#endif  //  GRAPHICS
    if (movOut) //if we have been writing a movie
    {
      if (WritingMovie) writeFrameBinary();
      movOut.close();
    }
  }

void Systm::SetEvolParams()
{ // called if any of dt, gamma, alpha, ... are changed
  // now depends on alpha, hence rect: gdt = gamma*dt/1+alpha^2;
  // dt determines the format of time display
#ifdef GRAPHICS
  // char* str1=new char; tried to find form get(str1,2) that would accept one character
  real unit=1.0/d; // was 1.0; now resolution is d*dt
  short places; // # places after decimal point
  for(places=0; unit>dt; places++)
     unit/=10;
  if (places>7) 
      {places=7; cout<<"dt too small to display all digits, press Enter"<<endl; cin.get();}
  //TimeFormat[3]=char(short('2')+places); // was initialized "t=%3.2f ns" in constructor
  //TimeFormat[5]=char(short('0')+places); 
  TimeDisp->prec=places; //cout<<"Setting time precision="<<places<<endl;
  // cout<<"TimeFormat="<<TimeFormat<<endl;
#endif  //  GRAPHICS
  if(!Repeat) cout<<"In SetEvolParams, dt = "<<dt<<endl;
  root->setRectangleParamRecurs();  //calc. parameters for all Rectangles
          // this is here because they depend on dt, damping,...
} // end function SetEvolParams

bool Systm::Cut_dt() // returns 1 if actually cuts
{  if(!Changed_dt) dt0=dt; // save original
   if(My->dt/My->dt0<0.001) {if(!Repeat) cout<<
	   "Already cut dt too much."<<endl; return 0;}
   if(!Repeat) cout<<"Cut_dt:"; ChangeInc(dt,0); // 0 means lower it; was dt*=0.5; 
   SetEvolParams(); Changed_dt=true; // Repeat=false; stopped too often
   return 1;
}

// generate a hash that should more or less uniquely identify a geometry, added with movieVersion 2.0
// should stop wrong pairings of movie and script files from working.
// puts geometry block in a string and hashes it
uint32_t Systm::calcFileHash(string filename) {
        string temp;
        uint32_t hashval = 0; char c;

        ifstream f (filename.c_str(), ios::binary);
        f>>noskipws;

        getline(f,temp,'\n');
        while (temp[0]!='G' || temp[2]!='o') { // reads "Geometry.."
            f>>ws;
            getline(f,temp,'\n');// Find geometry block
            if (temp.length() <= 2) temp = "Don't break bounds";
        }
        size_t begin = (size_t)f.tellg() - temp.length() - 1;


        getline(f,temp,'\n');
        while (temp[0]!='!' || temp[6]!='G') { // reads "! end Geometry block..."
            f>>ws;
            getline(f,temp,'\n');// Find geometry block end
            if (temp.length() <= 6) temp = "Don't break bounds";
        }

        size_t length = (size_t)f.tellg() - begin;
        cout<<"Calculating geometry hash with length of string "<<length<<endl;
        f.seekg((long)begin,ios::beg); // PV added (long) 1/2014
        // make the hash, simple algorithm adapted from http://www.cse.yorku.ca/~oz/hash.html
        for (unsigned short i=0; i<length; i++) {
            f>>c;
            if (c==' ' || c=='\n' || c=='\r' || c=='\t') continue; // skip whitespace characters, but count them
            hashval = ((hashval << 5) + hashval) + c; /* hash * 33 + c */
        }
        f.close();
        return hashval;
}
void Systm::Restore_dt0()
{  	if(Changed_dt) {dt=dt0; Changed_dt=false; SetEvolParams();} // restore
}

void Systm::ReadScript()
{ 
	//cout<<"Starting Systm::ReadScript"<<endl;
	if(!paramFile) 
	{cout<<"cannot access script file"<<endl; MYEXIT(61);}
    // Create some variables. You can create them in 'case' statements,
	// but NOT initialize: short i1=0 is illegal. short i1;i1=0 is ok?
	// and can only use in ONE case stmt unless in {}
    string temp, junk; // declare various temporaries (can't inside case stmt)
	// SbString charray; no, can't read in SbString
	char charray[101]; 
	// use iostream string instead of c-style string so we can manipulate
	// it to make other file names.
	// don't need: char c80[80]; // c-style string
	MVector ScratchVector;
	//cell *cellptr;
    char ch;
	real xdum,ydum,R,G,B; // multi-use dummies
	MVector vdum;
	short idum,jdum;
    float floatdum;
	paramFile>>ws; // eatwhite, get rid of tabs in 1st line
	if (paramFile.peek()==(int)'I') // want to override runID from .script name
	  {cout<<"Override default runID '"<<runID<<"' from .script filename with '";
	   paramFile>>junk>>runID; getline(paramFile,junk,'\n'); paramFile>>ws; 
	   cout<<runID<<","<<endl; }
// Now we know runID, ready to set up graphics:
#ifdef OPENGL
// set up opengl here: initialize window, camera, callbacks, etc., as for COIN below
#endif  //  OPENGL
#ifdef COIN // several pages
	//SoMaterial *EnergyMaterial=NULL;
	// Now actually set up graphics (cell system has been set up? NO!):
	  // This used to be in setupGraphics, then in ReadScript, then constructor, now ReadScript again! 
#ifdef WIN32 // SoWin uses different class and function names from SoXt:
	myWindow=SoWin::init((runID+": MagVis (U of Alabama Micromagnetic Simulator)").c_str());
	// cannot create any "So" objects until now?
    myViewer=new SoWinExaminerViewer(myWindow); // this may not be needed until later
	//  NEW:
#ifdef WINDOW2
	Win2 = new Window2; // create plot window
#endif // WINDOW2

#else %WIN32
	myWindow=SoXt::init("main"); // if ever use SoXt, need fix name
    myViewer=new SoXtExaminerViewer(myWindow);
#endif  //   |WIN32
        if(!myWindow) // do we need to initialize =NULL?
		  {cout<<"Couldn't create window"<<endl; MYEXIT(67);}
		assert(myViewer); // worked ok
		// rest of setup is still under 'D'isplay, below.
		// This stuff moved from setupGraphics:
        rootReal = new SoSeparator;		
		// cout<<"created rootReal at "<<rootReal<<endl;
        assert(rootReal);
		//cout<<"RefCount="<<rootReal->getRefCount();
		// to watch: rootReal->objdata.referencecount
        rootReal->ref(); // delete this when it works, not necessary.
		//cout<<", chg to "<<rootReal->getRefCount()<<endl;

  myEventCB = new SoEventCallback; //added by DMA for key press
  assert(myEventCB); // 
  // cout<<"Constructed myEventCB at "<<myEventCB<<endl;
  myEventCB->addEventCallback(SoKeyboardEvent::getClassTypeId(), myKeyPressCB, rootReal);
  rootReal->addChild(myEventCB); // this is the first child added to rootReal

  // moved from 'D'isplay:
    // We might someday move these to ReadScript, case 'C'amera?
        //orthoView = new SoOrthographicCamera; // ?? why need 2?
        //myViewer->setCameraType(orthoView->getClassTypeId());
        myViewer->setCameraType(myCamera->getClassTypeId());

        // move this later: myViewer->setSceneGraph(rootReal);
		// myViewer->setTitle((runID+": MagVis (U of Alabama Micromagnetic Simulator)").c_str()); 
		// this is done in SoWin::init
	    myViewer->setTransparencyType(SoGLRenderAction::SORTED_OBJECT_BLEND);
        myViewer->setBackgroundColor(SbColor(1.0,1.0,1.0)); 
		  // set later by 'B' line, set here as default

//inserting camera
  myCamera = new SoOrthographicCamera;
  assert (myCamera);
  //the following line is to change orientation of the camera
//  myCamera->orientation.setValue(SbRotation(SbVec3f(0.0,1.0,0.0), 1.57)); // rotate by pi about y axis
  myCamera->position.setValue(50,50,100); // look from front, z>0
  rootReal->addChild(myCamera);
  //cout<<"myCamera->viewportMapping="<<myCamera->viewportMapping.getValue()<<endl;
	  //0=CROP_VIEWPORT_FILL_FRAME, 1=CROP_VIEWPORT_LINE_FRAME, 
	  // 2=CROP_VIEWPORT_NO_FRAME, 3=ADJUST_CAMERA, 4=LEAVE_ALONE
  myCamera->viewportMapping.setValue(3);
  //cout<<"myCamera->viewportMapping="<<myCamera->viewportMapping.getValue()<<endl;
  SbVec2s Win_size; Win_size.setValue(1200,900); // increase default window size
  		// Have only ONE draw style, controls line width of arrows
		// and fill style of faces.  Must precede rootBase,rootPhong.
		// If wanted M and H vectors different, would need to insert
		// DrawStyle as part of (or before) each vector node.
		MyDrawStyle = new SoDrawStyle;
        assert (MyDrawStyle);
        MyDrawStyle->lineWidth.setValue(2);
        MyDrawStyle->style=SoDrawStyle::FILLED;
        rootReal->addChild(MyDrawStyle);

		rootBase = new SoSeparator;		
		//cout<<"created rootBase at "<<rootBase<<endl;
        assert(rootBase);
		SoLightModel *MyLightModel = new SoLightModel;
		MyLightModel->model=SoLightModel::BASE_COLOR;
		rootBase->addChild(MyLightModel);

        rootPhong = new SoSeparator;		
		//cout<<"created rootPhong at "<<rootPhong<<endl;
        assert(rootPhong); // defaults to PHONG model without SoLightModel:
		//rootPhong->addChild(MyLightModel); //make BOTH BASE!
		// then uses only diffuse color & transparency.
        rootReal->addChild(rootBase);
        rootReal->addChild(rootPhong);

        SoDirectionalLight *Lite = new SoDirectionalLight;
		Lite->direction.setValue(0,0,1);
		Lite->on=true;
	    rootPhong->addChild(Lite); //This has no effect?
		// Defaults to light at camera
		// if use diffuseColor light is not needed?

        // construct some stuff for MakeSceneTree recursion:

        myFont = new SoFont;
        assert(myFont);
        myFont->name.setValue("Times-Roman");
        myFont->size.setValue(20.0);
        rootBase->addChild(myFont); // all text will use Base color model
#endif  //  COIN

// moved stuff above to 'g'raphics_init then back to setupgraphics1, 
		//then back to 'g', then to system constructor, then back to ReadScript!
		// initialize, do not put cell tree in
// WHOOPS: #endif  //   // GRAPHICS
    // Removed log filebelow: send stuff to cout instead.
	// For batch operation, redirect to a file?  If hard, could change all cout to cout1
	// and #ifdef WINDOWS set cout1 to cout at compile time.
	//outlog.open((ScriptDir+runID+".log").c_str()); // c_str converts to c-string
	//if(outlog) cout<<"opened"; else cout<<"can't open";
	//cout<<" file "<<runID+".log"<<" for outlog"<<endl;
    // open Avgout later, when we know we need it
  //read in parameters
	cout<<"Reading script file,chars=";
  while ((ch = (char)paramFile.peek()) != '!') // Main loop over script-file lines
  {
	  cout<<ch; // string on 1 line(??)
   // cout<<"hitkey"<<endl; cin.get();
  switch(ch) // look at 1st character of line
   {
    case '#':
     // just reads & discards rest of line
      break;

	case '/': // skip between '/*' and '*/'
	  paramFile.get(ch); // this is '/'
	  paramFile.get(ch);
	  if(ch=='*') // we have comment '/*' 
	  {
		  paramFile.get(ch);
		  char ch2;
		  do 
		  {  paramFile.get(ch2); 
		     if((ch=='*')&&(ch2=='/')) break; // break jumps out of if, not just do
		       else ch=ch2; // & read new ch2
	      } while (paramFile); // quit if read error
	  } // end if commment
	  else {cout<<"comment? read / but not *"<<endl; MYEXIT(49);}
	  break;

	case 'V': // version number of program
      paramFile>>junk>>floatdum;
	  if (Version!=floatdum){cout<<endl<<"Wrong version: script file version="
		  <<floatdum<<", Program is version "<<Version<<endl; goto WriteAbort;}
//   DELETE: xdum=ReadReal(); floatdum=ReadReal();
//	  cout<<"read real:"<<xdum<<", float:"<<floatdum<<", x/f="<<xdum/floatdum<<endl;
      break;
      
      case 'p': //control CompAvg and projection
      paramFile>>junk>>CompAvg>>Proj;
      // cout<<"CompAvg="<<CompAvg<<", Proj="<<Proj<<endl;
// usually use defaults; Scriptfile lines were:
  //printout:    0        1      0 # boolean
  //#         CompAvg   Proj   movie

      if(CompAvg){
	  cout<<"Warning: CompAvg is not debugged!"<<endl; goto WriteAbort;
/*	  Avgout.open((ScriptDir+runID+".Avgout").c_str());
      if(Avgout) cout<<"opened"; else cout<<"can't open";
	  cout<<" file "<<runID+".Avgout"<<" for Avgout"<<endl;
	  cout<<"if get here, re-think Avgs! 101 no longer exists as CellType"<<endl;
      assert(0); // if ever get here, quit.
      Avgs[Max_Avgs-1] = new Average(Max_N); // 7 = number of quantities to average
      Avgs[Max_Avgs-1]->init(); // what was this last one for??
      Avgs[0] = new Average(Max_N);
      Avgs[0]->init();
*/
      }
      break;


    case 'D': // "Display" block
#ifdef GRAPHICS
	  MRad=0.2; MHeadLen=0.2; URad=0.1; ULen=0.1; // initialize arrow sizes, may override
#endif  //  GRAPHICS // these may not even exist if no graphics?
      getline(paramFile,junk); paramFile>>ws; // skip white space
      CheckScript(junk,"Display");
	  cout<<" Read DisplayBlock:";
      while((ch = (char)paramFile.peek()) != '!')  //inner nested while loop
	  {   cout<<ch; // debug
#ifdef GRAPHICS // if not, don't even look at character
	  // note that this is different from the way ifdef is handled in other cases;
	  // may want to uniformize this.
         switch(ch)
          {
            case '#':
              break;

             case 'M': // set size of M-vector for cells
				//cout<<"About to read MLenFudge:"<<endl;
				paramFile>>junk>>MHeadLen>>MRad;
				//cout<<"Setting  MHeadLen="<<MHeadLen<<", M radius="<<MRad<<endl; 
		        break;

		     case 'V': // setup vector for fields, etc
				paramFile>>junk>>ULen>>URad;
		        break;

 		   default: cout<<"Error in Display block"<<endl; goto WriteAbort;
		  } // end inner switch of Display block
#else %GRAPHICS // not GRAPHICS
	  cout<<"Ignoring Display setup line, no graphics."<<endl;
#endif  //   |GRAPHICS // end if GRAPHICS; read the rest anyhow, so don't need to change script file
	  	 if(!paramFile) {cout<<"Error in script, display block"<<endl; goto WriteAbort;}

		  getline(paramFile,junk,'\n'); // read end of line with ch
	      paramFile>>ws; // skip white space at beginning of next line
	  } // end while of DIsplay block - found !, ending list
      // leave end of its line for getline(paramFile,junk) to read

	  // NOW do actual setup of graphics, have gathered info:

	  // Use parameters gathered in Display block:
#ifdef COIN
	  UpVector = MakeUpVector(ULen,URad); // used for H fields, ..., not M
 	  MUpVector = MakeUpVector(MHeadLen,MRad); // for magnetization vectors
#endif  //  COIN
#ifdef OPENGL
   // make GL upvector, Mupvector?
#endif  //  OPENGL
	  break; // end case D, display block

	  case 'X': // "Xtras" to display block
       	paramFile>>junk>>xND>>yND>>dxND>>dyND>>NumCols; // see display.h|NumberDisplay for meaning
		CheckScript(junk,"Xtra"); // should say this
		paramFile>>ws; // skip white space on next line
		xND=root->Scale(0,xND); // rescale & shift to root cell size
		yND=root->Scale(1,yND);
		dxND*=2*root->half[0]; // just scale
		dyND*=2*root->half[1];
		PrevCol=999; // so we will always start new line
        cout<<"tra displays have "<<NumCols<<" cols, start at "<<xND<<","<<yND<<
		   ", increment "<<dxND<<","<<dyND<<endl;
		getline(paramFile,junk); paramFile>>ws;
      while((ch = (char)paramFile.peek()) != '!')  //inner nested while loop
      {  cout<<ch; // debug
	     stringstream StSt;
		 // NumberDisplay *ND; // used in several cases
         switch(ch)
          {
            case '#':
              break;

            case 't': // text display (can be several, independent locations)
				paramFile>>junk>>charray>>xdum>>ydum>>R>>G>>B; // text,posn,RGB
				// cout<<"text to display:"<<charray<<" @"<<xdum<<","<<ydum<<endl;
#ifdef GRAPHICS // this MUST come after setting up graphics, above	
				TextDisplay(charray,MVector(root->Scale(0,xdum),root->Scale(1,ydum),0),R,G,B);//rets. object, but needn't save?
				 // inserts in tree
				// rootReal->addChild(TextDisplay(charray,MVector(xdum,ydum,0),R,G,B).getNode());//insert in tree
#else %GRAPHICS
                cout<<"BUT, not in GRAPHICS mode!"<<endl;
#endif  //   |GRAPHICS
				break;

/*            case 'S': // spacings
				paramFile>>junk>>dxND>>dyND;
				dxND*=root->size[0]; // rescale w/o shift
		     	dyND*=root->size[1];
				cout<<"Spacings scale to"<<dxND<<", "<<dyND<<endl;
				break;
*/
			case 'C': // reset # of columns
				cout<<"Resetting number of columns from "<<NumCols;
				paramFile>>junk>>NumCols; // column numbers 0,1,2..
				CheckScript(junk,"ColumnNumber");
				cout<<" to "<<NumCols<<endl;
				PrevCol=999; // guarantee carriage return
				break;

            case 'T': // time display
               paramFile>>junk;
			   CheckScript(junk, "TimeDisplay");
				// was paramFile>>junk>>xdum>>ydum;
#ifdef GRAPHICS 
	          TimeDisp=UpNumberDisplay("t=",&tDisp,1.0," ns"); // 1.0=IOD=InternalOverDisplayUnit
			  // because ns is both the display and internal unit.
#endif  //  GRAPHICS

/*
#ifdef OPENGL
				// make time display in GL
#endif  //  OPENGL
#ifdef COIN // this MUST come after setting up graphics, above
			  cout<<"Display time"<<endl;
			  // in time case, but not other NumberDisplays, demand NCol:
			  if(NCol<=PrevCol) yND+=dyND; // carriage return upward
			  PrevCol=NCol;
       		  ND = new NumberDisplay("t=",1.0,2,&(tDisp)," ns",
				   xND+NCol*dxND,yND,0.0,0.0,0.0,0.5);
       		   NumbDisps.push_back(*ND); // add pointer to list
			   TimeDisp=&NumbDisps.back(); // returns ptr to NumberDisplay
#endif  //  COIN
*/			  
			  cout<<"DOne with case TimeDisplay"<<endl;
				break;

           case 'A': // exchange constant display. line says "A_Exchange xcord ycord
#ifdef GRAPHICS 
	          paramFile>>junk; CheckScript(junk,"A_Exchange");
			  UpNumberDisplay("A=",&(WatchedCell->rect->ExchangeA),1e-18," j/m");
			  //internal unit j'/nm = 1e-18*j/m, j/m=display unit
#endif  //  GRAPHICS
			  cout<<"DOne with case ExchDisplay"<<endl;
			  break;

           case 'E': // energy display
#ifdef GRAPHICS 
	          paramFile>>junk; CheckScript(junk,"Energy");
	          UpNumberDisplay("TotE/mu0=",&TotEnergy,1.0," uA^2 nm");
#endif  //  GRAPHICS
			  cout<<"Done with case EnergyDisplay"<<endl;
		      break;

           case 'I': // internal energy display
#ifdef GRAPHICS 
	          paramFile>>junk; CheckScript(junk,"InternalEn");
	          UpNumberDisplay("IntE/mu0=",&IntEnergy,1.0," uA^2 nm");
#endif  //  GRAPHICS
			  cout<<"Done with case IntEnergyDisplay"<<endl;
		      break;

			  case 'D': // MaxDM
#ifdef GRAPHICS 
	          paramFile>>junk; CheckScript(junk,"DeltaM_max");
	          UpNumberDisplay("MaxDM=",&MaxDM,1.0," kA/m");
#endif  //  GRAPHICS
			  cout<<"Done with case RotDisplay"<<endl;
			  if(!paramFile) EXIT1("reading script file");
		      break;

 case 'H': // field display
		paramFile>>junk>>xdum>>ydum>>idum>>jdum;// i,jdum=WhichComps,WhereLabel
#ifdef OPENGL
			  // do same for OPENGL:
#endif  //  OPENGL
#ifdef COIN // this MUST come after initializing graphics, above
		if(fieldScale<0.0) {cout<<"Need set Bigness of field"<<endl; MYEXIT(33);}
        externalField = new VectorDisplay("Hext(kA/m)",idum,jdum, // label,whichcomps,wherelabel
   //     MVector (root->center[0]+(xdum-0.5)*root->size[0],
   //	  root->center[1]+(ydum-0.5)*root->size[1],root->LowerLeft()[2]),
        MVector (root->Scale(0,xdum),root->Scale(1,ydum),root->LowerLeft()[2]),
		fieldScale, 0, 0, 1); // position by x,ydum
		//cout<<"about to add ExternalField display"<<endl;
        assert (externalField);
        rootBase->addChild(externalField->getNode()); 
#endif  //  COIN
	    //cout<<"DOne with case H_ext display"<<endl;
		break;

 		   default: cout<<"Error in Xtras block"<<endl; goto WriteAbort;
		  } // end inner switch of Display block
		  if(!paramFile) EXIT1("reading script file");

		  getline(paramFile,junk,'\n'); // read end of line with ch
	      paramFile>>ws; // skip white space at beginning of next line
		  if(!paramFile) EXIT1("Error reading script file");

	  } // end while of Extras block - found !, ending list
      // leave end of its line for getline(paramFile,junk) to read
	  break; // end case X, Extras block

	case 'F': // local Field display & label, MUST follow 'G'eometry
		paramFile>>junk;
		CheckScript(junk,"Fields,local");
#ifdef GRAPHICS
		cell::ReadLocalColor(); // will read rest of line anyway
#endif  //  GRAPHICS
	    break; // end case 'F'

case 'g': // game mode: for now, mainly, limit |H|.
#ifdef GRAPHICS
		paramFile>>junk>>FieldLimit;
		gameMode = true;
		TermCrit = MX; // terminate based on Mx
		cout<<"In game mode, FieldLimit="<<FieldLimit<<endl;
#else %GRAPHICS
	cout<<"Found 'g'ame mode, but no graphics"<<endl; 
#endif  //   |GRAPHICS
		break;

   case 'm': // "mode",sinusoidal or gradient
	    //if(!MyFMM) {cout<<"don't work without FMM!"<<endl; MYEXIT(9);}
		NModes++;
		if (NModes>MaxModes) {cout<<"Only allow "<<MaxModes<<" modes for now"<<endl; MYEXIT(10);}
		Modes[NModes] = new Mode();
		Modes[NModes]->ReadMode(); // reads rest of line (no getline, that's end of switch)
		break;
   case 'u': // add units line for mode values
	   paramFile>>junk;
#ifdef GRAPHICS
	   assert(junk[1]=='n');
	   // create text display below number displays
	   yMd+=dyMd; // increment one last time
	   TextDisplay("   muA nm^2",MVector(xMd,yMd,0),0.5,0.0,0.0); // dark red
	   TextDisplay("   kA/m",MVector(xMd+dxMd,yMd,0),0.0,0.0,0.5); // dark blue
		 // color, spacing shd agree with NumberDisplay calls
#else %GRAPHICS
	   cout<<"Found 'u'nits line, ignoring -- no graphics"<<endl;
#endif  //   |GRAPHICS
	   break;	

   case 'G': //Systm geometry, build the system according to script file
		if(!paramFile) {cout<<"can't read script file"; goto WriteAbort;}
	  paramFile>>junk>>quantum; CheckScript(junk,"Geometry;Quantum");
	  // paramFile>>junk; assert(junk[2]=='o');
	  // quantum=0.0; //now set quantum=size of smallest cell, later: initialize here.
      // cout<<"Read junk="<<junk<<endl;
	  if(!paramFile) cout<<endl<<"Error reading geometry";
	  getline(paramFile,junk,'\n'); // go to next line
      cout<<endl<<"About to read in Geometry of the system"<<endl;
      // assert (MyFMM->NumBelow);  //to check that the array was created

/*
#ifdef SEAPLUG // I guess this is broken now, other way to test SEAPLUG?
      // read in args for TestPlugin.
      paramFile>>junk>>vdum;
      CheckScript(junk,"TestPluginRect:");
      if (vdum[0]==(real)1.0 && vdum[1]==(real)1.0 && vdum[2]==(real)1.0) root = new LLGcell();
      else root = new cell();
      root->rect = new Rectang(root);
      TestPlugin(vdum[0],vdum[1],vdum[2]); // creates root, also CellFromID, sets rect->LLGsize

      root->rect->RectName="Layer1";
      root->rect->RectInfoPtr= new CellInfo(root->rect->LLGSize);
      root->moment = new mpole(); // use max order

      // read other info (anisotropy, etc) for all cells in the rect
      root->rect->setup();
      // this was stolen from MLBuild:
      real lengthScale; // used to set to largest side (will stick out on others)
	    lengthScale = root->rect->LLGSize[0];  // now set to SMALLEST side
        for (short iSide=1; iSide <3; iSide++)
          if (lengthScale > root->rect->LLGSize[iSide]) lengthScale = root->rect->LLGSize[iSide]; 
        lengthScale*= 0.45; //take <half of that
		// Obsolete: cout<<"lenthScale after mult by MLen"<<lengthScale<<" nm"<<endl;
        GlobalMScale = lengthScale/root->rect->Ms; // determined by LAST rectangle
  	      //so you can multiply this by M and get the right length 
	      // extra MLenFudge factor entered by user may be put in GlobalMScale later
#else // not SeaPlug
      root=parseBuild((cell*)NULL);  // creates tree from root, parent NULL
#ifdef DEBUG2
	  cout<<"Returned from top-level parseBuild"<<endl;
// #endif  //   // SEAPLUG
*/
     root=parseBuild((cell*)NULL);  // creates tree from root, parent NULL
	  // assumes we are at beginning of line
	    // reads rest of geometry, calls InitMRecursively for each rectangle

	  if(!MyFMM) root->moment = new mpole(1); // root has moment even if NO FMM.

     //by now, the tree has been constructed, NextSmCellID is the total number of small
     // cells in the system, initialize the array of cell pointers here
      //cout<<"About to allocate,now CellFromID="<<CellFromID<<endl;
      CellFromID = new LLGcell*[LLGcell::NextSmCellID];
	  //cout<<"Allocated CellFromID, now CellFromID="<<CellFromID<<endl;
	    // Know A, Ms for all rects, so we can initialize Jex
      // root->initJexRecursively(); // note ExchLen2 was set earlier
	  // could initMRecursively here too? instead of earlier when setup rect

      root->initIDRecursively(); //must follow geometry and precede use of CellFromID. 6/2013 may be faster in series, TODO: check in profiler
	  // cout<<"Ret. from initIDRecursively, CellFromID[LastCell="
      //  <<LLGcell::NextSmCellID-1<<"]="<<CellFromID[LLGcell::NextSmCellID-1]<<endl;
 //cout<<"Hello from thread "<<omp_get_thread_num()<<endl;
// removed #endif  //   here, didn't say what if it was ending!
	  /*
     #pragma omp parallel shared(My) num_threads(omp_get_num_procs())
	  { // CellFromID shared by default -- can't put an explicit shared(class data member)
		  //TODO: watch out -- could create race conditions if things like *= or += on shared variables. check and make sure we're ok.
          #pragma omp for
	      for(CellIDType i=0;i<LLGcell::NextSmCellID;i++) {
              CellFromID[i]->initM();
              CellFromID[i]->CreateFaces();
          }
          #pragma omp for
          for (CellIDType i=0;i<LLGcell::NextSmCellID;i++) {
              CellFromID[i]->initJex();
          }
	  }
     */
      for(CellIDType i=0;i<LLGcell::NextSmCellID;i++) {
           CellFromID[i]->initM();
           CellFromID[i]->CreateFaces();
      }
      for (CellIDType i=0;i<LLGcell::NextSmCellID;i++) {
              CellFromID[i]->initJex();
      }
  // FMM stuff was here, moved by PBV 6/26/09 to Rectang::setup:'D'
  // WHOOPS! Moved back here 2/6/12 then BACK to 'D'!:

   if(MyFMM) // get kernels, etc, only in FMM case
   {

// Here we should check whether we have a file with NearPartners
		// but this is not implemented yet.
// there is no layer kernel now:

// If no NearPartners file & not layered, probably need cube kernels
	  MyFMM->CubeKer = new std::map<int3,Kernel>; // CubeKer is pointer, this creates actual object
	 // go ahead with reading file:
	  MyFMM->ReadKernels(ExeDir+"Kernels.dat",MyFMM->CubeKer); // read file to CubeKer
/*	  string fileName1="Kernels.dat";
	  fstream KernelsFile; //file to input or output (CalcCubicKernels=1) cubic Kernels 
	  KernelsFile.open((ExeDir+fileName1).c_str(), ios::in); // needs c string
      if (!KernelsFile){cout<<"can't open kernel file "
		  <<ExeDir+fileName1<<endl; MYEXIT(31);}
      ReadKernels(); // reads it into CubeKer
*/
// loop rectangles again, if no NearPartners file, set RectKer:
    for(int iRect=0;iRect<NRects;iRect++)
	{  Rectang* rec=RectFromID[iRect];
       if (rec->LLGSize.Is111()) rec->RectKer=MyFMM->CubeKer; // can use same cubic kernels
	   else 
	   {  rec->RectKer = new std::map<int3, Kernel>; // make new kernel list
	      EXIT1("Whoops! Non-cubes not supported yet");
	   }
	    // later might check if other rect has same LLGSize???
	}

    CreatePartners(); // also reads external potential?

	  // set up partner lists after geometry set, but 
	  // before run or calc kernels recursively calculate Kernels
	cout<<endl<<"Using magnetostatic interactions -- created partners"<<endl;

	  // following was case 'R', but I can't imagine NOT reading kernels:
		// if(CalcCubicKernels) {cout<<"Can't read and calculate kernels!"<<endl; MYEXIT(30);}
		// inputFile>>junk>>fileName1; 
    } // end if(MyFMM)

// At this point, could call MakeSceneTree? Or dismember and put in ParseBuild!      
   // Now have all M's, could calculate fields -- but it is in case 'Q' below instead.
   // DELETE! getline(paramFile,junk); // finish line
	  paramFile>>ws; //eatwhite
	  getline(paramFile,junk); // read sanity-check line
	  if ((junk[0]!='!') || (junk[6]!='G')) //"! end Geometry block"
	    {cout<<"Error, should be '! end Geometry block', read "<<junk<<endl; 
	    MYEXIT(83);}
      break; // end 'G'eometry section
      /*  
This is redundant with movieRead command! Could add this capability (tstart, tshift) to that if desired
      case 'i':  // read initial condition; this must be after geometry
      { real tstart,tshift; // time to search for in file, possible shift
        paramFile>>junk>>ICname>>tstart>>tshift;
        cout<<"read init. cond. filename "<<ICname<<", starttime="<<tstart<<", shift="<<tshift<<endl;
        if(Psetup)  cout<<"read initial condition file, "<<ICname<<endl;
		// should use movIn!: ifstream infil;
        movIn.open ((ScriptDir+ICname).c_str());
         assert (movIn); // infil returns 0 if file didn't exist
         cout<<"Opened file "<<ICname<<", about to read it"<<endl;
         while (t<=tstart) // stops reading when t>=tstart
         readFrameBinary(movIn);
         cout<<"Found frame at t="<<t<<", should be >tstart="<<tstart<<endl;
         // look for if(WC): if(WatchedCell) cout<<"Read initial q="<<WatchedCell->q<<endl; // debug
         t-=tshift; // shift time origin (useful if t gets too large)
		 movIn.close();
        // writeFrameBinary(movOut); // just to debug
       } // end case i
       break; 
*/
case 'B': // bigness of M and field, relative 
#ifdef GRAPHICS
	  cout<<"About to read fieldScale (rel. to M), MLenFudge:"<<endl;
	  real MLenFudge;
      paramFile>>junk>>fieldScale>>MLenFudge; // fieldScale is RELATIVE to M scale here!
	  // cout<<"Setting fieldScale="<<fieldScale<<", fudging M scale by "<<MLenFudge<<endl; 
	  GlobalMScale*=MLenFudge; //GlobalMScale was set by LAST rectangle
      fieldScale*=GlobalMScale; // now absolute, in nm/(kA/m)
	  cout<<"Absolute field scale in  nm / (kA/m) is "<<fieldScale<<endl;
	  // VectDisp::multFactor=fieldScale; // this is done at Fields,local line
#endif  //  GRAPHICS
      break;

    case '&': //other parameters, & means 'rest'
      paramFile>>junk>>gamma>>Temp>>AvgInt;
	  CheckScript(junk,"&RestOf");
	  // note gamma is irrelevant if damping infinite
	  Noise = (Temp>0); // Boolean; note that this can NOT be assumed to
	    // be false if T=0, it is turned on in hysteresis loops.  
	    // It just controls whether we take the time to call RandomFieldforM().
	    // Should not be a problem as long as RandScale=0.
      cout<<endl<<"Read Gamma="<<gamma<<", Temp="<<Temp<<", AvgInt="<<AvgInt<<endl;
	  Temp*=(kB/mu_0); // now in energy units/mu_0, really field^2*volume
      break;
    case 'f': //fmm setup
	  // These MUST be early, needed for geometry, need know whether calc kernels
      // long before you actually do, so not to read kernels
      MyFMM = new FMM; //memory allocation for FMM
      MyFMM->CalcCubicKernels = FALSE; // not init. in FMM() because not needed for H_ONLY

	  assert (MyFMM);
      MyFMM->ReadScriptFMM(paramFile);//read rest of line
	    // and calculate NumBelow, then setup
 	  // end of fmm block
	  break;

	case 'H': // H_initial: set H_bias
	  paramFile>>junk>>H_bias;
      cout<<" found initial H, read initial field="<<H_bias<<endl;
      break;

	case 'l': // layered case
      paramFile>>junk>>Layer_normal>>Layer_offset;
	  Layered=true;
	  CheckScript(junk,"layered");
	  assert(Layer_normal>=0); assert(Layer_normal<3);
	  cout<<"layered case, read normal="<<(char)(Layer_normal+'x')<<endl;
	  assert(Layer_offset>-2); assert(Layer_offset<3);
	  if (Layer_offset>-1) cout<<"Layer_offset along "<<(char)(Layer_offset+'x')<<endl;
      break;

	case 'K': // calculate kernels (not maintained 2012)
	  if(!MyFMM) {cout<<"Error, can't calculate kernels without FMM"<<endl; goto WriteAbort;}
      paramFile>>junk>>MyFMM->OutName;
	  cout<<"beginning calculation of Kernels to put into file "<<MyFMM->OutName<<endl;
			//we need to write them down after calculation:
      // counter=0; //to prevent infinite repetition
      // MyFMM->setupCenterFaces(root);  //to set the center faces?????
      // while ((MyFMM->calcKernels() > 0.000001) && (counter < 30)) counter++;
      MyFMM->WriteKernels();
	  cout<<"Done writing kernels, exiting"<<endl; 
	  MYEXIT(76);
	  break;

    case 'P': // this must be in script file before geometry setup!
      paramFile>>junk>>periodic[0]>>periodic[1]>>periodic[2]>>PeriodVec;
      //cout<<"Reading periodicity: junk="<<junk<<", periodic[2]="<<
      //  periodic[2]<<", PeriodVec="<<PeriodVec<<endl;
      //paramFile>>junk; cout<<"Next junk="<<junk<<endl;
	  assert(paramFile.good());
      numOfPerDir = periodic[0]+periodic[1]+periodic[2]; //number of periodic directions
	  if (numOfPerDir>0) cout<<"Periodic BC bools:"<<periodic[0]<<", "<<
		  periodic[1]<<", "<<periodic[2]<<", periods="<<PeriodVec<<endl;
      break;

    case 'W': //Block to describe watched cell
		cout<<"Watched cell block:"<<endl;
      getline(paramFile,junk); paramFile>>ws; // skip white space
      CheckScript(junk,"Watched"); // "Watched cell block"
      while((ch = (char)paramFile.peek()) != '!')  //inner nested while loop
      {  cout<<ch; // debug
         switch(ch)
          {
            case '#':
              break;
            case 'P': //position of watched cell
		      paramFile>>junk>>W; 
              WatchedLLGCell = root->SubCellContaining(W);
              cout<<endl<<"Watched position "<<W<<", cell "; 
			  if(WatchedLLGCell) {WatID=WatchedLLGCell->SmCellID; 
			     cout<<*WatchedLLGCell<<", ID="<<WatID; }
			  else {cout<<"NULL, position wrong? Setting to 0th cell."; 
			    WatchedLLGCell=CellFromID[0]; WatID=0;} 
			  cout<<endl;
			  WatchedCell =static_cast<cell*>(My->WatchedLLGCell); // was reversed
			  // now WatchedLLGCell and WatchedCell are the same cell, but could change WatchedCell

			  //cout<<"Exchange field at +x face of watched cell: "
				//  <<WatchedCell->cellfacelist[1].myFace->Jex*WatchedCell->rect->Ms<<" kA/m"<<endl; //cin.get();
              break;

           case 'S': //position of watched source (partner)
		      paramFile>>junk>>W; 
              WatLLGPart = root->SubCellContaining(W);
              cout<<endl<<"Watched partner source "<<W<<", cell "; 
			  if(WatLLGPart) cout<<*WatLLGPart; 
			  else {cout<<"NULL, position wrong? Setting to 0th cell."; 
			    WatLLGPart=CellFromID[0];} 
			  cout<<endl;
/* DELETE:
#ifdef DBG_ // Now that we know WatLLGPart, we can create stuff for DebugGreen
		  if(DBG) DBG->SetDBG();
#endif // DBG_
		  */


              break;

            case 'H': // H_local: display "total" field H_av() at watched cell
				// note this is kind of broken by making HDisp show local field everywhere
				// HDisp has wrong type?
              assert (WatchedCell);
			  if(WatchedCell->child0) 
			    {cout<<"total field only for smallest cell"<<endl; goto WriteAbort;}
              #ifdef GRAPHICS   //Shuxia_pc
			  // -1,1 are WhichComps (to display -1 is none), WhereLabel(1=tip)..

			  TotField = new VectorDisplay("Htot",3,1,
                 WatchedCell->center+MVector(0,-0.5*WatchedCell->half[1],0), //displace
			     fieldScale, 0, 0.5, 0.5);
              assert (TotField);

//			  WatchedCell->HDisp = new VectDisp(WatchedCell->center, 0, 0.5, 0);
//	  VectDisp::SetMultFactor(fieldScale);
//	  VectDisp::multFactor=fieldScale;
//            assert (WatchedCell->HDisp); // VectorDisplay("H",-1,1 --> VectDisp(
//            // in constructor: rootReal->addChild(WatchedCell->HDisp->getNode()); 

              cout<<"Will display total field at watched cell"<<endl;
#endif  //  GRAPHICS
			  
#ifdef COIN
	           rootBase->addChild(TotField->getNode());   
#endif  //  COIN // what do you need to do to display VectorDisplay in OPENGL?

              break;

            case 'A': // display anisotropy field
#ifdef GRAPHICS
              assert (WatchedCell);
			  if(WatchedCell->child0) 
			    {cout<<"Anisotropy field only for smallest cell"<<endl; goto WriteAbort;}
			  AnisField = new VectorDisplay("Hanis",3,1,
              WatchedCell->center, fieldScale, 0, 0.5, 0.5);
              assert (AnisField);
              cout<<endl<<"Will display anisotropy field at watched cell"<<endl;
#endif  //  GRAPHICS

#ifdef COIN
	           rootBase->addChild(AnisField->getNode()); 
			
#endif  //  COIN // what do you need to do to display VectorDisplay in OPENGL? 
              break;
             
            case 'X': // display exchange field
#ifdef GRAPHICS
             assert (WatchedCell);
			 if(WatchedCell->child0) 
			    {cout<<"Exchange field only for smallest cell"<<endl; goto WriteAbort;}
			ExchField = new VectorDisplay("Hex",3,1,
            WatchedCell->center, fieldScale, 0.5, 0.0, 0.5);
            assert (ExchField);
            cout<<"Will display exchange field at watched cell"<<endl;
#endif  //  GRAPHICS
		
#ifdef COIN
	           rootBase->addChild(ExchField->getNode()); 
#endif  //  COIN // what do you need to do to display VectorDisplay in OPENGL? 
			  break;
        	case 'M': // display magnetostatic field
#ifdef GRAPHICS
              assert (WatchedCell);
			  if(WatchedCell->child0) 
			    {cout<<"Static field only for smallest cell"<<endl; goto WriteAbort;}
        	   // has label, whichComps, WhereLabel(1=tip); see display.h
			MStaticField = new VectorDisplay("Hmag",3,1,
            WatchedCell->center+MVector(0,-0.5*WatchedCell->half[1],0), //displace
		    fieldScale, 0.8,0.8,0);                  // to avoid other displays
            assert (MStaticField);
            cout<<endl<<"Show MS Field"<<endl;
#endif  //  GRAPHICS
#ifdef COIN
	           rootBase->addChild(MStaticField->getNode()); 
#endif  //  COIN // what do you need to do to display VectorDisplay in OPENGL? 

			break; // end case 'M'

        	case 'm': // display magnetization (AGAIN! - may overlay red one)
#ifdef GRAPHICS
              assert (WatchedCell);
			  if(WatchedCell->child0) 
			  {cout<<"Warning: displaying M of composite watched cell! Hit key"<<endl; cin.get();}
        	   // has label, whichComps, WhereLabel(1=tip); see display.h
			WatchedM = new VectorDisplay("M",3,0,WatchedCell->center
			// +MVector(0,0.25*WatchedCell->size[1],0) // too confusing to have 2!
		    ,GlobalMScale, 0.8,0.4,0);                 
            assert (WatchedM);
            cout<<endl<<"Show M components"<<endl;
#endif  //  GRAPHICS
#ifdef COIN
	           rootBase->addChild(WatchedM->getNode()); 
#endif  //  COIN // what do you need to do to display VectorDisplay in OPENGL? 
              break; // end case 'M'

            default:
              cout<<"Error in WatchedCell block"<<endl; goto WriteAbort;
            } //end of inner WatchedCell switch
		 	 if(!paramFile) {cout<<"Error in script, watched cell, near:"
		 <<junk<<endl; goto WriteAbort;}

          getline(paramFile,junk,'\n');
	      paramFile>>ws; // skip white space at beginning of next line
        } //end of inner WatchedCell while loop, shd be at !

      //cout<<"leaving watched-cell block"<<endl;
      break; //end of case 'W', Watched Cell block

	case 'Q': // Prepare to simulate -- read some information about colors, calculate initial fields 
      // for now, colors only relevant in COIN graphics, but read the lines anyway
      getline(paramFile,junk); paramFile>>ws; // skip white space
	  cout<<"Read 'Q', about to read color-setting lines"<<endl;
	  CheckScript(junk,"Q:");
#ifdef OPENGL
	  // not implemented in OpenGL yet
#endif  //  OPENGL
#ifdef COIN
      surfaceMaterial = new SoMaterial; // for faces; must be before reading colors
      assert(surfaceMaterial);
	  //? 
	  surfaceMaterial->diffuseColor.setValue(0.2,0.2,0.2); // defaults, overridden by script
	  surfaceMaterial->ambientColor.setValue(0.2,0.2,0.2);
	  surfaceMaterial->specularColor.setValue(0.2,0.2,0.2);
	  surfaceMaterial->emissiveColor.setValue(0.2,0.2,0.2);
	  //? 
	  surfaceMaterial->transparency = 0.82;// usually .82;

		//cout<<"surfaceMaterial spec="<<surfaceMaterial->specularColor.getValues(0)->getValue()<<endl;
		// cout<<" is diff"<<endl;
		//cout<<"dif.G="<<surfaceMaterial->diffuseColor.getValues(0)->green<<endl;
	  rootPhong->addChild(surfaceMaterial); // must add before any faces
#endif  //  COIN
#ifndef GRAPHICS
	  cout<<"Found 'Q' but can't start graphics"<<endl;
	  Repeat=true; Pausing=false;
#endif  //   |GRAPHICS 

	  //  have to read block even w/o graphics:
      while((ch = (char)paramFile.peek()) != '!')  //inner nested while loop
      {   cout<<ch; // debug
#ifdef COIN // don't even look at ch if not using graphics, wait for !
	     //decide later whether to implement this in OPENGL
         switch(ch)
          {
            case '#':
              break;
			case 'B': // BackgroundColor
			  paramFile>>junk>>R>>G>>B;
			  myViewer->setBackgroundColor(SbColor(R,G,B));
			  break;

			case 'd': // diffuseColor
		      paramFile>>junk>>R>>G>>B;
	          surfaceMaterial->diffuseColor.setValue(R,G,B);
              break;
			case 's': // specularColor
		      paramFile>>junk>>R>>G>>B;
	          surfaceMaterial->specularColor.setValue(R,G,B);
              break;
			case 'a': // ambientColor
		      paramFile>>junk>>R>>G>>B;
	          surfaceMaterial->ambientColor.setValue(R,G,B);
              break;
			case 'e': // emissiveColor
		      paramFile>>junk>>R>>G>>B;
	          surfaceMaterial->emissiveColor.setValue(R,G,B);
              break;
			case 't': // initial transparency
		      paramFile>>junk>>xdum;
	          surfaceMaterial->transparency=xdum;
              break;
            default:
              cout<<"Error in Q block"<<endl; goto WriteAbort;
            } //end of inner Q switch
		 	if(!paramFile) {cout<<"Error in script, Q block, near:"
	          	 <<junk<<endl; goto WriteAbort;}
#endif  //  COIN
          getline(paramFile,junk,'\n');
	      paramFile>>ws; // skip white space at beginning of next line
        } //end of inner Q while loop, shd be at !
      //cout<<"leaving Q block"<<endl;

	  // Some sanity checks:
 	  if(MyFMM && Layered) {cout<<"Can't have Layered with fmm!"<<endl; MYEXIT(71);}
    //cout<<"before setupGraphics, q="<<WatchedCell->q<<endl; // debug
    CBMode=READ_COMMAND; // must do this first in calback
    getline(paramFile,junk); // get rid of 'Q' line, ready for ReadScript2
	paramFile>>ws; // skip white space at beginning of next line
	cout<<"Ready to enter graphics; CalcAllFromM"<<endl;
	// know M at initial time, calculate H, etc. to display 
    CalcAllFromM(1); // arg=FirstUpdate

	  // Now set up more graphics

#ifdef COIN
	   // make scene tree 
       AllFaces = new SoSeparator; assert(AllFaces);
	   FaceCoords=new SoCoordinate3; assert(FaceCoords);
	   FaceStripSet = new SoTriangleStripSet; assert(FaceStripSet); 
	   
       FaceNormals = new SoNormal; assert(FaceNormals);

/* This seems to have no effect! applies only to surface material?
        myNormalBinding=new SoNormalBinding;
        assert(myNormalBinding); 
        myNormalBinding->value=SoNormalBinding::PER_PART;
        // means per triangle, with TriangleStripSets
        rootPhong->addChild(myNormalBinding);
*/
#endif  //  COIN

#ifdef GRAPHICS // note that COIN and OPENGL are nested INSIDE this.
// any kind of graphics, loop rectangles to draw cells 
   for(short iR=0;iR<NRects;iR++) 
   { // list vertices in rect
	   cell* iRcell=RectFromID[iR]->WholeRect;
#ifdef COIN // initialize counters to zero 
  numVert=0; // will count as insert in StripSet
  numFace=0; 
#endif  //  COIN
#ifdef OPENGL
  numVertGL=0; numIndex=0;
  OrigVec=iRcell->LowerLeft(); // so displacements positive       
  int3 s3=iRcell->half.ToInt(quantum); // # cells on each side
  NumVertBound=(s3[0]+1)*(s3[1]+1)*(s3[2]+1);
  VertexArray=new gl_Vertex[NumVertBound];
  IndexArray = new GLuint[12*NumVertBound]; // 3 4-vertex faces per cube
  //cout<<"NumVertBound="<<NumVertBound<<", VertexArrray size="<<sizeof(VertexArray)<<
  //  ", IndexArray="<<sizeof(IndexArray)<<endl;
    // concluded sizeof(array) is size of pointer (4 bytes)!
#endif  //  OPENGL
  iRcell->MakeSceneTree();
  // if OPENGL, puts vertices in vertex buffer, &vertex indices in index buffer, for faces
  // if COIN, makes whole tree
#ifdef OPENGL
  Verts.clear(); // don't need any more, is in buffer
  delete VertexArray; delete IndexArray;
#endif  //  OPENGL
  } // end for(iR)
	   // above replaces: root->MakeSceneTree(); // recurse on cells, insert faces
	   // could dismember this and put in 'G' case?
	   // needs to be before CalcAll.., which inserts field values, etc.
	   //AllFaces->addChild(FaceDrawStyle);
#endif  //  GRAPHICS // had 1 char '}' inside #ifdef..#endif  //   -- didn't like it at all!!!

#ifdef COIN
   // some things that aren't done per rectangle:
	   AllFaces->addChild(FaceNormals);
	   AllFaces->addChild(FaceCoords);
	   AllFaces->addChild(FaceStripSet);
	   rootPhong->addChild(AllFaces); // inserts all faces

	   /* // experiment with image:
	   SoSeparator *ImgNode;
	   ImgNode = new SoSeparator;
       // this fails inside switch!? SoTransform *ImgTransform = new SoTransform;
	   SoTransform *ImgTransform;
	   ImgTransform = new SoTransform;
	   ImgTransform->translation.setValue(6.0,0,0);
	   ImgNode->addChild(ImgTransform);
	   ImgNode->addChild(AllFaces); // add COPY!
       rootPhong->addChild(ImgNode);
	   */

	   myViewer->setSceneGraph(rootReal); // moved from earlier
	   //myViewer2.setSceneGraph(rootReal);
    // Now really show initial state:
       updateScene();
			  // cout<<"about to call show(), hit key"<<endl; cin.get();
              myViewer->setSize(Win_size);
			  myViewer->show();
              myViewer->viewAll();

  #ifdef WIN32 // unavoidable nested #ifdef
        SoWin::show(myWindow);
	    //  delete myViewer; causes failure, tho never reference again! Needs to still exist.
  #else %WIN32
        SoXt::show(myWindow);
  #endif  //   |WIN32
	mySensor = NULL; // to see if it gets created
//	if (!mySensor) mySensor = new SoTimerSensor(mySensorCallback, My); // thinks "My is float!
	mySensor = new SoTimerSensor(mySensorCallback, this);
	assert(mySensor);
	// mySensor->setInterval(1./25); // defaults to 1/30!
	// I assume interval is irrelevant if my callback takes longer than the interval, 
	// new callback triggers immediately when the callback returns because the interval is over
    mySensor->schedule(); // ?can we do this twice?
#endif  //  COIN
    // Now do some stuff that can be done any time after modes created:
    if(NModes) // if there are Modes
	{
	  for (idum=0;idum<LLGcell::NextSmCellID;idum++) // loop smallest cells
      {  LLGcell* c1=CellFromID[idum];
		 CalcShapeFCell(c1);
	     for (jdum=1;jdum<=NModes;jdum++) Modes[jdum]->IncMMax(c1->rect->Ms);
	  } // now scale MInc from fractional to absolute:
      for (jdum=1;jdum<=NModes;jdum++) Modes[jdum]->MInc*=Modes[jdum]->MMax;
	}
      break; // end of case 'Q'

	case 'e': // enter main loop
	  // make sure you're at first ReadScript2 command:
	  getline(paramFile,junk); // reads rest of last line for this case
	  if (!paramFile) {cout<<"Read e,error in script file, @ "<<junk<<endl; goto WriteAbort;}
	  // cout<<"At end of line, read "<<junk<<endl;
	  paramFile>>ws; // skip white space at beginning of next line
#ifdef COIN
         #ifdef WIN32
        SoWin::mainLoop(); // follow White Rabbit into hole, hope to return
   #else %WIN32
	    SoXt::mainLoop(); 
   #endif  //   |WIN32
#endif  //  COIN
#ifdef OPENGL
	  // this has a similar mainLoop(); for now just emulate it, just as without graphics:
	   idum=1; // flag
	   do idum=HandleCallback(); 
	   while(idum);
         //if returns idum=1 display scene & repeat
#endif  //  OPENGL
#ifndef GRAPHICS // if no graphics at all
	  // mimic mainLoop():
	   idum=1; // flag
       //was: while (t<tstop && idum)  idum=((Systm*)data)->HandleCallback();
	   do idum=HandleCallback(); 
	   while(idum);
         //if returns idum=1 display scene & repeat
	   // removed Shuxia's alternative mimic, see Systm.Dregs.cpp
#endif  //   |GRAPHICS
	  MYEXIT(79); // end of program (shd never get here?)
      break; // end case 'e'nter Mainloop

#ifdef _GROUPS
	case 'R': // Read group, in{} to allow declaring Nes
	  {bool NeedNH=!NH; // should set to true if NH not set yet
	  if(NeedNH) NH=new NestHolder(); // only for the first group
	  Nesting Nes;
	  Nes.ReadNesting(paramFile);
	  NH->Nestings.push_back(Nes);
	  if(NeedNH) Nes.MakeWedge(); // based on this nesting & subgroup, which should be the group of the physical system
	  }
      break;
   case 'r': // set up  to project to a continuous symmetry (l_, parity)
	  {// bool NeedSymH=!SymH; not needed
	  if(!SymH) SymH=new SymHolder();

       paramFile>>junk>>idum>>temp;
	   CheckScript(junk,"rotationGeneral");
	   bool odd1;
	   if(temp=="+") odd1=FALSE; 
	   else 
	   { if (temp=="-") odd1=TRUE; 
	     else {cout<<"Script error, parity character wrong, line "<<junk<<endl;
	       MYEXIT(39);}
	   }
	   Symmetry Sym(idum,odd1); // was Symmetry* Sym = new Symmetry(..)
       //assert(Sym);
      // NOte that Sym is destroyed on exit from ReadScript!
      // Only members listed (i.e. NbrID, NOT NbrID[0]!) are kept in list!
	   SymH->Symmetries.push_back(Sym);
       cout<<"Will rotate by pi/"<<idum<< 
	   // " = "<<Sym.l_<<
	     ", parity ";
	   if(odd1) cout<<"odd"; else cout<<"even";
	   cout<<endl;      
	  }
      break;
#endif  //  _GROUPS

#ifdef CONTOURS 
	case 'L': // Landscape contours
		CBMode = DO_CONTOURS;
		MyContours = new ContourHolder(); // reads line of script file
		//MyContours->ReadContours(); DELETE, this is done in constructor
		break;
#endif  //   
		
    default:
		cout<<"Found illegal 1st character in script line:"<<ch<<endl;
WriteAbort:
    getline(paramFile,junk);
    cout<<"At line in script1 file, beginning char="<<ch<<"=#"<<(short)ch<<
		 "; whole line="<<junk<<endl; MYEXIT(85);
    } //end switch(ch), read script file

    getline(paramFile,junk); // reads rest of last line for this case
	if (!paramFile) {cout<<"error in script file, @ "<<junk<<endl; goto WriteAbort;}
	// cout<<"At end of line, read "<<junk<<endl;
	paramFile>>ws; // skip white space at beginning of next line
  } //end while(ch); //must have hit ! marking end of file?

    // actually never get here, stay in mainLoop() forever???
    getline(paramFile,junk); // get rid of ! line.
	CheckScript(junk,"!");
	cout<<"Reached ! indicating end of script file  -- possibly in error?"<<endl; goto WriteAbort;

} // end of ReadScript()

void Systm::ReadScript2() // this one reads from graphics MainLoop
{ 
	cout<<endl<<"Starting ReadScript2:"<<endl;
	if(!paramFile) 
	{cout<<"cannot access script file"<<endl; MYEXIT(62);}

    string junk; // declare various temporaries (can't inside case stmt)
	string st1;
	char char12[12]; // dummy for binary read/write
	char ch,ch1; MVector ScratchVector; LLGcell* LL;
	Rectang *rect1;
	real xdum; short i1, NCons; float floatdum;
	Mode* M1; 
	ofstream CFsave; 	//addition by Kam 2013
	ifstream CFread;
	Config C1; // end addition by Kam

	//NumberDisplay* ND;
	// real* StepPtr; global!
	bool SceneChanged; // use this to break out of loop and display
	  // (relevant only if CBMode=READ_COMMAND so you would otherwise not display)

	// BEFORE, read only one line at a time; now loop:
  while ((ch = (char)paramFile.peek()) != '!') // Main loop over script-file lines
  {
//#ifdef _DEBUG // even user may want this!
	  cout<<ch<<'|';
//#endif  //   // than end user would want to see
  SceneChanged=false; // most commands don't require redisplay
#ifdef GRAPHICS
  unsigned int movbuffsize; char* buff;
#endif  //  
  switch(ch) // look at 1st character of line
   {
    case '#':
     // just reads & discards rest of line
      break;

	  	  // addition by Kam 2013
	case '$': // saving and reading config file from script
	   getline(paramFile,junk,' ');
	   getline(paramFile,st1,'\n');
	   // cout<<"Saving configuration in "<<junk<<endl;
	   if(junk[1]=='R'){
	   cout<<"Reading configuration from "<<st1<<endl;
      CFread.open(st1.c_str());
      if(!CFread.good()) {cout<<"Could not open file "<<st1<<endl; MYEXIT(61);}
	  C1.ImportfromFile(CFread);
	  C1.restore();
	  CFread.close();
	  SceneChanged=true;

	   } else{
       cout<<"Saving configuration in "<<st1<<endl;
      CFsave.open(st1.c_str());
	  C1.saveM();
      C1.ExporttoFile(CFsave);
	  CFsave.close();
	   }
      break;
	  // end addition by Kam 2013

	case '/': // skip between '/*' and '*/'
	  paramFile.get(ch); // this is '/'
	  paramFile.get(ch);
	  if(ch=='*') // we have comment '/*' 
	  {
		  paramFile.get(ch);
		  char ch2;
		  do 
		  {  paramFile.get(ch2); 
		     if((ch=='*')&&(ch2=='/')) break; // break jumps out of if, not just do
		       else ch=ch2; // & read new ch2
	      } while (paramFile); // quit if read error
	  } // end if commment
	  else {cout<<"comment? read / but not * in script2"<<endl; MYEXIT(59);}
	  break;

	case '+': // field increment
	  paramFile>>junk>>Increment;
	  // cout<<"Increment set to "<<Increment;
	  break;
	case '@': // reset damping
	  paramFile>>junk>>st1>>xdum;
	  CheckScript(junk,"@Reset_damping");
	  rect1=RectNamed(st1);
	  if(!rect1) {cout<<"resetting damping, unknown rect name "
		  <<st1<<endl; MYEXIT(83);}
      rect1->alpha=xdum;
      SetEvolParams(); // depend on alpha
	  cout<<"@: reset damping in rectangle "<<st1<<" to "<<rect1->alpha<<endl;
	  break;

	case 'a': // astroid (need grid first)
	  cout<<"Calculate astroid"<<endl;
      if (!GridInf1) cout<<"Need grid first!!"<<endl;
      GridInf1->CalcAstroid();
	  break;

    case 'A': //to choose the algorithm Heun(1),Euler(0)
      paramFile>>junk>>Heun>>DampingFinite; // removed >>use_q;
	  cout<<endl<<"Algorithm:";
	  if (Heun) cout<<"Heun, "; else cout<<"Euler, ";
	  if (!DampingFinite) cout<<"in";
	  cout<<"finite damping."<<endl;
      break;

	case 'C': // set Constraints block 
	  getline(paramFile,junk); paramFile>>ws; // skip white space
      CheckScript(junk,"Constraint_reset");
	  cout<<"Entering constraints block"<<endl;
	  for (i1=1;i1<=NModes;i1++) Modes[i1]->ConstrainIndex=-1; 
	    // most are unconstrained
	  NCons=0;
	  while((ch1 = (char)paramFile.peek()) != '!')  //inner nested while loop
      {  cout<<ch1; // debug
         switch(ch1)
         {   case '#':
             break;

             case 'M': // Mode to constrain
	         paramFile>>junk>>st1>>xdum; // modeName & tolerance
			 if(!paramFile) cout<<"Error reading mode to constrain, "
				 <<junk<<st1<<endl; 
			 CheckScript(junk,"Mode");
			 i1=Mode::ModeNamed(st1); if (!i1) cout<<"reading Constraint block"<<endl;
			 M1=Modes[i1]; assert(M1);
			 M1->Tol=xdum*M1->MMax; // now is absolute
			 if (M1->ConstrainIndex>=0) 
			   {cout<<"Trying to constrain a moment twice?"<<M1->ModeName<<endl; MYEXIT(53);}
			 M1->ConstrainIndex=NCons; // will be index in array
			   // value is reset in constructor, only matters that it's >-1
	         NCons++;
			 cout<<"Constraining mode "<<M1->ModeName<<endl;
	         break;

			 case 'S': // Setup constraints
	         if(cons) {cout<<"Resetting constraints -- may not be debugged!"<<
        		  " Press enter in this window"<<endl; cin.get();}
	         if(cons) cons->~Constrain(); //destructor!
        	 cons = new Constrain(NCons); // construct 
			 assert(cons);
			 //cout<<"Constructed new Constrain object with "<<
			 //	 NCons<<" constrained modes"<<endl;
			 cons->SetmWantToPresent(); // calcs present values, sets to them
             break; // end case setup

/* needs to default to SOMETHING! Default to this.  
         case 'W': // want present value
	         paramFile>>junk>>st1; // modeName 
			 assert(junk[1]=='a');
			 i1=Mode::ModeNamed(st1); if (!i1) cout<<"reading "<<junk<<endl;
			 M1=Modes[i1]; assert(M1);
			 assert(M1->ConstrainIndex>=0); assert(M1->ConstrainIndex<cons->NC);
			 CalcMoments(); // very inefficient! just want one.
			 cons->mWant[M1->ConstrainIndex]=M1->MVal;
			 cout<<"Want present value for mode "<<M1->ModeName<<", = "<<M1->MVal<<endl;
	         break;
*/
			 default:
				 getline(paramFile,junk);
				 cout<<"Unrecognized 1st character "<<ch1<<" in constraint block: "<<junk<<endl;
				 MYEXIT(83);
		  } // end inner switch of constraint block
		  if(!paramFile) {cout<<"; Error in script, constraint block, near:"
		    <<junk<<endl; MYEXIT(29);}
		  getline(paramFile,junk,'\n'); // read end of line with ch
	      paramFile>>ws; // skip white space at beginning of next line
	  } // end while of constraint block - found !, ending list
      // leave end of its line for getline(paramFile,junk) to read
	  break; // end case C, constraint block

	case 'D': // display iteration count
	{ // brackets are to allow local ND
      paramFile>>junk;
	  CheckScript(junk,"DisplayIterationCount");
#ifdef GRAPHICS
	  yND+=dyND;
	  NumberDisplay* ND = new NumberDisplay("",1.0,0,&(ConvSteps)," iterations",
	    xND,yND,0.0,0.0,0.0,0.5); // dark blue=00,0.5; precision=0 for integer
	  NumbDisps.push_back(*ND); // add to list
#endif  //  GRAPHICS
	}
	  break;

    case 'c': // to set a cell's magnetization, etc
	  float Mx,My,Mz; // temporaries; 
	  //bool ShowH; ShowH was once global
	  paramFile>>junk>>ScratchVector>>Mx>>My>>Mz; //had >>ShowH;
	  LL = root->SubCellContaining(ScratchVector); 
	  if(LL) // if not LLG cell, ptr=NULL
	  cout<<"Redefining M (case 'c') for cell "<<*LL<<" at "<<LL
		  <<" with radius="<<LL->half<<", M="<<Mx<<","<<My<<","<<Mz<<endl;  
	  else {cout<<"Error in 'c...' line, cell at"<<ScratchVector<<" not found"<<endl; goto WriteAbort2;}
	  LL->M = MVector(Mx,My,Mz); // M may not be normalized, just get q:
      //LL->q.rotateTo(MVector(Mx,My,Mz)); 
#ifdef LINEARIZE // just use M's if lineaerized, Mz=Ms.
	  LL->M[2]=LL->rect->Ms;
#else %LINEARIZE // not linearize: get M from q so it's normalized
      // LL->M =((LL->q * (LL->rect->M0)) * (~LL->q)).getMVector();
	  // or just normalize:
      LL->M.normalizeTo(LL->rect->Ms); // remove to allow un-normalized
#endif  //   |LINEARIZE
      //cout<<"Cell pointer = "<<CellFromID[iCell]<<", changed q to "<<CellFromID[iCell]->q<<endl;
#ifdef GRAPHICS
	  
	  // I'm not sure I understand this -- forget it for now:
// if (ShowH && cellptr->HDisp==NULL) // if H display wanted & not already done
// {  // note this code is almost repeated in Watched cell setup 'W'
///    cellptr->HDisp = new VectDisp(cellptr->center, 0, 0.5, 0.5);
//       assert (cellptr->HDisp); // VectorDisplay("H",-1,1 -> VectDisp
// cout<<"Script file line c... is creating H display -- not debugged!"<<endl;
// // crashes if HDisp_Which not set!!
// VectDisp::multFactor=fieldScale; // was set to this before if set at all?
		 // note this is problem: multFactor is static variable, governs alll cells' HDisp
         // rootReal->addChild(cellptr->HDisp->getNode()); 
// }
	  
#endif  //  GRAPHICS
      break; // ends case 'c'ell
		    
    case 'd':
      paramFile>>junk>>dt>>d>>DisplayConv;

      cout<<"set dt="<<dt<<", display after "<<d<<", DisplayConv="<<DisplayConv<<endl;
	  if(!paramFile) {cout<<"but error reading this"<<endl; MYEXIT(44);}
	  SetEvolParams(); // sets everything that depends on dt, damping, gamma
      break;

	case 'f': // CHANGE which local field to display 
		paramFile>>junk;
		CheckScript(junk,"fields,local");
#ifdef GRAPHICS
		cell::ReadNewLocalColor();
#endif  //  GRAPHICS
	    break;

	case 'E': //Export data block
      paramFile>>junk>>XportInt; // read interval; for both console and file
	  if(!paramFile){cout<<"Error reading Export block, interval="
		  <<XportInt<<", max=65535"<<endl; MYEXIT(57);}
      CheckScript(junk,"Export_data");
	  getline(paramFile,junk); paramFile>>ws; // skip white space
	  // Xporting=1; // means file exists? Use Xport.is_open() instead
	  
      // changed mind about not need: Xport_t=TRUE; // if write file at all, write t
	  XportM=XportMom=XportH=XpoBMode=XpoProd=0; 
	  XportE=XportIE=XportConsole=0; //XportmDotDir=XpomDir=XpoPerp=
	  // default to not print  // there is no XportMR, 
      // with any output object like MyMR it's assumed if it exists you want to export it
	  cout<<endl<<"xport block:";
      while((ch = (char)paramFile.peek()) != '!')  //inner nested while loop
      {   cout<<ch; // debug
         switch(ch)
          {
            case '#':
              break;
          case 'F': //export to file
			paramFile>>junk>>WriteFileInt; // whether to write at intervals, or just on key
	        Xport.open((ScriptDir+runID+".csv").c_str()); // c_str converts to c-string
            Xport.precision(12); // must be .csv to open automatically 
	        cout<<endl<<"Opening file "<<runID+".csv"<<" for data export, ";
			if(!Xport) {cout<<endl<<"FAILED -- is another program reading it?";	
	          goto WriteAbort2;}
		    break;
          case 'C': //export to console
              XportConsole=1; 
              cout<<"; will export to console as well as to data file"<<endl;
			  break;
            // case 't': //time now automatic Xport_t=1;
			  //cout<<"; will export time"<<endl; break;
            case 'M': //magnetization vector
              XportM=1;
              cout<<"; will export magnetization"<<endl;
              break;
            case 'm': //moment values & fields
			  paramFile>>junk>>XportMom; // # to export
              cout<<"; will export "<<XportMom<<" mode moments & fields"<<endl;
			  if(XportMom>NModes) {cout<<"That's more than we have ("
				  <<NModes<<")!"<<endl; MYEXIT(48);}
              break;
           case 'H': //magnetic field vector
              XportH=1; 
              cout<<"; will export magnetic field"<<endl;
			  break;
           case 'T': // Total moment
			  cout<<"Export total moment no longer supported"<<endl; 
			  MYEXIT(39);
              //paramFile>>junk>>XpomDir; 
              //cout<<"; will export "<<(char)(XpomDir+'x')<<" component of total m"<<endl;
			  //XpomDir++;  // because is index of mpole, x<->1 
			  break;

	    case 'B': // B-field mode
			  paramFile>>junk>>st1;
			  CheckScript(junk,"B-field");
			  XpoBMode=Mode::ModeNamed(st1);
			  cout<<"Will export B in mode "<<st1;
			  break;

			case 'P': // energy Product
              if(XpoBMode<0) EXIT1("can't set energy product before B-field mode");
              paramFile>>junk; 
			  XpoProd=true;
              cout<<"; also export energy product"<<endl;
			  break;
		/*
           case 'P': // Perpendicular comp. of mTot
			  cout<<"Export perp mTot no longer supported"<<endl; 
			  MYEXIT(39);
              paramFile>>junk>>XpoPerp; 
              cout<<"; will export component of total m perp. to "
			  	   <<(char)(XpoPerp+'x')<<"& its change"<<endl;
			  XpoPerp++;  // because is now index of mpole, x<->1 
			  break;

           case 'd': // dm/dt (total)
              paramFile>>junk>>XportmDotDir; 
              cout<<"; will export "<<(char)(XportmDotDir+'x')<<" component of total dm/dt"<<endl;
              XportmDotDir++; // because is index of mpole, x<->1 
			  break;
         */
		   case 'E': //total energy
              XportE=1; 
              cout<<"; will export total energy"<<endl;
			  break;

		   case 'I': // Internal energy
              XportIE=1; 
              cout<<"; will export Internal energy"<<endl;
			  break;

           case 'V': //compute volume integrals, added by zhuru
	          paramFile>>junk;	
              MyInt = new Integrals(2); //construct object of Integral class
        	  cout<<"; new Integral constructed>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"<<endl;
	          break;
		  } // end inner switch of export list
		 if(!paramFile) {cout<<"; Error in script, exporting, near:"
		    <<junk<<endl; MYEXIT(22);}
		  getline(paramFile,junk,'\n'); // read end of line with ch
	      paramFile>>ws; // skip white space at beginning of next line
	  } // end while of export list - found !, ending list
      // leave end of its line for getline(paramFile,junk) to read
	// Now write header line. Could avoid lots of "if"s by putting this in cases above,
	// but this ensures they are in a standard order:
	  Xport<<"t  "; // write header line-need to have automatic column (not preceded by d_)
	    // so it might as well be time; Xport_t is no longer used
	  // Xport.precision(4);
	  if(XportH) Xport<<d_<<" Hx  "<<d_<<"   Hy   "<<d_<<"   Hz   ";
	  if(XportM && WatchedCell) Xport<<d_<<" Mx  "<<d_<<"   My  "<<d_<<"    Mz   ";
      //if(XportMom) for(i1=1;i1<=XportMom;i1++) Xport<<d_<<" m["<<i1<<"] "<<d_<<" h["<<i1<<"] ";
      if(XportMom) for(i1=1;i1<=XportMom;i1++) Xport<<d_<<" m."<<Modes[i1]->ModeName<<d_<<" h."<<Modes[i1]->ModeName;
      //if(XpomDir) Xport<<d_<<" mTot_"<<(char)(XpomDir+'w');
	  //if(XpoPerp) Xport<<d_<<" mTot perp to "<<(char)(XpomDir+'w')<<d_<<" its change ";
	  //if(XportmDotDir) Xport<<d_<<"dm"<<(char)(XportmDotDir+'w')<<"/dt";
	  if(XpoBMode) Xport<<d_<<"B/mu_0";
	  if(XpoProd) Xport<<d_<<"'-BH(MGOe)";
	  if(XportE) Xport<<d_<<"E/mu_0";
	  if(XportIE) Xport<<d_<<"IntE/mu_0";
	  if(MyInt) Xport<<d_<<"Int1 "<<d_<<"  Int2"; // if exists, assume want to export
//	  if(MyMR) Xport<<d_<<"MR"; // Magnetoresistance
	  Xport<<d_<<"steps?"; // may be on some lines,not others
	  Xport<<endl;
	  // if plot B and BHmax, give critical h = Ms/2:
	  if(XpoBMode)
	  {  real MMax1=-Modes[XpoBMode]->MMax*2; // 2 is if relative to +MMax!!
	     real Ms2=MMax1/root->volume()/2.0; // Ms/2
		 real Prod=Ms2*Ms2*4*M_PI*1e-6; // in MGOe; without 4pie-6, is BH/mu_0=(Ms/2)^2%
		 Xport<<"m&Ms/2="<<d_<<MMax1<<d_<<Ms2<<d_<<d_<<d_<<d_<<Prod<<endl;
		 MMax1+=0.02*Modes[XpoBMode]->MMax; // make short line
	     Xport<<"      "<<d_<<MMax1<<d_<<Ms2<<d_<<d_<<d_<<d_<<Prod<<endl<<endl;

	  }
	  // ExportData here? May not be initialized! Moved to case 'R'un.
	  break; // end case E, export data

	case '&': //& Export 2nd data file block
      paramFile>>junk; 
      CheckScript(junk,"&2nd");
	  getline(paramFile,junk); paramFile>>ws; // skip white space
	  Xpor2ing=1; //true; if pointer, Xport = new ofstream;
	  Xpor2.open((ScriptDir+runID+".2.csv").c_str()); // c_str converts to c-string
          Xpor2.precision(12); // must be .csv to open automatically 
	  cout<<"; Opening file "<<runID+".2.csv"<<" for data export, ";
	  if(Xpor2.good()) cout<<"opened OK"<<endl; 
	  else {cout<<endl<<"FAILED -- is another program reading it?";	
	    goto WriteAbort2;}
	  
      // In 2010 this has only orbit writing capability
	  // Is in 2nd file so not limited to each time step

      while((ch = (char)paramFile.peek()) != '!')  //inner nested while loop
      {   cout<<ch; // debug
         switch(ch)
          {
            case '#':
              break;
            
		  } // end inner switch of expor2 list
		 if(!paramFile) {cout<<"Error in script, export2, near:"
		   <<junk<<endl; MYEXIT(42);}

		  getline(paramFile,junk,'\n'); // read end of line with ch
	      paramFile>>ws; // skip white space at beginning of next line
	  } // end while of export list - found !, ending list
      // leave end of its line for getline(paramFile,junk) to read
	  Xpor2<<"This is 2nd export file"; // write header line
	  Xpor2<<endl;
	  break; // end case 2, 2nd export file

	 case 'F': // final check
		finalCheck(); // reads more from script file.
        break;

	 case 'G': // grid
	  cout<<"Starting Grid block"<<endl;
      GridInf1=new GridInfo();
	  cout<<"Allocated GridInfo"<<endl;
	  GridInf1->ReadGrid(paramFile);
	  // cout<<"Read Grid"<<endl;
      BHmax=-Infty; // initialize
	  CBMode=GRID; // launch into grid convergence
	  // SceneChanged=true; // not needed if CBMode!=READ_COMMAND
	  break;

	 case 'H': // field-Changing block (was set in ReadScript())
		// cout<<"Starting H block, hit key"<<endl; cin.get();
      getline(paramFile,junk); paramFile>>ws; // skip white space
      CheckScript(junk,"H_block");
	  cout<<"-block:";
      while((ch = (char)paramFile.peek()) != '!')  //inner nested while loop
      {  cout<<ch; // debug
         switch(ch)
          {
            case '#':
              break;
            case 'B': // constant bias field
			  paramFile>>junk>>H_bias;
              cout<<"found 1st char in line (H), read value="<<H_bias<<endl;
              break;
            case 'P': //Pulsed Field part (PBV, 7/20/04)
			  // 2/2012: this predates scripting? Probably should replace this
		      // with scripted pulses.
              paramFile>>junk; // reads to white space
              short iPulse; paramFile>>iPulse; // reads number of pulse (1 or 2)
              cout<<"Case H, found pulse number "<<iPulse<<endl;
              PulsedField  **Ext_FieldPtr; // pointer to Ext_Field1 or Ext_Field2,
                 // a pulse object pointer
              switch(iPulse)
              {
                case 1:  Ext_FieldPtr=&Ext_Field1; break; // now *Ext_FieldPtr is Ext_Field1
                case 2:  Ext_FieldPtr=&Ext_Field2; break;
                default: cout<<"Error in iPulse"<<endl; goto WriteAbort2;
              } // end switch(iPulse), now *Ext_FieldPtr is the pointer to this pulse object
              if (! *Ext_FieldPtr) *Ext_FieldPtr = new PulsedField(); // create it if it doesn't exist
             (*Ext_FieldPtr)->Read(&paramFile); // read pulse information
             // cout<<"found 1st char(h), read field pulse, fields now "<<Ext_Field1->Hmax<<" and "<<Ext_Field2->Hmax<<endl; 
	         break; // end case P
	       case 'A': //AC pulse field
	      	 paramFile>>junk;
	     	 ACExt_FieldPtr = new ACPulsedField();
		     ACExt_FieldPtr->ReadAC(&paramFile);
		     break;
           case 'M': // field of moving media grain
              paramFile>>junk; CheckScript(junk,"Media"); //Media field
			  MyMedField = new MedField;
			  MyMedField->ReadMedField(&paramFile); // reads 1 line
			  cout<<"found 1st char in H (field) block (Media grain), MsMed4pi="<<MyMedField->MsMed4pi<<endl;
              break;
            default:
              cout<<"Error in H block"<<endl;
              goto WriteAbort2;
            } //end of inner H switch
		 	if(!paramFile) {cout<<"Error in script, H block, near:"
	        	 <<junk<<endl; MYEXIT(22);}

          getline(paramFile,junk,'\n');
	      paramFile>>ws; // skip white space at beginning of next line
        } //end of inner H while loop, shd be at !
      // leave end of its line for getline(paramFile,junk) to read
      // assert(junk[0] == '!');
	  // end of H block
	  break;

   // case 'I': //Insert Spin torque // gone, 5/2013

	case 'K': // calculate kernels
	  if(!MyFMM) {cout<<"Error, can't calculate kernels without FMM"<<endl; goto WriteAbort2;}
      paramFile>>junk>>MyFMM->OutName;
	  cout<<"beginning calculation of Kernels to put into file "<<MyFMM->OutName<<endl;
			//we need to write them down after calculation:
      // counter=0; //to prevent infinite repetition
      // MyFMM->setupCenterFaces(root);  //to set the center faces?????
      // while ((MyFMM->calcKernels() > 0.000001) && (counter < 30)) counter++;
      MyFMM->WriteKernels();
	  cout<<"Done writing kernels, exiting"<<endl; 
	  MYEXIT(76);
	  break;

	case 'L': // limits
	  paramFile>>junk>>st1>>ErrDM>>RotTol>>hTol>>MaxIters>>CosLimit; // st1 is IgnoreError
	  // note ErrDM is dimensionless here; fix a few lines down.
	  cout<<"Read limits: hTol="<<hTol<<", MaxIters="<<MaxIters<<", CosLimit="<<CosLimit<<endl;
	  if (RotTol<(MachineAcc*4)) 
	  {   cout<<"RotTol too small, ";
	      RotTol = MachineAcc*4.0;
		  cout<<", replacing by 4*MachineAccuracy = "<<RotTol<<endl;
	  }
	  switch(st1[0])
	  { case 'I': IgnoreError=true; break; // "IgnoreError
	    case 'P': IgnoreError=false; break; // "PauseOnError"
	    default: EXIT1("Illegal character in limits line, after "+junk);
	  }
	  ErrDM*=RectFromID[0]->Ms; // convert to kA/m units
	  DMTol=RotTol*RectFromID[0]->Ms; // assume 0th rectangle is typical!
	  cout<<"Reading limits, max DM="<<ErrDM<<", dM tol="<<DMTol<<", hTol="<<
		  hTol<<", MaxIters="<<MaxIters<<", CosLimit="<<CosLimit<<endl;
	  	  if(!paramFile) EXIT1("Error reading line "+junk);
	  break;

	case 'M': // reset value of mode 
	  paramFile>>junk>>st1>>xdum; // modeName & new value
	  if(!paramFile) cout<<"Error reading mode to reset, "<<junk<<st1<<endl; 
	  CheckScript(junk,"ModeReset");
	  i1=Mode::ModeNamed(st1); 
	  if (!i1) cout<<"Error reading name of mode to reset:"
		  <<st1<<" is not a mode name"<<endl;
	  M1=Modes[i1]; assert(M1);
	  short CI; CI=M1->ConstrainIndex;
	  cout<<"Reset mode "<<st1<<", number "<<i1<<"'s ";
	  if(CI>-1)  // constrained
	  {   cout<<"moment mWant["<<CI<<"]"; 
	      assert(cons); cons->mWant[CI]=xdum;
	  }
	  else // unconstrained, set field
	  {  cout<<"field Modes["<<i1<<"]->HVal";
	     M1->HVal=xdum;
	  }
	  cout<<" = "<<xdum<<endl;
	  SceneChanged=true; // display, come back for next command
	  break;

    case 'm': // read a movie, don't compute 
        {
#ifdef GRAPHICS
      paramFile>>junk>>Movname;
      CheckScript(junk,"movieRead");
      cout<<"read name of movie file: "<<Movname<<endl;
      movIn.open((ScriptDir+Movname).c_str(), ios::binary);
	  //Read12();
      cout<<"Movname="<<Movname<<endl;
      if(movOut) if(runID+".traj" == Movname)
        {cout<<"Can't read&write same file"<<endl; goto WriteAbort2;} //Shuxia_pc
      if(!movIn) cout<<"can't open file movIn"<<endl;
	  movbuffsize = 8192;
      buff = new char[movbuffsize]; // never deleted??
      movIn.rdbuf()->pubsetbuf(buff,movbuffsize);
      movIn.read(char12,8); CheckScript(char12,"Version "); // 8 bytes
	  movIn.read((char*)&floatdum,sizeof(floatdum)); // was movIn>>junk>>floatdum;
        //getline(movIn,junk,'\n'); // getline() is in header <string>
        //#ifdef GRAPHICS<-!!! #ifdef nested twice!
        cout<<"version of the movie is "<<floatdum
          <<", program can handle version "<<movieVersion<<endl;
        if (floatdum!=movieVersion)
        { cout<<"* * * incorrect version of the movie file! * * *"<<endl;
          goto WriteAbort2;}
        uint32_t hash=0,tmp=0;
      movIn.read(char12,6); CheckScript(char12,"\nHash "); // 6 bytes
	  movIn.read((char*)&tmp,sizeof(tmp)); // was movIn>>junk>>tmp;
        hash = calcFileHash(ParFileString);
        if (tmp!=hash) 
		{  cout<<"Stored movie hash ("<<tmp<<") does not match current Geometry hash ("<<hash<<
			"). Movie file was created with a different geometry, can't play!"<<endl;
            cout<<"Override this? (y/n)"; cin.get(ch); if (ch=='y'||ch=='Y') break;
			goto WriteAbort2;
		}

      // CBMode = STOP; //read the movie in forward direction (DMAm)
	    // ?? why not default to FORWARD? Or give user choice in script?
      // movIn.seekg(0,ios::beg);
	  CBMode = FORWARD; Pausing=false;
	  // SceneChanged=true; not needed
#else %GRAPHICS
		cout<<"'m'ovie? Surely not, not using GRAPHICS!"<<endl; MYEXIT(44);
#endif  //   |GRAPHICS
      } // end case 'm'

	  break;
    case 'w': // writeMovie
	  paramFile>>junk>>MovInt>>st1;
	  if(movOut.is_open()) EXIT1("error in movie test"); // make sure if() returns false if not open
	  if (MovInt) // 0 means no movie
	  {	 // open as binary; trunc means overwrite previous, if any (vs. app(end)|ate)
		  // see http://www.cplusplus.com/doc/tutorial/files/
		  movOut.open((ScriptDir+runID+".traj").c_str(),ios::binary|ios::trunc);
	     if(movOut) cout<<"opened"; else cout<<"can't open";
	     cout<<" file "<<runID+".traj"<<" for movOut"<<endl;
         // cin.get(); // debug
		 movOut.write("Version ",8); 
		 movOut.write((char*) &movieVersion,sizeof(movieVersion));
         // delete movOut<<"Version "<<movieVersion<<endl; //added by DMAm
		 uint32_t hash=calcFileHash(ParFileString);
		 movOut.write("\nHash ",6);
 		 movOut.write((char*) &hash,sizeof(hash));

        #ifdef GRAPHICS
         if (st1[0]=='n') WritingMovie=true; // 'n'ow simulation will write initial movie frames
         else assert(st1[0]=='l'); // 'l'ater, WritingMovie toggled with 'm' key
             //WritingMovie=false; // already initialized to false, not necessary
        #else
         if (st1[0]=='n') WritingMovie=true;
         else {
             assert(st1[0]=='l'); // 'l'ater, WritingMovie toggled with 'm' key
             cout<<"Write movie 'l'ater? But graphics are off -- writing now anyway."<<endl;
             WritingMovie=true;
         }
        #endif  //  

	  } 
      break;
	case 'o': // o=degree symbol! calculate angle dependence
	  short NumPoints;
      paramFile>>junk>>ScratchReal1>>NumPoints;
	  CheckScript(junk,"o.AngleDep");
	  ScratchReal1=0.0; // now is angle
	  ScratchReal2=2*M_PI/NumPoints;
      cout<<"Will calculate angle dependence at saturation using "<<NumPoints<<" points"<<endl;
	  CBMode = SATURATION;
	  // return to Main Loop to display; callback will call SaturationCB();
	  break;

	case 'O': // want to calculate orbits
#ifdef ORBITS
	  MyOrbs=new OrbitHolder; // this class has everything needed
	  // constructor reads next line of script file, doesn't finish it tho
	  CBMode= DO_ORBITS;
	  // not needed: SceneChanged=true;
#else
	  cout<<"Orbits not implemented yet, see Orbits.h"<<endl; MYEXIT(19); 
#endif  //   |ORBITS
break;

#ifdef _GROUPS
case 'p': // project onto group, one time only (mostly for debugging?)
	paramFile>>junk>>st1;  CheckScript(junk,"project");
   switch(st1[0])
   {case '4':
    { CheckScript(st1,"4fold");
      paramFile>>junk>>i1; // now junk=nesting label
		cout<<"Projecting onto group "<<i1<<" in nesting "<<junk<<". ";
      if(!NH) EXIT1("Can't project onto group, there are none!");
		std::list<Nesting>::iterator nit=NH->NesNamed(junk);
		if(nit==NH->Nestings.end()) // flag for error return
			cout<<"Can't find nesting"<<endl; 
		else
		 {
		  Config M0; M0.saveM(); 
		  Config M1=nit->Project(M0,i1);
		  M0=M1-M0; cout<<", integral of change^2="<<M0*M0<<", norm="<<M0.norm()<<endl;
		  M1.restoreAnd(); // restores to cell::M,And renormalizes,CalcAllFromM
		 }
    } // end case 4
      break;
   case 'G':
    {  
      CheckScript(st1,"General");
      paramFile>>i1>>junk; // now i1 is l_, junk is parity sign
      bool odd1;
	   if(junk=="+") odd1=FALSE; 
	   else 
	   { if (junk=="-") odd1=TRUE; 
	     else {cout<<"Script error, parity character wrong, line "<<junk<<endl;
	       MYEXIT(37);}
	   }
      std::list<Symmetry>::iterator sit=SymH->FindSym(i1,odd1);
      // FindSym aborts now, no need for:
      // if(sit==SymH->Symmetries->end()) cout<<"Can't find FindSym      sit->Project_l(
      // produces strange error: cout<<"Found "<<*sit<<", about to project"<<endl;
      cout<<"Found symmetry"<<i1<<","<<odd1<<endl;
      Config M0,M1; M0.saveM(); 
      cout<<"about to project M0="<<endl<<M0<<endl;
		sit->Project_l(M0,M1);
		  M0=M1-M0; cout<<", integral of change^2="<<M0*M0<<", norm="<<M0.norm()<<endl;
		  if(M1.ConfigArr[0].IsZero()) cout<<"M vanishes"<<endl;
		  M1.restoreAnd(); // restores to cell::M,And renormalizes,CalcAllFromM
    } // end case G
	} // end switch
       
		// Pausing=true; 
		SceneChanged=true;
		break;
case 'i': // initialize to winding-number state
   {  short j; bool odd1; real Amp;
      paramFile>>junk>>i1>>st1>>j>>Amp;//i1=winding #,st1=parity, j=which state
      CheckScript(junk,"initWinding");
      if(st1=="-") odd1=true; else if(st1=="+") odd1=false; else MYEXIT(29);
      cout<<"About to make seed with w="<<i1<<", ";
      if(odd1) cout<<"odd"; else cout<<"even";
      cout<<" parity, j="<<j<<endl;
      C1 = Config(MVector(0,0,RectFromID[0]->Ms)) +
		  Amp*MakeSeed(i1,odd1,j); // add a z component
      //cout<<"Made seed C1="<<C1<<endl;
/* delete
	  CellIDType WatID=WatchedLLGCell->SmCellID;
	  for (short jj=-5; jj<5;jj++)
		  cout<<"w="<<jj<<", seed="<<MakeSeed(jj,true,0).ConfigArr[WatID]<<endl;
*/

      C1.restore(); 
      //cout<<"restored seed C1="<<C1<<endl;
		// Pausing=true; 
		SceneChanged=true;
   }
		break;

#endif  //  _GROUPS		

#ifdef _NORMAL 
	case 'N': // get set to calculate normal modes
	  if (!My_norm) My_norm = new NorModes();
	  if(My_norm) cout<<endl<<"Creating normalModes"<<endl;
	  else {cout<<"Can't create My_norm!"<<endl; MYEXIT(49);}
	  My_norm->setNormalModeFromScript(paramFile); // was ReadNM(paramFile);
	  // not needed unless CBMode=READ_COMMAND!  SceneChanged=true;
	  break;
	  
	case 'n': // calc a new normal mode
	  My_norm->StartNewModeCx(paramFile); // finishes reading line w/info about symmetry
	  break;
#endif  //  _NORMAL

		case 'P': // set pause mode
		paramFile>>junk; CheckScript(junk,"Pause");
	  #ifdef GRAPHICS
        Repeat = false; Pausing=true;
		SceneChanged=true; // so user sees right thing while paused

      #endif  //  GRAPHICS
	    break;

	 case 'Q': // quit
		if(My_norm) My_norm->ModesOutF.close(); // close file first
	    cout<<"Read Quit command, press enter in this window to close"<<endl;
		MYEXIT(87);
		break;

	case 'R': // run (evolve via LLG) for a while
		char linestr[99]; paramFile.get(linestr,99);
		cout<<"Read line:"<<linestr<<endl;
		CheckScript(linestr,"Run");
		cout<<"At t="<<t<<", run: termination criterion is "<<TermStrings[TermCrit]<<endl;
	  cout<<"  & alpha="<<WatchedCell->rect->alpha<<endl;
#ifndef GRAPHICS
	  if(TermCrit==NONE) {cout<<"You need a termination criterion to run without graphics!"<<endl;
  	     MYEXIT(87);}
#endif  //   |GRAPHICS
	  //paramFile>>junk;
      //CheckScript(junk,"Run");
	  // do we really need this?:
      SetEvolParams(); // sets everything that depends on dt, damping, gamma
	  CBMode = RUN; // ??tho may be overwritten by STOP below?
	  ConvSteps=0; // count steps 

	  if(cons) cons->Prorating=false;
	  if(t<dt) {
          ExportData(WriteFileInt,XportConsole); // write t=0 on file &/or console
          if (WritingMovie) writeFrameBinary();
      }
      break;
	case 'r': // remark
	  // paramFile.get(junk,'\n');//param is streambuf,not string!; gets to EOL=\n but leaves EOL
	  getline(paramFile,junk,'!'); // must have delimiter, or will skip next line!
	  if (paramFile.peek()!=(int)'\n') {cout<<"Need delimiter ! after remark."<<endl; MYEXIT(39);}
	  cout<<junk<<endl;
	  break;
	case 's': // Sweep some field (not moment, for now) at constant rate
	  paramFile>>junk>>StepMode>>floatdum>>StepStop>>xdum; //xdum is dH/dt
      CheckScript(junk,"sweep");
	  M1=Modes[StepMode]; assert(M1);
	  M1->HVal=floatdum; // starting point
	  StepInc=xdum*dt;
	  cout<<"Found 'sweep', mode "<<StepMode<<",start@"<<floatdum<<", stop@ "<<StepStop<<
		  "rate="<<xdum<<", inc="<<StepInc<<endl;
	  TermCrit=MODEVAL; // stop when hit StepStop
	  // do we really need this?:
      SetEvolParams(); // sets everything that depends on dt, damping, gamma
	  CBMode = RUN; // or STEP?? tho may be overwritten by STOP below?
	  // not needed: SceneChanged=true;
      break;

      case 't': //set termination criterion
          paramFile>>junk>>st1;

		  CheckScript(junk,"termin");
		  switch(st1[0])
		  { // there are other values, not set here
        	case 't':
			  paramFile>>xdum;
	          tstop=t+xdum;
		      TermCrit=TIME;
              cout<<"term. crit. is time, new tstop="<<tstop<<endl;
			  if(!paramFile) EXIT1("Error reading termination time");
              break;
        	case 'n':
		      TermCrit=NONE;
              cout<<"No term. criterion"<<endl;
              break;
			case 'o': // order loop; start at max order
			  TermCrit=LOOP_ORD; // want to write file with answers
			  paramFile>>TestCase; // this should be separate function, but where?
			  switch(TestCase)
			  {case '1': // 1D test
			  { // create scope to ensure destruction of mpoles
			     real hf = root->half[1]; // half cell size
			     assert(hf==WatchedCell->half[1]);// one cell wide in y dir
			     assert(hf==WatchedCell->half[2]);// & z dir
				 mpole* qSquare=new mpole(MyFMM->MaxOrder); // will be moment of square
 
				 qSquare->SetToZero();
              real val = WatLLGPart->rect->Ms*4*hf*hf; // charge=M*area
				 qSquare->values[0]=val;

             if(MyFMM->MaxOrder<2) goto DoneqSquare; // very unlikely! do 2nd order:
				 val*=0.5*hf*hf; // first value to use, worked out by hand
				 qSquare->values[MyFMM->index[0][2][0]] = val; // p. 21.4r
				 qSquare->values[MyFMM->index[0][0][2]] = val; // probably enough!
             if(MyFMM->MaxOrder<4) goto DoneqSquare; // do 4th order:  
				 val*=hf*hf/12; // total hf^4/24
				 qSquare->values[MyFMM->index[0][4][0]] = val; 
				 qSquare->values[MyFMM->index[0][2][2]] = 6*val;
				 qSquare->values[MyFMM->index[0][0][4]] = val; // 
             if(MyFMM->MaxOrder<6) goto DoneqSquare; // 6th order:
				 val*=hf*hf/30;  // total hf^6/720
				 qSquare->values[MyFMM->index[0][6][0]] = val; 
				 qSquare->values[MyFMM->index[0][4][2]] = 15*val;
				 qSquare->values[MyFMM->index[0][2][4]] = 15*val;
				 qSquare->values[MyFMM->index[0][0][6]] = val; // stop 6th order
DoneqSquare:
             qSquare->printMP("\n1D test: q of square",cout);
				 mpole* Pot=new mpole(MyFMM->MaxOrder);
				 mpole* scra1=new mpole(MyFMM->MaxOrderD);
				 mpole* scra2=new mpole(MyFMM->MaxOrderD);
				 real FromRight= WatchedCell->center[0]-(root->center[0] + root->half[0]);//<0
             real FromLeft=WatchedCell->center[0] - (root->center[0] - root->half[0]); // >0
				 cout<<endl<<"  FromLeft="<<FromLeft<<", FromRight="<<FromRight<<endl;
				 Pot->SetToZero();
				 Pot->AddTaylor(qSquare,MVector(FromRight,0,0),scra1,scra2); // add pot. from left
             *qSquare *=(-1.0); // reverse sign for south pole
				 Pot->AddTaylor(qSquare,MVector(FromLeft,0,0),scra1,scra2);
             // Pot->printMP("Pot at watched cell",cout);
				 WatchedH.SetToZero();
				 WatchedH[0] = Pot->values[1]; // expected value
				 cout<<"Pot.x="<<Pot->values[1]<<", WatchedH="<<WatchedH<<endl;
			  }
				 break;
			  case '2': // 2D test
				  cout<<"2D disk test not written yet"<<endl;
				  break;
			  case '3': // 3D test
				   cout<<"sphere test not written yet"<<endl;
			  case 'l': // no test case, just loop order
              cout<<"Termination criterion = loop order"<<endl;
				  break;
			  default: cout<<"Unrecognized test, line TermCrit o..  "<<endl; MYEXIT(82);
			  } // end switch
			  OrdFile = new ofstream;
			  OrdFile->open((ScriptDir+runID+".Ord.csv").c_str());
			  if(!OrdFile->is_open()) {cout<<"Can't write file *.Ord.csv"<<
		           "-- is it in use?"<<endl; MYEXIT(38);}
			  *OrdFile<<"order"<<d_<<"Error"<<d_<<"Mx"<<d_<<"My"<<d_<<"Mz"<<endl;
			    // header line
			  break;
			case 'G': // calc Green function (also in loop over order)
#ifdef DBG_
				TermCrit=CALC_GREEN;
			 // We can create stuff for DebugGreen
				short Gdir1;
				paramFile>>Gdir1; // direction to plot along
				DBG = new DebugGreen(Gdir1); 
			   if(!OrdFile) 
            {
               OrdFile = new ofstream;
               OrdFile->open((ScriptDir+runID+".Ord.csv").c_str());
            }
			  if(!OrdFile->is_open()) {cout<<
              "Error, no file to write Green fn data to"<<endl; MYEXIT(28);}
			  *OrdFile<<"Calculate Green functions now"<<endl
              <<"order"<<d_<<"SumAbs"<<d_<<"MeanSq"<<d_<<"MaxErr"<<
                d_<<"WorstCellx"<<d_<<"y"<<d_<<"z"<<endl; // header line

            DBG->CalcGreen(); // runs whole calcualtion and prints
		      // DBG->SetDBG(); now in constructor; already have FMM
            // now go to display, then in callback call CalcGreen again
#else
				cout<<"DBG_ undefined, ignoring 'Green' termination criterion"<<endl;
#endif //!DBG_
                break;
			default: cout<<"unknown criterion: "<<junk<<", "<<st1<<endl; MYEXIT(41);
		  } // end switch
		  if(!paramFile) cout<<"Bizarre error5"<<endl;
		  break;

	  //case 'U': // update fields (now always done if can be seen)
		//CalcAllFromM(1); // arguments force recalc of H
		//updateOIScene();
		  //SceneChanged=true;
        //break;

	  case 'W': // wait for user before starting
        #ifdef GRAPHICS
		  Pausing=true;
          //cout<<"Setting CBMode to STOP"<<endl;CBMode = STOP; 
         #else %GRAPHICS // note this option creates problems debugging, just freezes
           cout<<"********* Bad option T | W, not using graphics"<<endl;
         #endif  //   |GRAPHICS
           break;

	 case 'z': // zero out time (set initial t=0)
        cout<<"Resetting time to zero"<<endl;
        t = 0; tDisp=0;
        break;

/* SOme ReadScript2 commands that are not now implemented:
  short counter=0; //to prevent infinite repetition in calc kernels
  char junk[MAXLENFMM+1], fileName2[MAXLENFMM+1]; 
from script file:
  #exteriorPotentialCompute Image1.dat
  #ExteriorPotentialRead Images.dat
  #NoInterRectKernels # use multipole field for different rect, even if close

         case 'e': //Calculate cored potential of exterior (periodic systems)
		    inputFile>>junk>>fileName2;
			if (!(periodic[0] || periodic[1] || periodic[2])) 
			{cout<<"Don't calculate exterior potential if not periodic!"<<endl; 
			  MYEXIT(94);}
			ImageFile.open((ExeDir+fileName2).c_str(), ios::out);
  		    calcVextAndWriteImages();
		    cout<<"Done calculating images, exiting"<<endl;  MYEXIT(27);
            break; // COULD just go on to do calculation here! 
			  // with or without re-read.  Think about that later.
		  case 'E': // read cored potential (Exterior) & images
            inputFile>>junk>>fileName2;
			if (!(periodic[0] || periodic[1] || periodic[2])) 
			{cout<<"Don't read exterior potential if not periodic!"<<endl; 
			  MYEXIT(28);}
			ImageFile.open((ExeDir+fileName2).c_str(),ios::in);
			if (!ImageFile)
			{cerr<<"can't open file "<<fileName2<<endl; MYEXIT(29);}
              // can't actually read it here, need geometry first
			break;
		  case 'N': // between rectangles, use mpoles, not kernels
			NoInterRectKernels=1;
			break;
	// Following is from fmm reading; keep til images work, then delete:
	if (Psetup)cout<<"KernelsFile file is "<<fileName1;
    if(ImageFile) cout<<", images file is "<<fileName2;
    cout<<"\nCalcCubicKernels = "<<CalcCubicKernels
    <<", MaxOrder = "<<MaxOrder
	<<", NoInterRectKernels="<<NoInterRectKernels<<endl;}
*/
	 default:
		WriteAbort2:
        getline(paramFile,junk);
		cout<<"Unrecognized 1st char:"<<ch<<"=#"<<(short)ch<<
		 "; whole line="<<junk<<endl; MYEXIT(88);
} // end switch(ch) of ReadScript2.
	 if(!paramFile) {cout<<"Error in script2, near:"
		  <<junk<<endl; MYEXIT(22);}
        getline(paramFile,junk,'\n'); // prepare for next line
	    paramFile>>ws; // skip white space at beginning of next line

#ifdef GRAPHICS
		if(Pausing) // if not, next callback will update scene  
		{
		  if (SceneChanged) 
		  {  CalcAllFromM(0);
		     updateScene();	
   		  }
	    return; // want to display result of each scriptfile command
		}
#endif  //  GRAPHICS
		if(CBMode!=READ_COMMAND) return; // to HandleCallback()
		// else go back and read another ch
  } //end while; must have hit ! marking end of file; quit program:
  CBMode=EXIT;  
// what was this about??? #ifdef GRAPHICS Pausing=false;#endif  //    Repeat=true;
} // end ReadScript2; return to HandleCallback()

cell* Systm::parseBuild(cell *parent1) // creates subtree
 // by reading paramFile.  Returns pointer to subtree.
 // initial call is root=parseBuild(NULL)
 // needs parent1 only to set parent=parent1.
{  	
	//assumes input stream pointer at beginning of line (before tabs)
        // leaves input stream pointer at the beginning of the next line
  char ch; //short idum;
  //cout<<"Enter parseBuild() function with parent1="<<parent1<<endl;
  //cout<<endl<<"In parseBuild:";
  string junk;
  cell* ptr; // this is to be returned, after constructing it

  paramFile>>ws; // eatwhite, get rid of tabs
  // cout<<"After ws";  
  //  for (short i=0; i<10; i++) {paramFile>>ch; cout<<ch<<"(#"<<(short)ch<<")";}
	  //cout<<"| was next"<<endl; // dumps 10 characters to cout

  // Read lines in loop. Actually you will read only ONE line, but it is in loop
  // to allow user to stick in comments and blank lines for readability.
    while ((ch = (char)paramFile.peek()) != '!')   // '!' indicates end of rects in ML
  {  //if(ch!='#') cout<<"In ParseBuild, read char #"<<(short)ch<<":"<<ch<<endl;
   switch(ch)
   {
    case '#': 
      getline(paramFile,junk,'\n');
      break;

    case 'M': // Multilayer Build, construct children & build them
		// means parseBuild will read an ML.object, ending with ! end ML
      //cout<<"in parseBuild(), in case M"<<endl;
        //parseBuild assumes we're at the beginning of a line.
	  // don't even read rest of line, let MLBuild do that.
      ptr=MLBuild(parent1); // read rectangles, put under ptr
	  // reads ML object: 'M' line through "! end ML"
	  // don't know size and center, must get recurursively
      //ptr->GetSizeCenterRecursively(); // also sets radius2   Don't need this anymore? (June 2013)
      // cout<<"MLBuild ret ptr="<<ptr<<" cell="<<*ptr<<
		  //" to parseBuild, now ret to caller"<<endl; 
	  goto ReturnNow; // if this fails, copy return stmt to here & below
      break;

	case 'P': // ParentBuild, construct children & build them
	  cout<<"Option P is really screwed up -- fix this"<<endl; MYEXIT(88);
/*
	  if(Psetup) cout<<"in parseBuild(), in case N"<<endl;
        //parseBuild assumes we're at the beginning of a line.
	  ptr = new cell(); ptr->parent=parent1;
	  paramFile>>junk>>idum; // 1 if we show M of this composite cell
#ifdef GRAPHICS 
	  // no ShowM now; what to do here? ptr->ShowM=idum;
#endif  //  GRAPHICS
	  assert(junk[1]=='o'); // as in Non... !!!!!!!!!!! only worked if said "Po..."
      getline(paramFile,junk,'\n'); //EOL,ready to call parseBuild

      ptr->child0=parseBuild(ptr); // reads object1 (about child0)
	// for 0th child, parseBuild leaves us at the beginning of a line.

      ptr->child1=parseBuild(ptr); 
	  paramFile>>ws; // eatwhite
	  getline(paramFile,junk); // read technically unnecessary sanity line
      if ((junk[0]!='!') || (junk[6]!='n'))
	    {cout<<"Error ending ParentNode object, expected ! end node, read "
	      <<junk<<endl; MYEXIT(88);}
	  // Now have both children, construct parent size
	  //ptr->GetSizeCenter(); // also gets radius2     Don't need this anymore? (June 2013)
      cout<<endl<<"parseBuild made non-rect "<<ptr<<",halfsize="<<ptr->half<<"center="<<ptr->center<<endl;
	  goto ReturnNow; // end of case P
*/

    default:
      cout<<endl<<"Error in parseBuild() input file, 1st char is:"<<ch<<endl; 
      MYEXIT(23); // exit(2);
    } // end of switch, should be at '!' now
   paramFile>>ws; // eatwhite, get rid of tabs

   } // while loop never terminates, you jump out via "return ptr"(no ! terminator) 
   // if(!paramFile) {
	cout<<endl<<"Error in geometry script, near:"<<junk<<endl; MYEXIT(24);
ReturnNow:
   // already at next line!! getline(paramFile,junk,'\n'); 
    // so you leave this function at beginning of next line
   // cout<<"Returning "<<ptr<<" from parseBuild"<<endl;
   return ptr;
} // end parseBuild()

cell* Systm::MLBuild(cell *parent1)
{  // read rectangles to build MultiLayer. Returns ptr to ML subtree
	// called from ParseBuild if reads "MLBuild" 
	// (hasn't read line, just peeked at 'M').
	// parent1 can be NULL, if whole system is one ML.
        // leaves input stream pointer at the beginning of the next line
	cell* ptrML=NULL; // this will be returned, is entire subtree of ML
  char ch;
  //cout<<"enter MLBuild() with parent1="<<parent1<<endl;
  //cout<<endl<<"In MLBuild:";
  string junk;

  bool ShowM1; // save to put in ptrML->ShowM
	  paramFile>>junk>>ShowM1; // 1 if we show M of this composite cell
	  CheckScript(junk,"MLBuild");
      getline(paramFile,junk,'\n'); //EOL,ready to read first rectangle

  paramFile>>ws; // eatwhite, get rid of tabs
  // cout<<"After ws";  
  //  for (short i=0; i<10; i++) {paramFile>>ch; cout<<ch<<"(#"<<(short)ch<<")";}
	  //cout<<"| was next"<<endl; // dumps 10 characters to cout

  while ((ch = (char)paramFile.peek()) != '!')   // '!' indicates end of rects in ML
  { //if(ch!='#')cout<<"In MLBuild, read char #"<<(short)ch<<":"<<ch<<endl;
   if(!paramFile) MYEXIT(88);
   switch(ch)
   {
    case '#': 
      getline(paramFile,junk,'\n');
      break;

    case 'R': // RectBuild, construct entire rectangle
	  cell* ptr; // can't combine these 2 lines! Very odd.
	  ptr = new cell(); // pointer to new cell
      // note parent defaults to NULL, must be set later.
	    // First decide where to put it
	    // find hook to hang new cell 
		// march down left and right sides of subtree
      if(ptrML) // not first rect in ML
	  {  cell* ptrLeft; ptrLeft=ptrML;
		 cell* hook; // on right side of tree
		 hook=ptrML; // find occupied location without rect. children
	     while (ptrLeft->rect==NULL) // still above rect. level
		 {
		    if (hook->rect) goto HangOnHook;
            ptrLeft=ptrLeft->child0; hook=hook->child1; //go down
		 }  // if finish loop (reach rect on left),no room:
		 hook=ptrML; // hang at top  
HangOnHook:
/*		 while (hook->child1) // find highest one w/o children
		 {
			 ptrLeft=ptrLeft->child0; hook=hook->child1; // go down
		 } 
		 //if(hook==ptrML) MYEXIT(99);
		 if(ptrLeft!=NULL) // bottom of tree, can't fit new node
			 hook=ptrML; // hang at top
		     // else tree goes deeper at left, there's room
*/
         // Now we know where we want to hang new node (on hook)

		 assert(hook); // shouldn't be null, because we checked ptrML wasn't
	     cell* hookNew = new cell(); // will replace hook by this
#ifdef DEBUG2
		 cout<<"Hanging non-1st rect from "<<hook<<
			 ", which moves down and is replaced by "<<hookNew<<endl;
#endif  //  
		 //hookNew->TransferParentPointer(hook); // fixes both directions
		 // hook->parent->child1=hookNew;
		 // don't NEED to fix both directions! Just need
		 hookNew->parent=hook->parent; 
		 if(hook==ptrML)  // we are at the top
		    ptrML=hookNew; //pointer changes
		 	//hookNew->parent->childX fixed on return from Parsebuild
		 else // not at top, fix downward pointer
			 hookNew->parent->child1=hookNew; // always to right
         // PBV4 hookNew->parent->child1=hookNew; 
		    // hookNew->parent->childX set after return to parseBuild!
         hookNew->child0=hook; hook->parent=hookNew;
		 hookNew->child1=ptr; ptr->parent=hookNew;
		   // No, set recursively in parseBuild, after MLBuild: We get here if there's more than one rectangle, so diameter undefined??
           // ptrML->radius2=9e9;

		 } // end if ptrML
      else // ptrML=NULL, i.e. 1st rectangle
	 { assert(!ptrML); // delete this
		 ptrML=ptr; // this will be our first node
		 //cout<<"1st rectangle, ="<<ptrML<<endl;
		 ptrML->parent = parent1; // note that reverse pointer child0 
		     // or 1 will be set upon return to parseBuild?
	  }

   // Now know where rectangle goes (ptr), read it & put there.

	  ptr->rect = new Rectang(ptr);
	  ptr->readSize(); // size info at end of line, store at ptr
          //create an instance of Rectang whose WholeCell is ptr
      cout<<endl<<"In MLBuild, rect halfsize="<<ptr->half<<endl;

	  //if(Psetup) {cout<<"List Cells in ML (last not subdiv!)"<<endl; ptrML->ListCellsRecursively();cout<<"EndList"<<endl;} // to debug
      assert (ptr->rect);
	 cell* oldptr;
	 oldptr = ptr;
     ptr = ptr->rect->setup(); // reads more lines; separate function to refer to rect
	 // pointer to root of rect is returned, in case it has changed from splitRecursively (changed 6/2013 RJR)
	 if (oldptr!=ptr && oldptr==ptrML) ptrML=ptr; // update ptrML if necessary
        // divide cell, set properties, etc, until '!' end rect
	    // next line should be next RectBuild, or ! end ML

//changed by DMAk; NOT debugged recently (8/05)
//root cell size has been set in case 'P', if we calculate Kernels we leave it unchanged
      if (MyFMM) 
	  if (MyFMM->CalcCubicKernels)   //we calculate Kernels, added by DMAk, DMAfs
      {  //we need to find the smallest required size of the system
        assert(ptr==root); 
        real dMin = sqrt (ptr->rect->LLGSize*ptr->rect->LLGSize/MyFMM->MaxAngle2);
                                    //LLGSize*LLGSize is radius2 for smallest cell
        short id[3];                  //size of the system in units of smallest cell
        id[0]=id[1]=id[2]=4;        //initialization
        for (short iSizeDet=0; iSizeDet<3; iSizeDet++)
        { 
          while (ptr->rect->LLGSize[iSizeDet]*0.5*id[iSizeDet] < dMin)
            id[iSizeDet]*=2;
       //we use coarse grain cells for Kernel calculation (must be 2 times larger)
          PeriodVec[iSizeDet] = 2.*id[iSizeDet]*ptr->rect->LLGSize[iSizeDet];
          ptr->half = PeriodVec*0.5;
          ptr->center = ptr->half;
        }
        cout<<"the size of the system is found: "<<PeriodVec[0]<<" x "
          <<PeriodVec[1]<<" x "<<PeriodVec[2]<<endl;
      } // end if(CalcCubicKernels)
      //find the largest size of the cell (used to scale the M vector in Arko's experiment)
      
        assert(ptr->rect); 
		real lengthScale; // used to set to largest side (will stick out on others)
	    lengthScale = ptr->rect->LLGSize[0];  // now set to SMALLEST (half) side
        for (short iSide=1; iSide <3; iSide++)
          if (lengthScale > ptr->rect->LLGSize[iSide]) lengthScale = ptr->rect->LLGSize[iSide]; 
        lengthScale*= 0.45; //take <half of that
		// Obsolete: cout<<"lenthScale after mult by MLen"<<lengthScale<<" nm"<<endl;
        GlobalMScale = lengthScale/ptr->rect->Ms; // determined by LAST rectangle
  	      //so you can multiply this by M and get the right length 
	      // extra MLenFudge factor entered by user may be put in GlobalMScale later

#ifdef DEBUG2
		  cout<<"in MLBuild(), case R:"; 
          ptr->half.print("rect halfsize=");
#endif  //  }

      //ptr->initMRecursively(); // to constant or spin wave, as read in rect->init.
	    //  May be overwritten later by initial condition file, or option 'c'.
      cout<<" MLBuild 'R' made rect "<<ptr<<", "<<*ptr<<endl;

	  //cout<<"So far, ML="<<ptrML<<" "<<*ptrML<<endl;
	  break; // end of case 'R'

 default:
      cout<<endl<<"Error in MLBuild() input file, 1st char is:"<<ch<<endl; 
      MYEXIT(24); // exit(2);
    } // end of switch, should be at '!' now
   paramFile>>ws; // eatwhite, get rid of tabs
   } // while loop terminates when peeks at !end 
   if(!paramFile) {cout<<endl<<"Error in geometry script, near:"
		 <<junk<<endl; MYEXIT(24);}

   getline(paramFile,junk,'\n'); 
    // so you leave this function at beginning of next line
#ifdef _DEBUG
   if (junk[0]!='!' || junk[6]!='M') 
      {cout<<"Error, should ! end ML, read"<<junk<<endl;MYEXIT(27);}
   cout<<"at end of MLBuild, read line terminating ML:"<<junk<<endl;
   //cout<<"return from MLBuild-ing cell@"<<ptrML<<"which is "<<*ptrML<<endl;
#endif  //  _DEBUG
#ifdef GRAPHICS
   if(ptrML->rect)// there was only one rect in ML
	   if(ptrML->rect->ShowM!=ShowM1) {cout<<"Conflicting ShowM flags rect->ShowM="
         <<ptrML->rect->ShowM<<", ShowM1(ML)="<<ShowM1<<" for single-rectangle "
		   <<"ML, will show it anyway;"<<endl; ptrML->rect->ShowM=true;
           cout<<"Should fix script file, near "<<junk<<", press enter"<<endl; cin.get();}
	   // may be puzzling, if both ShowM are 0, because set=1 by setup()
#endif  //  GRAPHICS
   // note we have not kept up with composite cell->size, do after return
   return ptrML;
} // end MLBuild()
 
Rectang* Systm::RectNamed(string Nam)
{   for (int i=0;i<NRects;i++)
      if(Nam==RectFromID[i]->RectName) return RectFromID[i];
    cout<<"Rectangle name not found:"<<Nam<<endl; MYEXIT(21);
	//return NULL;
}

void Systm::CalcAllFromM(bool FirstUpdate) // , bool LastUpdate) 
  // assumes small-cell M(t) known, calculates all from them:
  // external+pulse field (depends on t)
  // moments, magnetostatic field, exchange, 
  // updates graphics (updateOITree, externalField->UpdateVector)
  //  if (!FirstUpdate) some calcs skipped; //if(LastUpdate) refresh display
{
   // cout<<"Entered CalcAllFromM, hit key"<<endl; cin.get();
	if(MyFMM) root->updateMoment(); //update moments, initialize SmoothField, DMAf //possibly not worth parallelizing? Rosati Jun 2013
	// if no FMM, once calculated root moment "by hand";
   //root->moment->printMP("root moment",cout);
	//total field: bias, pulse and demagField from the far(???) surface
   if (FirstUpdate) // calculate stuff that depends only on t (needn't recalculate later)
	     // NOTE 2011: Try to minimize these tests! Hyst just changes bias field, BEFORE calling
		 // evolve(), which calls this CalcAllFromM.
   {  H_ext = H_bias;
      if(Ext_Field1) //pulsed field is present
		   H_ext += Ext_Field1->GetValue(t);
      if(Ext_Field2) H_ext += Ext_Field2->GetValue(t); // allow only 2
	  // note that modes are NOT added here, because non-uniform
	  if(ACExt_FieldPtr) H_ext += ACExt_FieldPtr->GetACField();
	  //cout<<"About to add MedField, H_ext="<<H_ext<<endl;
	  if(MyMedField) MyMedField->AddMedField(H_ext); // add field of media grain
	  //cout<<"After add MedField, H_ext="<<H_ext<<endl;
	  // cout<<", H_ext="<<H_ext<<endl;
    } // end if FirstUpdate

   if (MyFMM) root->SmoothField->SetToZero();
	//#pragma omp parallel
   root->updateStat(); // no longer calculates energy
	  // but does calculate all local fields

   /* DELETE
#ifdef DBG_ // if debugging Green fn, don't need to go any further!
   if(DBG) DBG->ReportResults();
     // And would like to display correctly, so M must be nonzero:
   for (CellIDType ic = 0; ic < LLGcell::NextSmCellID; ic++) CellFromID[ic]->M[2]+=0.000001;
    Repeat=false; // can't find clean way to terminate,
      // will complain next time that wrong moments nonzero.
	Pausing=true; // callback just repeats if true; 
	  //  can use keyboard Q
#endif // DBG_
	  */
}  // end CalcAllFromM

void Systm::evolve() // on entry AND exit, M and H all known at t, ready to display
{  // called from main() and Systm::HandleCallback()
   LLGcell* c1=NULL; // dummy for loops
   CellIDType i; short j; real x;
#ifdef DEBUG3
   cout<<"In Systm::evolve() now, t="<<t<<"!!!!!!!!!!!!!!!!!!!!"<<endl;
#endif  //  DEBUG3
   // save stuff for differences (used in ExportData & Delta_H)-note
   // that you could save time by omitting this except when  ExportData

   // calculate delM for each cell
   MaxDM = 0.; // quicker to just do it than to check if needed!
   //if (!Repeat) cout<<"DEB: enter Evolve w/ cell#0 Mx="<<CellFromID[0]->M[0]<<endl; 

   for (i=0;i<LLGcell::NextSmCellID;i++) // loop smallest cells
   {  c1=CellFromID[i];
      if(Noise) c1->H_rand = c1->RandomFieldForM(); 
          // note that this is random field from t-dt to t; don't recalculate 2nd time through 
	      // note that sometimes Noise is on even though T=0; trust 
      c1->CalcDelM();   
	  //if (i==0) cout<<"DEB: got DelM="<<c1->DelM<<endl;
	  for (j=0;j<3;j++) {x=fabs(c1->DelM[j]); if(x>MaxDM) MaxDM=x;} 
	  if(Heun) c1->DelMpre=c1->DelM; // save delM ; always?
	  //if(!Repeat) if (i==0) cout<<"DEB: abt to add "<<c1->DelM<<" to "<<c1->DelM<<endl;
	  c1->M+=c1->DelM; // This looks dangerous! Because we change this cell's M, 
		// on which the next cell's fields depend.  But these fields have already been
		// calculated, so changing M does no harm.  
		// The only M used explicitly in CalcDelM is this->M.
   } // end for i
   j=0; // place holder
   //if (!Repeat) cout<<"DEB: in Evolve, chgd cell#0->Mx to "<<CellFromID[0]->M[0]<<endl; 
   //if(CBMode==STEP) {Noise=false; 
     //if(NudgeMode) {Modes[NudgeMode]->HVal=NudgeAmt;}
   // maybe turned on as nudge; done with noise or modes

#ifdef ORBITS
   if(MyOrbs) // doing orbits, need normalize DelM to OrbSep
   {
	   real DeltaM=WatchedCell->DelM.Magnitude();
	   if(DeltaM>0) 
	   {  dt = dt*MyOrbs->OrbSepHalf/DeltaM;
	       // was dt = MyOrbs->FixDt(WatchedCell->DelM.Magnitude()); 
	       SetEvolParams(); // because dt was changed
	   }
   }
#endif  //  ORBITS
   t+=dt; tDisp=(real)t; nt++; // now M is known at t, H at t-dt
#ifdef GRAPHICS
   if(Strobe) DisplayTime=false; // display only at cycle end
   else DisplayTime=(nt%d==0); // display every d steps
#endif  //  GRAPHICS
   //cout<<"new nt = "<<nt<<", dt = "<<dt<<endl;

   // Now we have new M, old H, see if converged so we can reset HVal BEFORE CalcAll.
   //?? why did we need to remember H to see if it converged?

   //  idea: why don't I normalize BEFORE calcAllFromM? Just to save having 
   // 2 loops over cells (norm & Heun)? Think about it.
  // use this M to calculate fields, etc at time t:
   CalcAllFromM(1);  // args=FirstUpdate(calculate stuff dep. only on t, same for 2nd)
     // now M & H all known at time t

   // for debugging 06/2013
   //if (WatchedLLGCell) cout<<"Watched M="<<WatchedLLGCell->M<<", H_quad="
	//   <<WatchedLLGCell->H_quad<<", H_lin="<<WatchedLLGCell->H_lin<<endl;

   if (MaxDM>ErrDM)  // check for rotation too large
   { cout<<"Warning: DeltaM="<<MaxDM<<" > "<<ErrDM<<"=allowed"<<endl;
     if(cons) // constraining, might be because of artificial field large
	 {	cons->MaxField*=0.5;
	    cout<<"Decreasing MaxField to "<<cons->MaxField<<endl;
	 }
	 else // not constraining, just decrease dt
	 { cout<<"decreasing dt & reverting to Pause mode."<<endl;
       Cut_dt(); Repeat=false; Pausing=true; HandleError();
	 }
   } // end if(MaxDM>

 // In this loop, normalize, but if Heun also make another pass to correct M
   for (CellIDType i=0;i<LLGcell::NextSmCellID;i++)        
     {  c1=CellFromID[i];
       //if(!c1->child0) // loop smallest cells {
		  if (Heun) {c1->CalcDelM(); c1->M+=(c1->DelM-c1->DelMpre)/2;}
		  //cout<<"2nd loop, DelM="<<c1->DelM<<", new M="<<c1->M<<endl;
#ifndef LINEARIZE
		 c1->M.normalizeTo(c1->rect->Ms); 
#endif  //   |LINEARIZE
    // Now have final correct M
      //c1->q.rotateTo(c1->M); // calculate q before displaying!
          // do some debug checks:
#ifdef _DEBUG  
          // c1->Mqsane(); no longer use q here
#endif  //  _DEBUG
    } // end for i
     // if (!Repeat) cout<<"DEB: later in Evolve, normalize c1->Mx to "<<c1->M[0]<<endl; 
	 if(Heun) CalcAllFromM(0); // 0 means not FirstUpdate

#ifndef LINEARIZE  // crude but fast sanity check on last cell's M:
	 if(fabs(c1->M[1])>0.001+c1->rect->Ms) {cout<<"Error, last cell's My="
		 <<c1->M[1]<<" > Ms="<<c1->rect->Ms<<endl; MYEXIT(91);}
#endif  //   |LINEARIZE
   // Now everything is known at time t
   
  if((nt%XportInt) == 0) ExportData(WriteFileInt,XportConsole); // Calculates data not usually needed, 
   // such as energy and MR, and writes to file *.data
    // Annoying thing: if Xport isn't pointer, if(Xport(good()) or if(Xport) returns TRUE if file hasn't been opened! 
#ifdef DEBUG3 //
  if(WatchedCell) cout<<"M0="<<CellFromID[0]->M<<", M1="<<CellFromID[1]->M<<endl;
#endif  //  DEBUG3
   if(WritingMovie) if ((nt%MovInt) == 0) writeFrameBinary();
// no longer test for termination here
} // end evolve()

void Systm::ExportData(bool XF, bool XC) // writes to file *.data if XF, console if XC
   {  // First calculate outputs that haven't been needed for updating M, in loop over cells:
   // Initialize Energy using correction (omit half of Zeeman energy of external field below)
   // real dum; // moment to export, dummy
   if ((!XF) && (!XC)) return; // no need
   //if (XportE) CalcEnergy(); // ?? why is this here rather than inside if(XportE) below?
  // may be broken:
//   if(MyMR) MyMR->MR=0.0;  // initialize possible magnetoresistance calc:

/*	 for (CellIDType i=0;i<LLGcell::NextSmCellID;i++) // loop small cells       
     {LLGcell* c1=CellFromID[i];
//broken
if(MyMR) // calculating magnetoresistance
		    if (c1->rect==MyMR->Rect1)  // this rectangle is the one above the junction
			  if (c1->InterPartner) MyMR->MR += c1->M*c1->InterPartner->M; // dot prod, not normalized yet
    } // end for i
   if(MyMR) MyMR->MR/=MyMR->Rect1->Ms*MyMR->Rect0->Ms;  // normalize MR to sum of cosines
*/
      // Now done calculating Energy and MR, write all out
	   //Xport.precision(7); // set when headers writen
   if(XF) Xport<<t; if(XC) cout<<"t="<<t; // tried using endl @ beginning, too confusing
     //   if (Xport_t){} gone here
   if (XportH) {
	   if(XF) Xport<<d_<<H_ext; if(XC) cout<<", H="<<H_ext;}
   if (XportM && WatchedLLGCell) {if(XF) Xport<<d_<<WatchedLLGCell->M; 
      if(XC)cout<<", watched M="<<WatchedLLGCell->M;}
   if (XportMom) 
   {  CalcMoments(); // may have calcd constrained ones, but not all
	  for (int i1=1;i1<=XportMom;i1++) {// mode moments & fields
		real M1=Modes[i1]->MVal-Modes[i1]->MMax; // later read in whether to subract MMax this
	    if(XF) Xport<<d_<<M1<<d_<<Modes[i1]->HVal;
	    if(XC) cout<<" ModeM&H["<<i1<<"]:"<<M1<<"&"<<Modes[i1]->HVal;}
   }

   /*
   // should delete XpomDir and XpoPerp...
   if (XpomDir) {
	   if (MyFMM) dum=-root->moment->values[XpomDir];
	   else 
	   { dum=root->M[XpomDir-1]*root->volume(); // moment doesn't exist
	     if(root->child0) {cout<<"Check whether root->M exists!"<<endl; 
		   MYEXIT(73);} // if only root, you're ok (but this is pointless!)
	   }
       if(XF) Xport<<d_<<dum; //  
       if(XC) cout<<", m"<<(char)(XpomDir+'w')<<"="<<dum;}
    */
   real B1;
   if (XpoBMode || XpoProd) 
   {  B1=Modes[XpoBMode]->B_mu0(); 
      if (XpoBMode) {if(XF) Xport<<d_<<B1; if(XC) cout<<", B="<<B1;} 
      if (XpoProd) {real Prod=-B1*Modes[XpoBMode]->HVal*4*M_PI*1e-6;
        // in MGOe; without 4pie-6, is BH/mu_0
        if(XF) Xport<<d_<<Prod; if(XC) cout<<",-BH="<<Prod; 
	    if(Prod>BHmax)BHmax=Prod;} 
   }
   if (XportE || XportIE) CalcEnergy(); // int. or total

	if (XportE) { 
		  //Xport.precision(6); 
		  if(XF) Xport<<d_<<TotEnergy; 
		  //Xport.precision(4);
		  if(XC) cout<<", tot.E="<<TotEnergy; 
	      }

	  if (XportIE) { // is internal energy
		  if(XF) Xport<<d_<<IntEnergy; 
		  if(XC) cout<<endl<<", IntE="<<IntEnergy<<", ModeE="<<ModeEnergy()<<", Zeeman="<<ZeemanEnergy; 
	      } // stuck in extra ModeEnergy & Zeeman for debug, put on separate line

	  if (MyInt) //This is null if we are not using integrals
		  {// compute and write integrals
			  MyInt->InitializeInt(); //reset integrals
			  EXIT1("not debugged"); //root->integrate(); //this is a recursive function, everything gets integrated
/*			  if(XC) cout<<", Ints=";
		      for(short i=0; i<MyInt->NumInt; i++)  //write the integrals
			    {if(XF) Xport<<d_<<MyInt->ValInt[i]; 
			    if(XC) cout<<MyInt->ValInt[i]<<", ";}
*/
		  }
/* BROKEN
if (MyMR) // magnetoresistance, calcd above
	  {if(XF) Xport<<d_<< MyMR->MR; if(XC) cout<<", MR="<<MyMR->MR;}
*/
	  if((CBMode==GRID)||(CBMode==STEP)) {// 
		  if(XF)Xport<<d_<<ConvSteps; // no header in file because 
		  // didn't know =STEP when wrote header "steps?"
	     if(XC) cout<<", "<<ConvSteps<<" steps";}

	  	  //addtion by Kam 2013
	  if(My->My_norm){ // shd have Bool to set?
		  //export to normal.csv
		  Xport<<d_<<My_norm->PoPt;
		  Xport<<d_<<My_norm->PodPt;
	  }
	  //end addition by Kam 2013

	  if(XF) Xport<<endl;  if(XC) cout<<endl;
   } // end of ExportData

/* delete this, is in F:\AlaMag\OldCodeFragments\TestingBinaryWrite.txt
char hex(char ch) //returns hex representation of (int)ch: 0 - F
{  if((int)ch<=9) return (char)((int)'0'+(int)ch);
   else if((int)ch<16) return (char)((int)'A'+(int)ch-10);
   else return '#';
}
char hexL(char ch) {return hex((char)(((int)(unsigned char)ch)>>4));} // high bits
char hexR(char ch) {return hex(ch&15);} // low bits
*/

void Systm::writeFrameBinary() // to ofstream movOut)
{  	// like writeFrame, but writes exact data for restart
	// redo 4/1/08; designed to be somewhat readable by Notepad, can see T: at the 
	// beginning of lines, etc.
    assert(movOut); // movie file is open
  char char12[12];
  movOut.write("\nT:",3); // writes 3 bytes of text, for readability in notepad
  movOut.write((char*)&t,sizeof(t));
  // writeBase64Obj(movOut, &t, sizeof(t));
 // Now save binary seed. 
 //cout.precision(4);
 //cout<<"Write frame, t="<<t<<" R: "<<getSeed()<<endl;
 //if(WatchedCell) cout<<", q="<<WatchedCell->q<<", M="<<WatchedCell->M<<endl;
 //movOut<<"R:"<<dec<<getSeed()<<endl;
  unsigned long seed; seed = getSeed();
  movOut.write("\nR:",3); movOut.write((char*)&seed,sizeof(seed));
 Config mag = Config(); mag.saveM();
 // don't really need this: movOut.write("\nConfig:",8);
 mag.writeBinary(movOut);
 movOut.write("\n!end frame",11); // was: "at step "<<dec<<nt<<", time "<<t<<endl;

#ifdef WIN32
        sprintf_s(char12,12,"#%.4i",int(nt%10000)); // use last 4 digits
		// oddly, sets unused chars to -3=FD, last to 0
#else
		sprintf(char12,"#%.4i",nt%10000);
#endif  //  
	   movOut.write(char12,5);
} // end writeFrameBinary

//the version of the movie file this readFrameBinary() can read
//is stored in movieVersion. Each time we make a crucial change
//to this function we need to change movieVersion accordingly.
//Possibly we can have a switch statement in readFrameBinary() that
//allows us to read different versions of the movie.:

bool Systm::readFrameBinary() // from movIn
 {// Why we do this in binary: When something interesting happens, 
  // we want to be able to start from the previous saved frame EXACTLY
  // so we need binary M and/or q.
#ifdef GRAPHICS
  char char12[12];
  unsigned long seed;
  string junk;
  //Config C1;
  MVector TempM;
  //really noe point allowing order to vary; just read in order! 
  movIn.read(char12,3); 
  if(!movIn.good()) {movIn.clear(); return 0;} // probably end of file, clear so can rewind
  CheckScript(char12,"\nT:"); // should begin with time
  movIn.read((char*)&t,sizeof(t));
  movIn.read(char12,3); CheckScript(char12,"\nR:"); // random seed
  movIn.read((char*)&seed,sizeof(seed));
  setSeed(seed); //cout<<" seed="<<seed<<" at t="<<t<<endl;

  Config mag = Config(); 
  // gone: movIn.read(char12,8); CheckScript(char12,"\nConfig:");
  mag.readBinary(movIn);
  mag.restore(); // puts this config in M
  // give up on giving time in text? Makes length of frame unpredictable, 
  // complicates positioning. Might just give last few digits of frame #.
  //movOut<<"!end of frame at step "<<dec<<nt<<", time "<<t<<endl;
  movIn.read(char12,11); CheckScript(char12,"\n!end frame"); 
  movIn.read(char12,5); // this was the frame number, just ignore it for now
    // could get nt from atoi(char12), may need to remove '#'

  if(!movIn.good()) EXIT1("problem in readFrameBinary");
  //My->readBase64Obj(inFile, &t, sizeof(t)); 
  #endif  //  
	return true; // successful return
    
/* Alas, this stuff broken 2011:
      if (C1->child0) // not smallest
	  {if (C1->rect) TempM= C1->rect->M0*(absM/C1->rect->Ms);
          // rect, but not smallest; correct the length
        else TempM = MVector(0,0,absM); // non-rectangle}
      else // smallest
 // {TempM=C1->rect->M0;}// In 2/05, was root->rect!! Wrong.
*/

} // end ReadFrameBinary

 void Systm::writeSnapshot()
{  // like writeFrameBinary, but writes formatted data for excel plot
  LLGcell *C1;
  ofstream SnapFile;

  char NameString[20]; 
    // strset(NameString,char(0)); //sets all chars to null? JUST 0th!!
#ifdef WIN32
  strncpy_s
#else
  strncpy
#endif  //  
	  (NameString,TimeString,strlen(TimeString)-3); //copies, no null! 
    //gcc doens't support strncpy_s, changed to strncpy Jun 2013. Wish they would agree!
    // but it has " ns" at end, chop off
  // size_t jj=strlen(TimeString);
  NameString[strlen(TimeString)-3]='\0'; // terminate it

  // char d_=','; // delimiter now in lowest.h
  char* suffix=".csv"; // could read flag in from script, blank delimiter option
  SnapFile.open((ScriptDir+runID+"@"+NameString+suffix).c_str());
  cout<<"Opened snapFile to write"<<endl;
  // TimeString and TimeFormat(c-string version) are 
  // defined if time is displayed

  SnapFile<<"Snapshot at t="<<d_<<t<<", H="<<d_<<H_ext<<", columns are:"<<endl;
  SnapFile<<"x   "<<d_<<"y   "<<d_<<"z   "<<d_<<"Mx   "<<d_<<"My   "<<d_<<"Mz   "<<d_<<
	  "theta  "<<d_<<"Hk/Ms "; // endl is in loop
  
  for (CellIDType i=0; i<(LLGcell::NextSmCellID); i++)
    {
    C1=CellFromID[i];
    SnapFile<<endl;
    //SnapFile<<C1->center[0]<<d_<<C1->center[1]<<d_<<C1->center[2]<<d_
	//	<<C1->M[0]<<d_<<C1->M[1]<<d_<<C1->M[2]<<d_<<acos(C1->M[0]/C1->rect->Ms)
	//	<<d_<<C1->anisCoef;
	SnapFile<<C1->center<<d_<<C1->M<<d_<<acos(C1->M[1]/C1->rect->Ms)<<d_<<C1->anisCoef;
     } //end of for()
  cout<<"Wrote snapshot"<<endl;
  SnapFile.close();
} // end writeSnapshot; cellls aren't in order but can sort by z in excel.

//the version of the movie file this readFrameBinary() can read
//is stored in movieVersion. Each time we make a crucial change
//to this function we need to change movieVersion accordingly.
//Possibly we can have a switch statement in readFrameBinary() that
//allows us to read different versions of the movie.:

 void Systm::SetUniformM(MVector dirn)
 {for(CellIDType i=0;i<LLGcell::NextSmCellID;i++) 
  {LLGcell* c1=CellFromID[i]; c1->M=dirn*c1->rect->Ms;}
}

 
void Systm::CalcEnergy(){ // this is E/mu_0 as always
	IntEnergy = 0; ZeemanEnergy=0.; // Int is quadratic; Zeeman is cellwise H_lin.M
	for (CellIDType i=0; i<(LLGcell::NextSmCellID); i++)
		   CellFromID[i]->AddCellEnergy();
	TotEnergy = IntEnergy + ZeemanEnergy;
}

void Systm::CalcMoments() // calc. MVal-- note this calcs ALL of them
{ // (CalcMomentsdDmdh calcs only CONSTRAINED ones!)
   // This is used only for display or writing file
    CellIDType i; short im;
	Mode* IM;
	for (im=1; im<=NModes; im++) Modes[im]->MVal=0.0; // initialize moments
    // cout<<"In CalcMoments()"<<endl;
   for (i=0;i<LLGcell::NextSmCellID;i++) 
   { // this is only over SMALLEST cells
	   LLGcell* c1 = CellFromID[i];
       CalcShapeFCell(c1);
       for(im=1;im<=NModes;im++) 
	   {  IM=Modes[im];
	   	  IM->MVal+=IM->ShapeF*c1->M; // actual moment
	   } // end for im
   } // end for i
} // end function CalcMoments

real Systm::ModeEnergy(){ // mode Zeeman energies -- just for debugging!
	real ModeE=0.0; //  Shd be same as Zeeman.
	// cout<<"In ModeEnergy()"<<endl;
	CalcMoments();
	for (short i=1; i<=NModes; i++)
	   {   assert(Modes[i]);
		      if(!Modes[i]->NonField) ModeE-=Modes[i]->HVal*Modes[i]->MVal;
              // non-field-like modes contribute M dot(MxF)=0 to E
	   }
	return ModeE;
}

void Systm::finalCheck()
{
  cout<<"Starting Systm::finalCheck"<<endl;
  assert(paramFile);

  char ch; // just like in ::ReadScript()
  string junk; real AllowedErr=0; // will be read in
  MVector V1; // for some reason you can't declare V1 inside switch!
  getline(paramFile,junk); // should say "Final Check"
  if (junk[0]!='F') {cout<<"Program done (no final check), hit key to exit(0)"<<endl; 
    cin.get(); exit(0);} // was _cexit() 
  paramFile>>ws; // ready for first subcommand under F
  while ((ch = (char)paramFile.peek()) != '!')
  {
   switch(ch)
   {
    case '#': // comment
      break;
    case 'E':
      paramFile>>junk>>AllowedErr;
      cout<<"read in Allowed error = "<<AllowedErr<<endl;
      break;
    case 'M': // expected value of M at watched cell
      real m0, m1, m2;
      paramFile>>junk>>m0>>m1>>m2;
      V1 = MVector(m0,m1,m2);
      cout<<"read expected M value for watched cell, V1 = ("<<V1[0]<<","
             <<V1[1]<<","<<V1[2]<<")"<<endl;
	  assert(WatchedCell);
	  assert(typeid(WatchedCell)==typeid(LLGcell));
      cout<<"WatchedCell->M="<<((LLGcell*)WatchedCell)->M<<endl; 
      real Error;
      Error = (V1-((LLGcell*)WatchedCell)->M).Magnitude();     
      cout<<"In FinalCheck, watched cell's M="<<((LLGcell*)WatchedCell)->M<<endl;
	  cout<<"Expected "<<V1<<endl;
	  cout<<"Error = "<<Error<<", allowed error="<<AllowedErr<<endl;
      (((LLGcell*)WatchedCell)->M).print("Watched cell had M=");
      V1.print("Expected value =");
      // this was for special case:
	  //cout<<"******** actual LAMDA*2M_PI/w is: "<<
		  //log(sqrt((WatchedCell->M[0]*WatchedCell->M[0]+WatchedCell->M[1]*WatchedCell->M[1])/(M1[0]*M1[0]+M1[1]*M1[1])))<<endl;
      cout<<"Error = "<<Error<<endl;
      cout<<"***********  "; 
	  if(Error<AllowedErr) cout<<"PASSED"; else {cout<<"FAILED"; MYEXIT(95);} 
	  cout<<" FINAL CHECK of M ********************"<<endl; // this line gives error C2143
	  //missing ; before "string"
      break;
case 'H': // expected field at watched cell
      real h0, h1, h2;
      paramFile>>junk>>h0>>h1>>h2;
      V1 = MVector(h0,h1,h2);
      cout<<"read expected field at watched cell ("<<V1[0]<<","
             <<V1[1]<<","<<V1[2]<<"), or "<<V1<<endl;
	  assert(WatchedLLGCell);
      cout<<"WatchedLLGCell->H="<<WatchedLLGCell->H_av()<<endl; 
      Error = (V1-WatchedLLGCell->H_av()).Magnitude();     
      cout<<"In FinalCheck, watched cell's H_av="<<WatchedLLGCell->H_av()<<endl;
	  cout<<"Expected "<<V1<<endl;
	  cout<<"Error = "<<Error<<", allowed error="<<AllowedErr<<endl;
      (WatchedLLGCell->H_av()).print("Watched cell had H_av=");
      V1.print("Expected value =");
      // this was for special case:
	  //cout<<"******** actual LAMDA*2M_PI/w is: "<<
		  //log(sqrt((WatchedCell->M[0]*WatchedCell->M[0]+WatchedCell->M[1]*WatchedCell->M[1])/(V1[0]*V1[0]+V1[1]*V1[1])))<<endl;
      cout<<"Error = "<<Error<<endl;
      cout<<"***********  "; 
	  if(Error<AllowedErr)cout<<"PASSED"; else {cout<<"FAILED"; MYEXIT(96);} 
	  cout<<" FINAL CHECK of H_av ********************"<<endl;
      break;

    default:
      cout<<"Error in final check"<<ch<<endl; MYEXIT(90);
      break;
   } // end switch
   	if(!paramFile) {cout<<"Error in script, FinalCheck, near:"
		 <<junk<<endl; MYEXIT(52);}

      getline(paramFile,junk); // for each case
	  paramFile>>ws; // eat white space from next line
  } // end while
  
  // cout<<"close window by clicking upper right"<<endl;
}

//handle the callback action, called from mySensorCallback or emulation loop
bool Systm::HandleCallback() //always returns 1! 
{    //  If 0 would stop, not used.
  //cout<<"InHandleCallback"<<endl;
	real Error; // scratch
#ifdef GRAPHICS
  if(Pausing) 
  {    //cout<<"Pausing, about to return from HandleCallback"<<endl;
		return 1;// just display same scene again
  }
#endif  //  GRAPHICS

AfterDisplay: // here either just displayed, or just skipped displaying
  switch (CBMode) // this flag tells what I was doing before I left callback
  { // see MuMag Code History p. 20.2 for rationale of CBMode
#ifdef GRAPHICS // ~2 pages
// Movie cases:
//this_frame always corresponds to the position of the get-pointer
//in movIn file.
    case FORWARD:  //movie file is read in forward direction
      if (frame_size>0 && frame_offset>0) {
          movIn.seekg((this_frame+numberSkipFrames-1)*frame_size+frame_offset, ios::beg);
          if (readFrameBinary())
          { cout<<" t="<<t<<endl; this_frame+=numberSkipFrames; }
          else
          { cout<<"At latest recorded time, t="<<t<<endl;
            Pausing=true;
            break;
          }
      } else {
          assert(this_frame==0);
          frame_offset=movIn.tellg();
          if (readFrameBinary()) {
            cout<<" t="<<t<<endl; this_frame++;
            frame_size=(short)movIn.tellg()-frame_offset;
          } else EXIT1("Error, could not read first frame of movie file!");
      }
      if (!Repeat)
        Pausing=true; //to pause the movie

      break;
    case BACKWARD: //skip several frames, moving backwards
      if (this_frame>=numberSkipFrames)
      {  //beginning is not reached
        this_frame-=numberSkipFrames;
        movIn.seekg(frame_size*this_frame+frame_offset, ios::beg);
        readFrameBinary();
        cout<<" t = "<<t<<endl;
        if (!Repeat)  //pause the movie
          Pausing=true;
      }
      else   //beginning of the movie is reached
        // CBMode = HOME; 
      {
        CBMode=FORWARD; Pausing=true;
        cout<<"At earliest recorded time, t="<<t<<endl;
      }
      break;
#endif  //  GRAPHICS

	case RUN:
		  switch(TermCrit)
		   { // determines what to do when terminate, not just when to.
	       case TIME: 
			   if (t > (tstop-.5*dt)) // "time up"
			   {  CBMode = READ_COMMAND; } 
             break;
		   case CYCLE_OVER: // wait for end of cycle
			   My_norm->CheckForCycleEnd(); // part of former updateNormalMode()
			   // if cycle not over, just returns; if over, checks for convergence, returns here to start next cycle.
			   // If converged, sets up for next mode
			   if(TermCrit==QUARTER_CYCLE) // means converged
				   goto DisplayNow;
               break;

		   case QUARTER_CYCLE: // doing one last quarter cycle to get "im" config.
			   if (t > (tstop-.5*dt)){ // "time up"
				   My_norm->QuarterEnd();
				   DisplayTime=true; // force display
				   Pausing=true; Repeat=false; // turn on pause
				   goto DisplayNow; // to skip evolve, show user this config.
			       //My_norm->StartNewMode();
			 //  CBMode = READ_COMMAND;
			   }  else {
				   My_norm->UpdateNormMode();
			   }
			   break;

	   	   case MODEVAL: // sweeping mode, stop at StepStop (shd chg name!)
			   // this has not been used for a while, 4/2013
			  Modes[StepMode]->HVal+=StepInc; // note this is field, NOT moment!
			            // would never sweep moment, only step.
			  if((Modes[StepMode]->HVal/StepInc)>(StepStop/StepInc)) // done stepping
				 CBMode=READ_COMMAND;
              break;         
		   case MX: // this is for "game mode"
		   if(WatchedLLGCell->M[0]>0.5*WatchedLLGCell->rect->Ms)
		    {cout<<endl<<"Switched! Time was "<<t<<" nanoseconds."<<endl;
		     TermCrit=NONE; }
              break;

		   case LOOP_ORD: // leave M's the same, but recalculate for lower order
			   // & write to file
			   if(MyFMM->UsedOrder==MyFMM->MaxOrder) // just starting
            {  
               if(TestCase=='l') // just loop, don't know exact answer
				     WatchedH=WatchedLLGCell->H_quad; // highest-order estimate
               cout<<"Looping order: exact WatchedH="<<WatchedH<<endl;
            }
			   Error = sqrt((WatchedLLGCell->H_quad-WatchedH).Square());
			   *OrdFile<<MyFMM->UsedOrder<<d_<<Error<<d_<<WatchedLLGCell->H_quad<<endl;
			   // this is triggered by Order_loop command in script file2
            MyFMM->UsedOrder--;

            cout<<"   present value "<<WatchedLLGCell->H_quad<<", decrement UsedOrder to "<<MyFMM->UsedOrder<<endl;
			   if(MyFMM->UsedOrder==0) 
			   {
				  //OrdFile->close(); might re-use
              cout<<"Done looping order, return to MaxOrder="<<MyFMM->MaxOrder<<endl;
			      CBMode = READ_COMMAND; // done with this command
				  MyFMM->UsedOrder=MyFMM->MaxOrder; // restore it 
			   }
            root->SetOrderRec(); // actually change orders of moment, SmoothField
              // (which control accuracy) -- size of array stays the same.
            //cout<<"Before CalcAllFromM, WatchedLLGCell->H_quad="<<WatchedLLGCell->H_quad<<endl;
            CalcAllFromM(false); // updateMoments & updateStat (get field)
            //cout<<"After CalcAllFromM, WatchedLLGCell->H_quad="<<WatchedLLGCell->H_quad<<endl;
			   goto DisplayNow; // skip rest of evolve (updating M)
			   break;

		   case CALC_GREEN: // lower order by one
#ifdef DBG_
			   assert(DBG);
            // just displayed
			   // triggered by script file line
			   // reset stuff each time; M messed up below for display
            MyFMM->UsedOrder--;
			   if(MyFMM->UsedOrder==0) // done, set things so 
			   {  MyFMM->UsedOrder = DBG->SaveUsedOrder; // restore 
                 Repeat=false; // can't find clean way to terminate,
                 // will complain next time that wrong moments nonzero.
               	 Pausing=true; // callback just repeats if true; 
	             //  can use keyboard Q
               CBMode = READ_COMMAND; // done with Green, probably just quit
			   } 
			  else // not done
			  {
             DBG->CalcGreen(); // calculate everything, ready for next display
			  }
           goto DisplayNow; // skip rest of evolve (updating M)

#else
			   cout<<"Can't calculate Green fn, no DBG_"<<endl;
#endif //!DBG_
			   break;
		   case NONE: // don't test anything, wait for user to pause.
		     break;
	        } // end switch(TermCrit) 
		   // return; // display, then come back to callback

        if(cons && (!My->My_norm) ) // adjust fields if constraint
          {   // but if doing cycle, wait (later may have better test for doing cycle, now use My_norm)

		    if(!cons->AdjustFields()) // gets new mode fields HVal & fixes H_lin
		    // returns false if fails, then exits callback
			{ // jump to here upon other disaster?
				cout<<"Failed to AdjustFields -- setting to zero, entering pause mode"<<endl;
				cout<<"This configuration may be unfixable, you may have to exit."<<endl;
				Repeat=false; 
                cons->SetConsH0(); // set conserved modes to zero, may be garbage
				DisplayTime=true; // display
				// crashed on display, must not set stuff right here. Needs debugging.
				return 1; //was: break; // jump out of do loop, so display before next evolve
			} // end if AdjustField fails)
		  } // end if cons(straints)

 		// loop through all cells, changing their magnetization etc:
	    evolve();
		//if(!Repeat) cout<<"t="<<t<<endl; // had ", M="<<WatchedCell->M<<endl;
      break; // break case RUN of switch(CBMode)

	case GRID:
        GridInf1->GridCB();
	  //updateScene(); // now it's display time
		break; // out to display; shd restore pausing

	case SATURATION:
		SaturationCB();
		break;

    case EXIT:      //unschedule? and return from the program
		cout<<"Exiting the program, t="<<t<<": press enter in window to close"<<endl; 
	 //cout<<"Please close window by clicking upper right"<<endl;
	 cin.get();
	 exit(0); // try putting exit(0) here???
      // mySensor->unschedule(); not sure why did this -- let user close window
      // cout<<"Unscheduled"<<flush<<endl;
      // CBMode = WAITFORUSERTOCLOSE;
      // return 1; // was 0, closed window
      break;

	case READ_COMMAND:
	  ReadScript2();
	  break;

#ifdef CONTOURS
	case DO_CONTOURS: // computing contours
       EXIT1("Contours not implemented yet");
       //MyContours->ContoursCB();
	  break;
#endif  //  

#ifdef ORBITS
    case DO_ORBITS: // am computing orbits, in class OrbitHolder
		MyOrbs->OrbitsCallback(); if(!Repeat) CBMode=DO_ORBITS_PAUSE;// in pause mode
	    // just display until user presses right arrow
      break;
#endif  //  ORBITS

/* gone:
	case CALC_NORM_MODES:
		assert(My_norm);
		My_norm->NormCB();
		break;
*/

  } //end of switch (CBMode)
  if(!Repeat) Pausing=true; // if no GRAPHICS, Repeat should be true
  // if (!Repeat) cout<<'.'<<endl; // for readability, end of callback
	  //cout<<"Returning from callback with Repeat="<<Repeat<<
	  //   ", Pausing="<<Pausing<<", CBMode="<<CBMode<<endl;// 4=compute, 5=READ_COMMAND

  // cout<<"Abt to lv CB *************"<<endl;
  if(!DisplayTime) goto AfterDisplay; // top of function again, skip display
DisplayNow: // jump to here to skip evolve
  #ifdef GRAPHICS
	  updateScene(); // now it's display time
  #endif  //  GRAPHICS
  return 1;
} //end of HandleCallback

void Systm::SaturationCB() // calculates E(angle) at saturation, writes 
{  // called by ReadScript2 on command 
	// rotates in xy plane only, ScratchReal1=angle from x axis
    // doesn't use evolve() at all, just calculates energy of known configuration
   MVector Mag=MVector(cos(ScratchReal1),sin(ScratchReal1),0.0)*CellFromID[0]->rect->Ms;
   for (CellIDType i=0;i!=LLGcell::NextSmCellID; i++) CellFromID[i]->M = Mag;
   CalcAllFromM(true);
   CalcEnergy();
   // *SFile<<
   ScratchReal1+=ScratchReal2; // increment angle
   if (ScratchReal1>2*M_PI) CBMode = READ_COMMAND; 
   DisplayTime=true; // return from callback to display
}
   
// next 2 functions really relate to pulse.cpp, mode class
void Systm::CalcShapeFPoint(Rectang *rect,MVector r) // this and following function almost same,
	                         // except that this one evaluates at point r 
	{ // initialize shape function to zero:
	   for (short i=1;i<=NModes;i++) Modes[i]->ShapeF.SetToZero();
	   list<Term>::iterator iT;
	   for (iT=rect->Terms.begin(); iT!=rect->Terms.end(); iT++)
	     iT->ItsMode->ShapeF += iT->EvalAtPoint(rect->WholeRect,r);
	} // end CalcShapeF

void Systm::CalcShapeFCell(LLGcell* c1) // calcs CELL INTEGRAL of F_i for all modes i
          // NOT AVERAGE?? Used in LL eqn for field, must div by vol
	{ // initialize shape function to zero:
	for (short i=1;i<=NModes;i++) Modes[i]->ShapeF.SetToZero();
	list<Term>::iterator iT;
	for (iT=c1->rect->Terms.begin(); iT!=c1->rect->Terms.end(); iT++)
		iT->ItsMode->ShapeF += iT->IntOverCell(c1);
	//if(!Repeat) cout<<"CalcShapeFCell@"<<c1->center[0]<<", mode 2= "<<Modes[2]->ShapeF<<endl;
	} // end CalcShapeF

//addtion by Kam 2013
//Calculation shape function just for one mode (specified for whichmode in the mode list)
void Systm::CalcShapeFCellJustNormalMode(LLGcell* c1, Mode* whichMode) // calcs CELL INTEGRAL of F_i for all modes i
          // NOT AVERAGE?? Used in LL eqn for field, must div by vol
	{ // initialize shape function to zero:
	whichMode->ShapeF.SetToZero();
	list<Term>::iterator iT;
	for (iT=c1->rect->Terms.begin(); iT!=c1->rect->Terms.end(); iT++){
		if((iT->ItsMode)==whichMode){
		iT->ItsMode->ShapeF += iT->IntOverCell(c1);
		}
	}
	//if(!Repeat) cout<<"CalcShapeFCell@"<<c1->center[0]<<", mode 2= "<<Modes[2]->ShapeF<<endl;
	} // end CalcShape...
//end addtion by Kam 2013

#ifdef GRAPHICS // OpenGL and COIN nested inside
void Systm::updateTree(cell* curCell)
{  
  //if(curCell->rect->WholeRect=curCell) // is a rectangle
	//  curCell->rect->UpdateFieldDisp();
  if (curCell->child0)  //not smallest cell
  {
    updateTree (curCell->child0);
    updateTree (curCell->child1);  //recursive calls
    // curCell->M = (curCell->child0->M + curCell->child1->M)*cuberoothalf; // need calc M
    // Note that this is wierdly normalized, so vol of vector ~ magnitude!
	// if need this for non-LLG cells, use M from mpole moment[1-3]
  }
  
  LLGcell* curLLG=dynamic_cast<LLGcell*>(curCell);
  if (curCell->DispInfoPtr)
  { 
    if(curLLG==NULL) EXIT1("Trying to display non-LLG cell; need to write separate function");
    // if(curCell->child0) always do this// not smallest, need calc q (RARE)
	curLLG->q.rotateTo(curLLG->M); 
	  
    real x0,x1,x2,x3;
    curLLG->q.CopyTo(x0,x1,x2,x3);
#ifdef DEBUG3
      cout<<"in updateTree, cell center is "<<curCell->center<<endl;
#endif  //  DEBUG3

#ifdef COIN
    curLLG->DispInfoPtr->myMTransform->rotation.setValue(x1,x2,x3,x0);
                     //OI uses wrong order!
    real lengthScale = GlobalMScale*curLLG->M.Magnitude(); // in nm
    curLLG->DispInfoPtr->myMTransform->scaleFactor.setValue(lengthScale, lengthScale, lengthScale);
#endif  //  COIN
#ifdef OPENGL
	// put something here!
#endif  //  OPENGL
  } // end if DispInfo

  if (curCell->DispInfoPtr) // if pointer exists
  {  // put correct fields into HDisp displays,
	  assert(typeid(*curCell)==typeid(LLGcell));
      LLGcell* LL=static_cast<LLGcell*>(curCell);
	  LL->InsertHDisp(); // updates H_max too
  } // end if pointer
} // end Systm::updateTree

void Systm::updateScene() // called only if GRAPHICS
{ // first update field, time display (not tree)
  if(externalField) externalField->updateVector(H_ext); // if pointer non-null
       //cout<<"updated H_ext display="<<H_ext<<",now call updateScene(root)"<<endl;

  if(NModes) CalcMoments(); // some of NumberDisplays are moments	  
  if(NeedCalcEnergy) CalcEnergy();
  for(short i=0;i<cell::NumLocalColor;i++) 
	  //if(cell::HDisp_Which[i]>0) we DO want label to be correct even if 0
	  {   //cout<<"about to call UPdateND"<<endl;
		  cell::HDispNumb[i]->UpdateND(); //"Show:Anis..~2.0"
		  //cout<<"Returned from UpdateND"<<endl;
	  }
  //cout<<"Done with for i loop"<<endl;
  list<NumberDisplay>::iterator iND;
  for (iND=NumbDisps.begin(); iND!=NumbDisps.end();iND++) 
	  iND->UpdateND();

#ifdef COIN 
// Have not used these lately: consider eliminating?:
  if(WatchedLLGCell) if (!WatchedLLGCell->child0) // can show these things only for smallest cells
  {
	assert(typeid(*WatchedLLGCell)==typeid(LLGcell));
	LLGcell * LL = WatchedLLGCell;
  if (AnisField) AnisField->updateVector(LL->H_anis()); 
    //changing Show... for pointer test.********
  if (MStaticField) MStaticField->updateVector(
	   LL->H_quad-LL->H_anis()-LL->H_exch());
  if (ExchField) ExchField->updateVector(LL->H_exch());
  if (TotField) TotField->updateVector(LL->H_av());
  if (WatchedM) WatchedM->updateVector(LL->M);
  }
#endif  //  COIN
  UpdateFieldDisp(); // Mode displays
  
  for (short i=0;i<cell::NumLocalColor;i++) cell::H_max[i]=0.0; // initialize
  updateTree(root); // recursively; also calcs H_max
  #define RELSCALE cell::H_max[i]*cell::H_scale[i]*0.5/WatchedCell->half[0]
  for (short i=0;i<cell::NumLocalColor;i++)
	if(cell::H_scaleAuto[i]) // auto-scale
    { while (RELSCALE>2.0) cell::H_scale[i]*=0.5; // leave it close to 1
      if (cell::H_max[i]<0.001) break; // field essentially zero, don't raise scale
	  while ((RELSCALE<0.25)&&(cell::H_scale[i]<1e5)) cell::H_scale[i]*=4.0; // don't exceed 1e5
	  cell::H_scaleToDisplay[i]=2*WatchedCell->half[0]/cell::H_scale[i]; // would display as cell size
    }
  // Because scale calc. AFTER display, it will be wrong the 1st time -- live with it.
} // end updateScene

NumberDisplay* Systm::UpNumberDisplay(string Prefix,real* NumIn,real IOD,string Units)
{  // reads info to set up number display scrolling upward
	// NumIn is in "internal" units, e.g. IntUnit=J'/nm.  Display unit DispUnit = j/m.
	// And IntUnit = IOD* DispUnit, where IOD="InternalOverDisplay"=10^-18.
	// value = NumIn*IntUnit = NumIn*10^-Exp10 e(Exp10) IntUnit =
	// NumIn*10^-Exp10*IOD e(Exp10) DispUnit = what we write
	short Exp10; short NCol; stringstream StSt; 
	NumberDisplay* ND;
   paramFile>>Exp10; // assumes you've read 'junk'; could also read colors
   NCol=PrevCol+1;
   // Find new position
   if(NCol>=NumCols) {NCol=0; yND+=dyND;} // carriage return upward
   PrevCol=NCol; 
   if(Exp10) StSt<<" e"<<Exp10; else StSt<<"";
   StSt<<Units; // like "E*10^-3 e3 uA^2 nm"
   ND = new NumberDisplay(Prefix,pow(10.0,-Exp10)*IOD,3,
	  NumIn,StSt.str(), xND+NCol*dxND,yND,0.0,0.0,0.0,0.5);
   NumbDisps.push_back(*ND); // add pointer to list
   return &NumbDisps.back(); // return pointer for later use -- hope it doesn't change!
}

void Systm::UpdateFieldDisp() // shows only ONE mode, for a specific rectangle
{   list<FieldDisplay>::iterator iFD;
	for (iFD=FieldDisplays.begin(); iFD!=FieldDisplays.end();iFD++) // loop field displays  
	{   Mode* MP=iFD->ItsMode; // mode pointer
	    CalcShapeFPoint(iFD->ItsRect,iFD->whereis()); // get ShapeF for ALL modes, inefficient! 
		iFD->updateVector(MP->ShapeF*MP->HVal); 
	// or could make fn. iFD->UpdateFD() that calcs MP and calls 
	// UpdateVector(ItsMode->EvalFAtPoint(whereis())*ItsMode->HVal), only sl. < cumbersome 
//   was FieldDisp[i]->updateVector(SumModesAtPoint(FieldDisp[i])); 
	// may someday want to display all modes in one display?
	}
}

void Systm::FixReal(real &v, real vDefault) // util. for display.cpp; in class system for access to ShiftIsDown
{	if(ControlIsDown) {v=0.0; cout<<"Set to zero"; return;} // means we want to set to 0
      
if(ShiftIsDown) {
	if (v==0.0) {v=vDefault; cout<<"made nonzero";}
    else {v*=decibel; cout<<"Increased by 1 dB";}
    }
else {v/=decibel; cout<<"Decreased by 1 dB";}
} // end ChangeHDisp()

void Systm::HandleKey(short iM,short sgn) // handle keypress, need list of Modes
{
	if (iM<=NModes) 
	{
		assert(Modes[iM]); // that mode exists
		if(ShiftIsDown)
			if(ControlIsDown) Modes[iM]->ToggleConstraint();
			// could use sgn to distinguish '1' from 'F1'
			else // just shift,// want to change increment
		{  
			cout<<"For Mode #"<<iM; 
			// if(sgn>0) cout<<" in"; else cout<<" de";cout<<"creased inc"; 
			Modes[iM]->ChangeModeInc(sgn>0); // up if true; couts chg
		}
		else if(ControlIsDown) // & shift is not
		  {Modes[iM]->HVal=0.0; cout<<"Zero out mode #"<<iM<<endl;}
		else // no control or shift
		{ cout<<"For Mode #"<<iM<<"+="<<sgn; // no endl, IncMode finishes line
		  Modes[iM]->IncMode(sgn); //return; // call new function so it knows HGrad object
		}
	} 
	else cout<<"Wrong key, no mode #"<<iM<<endl;
}  // end HandleKey()

#endif  //  GRAPHICS

void Systm::CheckScript(string junk, string correct) // check junk begins correctly
  {size_t n1=correct.length(); // does not check junk[0]==peeked ch
  for (size_t i=0;i<n1;i++) if(junk[i]!=correct[i])
  {cout<<"line begins "<<junk<<", should be "<<correct<<endl; MYEXIT(38);}
  }

void Systm::CheckGood(string junk)
{  if(!paramFile) //was if(paramFile.good()) 
   {cout<<"Error in script file line beginning:"<<junk<<endl;MYEXIT(68);}
}

#ifdef OPENGL
   void Systm::PutInArray(MVector v) 
   {   std::map<int3,vertindex>::iterator it;
       CellIDType ind; // either current numVertGL or old one
	   int3 key=(v-OrigVec).ToInt(2*quantum);
       it = Verts.find(key); 
	   if (it==Verts.end()) // not found
	   {  ind=numVertGL++; // inc AFTER setting ind
		  Verts[key]=vertindex(v,ind); // put it in map
		  // and also into array to send to buffer:
		  VertexArray[ind]=gl_Vertex(v); // constructor; need color, w?
		  cout<<"PutInArray: v="<<v<<", key="<<key<<endl;
		  if(ind>NumVertBound) {cout<<ind<<" is too many vertices, max="
			  <<NumVertBound<<endl; MYEXIT(91);}
	   }
	   else ind=it->second.Index1; // found
	   cout<<"Insert "<<v<<" =vert.#"<<ind<<" at loc. numIndex="<<numIndex<<
		  " in IndexArray, a="<<VertexArray[ind].color[3]<<endl;
       IndexArray[numIndex++]=ind;
	   // OK, we put vertex in vertex array -- but don't we need to construct an 
	   // array of vertex indices to make the TriangleStrip????
   }
#endif  //  OPENGL

#endif  //  H_ONLY

#ifdef DBG_ // debug Green fn
DebugGreen::DebugGreen(short dir1) // constructor
{ Gdir= dir1;
  PlotMax = 100; // hard-wire, for now; later read?
  MaxErr = SumAbs = SumSq = 0.0; // initialize
  GFile.open((My->ScriptDir+My->runID+".Green.csv").c_str());
  if(!GFile.is_open()) {cout<<"Couldn't open Green.csv for writing"<<endl; MYEXIT(61);}
  Gmom = new mpole(My->MyFMM->MaxOrder); // moment of source cell WatLLGPart
  //Smoo = new mpole(My->MyFMM->MaxOrder); // used as scratch, smooth field
  SaveUsedOrder = My->MyFMM->UsedOrder; // so you can restore it
  // Usually there will be only one rectangle? Source is WatLLGPart.
  GAvCoef = My->MyFMM->MakeAvCoef(My->WatLLGPart->rect->LLGSize,My->MyFMM->MaxOrder); 
  Gmom->SetToZero();
  for (short i=0;i<VDIMENSION;i++)
	 Gmom->AddMult(GAvCoef[i],My->WatLLGPart->volume()*My->WatLLGPart->M[i]); 
  cout<<"Will calculate Green fns, source at "<<*My->WatLLGPart<<", MaxOrder="<<My->MyFMM->MaxOrder<<endl;
}

DebugGreen::~DebugGreen()
{  delete Gmom;
   for (short i=0;i<3;i++) delete GAvCoef[i];
   delete GAvCoef; 
}

/* moved to constructor now, delete this:
void DebugGreen::SetDBG()
{  cout<<"Create stuff for DebugGreen: GAvCOef..."<<endl;
   SaveUsedOrder = My->MyFMM->UsedOrder; // so you can restore it
   // Usually there will be only one rectangle? Source is WatLLGPart.
   GAvCoef = My->MyFMM->MakeAvCoef(My->WatLLGPart->rect->LLGSize,My->MyFMM->MaxOrder); 
   Gmom->SetToZero();
   for (short i=0;i<VDIMENSION;i++)
	 Gmom->AddMult(GAvCoef[i],My->WatLLGPart->volume()*My->WatLLGPart->M[i]); 
     // this is same as in cell::updateMoment(), for consistency.
   //TestDeriv(MVector(2,3,4),MVector(1,-1,1));  // tests SetToDeriv function; comment out later
   // did tests (100; 0.5,0,0);(1,0,0;0.2,0,0); (2,3,4;1,-1,1);
   // sometimes convergence bouncy, but average rate ~ "Small".
   // Conclude SetToDeriv is correct, 2/9/2014.
}
*/

void DebugGreen::TestDeriv(MVector r, MVector d) // tests SetToDeriv function
{  // calculates derivs of 1/r at r, evaluates 1/|r+d|
  real Answer=1.0/((r+d).Magnitude()*4.0*M_PI);
  real Small = sqrt(d.Square()/r.Square()); // rough ratio of terms,error Small^order+1
  // orders: FMM::setup calculates arrays to order MaxOrder or (for Deriv) MaxOrderD.
  mpole* scratD = new mpole(My->MyFMM->MaxOrderD);
  mpole* powers = new mpole(My->MyFMM->MaxOrderD);

  scratD->SetToDeriv(r,powers); // powers used as scratch here;
    // order is MaxOrderD, set by powers->order
  scratD->printMP("Deriv=",cout);
  powers->SetToPowers(-d); // calc V(d)=W(-d)
  powers->printMP("powers",cout);
  cout<<"TestDeriv with r="<<r<<", d = "<<d<<", 1/4pi|r+d|="<<Answer<<
	  ", conv.rate="<<Small<<endl;

  short iBegin, iEnd=0;
  real dotProduct = 0.0;
  for (short ord1 = 0; ord1 <= My->MyFMM->MaxOrderD; ord1++)//loop order
  {//
    iBegin = iEnd; iEnd = My->MyFMM->NumBelow[ord1];
    for (short i = iBegin; i < iEnd; i++)
    {  dotProduct += scratD->values[i]*powers->values[i];
	   cout<<"Add D["<<My->MyFMM->IndArr[i]<<"] * d^n, get"<<dotProduct<<endl;
	}
	cout<<"     At order "<<ord1<<", V_r+d="<<dotProduct<<
		 ", error="<<dotProduct-Answer<<endl;
  } // end for ord1
}  // end TestDeriv

void DebugGreen::CheckField(MVector H_q, cell* c1)
{ //cout<<"Enter CheckField with H_q="<<H_q<<", center="<<c1->center<<endl;
  MVector FieldM, BestGuess; // high-order estimate
  mpole* Smoo=  new mpole(My->MyFMM->MaxOrder); // smooth field
     // at one point was in class DebugGreen, but that creates clashes between cores!
  // actually, extra orderD needed only for plot loop, but may as well create it here.
  mpole* scrat1=new mpole(My->MyFMM->MaxOrderD); 
  mpole* scrat2=new mpole(My->MyFMM->MaxOrderD);   

  // check against stored kernel, if possible
  MVector sep =c1->center - My->WatLLGPart->center; // separation of cells
  int3 key = sep.ToInt(My->WatLLGPart->rect->LLGSize*2); //key for Kernel
  bool KerFound;
  if(key.isZero()) goto SeekKernel; // high-order estimate FieldM would diverge!
  if(H_q.Square()>1E7) {cout<<"Big H_q="<<H_q<<endl; MYEXIT(39);} // had bug
  //Calculate "exact" Green's function; first smooth field Smoo:
  Smoo->SetToZero();
 // My->PD4=true;
  //Gmom->printMP("Gmom before AddTaylor",cout);
  Smoo->AddTaylor(Gmom,sep,scrat1,scrat2); 
       // gets smooth field due just to WatLLGPart
  // this code borrowed from AverageMpole -- but we may change that?
  for(short i=0;i<3;i++) 
    {
	  FieldM[i] = (*(GAvCoef[i])*(*Smoo));
    }

SeekKernel: // check against stored kernel
  std::map<int3, Kernel>::iterator iterMap;//iterator for Kernels
  //cout<<"Checking @ sep="<<sep<<", look for key="<<key;
  // why did I do this? If too big, find(key) will tell me!
  //if(key.Square()>20) {cout<<", prob. too big";KerFound=false; goto DidntFind;} // no hope to find in list?


#ifdef MOREKERNELS
  // put this here because iterMap, scrat1,scrat2,Smoo have been conveniently created.
  short Dmax=5;
  Kernel KTemp;
  for (short ix=-Dmax;ix<=Dmax;ix++)
	for (short iy=-Dmax;iy<=Dmax;iy++)
      for (short iz=-Dmax;iz<=Dmax;iz++)
	  {  key = int3(ix,iy,iz);
         if(key.isZero()) continue; // skip this
		 real Vol=My->WatLLGPart->volume();
		 sep=My->WatLLGPart->half[0]*2*key; // real separation
		 // Now loop over elements of kernel matrix, but by hand
		 Smoo->SetToZero();
         Smoo->AddTaylor(GAvCoef[0],sep,scrat1,scrat2); // source along x
		 KTemp.Kxx = (*(GAvCoef[0])*(*Smoo))*Vol; // avg field along x
		 KTemp.Kxy = (*(GAvCoef[1])*(*Smoo))*Vol; // avg field along y
		 KTemp.Kxz = (*(GAvCoef[2])*(*Smoo))*Vol; // avg field along z

		 Smoo->SetToZero();
         Smoo->AddTaylor(GAvCoef[1],sep,scrat1,scrat2); // source~y
		 KTemp.Kyy = (*(GAvCoef[1])*(*Smoo))*Vol; // avg field along y
		 KTemp.Kyz = (*(GAvCoef[2])*(*Smoo))*Vol; // avg field along z

		 Smoo->SetToZero();
         Smoo->AddTaylor(GAvCoef[2],sep,scrat1,scrat2); // source~z
		 KTemp.Kzz = (*(GAvCoef[2])*(*Smoo))*Vol; // avg field along z

	     iterMap = My->WatLLGPart->rect->RectKer->find(key);
		 KerFound=iterMap!=My->WatLLGPart->rect->RectKer->end();		

		 cout<<"Calcd kernel at "<<key<<": "<<KTemp<<endl;
		 if (KerFound)
		 { real Err;
		   Err=sqrt((KTemp-iterMap->second).Square());
		   cout<<"Found kernel, "<<iterMap->second<<", error="<<Err<<endl;
		 } // end if found
		 else // not found, make new one
		 { cout<<"Didn't find, insert. ";

		 //My->WatLLGPart->rect->RectKer->insert(std::pair<int3, Kernel>(key,KTemp));
		   if (!((My->WatLLGPart->rect->RectKer->insert(std::pair<int3, Kernel>(key,KTemp))).second))
	              {cout<<"Error inserting kernel"<<endl; MYEXIT(39);}
	       else cout<<"Inserted kernel"<<endl;
		 } // end else not found
  } // end loop on key
  My->MyFMM->OutName="NewKernels.dat";
  My->MyFMM->WriteKernels();
  MYEXIT(33); // this is all you wanted to do
#endif // MOREKERNELS

  iterMap = My->WatLLGPart->rect->RectKer->find(key);
  KerFound=iterMap!=My->WatLLGPart->rect->RectKer->end();		
  if (KerFound)
  {  
    BestGuess = iterMap->second.Dot(My->WatLLGPart->M); // kernel
	 //cout<<", found kernel="<<BestGuess<<endl; 
    if(!key.isZero()) // FieldM is undefined at 000
    {
	   real KernErr=(BestGuess-FieldM).Magnitude();
	   cout<<"@ "<<key<<", compare kernel"<<BestGuess<<endl<<
		  "      to high-order"<<FieldM<<", error="<<KernErr<<endl;
    }
  }
  else 
  {
// DidntFind: // delete?? 
	  //cout<<", didn't find"<<endl;
	  BestGuess = FieldM;
  }
  real error2 = (BestGuess - H_q).Square();// actual error
  real errG = sqrt(error2); // error in Green fn.
  SumAbs+=errG;  SumSq+=error2;
  //real errFrac = error2/BestGuess.Square(); 
  if (errG>MaxErr) 
    {MaxErr = errG; WorstCell=c1;
     cout<<"New MaxErr "<<MaxErr<<", worst cell = "<<*c1<<endl
        <<"  GF="<<H_q<<", guess(";
	 if(KerFound) cout<<"stored kernel"; else cout<<"order "<<My->MyFMM->MaxOrder;
	 cout<<")="<<BestGuess<<endl;
  }

  // NOW PLOT FIELDS, if this is watched cell
  if(c1!=My->WatchedLLGCell) return;
  if(My->MyFMM->UsedOrder<My->MyFMM->MaxOrder) return; // comment this line out
    // to graph all orders
  cout<<"Now at watched cell, plot fields along axis "<<Plotdir<<endl;
  if(!SourceAnc) 
  { 
    cout<<"Watched cell close; kernel used"<<endl;
    SourceAnc=My->WatLLGPart;
    FieldAnc=My->WatchedLLGCell;
  }
   real Left=SourceAnc->LowerLeft()[Plotdir];
   //delete mpole* scrat1=new mpole(My->MyFMM->MaxOrder); 
   //mpole* scrat2=new mpole(My->MyFMM->MaxOrder);
   if (FieldAnc->LowerLeft()[Plotdir]<Left) Left = FieldAnc->LowerLeft()[Plotdir]; // furthest left
   real Right=SourceAnc->UpperRight()[Plotdir];
   if (FieldAnc->UpperRight()[Plotdir]>Right) Right = FieldAnc->UpperRight()[Plotdir]; // furthest right
   MVector r=My->WatchedLLGCell->center; // line goes through field cell
   //MVector sep;

   // Print some mpoles:
   //Gmom->printMP("Gmom=",cout);
   //SourceAnc->moment->printMP("SourceAnc->moment=",cout);
   //scrat1->SetToDeriv(MVector(30,0,0),scrat2);
   //scrat1->printMP("D at 30 nm",cout); // 300 component shd be nonzero

   // WRITE PLOT FILE:
   r[Plotdir]=Left; real dr=0.1*My->WatLLGPart->rect->LLGSize[Plotdir];
   real MinSq=dr*dr/4; // for zero-tests
   // file header:
   GFile<<"x, LLG's Field, ShiftMom Field, AncSmooth, LLGSmooth"<<endl;
   cout<<"PlotFields in LLGcell"<<*My->WatchedLLGCell<<" in ancestor"<<*FieldAnc<<endl;
   cout<<" due to LLGcell"<<*My->WatLLGPart<<" in ancestor "<<*SourceAnc<<endl; // could write to GFile too?

   while (r[Plotdir]<=Right)
   { cout<<"$"<<r[Plotdir]<<",";
     GFile<<r[Plotdir]<<d_; // write .csv file; d_ is comma
     // Plot UsedOrder (which can, however, be made small!) field of small cell
     // if UsedOrder=1, is just dipole field

#ifdef DEBUG4
	 // Details are most interesting at center of field cell:
	 // if( (r-c1->center).Square()<MinSq) My->PD4=true; // write them
#endif // DEBUG4

     //First calculate the field of source cell to MaxOrder,header " LLG's Field":
     assert(Gmom->order==My->MyFMM->MaxOrder);
     // use max order,not SmoothField->AddTaylor(WatLLGPart->moment, r-WatLLGPart->center,...
     Smoo->SetToZero();
	 sep=r-My->WatLLGPart->center;
	 if(sep.Square()>MinSq) 
	 {
       Smoo->AddTaylor(Gmom,sep,scrat1,scrat2); // only this use of scrat requires MaxOrder
       //Smoo->printMP("Smoo from Gmom",cout);
	    WriteSmoo(Smoo);// offset in mpole, is H value 
	 }
     // may someday want to plot BOTH orders?  For now, not; do another run for another.
     // for consistency, use MaxOrder=UsedOrder?
	 GFile<<d_;  // need delimiter even if left blank

     // now plot field of shifted moment, only to UsedOrder 
	 // (only have moment to UsedOrder),header is "ShiftMom Field"
     scrat1->order = scrat2->order = My->MyFMM->UsedOrder+My->MyFMM->ExtraOrderD; 
       // to control order of calc; reset to UsedOrder or MaxOrder below
     Smoo->SetToZero();
	 sep=r-SourceAnc->center;
 	 if(sep.Square()>MinSq) 
	 {
	   //My->PD4=true; // turn on debug printing
       Smoo->AddTaylor(SourceAnc->moment,sep,scrat1,scrat2);
       //Smoo->printMP("SourceAnc Smoo",cout);
	   WriteSmoo(Smoo); // offset in mpole, is H value
	 }
	 GFile<<d_; // put delimiter in even if blank

     // now plot "AncSmooth", Smooth Field in Field Ancestor
     Smoo->SetToZero();
	 scrat2->order = My->MyFMM->UsedOrder; // scrat1 is still bigger
     scrat2->SetToPowers(FieldAnc->center-r); // reverse sign 
	   // because SmoothField gives W_n, not V_n (- sign if n odd)
     //to translate the origin of SmoothField by r-center
     Smoo->AddDownConv(FieldAnc->SmoothField, scrat2,0); // this is expansion about r
     WriteSmoo(Smoo);
	 GFile<<d_;
	 
	 // now plot "LLGSmooth" (in Field cell WatchedLLGCell)
     Smoo->SetToZero();
     scrat2->SetToPowers(My->WatchedLLGCell->center-r); // reverse sign
#ifdef DEBUG4
  if(My->PD4) scrat2->printMP("Powers of LLG-r",cout);
#endif // DEBUG4
     //to translate the origin of SmoothField by r-center
	 //scrat2->printMP("Powers of dist to field LLG",cout);
	 //My->WatchedLLGCell->SmoothField->printMP("SmoothField coefs of LLG",cout);
     Smoo->AddDownConv(My->WatchedLLGCell->SmoothField, scrat2,0); // this is expansion about r
#ifdef DEBUG4
  if(My->PD4) Smoo->printMP("Smooth field of LLG, at r",cout);
#endif // DEBUG4	 
     scrat1->order = My->MyFMM->MaxOrderD; scrat2->order = My->MyFMM->MaxOrderD; // restore
	 WriteSmoo(Smoo);
	 GFile<<endl; // no delimiter at end
#ifdef DEBUG4
	 My->PD4=false; // reset in case we turned it on
#endif // DEBUG4

     r[Plotdir]+=dr;
   }
   //Print some more stuff, just for the last point
    //scrat1->printMP("powers of last point-anc",cout);
	 //scrat2->printMP("powers of last point-LLG",cout);
   cout<<"Done making plot"<<endl;
} // end fn CheckField

void DebugGreen::CalcGreen()
{  // call after each change in order, including at beginning with MaxOrder
   cout<<endl<<"Enter CalcGreen with UsedOrder="<<My->MyFMM->UsedOrder<<endl;
   for (CellIDType i=0;i<LLGcell::NextSmCellID;i++) // reset M
      My->CellFromID[i]->M.SetToZero();
   My->WatLLGPart->M[Gdir]=My->WatLLGPart->rect->Ms; // just one cell has nonzero M
   SourceAnc=FieldAnc=NULL; // determined in updateStat
   SumAbs=SumSq=MaxErr=0.0; 
   My->root->SetOrderRec(); // actually change orders of moment, SmoothField
     // don't have to repeat updateMoment(), hasn't changed? order has!! 
	  //root->updateStat(); // doesn't update moment! need:
   My->CalcAllFromM(false); // does updateMoment, updateStat; gets Green fn
      // FirstUpdate=false, needn't update some things.
	ReportResults();
     // And would like to display correctly, so M must be nonzero:
   for (CellIDType ic = 0; ic < LLGcell::NextSmCellID; ic++) 
      My->CellFromID[ic]->M[2]+=0.000001;
} // end CalcGreen

void DebugGreen::WriteSmoo(mpole* Smoo1) // writes field, limiting range
{  real Hplot = Smoo1->values[Plotdir+1];
   if(abs(Hplot)<PlotMax) GFile<<Hplot;
}

void DebugGreen::ReportResults()
{  real Mnorm = My->WatLLGPart->M[Gdir]; // normalization factor
SumAbs/=Mnorm;  MaxErr/=Mnorm;
real SSq=sqrt(SumSq)/Mnorm;
cout<<"Done Green fn with order "<<My->MyFMM->UsedOrder<<
  " normalized sum of |error|="<<SumAbs<<endl<<
  "  Sqrt(SumSquares)="<<SSq<<endl;
   cout<<"Max local error = "<<MaxErr<<" in cell "<<*WorstCell<<endl;
   *My->OrdFile<<My->MyFMM->UsedOrder<<d_<<SumAbs<<d_<<SSq<<d_<<MaxErr<<d_<<WorstCell->center<<endl;
} // end ReportResults
#endif //DBG_
