// This file is called "systm" in case "system" is a reserved word (tho seems not to be)
#ifndef SYSTM_H
#define SYSTM_H // moving this to end produces WIERD errors; maybe other includes try to read again.
#include "MVector.h" // to declare MVector members of system
//#include "Lowest.h"


#include <omp.h>
#include <iomanip>
#ifndef GRAPHICS
#ifndef WIN32 // cstdint is not on pc, and not needed?
#include <cstdint> //including the COIN stuff already defines these typedefs
#endif  //  WIN32
#endif  //  GRAPHICS

class cell;
class Rectang;
class LLGcell;
class FMM;
#ifdef DBG_ // debug Green's fns
  class DebugGreen;
#endif / DBG_

#ifndef H_ONLY
#define Max_Avgs 5 // maximum number of cell types, used only for Avgs now 
// OLD: #define Max_Type 103 // maximum number of cell types: not really so many,
  // 101,102 are special
//#define MAX_FRAMES 10000 //for movie, added by DMAm
//#define FALSE 0 lower case defined elsewhere?
//#define TRUE 1
#include "Quaternion.h"
#include <fstream> // not enuf to declare class name
#include <string> 
#include "acfield.h" // for class ACField
#include "pulse.h" // for class PulsedField, Mode, MedField; 
#include "config.h"
#include "cons.h"
class Average;
// class Magnetoresistance;
class Integrals;
#endif  //  H_ONLY

#ifdef _GROUPS
// #include "_Groups.h"
// instead, use
class NestHolder;
class SymHolder;
class Nesting;
#endif  //  

#ifdef ORBITS
   #include "Orbits.h"
#endif  //  ORBITS

#ifdef CONTOURS
   #include "Contours.h"
#endif  //  

#ifdef _NORMAL
#include "NorModes.h"
#endif  //  

#ifdef COIN
#define GRAPHICS
   class SoSeparator;
   class SoTimerSensor; // this is used near end
   class SoMaterial; // to avoid needing display.h
   class SoText2;
#endif  //   --COIN

#ifdef OPENGL
#define GRAPHICS
#endif  //   OPENGL

//using namespace std;


class Systm{
public:
	// constructor & destructor
  Systm();
  ~Systm();

  //***************data members*********************
	int num_threads; // approximate current number of threads running, needed for parallelized recursive functions
  real MachineAcc; // machine precision
  FMM *MyFMM;  //pointer to FMM class (DMAfs)
  cell *root;
  LLGcell **CellFromID;
     //array of pointers to store the pointer to a cell
  static const short MaxRects=20;
  Rectang *RectFromID[MaxRects];
  short NRects;
  MVector W; //position to watch, read in from input file
  MVector OrigVec; // dummy origin, accessible from all classes
  cell *WatchedCell; // at which we may display things
  LLGcell *WatLLGPart; // watched partner
  LLGcell *WatchedLLGCell;
  MVector WatchedH;
  CellIDType WatID; 
  MVector PeriodVec; // periodic length when the system is periodic and the root
                     // cell is not a rectangle (include the air gap)
#ifdef DEBUG4
  static bool PD4;
#endif // DEBUG4

  //member functions, always needed for FMM
    void CreatePartners (); //added by DMAk, create Partners and NearPartners lists
    MVector Displace(MVector, int3); //const short*);
	bool almost_same(real, real);   //added by DMApf for comparison of real numbers
    bool almost_same(MVector, MVector);

#ifdef DBG_ // debug Green's fns
  DebugGreen* DBG; //constructed  in ????
#endif / DBG_

#ifndef H_ONLY
  // MVector MVector0; // unsigned short MaxShort;
  ifstream paramFile;
  ofstream Avgout; //output file storing the average
  ofstream Xport, Xpor2; // file for routine plotting of data
  bool Xpor2ing; // flag to use Xport, erased Xporting
  // ofstream paramin; //file to store input parameters, for debugging
  ofstream Errorlog; // for startup errors
  //ofstream outlog; //log file for bulky output, for debugging
  ofstream movOut; // File of frames for movie
  ofstream* OrdFile; // file with convergence data
  string ScriptDir, ExeDir; // 2 directories are used, script & exe
  string ICname;
  string ParFileString;
  bool WritingMovie; // if true, will write to movie file every MovInt frames
  bool CompAvg,Proj, use_q,Heun,FirstUpdate;//,LastUpdate;
  //Removed boolean vbls Psetup--split cells, Pcpt--compute 
  //to control print out; replaced by DEBUG2&3
     //CompAvg--compute average
     //Proj-project LocalAxis perpendicular to M
     //use_q for dmdt algorithm, use_q=0 to use dmdt
  string runID;
  //the above data members are used in ReadScript() function

  //std::list<IntFace> IntFaces; 
  //std::list<Interface>::iterator iInt; // could be just where used, tho

  double t; //time variable (only double in code!)
  real tDisp; // display assumes real!
  int nt; //number of time steps elapsed
  short AvgInt; //averaging interval
  // delete short IncInt; //increment interval
  short MovInt; //interval to write out movie file
  //static short ZeroDisp[3];
  real quantum; // distance quantum, smallest distance used. Could later be MVector.
  bool periodic[3]; //boolean, periodic[0]=1 if periodic in x direction
  short numOfPerDir;
  bool Layered; // if 1D thin film, indep of other coords
  short Layer_normal,Layer_offset; // 0 if x=direction
  bool XportM, XportH,XportE,XportIE,XpoProd; //Xport_t, 
   // Boolean, to write fields to file Xport
  real BHmax;
  unsigned short XportMom,XpoBMode; //XportmDotDir,XpomDir,XpoPerp,
  bool XportConsole; // to write to console as well as file
  bool WriteFileInt; // to write to file at intervals
  unsigned short XportInt; // number of dt's to wait, which perp comp to export (0=none)
  Average *Avgs[Max_Avgs];
  Average *WatchedAvgs[Max_Avgs];

  //data members from former Parameters class
  real Increment; // increment for bias field & ...
  MVector H_bias; //bias field (DMAh)
  MVector H_ext; // includes transient pulse
  char TimeString[20]; // used in snapshot as well as graphics

  // these not really relevant w/o GRAPHICS, but convenient:
  bool DisplayTime; // whether will display at this time; false if no GRAPHICS
  real xND,yND,dxND,dyND; // position of number displays (want to read these even if no graphics)
  real xMd,yMd,dxMd,dyMd;  // same for mode displays
  short NumCols,PrevCol;// column of previous number display ND
  bool DisplayConv; // special display mode for GridCB
  short d; //Display frequency, number of dts per display

#ifdef GRAPHICS // generic graphics
  real MRad, MHeadLen, URad, ULen; // arrow sizes, used in ReadScript
  VectorDisplay *externalField, *MStaticField, *ExchField, *AnisField, *TotField, *WatchedM; //added by PBV 
  //bool ShowWatchedMStatic, ShowWatchedHk, ShowWatchedExch, ShowWatchedTot; // display vector
  std::list<NumberDisplay> NumbDisps; // like energy, etc.
  NumberDisplay *TimeDisp; // save this separately, to use to change TimeFormat later
  bool NeedCalcEnergy; // putting on screen or writing, need calc in updateScene
  std::list<FieldDisplay> FieldDisplays; // displays pertaining to Modes
  //char TimeFormat[11];
  string Movname; // to READ (used as flag that we're reading, if non-null)
  ifstream movIn; // input stream to read in movie file
  bool ShiftIsDown,ControlIsDown,AltIsDown; // alt not used yet!
    // to keep track of shift&alt keys, see display.cpp
  // following may need to be outside #ifdef GRAPHICS??
  bool m; // =1, read in movie file; =0, display and calculate at the same time 
   //needed for movie: (added by DMAm)
   unsigned short numberSkipFrames; //number of frames to skip for fast-forward, -backward
   unsigned short this_frame, frame_offset; //current frame, DMAm
   unsigned short frame_size;
   //streampos pos_frame[MAX_FRAMES]; //position of a frame, DMAm
   void updateTree(cell*);//to update the cell tree recursively
   void updateScene();//to update the whole scene
   NumberDisplay* UpNumberDisplay(string Prefix,real* NumIn,real IOD,string Units);
   void UpdateFieldDisp();
   // void setupGraphics();
   SoSeparator *MakeUpVector(real HeadLength,real HeadRad);
      // functions for modes:
   void FixReal(real &v, real vDefault); // utility for Display.cpp
   void HandleKey(short iM,short sgn); // for Mode keys
   bool gameMode;
   void LimitField(); // in display.cpp,for game mode
   real FieldLimit;
#endif  //   --GRAPHICS

#ifdef COIN // declare graphics stuff, not allowed to in case stmts below:

#ifdef WIN32 // SoWin uses different class and function names from SoXt:
	HWND myWindow;
    SoWinExaminerViewer *myViewer;
 #else
	Widget myWindow;
    SoXtExaminerViewer *myViewer;
  #endif  //   // WIN32

  //SoOrthographicCamera *orthoView; // these are created in Systm::Systm() constructor
  SoOrthographicCamera *myCamera;
  SoEventCallback *myEventCB;
  SoNormalBinding *myNormalBinding;
  SoFont *myFont;
  SoDrawStyle *MyDrawStyle;
// below was in 'g'raphics of readScript (delete this line!)
// MVector kVector; //now in Rectang
  SoSeparator *rootReal; //roots for scene graphs, in real&tree windows
  SoSeparator *rootBase,*rootPhong; //under rootReal, color models
  SoSeparator *UpVector; // unit vector in z direction, with cone point,used for H.
  SoSeparator *MUpVector; // used for M.
  // now just modes; eventually will replace these:
  SoText2 *ExchText, *TimeText, *EnergyText, *IntEnergyText, *RotText; // exists only if we display time 
  SbString EnergyString, IntEnergyString, ExchString, RotString;
  //, ExchString[20]; //,EnergyString[36]; // temporary, <string> class won't support sprintf
  // Note about char arrays: older code uses fns like strcat_s, newer uses COIN
  // class SbString; could replace old by SbString, but since it works now, why?
   SoMaterial *surfaceMaterial;  //DMA
   SoSeparator *AllFaces;
   SoCoordinate3 *FaceCoords;
   SoTriangleStripSet *FaceStripSet;
   // SoDrawStyle *FaceDrawStyle;
   SoNormal *FaceNormals;
   SoTimerSensor* mySensor;
  int numFace, numVert;
#endif  //   --COIN

#ifdef OPENGL
//	 string EnergyString, IntEnergyString, ExchString, RotString;//????
     std::map<int3,vertindex> Verts;
	 gl_Vertex *VertexArray;
	 GLuint *IndexArray; 
	 GLuint numIndex, numVertGL;
	 GLuint NumVertBound;
     void PutInArray(MVector v);
#endif  //   --OPENGL
//
  Quaternion q_init; //used to set initial quaternion for a cell
  real fieldScale;
  real GlobalMScale; // length in nm of displayed M vector / |M|
  real gamma; //gyromagnetic ratio;
  bool DampingFinite; // means alpha < infinity, see CodeHistory p. 9.62
    // note that t has a different meaning ("tau" when damping infinite)
  bool Noise;
  real D;
  real dt,dt0; //time step, orig. step if changed
  real tstop; //time to stop
 //  real tstart; //time to start
  real Temp; //temperature
  // real gdt; now in rect // is gamma*dt, for convenience
  PulsedField *Ext_Field1, *Ext_Field2;   //pulsed field (PBV 7/20/04)
  ACPulsedField *ACExt_FieldPtr; //added by zhuru 01/07/2009

  // moved to rectangle, then back!!:
#define MaxModes 20 // modes to maybe constrain, or tweak; only 7 displayed
   // used array, not list, because probably < 20 of them
  Mode* Modes[MaxModes+1]; // array of pointers,[0] unused to map to keys 1..
  short NModes; // up to MaxModes; note #0 is always unused
  //delete bool ModesOn; // for quickly turning on and off

  bool Repeat; // should logically be in GRAPHICS but is used to control cout statements.
    // Repeat=mode designation;=0 if the movie reading pauses after reading numberSkipFrames frames
  bool Pausing; // current state; turns on and off while Repeat stays 0
    // Very confusing: to user, "pause mode" means !Repeat
    // Need remember Repeat while Pausing turns on & off.
  bool Strobe; // mode in which you display only once per cycle
    // Strobe toggled by 'S' key -- no script file control so far.
  // Spin torque stuff, moved from rectangle:
  //bool SpinTorque; // later this could be class, have several, in different rectangles
  // GONE: short STMode; // Magnetization of the pinned layer follows this mode  
    // if zero, no spin torque
  //real aJ,bJ;   //coefficients of the spin torque


  MedField *MyMedField; // field of passing media grain
  // bool Converged;

  GridInfo* GridInf1;
  // Config *StartConfig, *LineStart; 
  // bool NeedNewPt, LineOK; // Gone! Replaced by StartConfigs list.

  real StepStart, StepStop, StepInc;
  //real Step2Min, Step2Start, Step2Max, Step2Inc, 
  real NudgeAmt; // for stepping, hysteresis
  real ScratchReal1,ScratchReal2; // dummies, used in SaturationCB
  short NudgeMode, StepMode; // Step2Mode, 
  real ConvSteps; // stupid kludge so I can display w/NumberDisplay
    // should be short, of course
  short NextDisp; // value of ConvSteps at which we will display next
    //if 0, always display; doubles each display so you display @powers of 2
  //real *StepPtr, *Step2Ptr; // what to increment
  bool Sweep; // means sweeping, not stepping, the StepMode
  Constrain *cons;
  Integrals *MyInt; // to do spacial integrals, zhuru
  // Magnetoresistance *MyMR; // magnetoresistance output

#ifdef _GROUPS
//  class NestHolder; this was bad news! Because it was inside declaration of class Systm,
// compiler thought NestHolder was part of Systm! Two "NestHolders".
  NestHolder *NH;
  SymHolder *SymH; // rotation holder
#endif  //  

#ifdef CONTOURS  
  ContourHolder *MyContours;
#endif  //  

#ifdef _NORMAL
  NorModes *My_norm;
#endif  //  

#ifdef ORBITS
  OrbitHolder *MyOrbs; // see Orbits.h
#endif  //  ORBITS

  real MaxDM; // maximum delM (un-normalized)
  real ErrDM; // error if above this
  real RotTol, DMTol, hTol; // converged if below this
  int MaxIters;
  bool IgnoreError;
  real CosLimit; // sometimes require nbr angle<this
  real ZeemanEnergy, IntEnergy, TotEnergy;

  //**************member functions******************
  void HandleError();
  void SetEvolParams();
  bool Cut_dt(); // halves
  void Restore_dt0(); // restores
  uint32_t calcFileHash(string);
  void ReadScript();
  void ReadScript2();
  cell* MLBuild(cell*);
  cell* parseBuild(cell*); //parse build cells according to the script input file
  Rectang* RectNamed(string Nam);
  void ExportData(bool,bool); // writes to file *.data
  void writeSnapshot();
  void writeFrameBinary(); //Store necessary information for restart in movOut
  bool readFrameBinary(); // Read in the data written by writeFrameBinary 
  void prepareExit (); //write final information and close files Shuxia
  void finalCheck(); //added 06/08/00, checks if we get the same result for
                     //the same cell when we changed the code
  void CalcAllFromM(bool);
  void evolve(); //added 05/30/00, for purpose of seperating Graphics part
// Utility functions
  void CheckScript(string junk, string correct);
  void CheckGood(string junk);
  void CalcEnergy(); // calcs Int & Zeeman energies
  void CalcMoments();
  real ModeEnergy(); // calcs mode Zeeman energies
  void CalcShapeFPoint(Rectang *rect,MVector r);
  void CalcShapeFCell(LLGcell* c1);
  void CalcShapeFCellJustNormalMode(LLGcell* c1, Mode* whichMode);
  void ListCells(ostream& fstrm);
  void SetUniformM(MVector); 
  real MinCos();
  bool HandleCallback();  //used to handle the callback action, DMAm
   void SaturationCB(); // most callbacks are in other classes
  enum CBMODE  // started as movie direction, now just "what were we doing"
  {FORWARD, BACKWARD, EXIT, HOME, RUN, GRID, STEP, 
     SATURATION,READ_COMMAND,WAITFORUSERTOCLOSE
//  some extra modes
#ifdef CONTOURS
   ,DO_CONTOURS
#endif  //    
#ifdef ORBITS
   ,DO_ORBITS,TRIANGULATE
#endif  //    
#ifdef _NORMAL
   , CALC_NORM_MODES
#endif  //   _NORMAL  
     } CBMode; 
  // note some of these are only active with certain compiler options, such as DO_ORBITS
   enum TERMCRIT {TIME, CYCLE_OVER, QUARTER_CYCLE, MODEVAL, MX, LOOP_ORD, CALC_GREEN, NONE} TermCrit;
   string TermStrings[8]; // = {"TIME","MODEVAL","MX","NONE"}; fails initialized in Systm::Systm
   // char enumStrings[][7]={"TIME","MODEVAL","MX","NONE"}; tried all permutations!!
   // static const char *enumStrings[]="TIME","MODEVAL","MX","NONE";
   char TestCase; // is 1 for 1D, 2,3, or 'l' for just loop, don't know exact answer
   real Infty; // set in ReadScript
   float movieVersion, Version;//don't rewrite all movies if chg Version
     //movie version that this program can read, depends on readFrame()
   bool Changed_dt; // mostly for steppping modes
#endif  //  H_ONLY
}; // end class Systm (power to the people!)

#ifdef DBG_
class DebugGreen
{
public:
  cell* WorstCell;
  real MaxErr; // omp: how do we make sure 2 cores don't change this at once?
  real SumAbs, SumSq;
  short Gdir; // which M component nonzero
  short SaveUsedOrder;
  mpole* Gmom; // moment of source cell WatLLGPart
  mpole** GAvCoef;
  static const short Plotdir=0; // which direction to plot fields (later read in)
  cell *SourceAnc, *FieldAnc;
  ofstream GFile;
  real PlotMax; // limit for plots
  DebugGreen(short dir1);
  ~DebugGreen();
  void SetDBG();
  void TestDeriv(MVector r, MVector d);
  void CheckField(MVector H_q, cell* c1);
  void CalcGreen();
  void WriteSmoo(mpole* Smoo1);
  void ReportResults();
  //void PlotFields();
}; // end class DebugGreen
#endif // DBG_

#ifndef H_ONLY  
  void CleanUpSmall(real x);
  void ChangeInc(real& x, bool Up);
#endif  //  

extern Systm *My;

#endif  //   // ends #ifndef SYSTEM.H? suddenly "unexpected" so I removed it; later added w/o problem (???)
