// AlaMag -- Copyright(C) 1999-2009 Pieter B. Visscher, see main.cpp for notices
// This is Groups.cpp

#include "Lowest.h" // #includes <cmath>
#include "_Groups.h" // describes format of groups and rotations
#include "Systm.h" // high-level, avoiding nesting in *.h
#include "cell.h"
extern Systm *My;
extern const string RotNames[16];
// some functions not in any class:

MVector RotV(RotType R, MVector V) // uses just the 1st 4 bits of R
{  MVector RV;
   if (R&1) RV[0]=-V[0]; else RV[0]=V[0];
   if (R&2) RV[1]=-V[1]; else RV[1]=V[1];
   if (R&4) RV[2]=-V[2]; else RV[2]=V[2];
   if (R&8) {real xdum=RV[0]; RV[0]=RV[1]; RV[1]=xdum;} // swap x&y
   return RV;
}

NestHolder::NestHolder() 
{  ProjToGp=-1; // could use as unitialized flag
   ProjToNes=Nestings.end(); // "don't project"
}

CellIDType** Nesting::RotCell=NULL; // static vbls, need initialize outside any class
CellIDType Nesting::NWedge=0; 
//  or you get "unresolved externals" error

//Nesting::~Nesting(){};

std::list<Nesting>::iterator NestHolder::NesNamed(string Nam)
{   std::list<Nesting>::iterator nit;
    for (nit=Nestings.begin(); nit!=Nestings.end();nit++) 
	{  //cout<<"Search for "<<Nam<<", try "<<nit->NesName<<endl;
		if (Nam==nit->NesName) return nit;
	}
    cout<<"Group Nesting name not found:"<<Nam<<endl; return Nestings.end(); // flag
}

void Nesting::MakeWedge()
{   // wedge is 1/16 of space with x>y>0,z>0

	LLGcell *c1, *cr; MVector RotCen;
	CellIDType MaxWedge, crID, iW=0; // index in wedge
   MaxWedge=10*LLGcell::NextSmCellID; // #cells in wedge? ; 10/9 is wild guess at safety factor
	if(!My->WatchedCell){cout<<"No watched cell!"<<endl; MYEXIT(33);}
	short NL = floor( (My->WatchedCell->rect->WholeRect->half[2]/My->WatchedCell->half[2])+0.1 );
	 // NL = number of cells in z thickness
	if(NL&1) MaxWedge*=((NL>>1)+1); else MaxWedge*=(NL>>1); // # of wedge pts along z
	MaxWedge/=(72*NL); // 9*8; 8=symmetry in xy, NL along z
	MaxWedge+=6; // wild guess; 5 is too small for 5x5x3 system
	cout<<"Thickness="<<NL<<", guessing we'll need "<<MaxWedge<<" of our "<<
		LLGcell::NextSmCellID<<" cells in wedge."<<endl;
	RotCell = new CellIDType*[MaxWedge];

	bool *GotHere = new bool[LLGcell::NextSmCellID]; // flag array
    for (CellIDType ic = 0; ic < LLGcell::NextSmCellID; ic++) GotHere[ic]=false;
    for (CellIDType ic = 0; ic < LLGcell::NextSmCellID; ic++)
	{  c1=My->CellFromID[ic]; // 0.1 is arbitrary roundoff tolerance
	   // skip unless is in wedge x>y>0,z>0
	   for (short j=0;j<3;j++) if(c1->center[j] < -0.2*c1->half[j]) goto Next_ic;
	   // means we already did this - equivalent to "break"? but less ambiguous
	   if (c1->center[0] < c1->center[1]-0.2*c1->half[1]) goto Next_ic; // insist x>y
	   assert(!GotHere[ic]); // would be strange if we'd been here
	   if(iW>=MaxWedge) {cout<<iW<<" wedges is too many"<<endl; MYEXIT(98);}
	   RotCell[iW]= new CellIDType[NumRots];
	   //cout<<"Have wedge cell #"<<iW<<", SmCellID="<<ic<<" at "<<c1->center<<endl;
	   for (RotType r=0;r<NumRots;r++) // loop rotations
	   {  RotCen = RotV(r,c1->center); // rotated position
          cr=My->root->SubCellContaining(RotCen);
		  if(!cr){cout<<"No cell with center "<<RotCen<<
			  " rotated from "<<c1<<endl; MYEXIT(43);}
		  if ((cr->center-RotCen).Magnitude()>0.004*c1->half*c1->half)
		    {cout<<"Rotated cell in wrong place:"<<cr->center<<"!="<<RotCen<<endl; 
		     cin.get();MYEXIT(44);}
		  crID=cr->SmCellID;
		  // cout<<"  Rotate by #"<<(short)r<<"="<<RotNames[r]<<" get SmCellID="<<crID<<" at "<<RotCen<<endl;
		  RotCell[iW][r]=crID; // save for use in projecting
		  GotHere[crID]=true;
	   }
	iW++; // increment number of cells in wedge
Next_ic:
	crID=0; // dummy line to allow compile
	} // end for(ic)
  //cout<<"GotHere="<<GotHere;
  delete GotHere; // works?
  //cout<<", after deletion = "<<GotHere<<endl; // no change! How do I know it was deleted?
  NWedge=iW; // # of actual wedges; static data member
  cout<<"Total number of cells in wedge is "<<NWedge<<" , max="<<MaxWedge<<
	    " of total "<<LLGcell::NextSmCellID<<endl;

// debug check, can comment out cout:
    RotType R,R1;
    for (short i=0;i<NumRots;i++)
	{  R=i; R1=i; 
	   RotType Prod=Mult(R,R1); // should be 1 or R90:
	   //cout<<"R1="<<(short)R<<"="<<RotNames[R]<<"; "<<
		//"R2="<<(short)R1<<"="<<RotNames[R1]<<"; "<<
		//"R1*R2="<<(short)Prod<<"="<<RotNames[Prod]<<endl;
	   if((Prod!=0) && (Prod!=3)) EXIT1("Error in Mult check, Groups.cpp");
       R1=Mult(R,Inverse(R));
	   if(R1!=0) cout<<"R*inverse = "<<RotNames[R1]<<", should be 1!"<<endl;
	}
} //end MakeWedge

Config Nesting::Project(Config M0,RotType iGroup) // use iGroup-th group in nesting
{   // write group element as
    Config NewM; 	
	MVector Sum,M0c; 
	CellIDType iW; // wedge-cell index
	CellIDType rcID; // rotated cell index
	RotType Ri,i,iC; // coset index
	assert(SizeSub[iGroup]<17); // sanity, 16 is max size
	for (iW=0;iW<NWedge;iW++) // loop wedge points
	  for (iC=0;iC<NumRots;iC+=SizeSub[iGroup]) // jump over group to get coset members
	  { //cout<<"iW="<<iW<<"@"<<My->CellFromID[RotCell[iW][0]]->center <<",iC="<<(short)iC<<" is "<<
	    //  RotNames[GRots[iC]&15]<<" at "<<My->CellFromID[RotCell[iW][GRots[iC]&15]]->center<<endl;
		Sum.SetToZero(); 
		for(i=0;i<SizeSub[iGroup];i++) // loop subgroup to add M's
		{   Ri=GRots[i];
			if(Mult(Ri,GRots[iC])!=GRots[iC+i]) EXIT1("Error in GRots");
			rcID=RotCell[iW][GRots[iC+i]&15]; // really Grots should be on left, of course
            if(Ri&16) Ri=Ri^7; // reverses x,y,z
		    Sum+=RotV(Inverse(Ri),M0[rcID]); // now Ri contains extra spin inversion
			//cout<<"From cell@"<<My->CellFromID[rcID]->center<<" Added "<<RotV(Inverse(Ri),M0[rcID])<<endl;
		}
	    Sum/=SizeSub[iGroup]; // divide by no. of rotations we did

		for(i=0;i<SizeSub[iGroup];i++) // loop subgroup again to write new M's
		{   Ri=GRots[i];
			rcID=RotCell[iW][GRots[iC+i]&15]; // really Prod should be on left, of course
            if(Ri&16) Ri=Ri^7; // reverses x,y,z
		    NewM[rcID]=RotV(Ri,Sum); // now Ri contains extra spin inversion
			//cout<<"To cell@"<<My->CellFromID[rcID]->center<<" put "<<RotV(Ri,Sum)<<endl;
		}
	  } // end for(iC)
	return NewM;
} //end Project()

void Nesting::ReadNesting(ifstream& Fil)
{  // MAY some day want to separate SubGroup generation from reading!
	// so you can do it for several subgroups within the nesting?
	string junk; RotType R, Ri, i, j, Pow, iGroup;
	NumGens=0; // count generators
	bool Sieve[NumRots]; // flag array: argument is SPACE rotation only.
	  // 0 if rot not in group, 1 if is, 2 if is w/spin inversion
    GRots[0]=0; SizeSub[0]=1; // start with identity group
	Preserved[0]=true; // LL equation preserves this symmetry trivially
	Sieve[0]=true; // start with identity in group (never has spin inversion)
	for (i=1;i<NumRots;i++) Sieve[i]=false; // haven't found that rotation
	Fil>>junk>>NesName; My->CheckScript(junk,"4-fold"); // checks first part of string
	cout<<endl<<"ReadNesting "<<NesName<<endl;
	// We generate the group via Sieve, just in case the generators aren't complete 
	// and we need to add more

	while(SizeSub[NumGens]<NumRots) // enlarge group by reading or picking generators
	{   if(Fil.peek()==(int)'!') // no more generators to read
	    { // look for rotation R not yet generated
	      R=NumRots; // flag means haven't found gap
	      for (i=0;i<NumRots;i++) if(!Sieve[i]) {R=i; break;}
          if(R==NumRots) EXIT1("Can't find unused space rotation for generator"); // 
	    }
	    else // can read generator
	    {  Fil>>junk>>ws; // read name & remove white space to read terminator ! 
		   size_t found = junk.find('&'); // look for "&i"
           bool HasInv=(found!=string::npos); // found &i
		   if(HasInv) junk=junk.substr(0,found); // remove "&i"
		   for (R=0; R<NumRots;R++) if(RotNames[R]==junk) goto GotIt;
           EXIT1(junk+" is not a rotation");
GotIt:
	       cout<<"- ReadNesting read generator="<<RotNames[R];if(HasInv) cout<<"&inv.spin";cout<<endl;
           R=R | ((RotType)HasInv<<4); // insert spin-inv bit 
	    }
	    // Got new generator R
		iGroup=NumGens; // index of group we HAVE
		Gen[NumGens++]=R; //save & postincrement
        Preserved[NumGens]=Preserved[iGroup] && !((R&1)^((R&2)>>1)^((R&4)>>2)^((R&8)>>3)^((R&16)>>4));
		  // each of these 5 bits represents an odd-parity factor; odd parity not preserved
		// now iGroup = index of group we have = NumGens-1 (NumGens is the one we're generating)
       // Fil>>ws; // get rid of white space to read terminator ! 

       Pow=R; // power of R, start with 1st
	   short SizeSet=SizeSub[iGroup]; // expand G to set G+GR
	   j=SizeSet; // index in GRots of new entries, already have 0...SizeSub-1
	   while(!Sieve[Pow&15]) // mult. group by power of R,    
	   { //but not if its space rot'n already used (means it's in group already)
	     for (i=0;i<SizeSet;i++) //multiply set by Pow and append
	     {  Ri=Mult(GRots[i],Pow); 
	        GRots[j++]=Ri; // insert product; increment j
			//if(Sieve[Ri&15]) EXIT1("NOT Error: Generator was already in group?");
			Sieve[Ri&15]=true; // don't need spin inv info
			//for (short k9=0;k9<NumRots;k9++) if(Sieve[k9]) cout<<'@'; else cout<<"."; 
			//cout<<" Pow="<<Pow<<";i="<<i<<", GRots="<<GRots[i]<<",Ri="<<Ri<<endl;
	     } 
		 //assert(j==(2*SizeSet)); should have doubled? NOT ALWAYS!
		 SizeSet=j; 
		 Pow=Mult(Pow,Pow); // square
	   } // end while Pow
	   SizeSub[NumGens]=j; // new group size
/*	   // now print out lots of stuff, to debug groups:
       cout<<"SubGroup # "<<NumGens<<":";
	   for (j=0;j<SizeSub[NumGens];j++) 
	     {cout<<RotNames[GRots[j]&15];if(GRots[j]&16) cout<<"&i";cout<<",";}   
	   cout<<endl;

	   short hash1=0, hash2=0; // make unique identifier for space group
	   for (j=0;j<SizeSub[NumGens];j++) {i=GRots[j]&15; hash1+=i; hash2+=i*i;}
	   cout<<"Space,"<<NesName<<d_<<hash1<<d_<<hash2<<d_;
	   for (j=0;j<SizeSub[NumGens];j++) {cout<<RotNames[GRots[j]&15]<<d_;}
	   cout<<endl;

	   hash1=0; hash2=0; // make unique identifier for total group
	   for (j=0;j<SizeSub[NumGens];j++) {i=GRots[j]; hash1+=i; hash2+=i*i;}
	   cout<<"Spin,"<<NesName<<","<<hash1<<","<<hash2<<",";
	   for (j=0;j<SizeSub[NumGens];j++) {i=GRots[j]; cout<<RotNames[i&15]; if(i&16) cout<<"&i";cout<<",";}
	   cout<<endl;
done with debug groups prints */

	} 	// end while; done finding generators
    // NO!! getline(Fil,junk,'\n');

    cout<<"Got all "<<NumRots<<" rotations, done ReadNesting; NumGens="<<
		(short)NumGens<<endl;
	cout<<"Generators are ";
	for (i=0;i<NumGens;i++) {cout<<RotNames[Gen[i]&15]; 
	  if(Gen[i]>>4) cout<<"&i"; cout<<", ";} //Gen[i]>>4 is spin-inv bit
	cout<<endl;
	cout<<"Group sizes & preservation of LL are "; 
	for (i=0;i<=NumGens;i++) {cout<<(short)SizeSub[i]; if(Preserved[i]) cout<<"P"; cout<<", ";}
	cout<<endl;
} // end ReadNesting

// Deal with general rotations: implement holder class Symmetry
Symmetry::Symmetry(short l1,bool odd1) //  constructor
{ l_=l1; odd=odd1; usage=Ignore; // reset later
  if(l1==0){cout<<"Called symmetry constructor with l=0;"<<
     "sorry, but there's no finite group corresponding to l=0"<<endl;
    MYEXIT(111);}
  real NbrDir, x1, y1, ang = M_PI/l_;
  LLGcell *cR, *cNbr;
  // bool NbrLeft; // true if cell cNbr is to left of cR
  MVector rNbr,r2;
  Cos = cos(ang); Sin = sin(ang);
  cout<<"Entering contructor Symmetry()with lP="<<l1;
  if(odd1)cout<<"-"; else cout<<"+";
  cout<<", Cos="<<Cos<<
      ", Sin="<<Sin<<endl;
  // Now create weight coefficients:
  NbrID=new CellIDType*[LLGcell::NextSmCellID];
  NbrW = new real*[LLGcell::NextSmCellID];
  for (CellIDType ic = 0; ic < LLGcell::NextSmCellID; ic++)
  {  NbrID[ic]=new CellIDType[3]; // 3 cells weighted
     NbrW[ic] = new real[3];\
        // DEBUGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG
     My->CellFromID[ic]->M=My->CellFromID[ic]->center; // DEBUG for radial M
  }
  // and initialize them
  for (CellIDType ic = 0; ic < LLGcell::NextSmCellID; ic++) // loop cells
  { cell* c1=My->CellFromID[ic];
    MVector r1; // position, rotated cw by pi/l & mirrored in z 
    r1[0]=Cos*c1->center[0]+Sin*c1->center[1];
    r1[1]=Cos*c1->center[1]-Sin*c1->center[0]; r1[2]=c1->center[2];
	if(odd) r1[2]=-r1[2]; // parity
    //cout<<"cell "<<*c1<<", center at "<<c1->center<<", rotates to r1="<<r1<<endl;
    cR=My->root->SubCellContaining(r1); // rotated cell
    if (!cR) // r1 was not in a cell
    { x1 = fabs(r1[0]); y1 = fabs(r1[1]);
      r2 = r1; // copy to move around
      if (x1>y1) // near x axis, move toward origin
       // Fails: r2[0]-=copysign(2*c1->half[0],r2[0]); // magnitude 2*half, sign of r2
       // copysign not in cmath in 2005?
       if(r2[0]>0) r2[0]-=2*c1->half[0]; else r2[0]+=2*c1->half[0];
      else // move in y direction
       if(r2[1]>0) r2[1]-=2*c1->half[1]; else r2[1]+=2*c1->half[1];
      cR=My->root->SubCellContaining(r2); // Try again
      if(!cR) {cout<<"Moved r1="<<r1<<" inward to r2="<<r2<<
         ", STILL not in system!"<<endl; MYEXIT(48);} // can this happen?
    }  // now cR is the closest cell center to r1
      
    NbrID[ic][0]=cR->SmCellID; // use this as 0th nbr

    // Find neighbor in x direction:
    if(r1[0]>cR->center[0]) NbrDir=1.0; else NbrDir=-1.0; // +1 if r1 to right, -1 if left
    // fails NbrDir=copysign(1.0,r1[0]-cR->center[0]);
    // NbrLeft = signbit(r1[0]-rNbr[0]); // true if r1 to left
    //if(NbrLeft) NbrW[ic,1]*=-1;
    // assert(half[0]==half[1]); // could use either
    rNbr = cR->center; // now displace this in direction of r1:
	 rNbr[0]+=NbrDir*2*cR->half[0]; 
    cNbr=My->root->SubCellContaining(rNbr);
    if(!cNbr) // this isn't cell, must go other direction
    { NbrDir=-NbrDir;
      rNbr[0]=cR->center[0]+NbrDir*2*cR->half[0]; 
      cNbr=My->root->SubCellContaining(rNbr); 
      if(!cNbr) {cout<<"Can't find x-neighbor of cell "<<
         *cR<<", quitting";MYEXIT(38);} 
    } 
    NbrID[ic][1]=cNbr->SmCellID; // use this as 1st nbr
    NbrW[ic][1]=NbrDir*(r1[0]-cR->center[0])/(2*cR->half[0]); // x-distance

    // Do the same for y direction 
    if(r1[1]>cR->center[1]) NbrDir=1.0; else NbrDir=-1.0; 
       // +1 if r1 up, -1 if down
    rNbr = cR->center; // now displace this in direction of r1:
	 rNbr[1]+=NbrDir*2*cR->half[1]; 
    cNbr=My->root->SubCellContaining(rNbr);
    if(!cNbr) // this isn't cell, must go other direction
    { NbrDir=-NbrDir;
      rNbr[1]=cR->center[1]+NbrDir*2*cR->half[1]; 
      cNbr=My->root->SubCellContaining(rNbr); 
      if(!cNbr) {cout<<"Can't find y-neighbor of cell "<<
         *cR<<", quitting";MYEXIT(39);} 
    } 
    NbrID[ic][2]=cNbr->SmCellID; // use this as 2nd nbr
    NbrW[ic][2]=NbrDir*(r1[1]-cR->center[1])/(2*cR->half[1]); // y-distance

    NbrW[ic][0]=1.0-NbrW[ic][1]-NbrW[ic][2]; // weights must add to 1
  } // end loop on ic


 // DEBUG only:
Config M0,M1;
M0.saveM();
cout<<"Test: start M(0)="<<M0.ConfigArr[0]<<", M(1)="<<M0.ConfigArr[1]<<endl;

RotC(M0,M1);

cout<<"Rotated, M(0)="<<M1.ConfigArr[0]<<", M(1)="<<M1.ConfigArr[1]<<endl;

M1.restore();

} // end constructor Symmetry()

Symmetry::~Symmetry()
{  /* NO!!!! This is called when you exit ReadScript after pushing
   it onto list.  You still need these arrays!
   for (CellIDType ic = 0; ic < LLGcell::NextSmCellID; ic++) // calc
      {delete NbrID[ic]; delete NbrW[ic];}
   delete NbrID; delete NbrW;
   */
}

SymHolder::SymHolder()
{
   // can't really think of anything we need
}

ostream& operator<<(ostream &fout, Symmetry s){ // overload <<
  fout<<"Symmetry lP="<<s.l_; if(s.odd) fout<<'-'; else fout<<'+';
  return fout; }

std::list<Symmetry>::iterator SymHolder::FindSym(short l_1,bool odd1)
{   std::list<Symmetry>::iterator sit;
    char parity;
    if(odd1) parity='-'; else parity='+';
    for (sit=Symmetries.begin(); sit!=Symmetries.end();sit++)
	{  cout<<"Search for symmetry l="<<l_1<<parity<<endl;
      cout<<", try "<<sit->l_<<", parity "<<sit->odd<<endl;
		if ((sit->l_==l_1) && (sit->odd==odd1)) return sit;
	}
    cout<<"Symmetry not found, l="<<l_1<<parity<<endl; 
    MYEXIT(48); // for now; else need check after return
    //return Symmetries.end(); // flag as unfound
}

Config MakeSeed(short w, bool odd,short j) // see p. 4.536
{ // makes the j^th (j=0,1) config with winding number w
   // odd refers to parity under horiz mirror mz
   // normally odd=true; even parity needs factor of z
   // Seed normalized to 1 at periphery
   short i;
   bool wNeg=(w<0); 
   short wa; if(wNeg) wa=-w; else wa=w; // wa=|w|
   real *xPow, *yPow;
   real Radw, Rad=My->root->half[0]; // circle radius
   short* BinCoefs;
   xPow=new real[wa+1];
   yPow=new real[wa+1];
   BinCoefs=new short[wa+2]; // need BinCoefs[wa+1]=0
   for (i=1;i<wa+2;i++)  BinCoefs[i]=0.0;
   BinCoefs[0]=1.0;  // initialize to 1,0,0,...
   Radw=1.0; // initialize
   for (short v=1;v<=wa;v++) // calculate coeffs for order v from v-1
   {  //cout<<"Calc. BinCoef, order "<<v<<"(backwards):";
      for (i=v;i>0;i--) // backwards to avoid over-writing
      {  BinCoefs[i]+=BinCoefs[i-1];
         //cout<<" BinCoefs["<<i<<"]="<<BinCoefs[i];
      }
      //cout<<endl;
	  Radw*=Rad; // radius^wa for normalization 
   }
  Config MS; real Sgn, xyB; short iPow,q;
  for (CellIDType ic = 0; ic < LLGcell::NextSmCellID; ic++)
  {  LLGcell* c1=My->CellFromID[ic];
     //cout<<"Cell "<<*c1<<endl;
     xPow[0]=1.0; yPow[0]=1.0; // xPow[i]=x^i
     for (i=1;i<=wa;i++) {xPow[i]=xPow[i-1]*c1->center[0];
       yPow[i]=yPow[i-1]*c1->center[1]; 
       //cout<<"Calc "<<i<<"-th power of x="<<xPow[i]<<", y="<<yPow[i]<<endl;
       if(wa==0) MYEXIT(54);}
     MS.ConfigArr[ic].SetToZero();
     Sgn=1.0; // will be (sign w)^q
     for (q=0;q<=wa;q++) // see 4.5362
     { iPow=q-j+4; // +4 avoids confusion if iPow<0; iPow=exponent of i; 
	   // iPow = iPow&1 + iPow&2 + multiple of 4, so i^iPow = i^iPow&1 * i^iPow&2;
	   xyB=Sgn*BinCoefs[q]*xPow[wa-q]*yPow[q]/Radw; // term to add, with i^iPow coefficient
	   if (iPow&2) xyB=-xyB; // i^iPow&2=-1 (else =1 if iPow&2=0)
	   if (iPow&1) // i^iPow&1 is i, contribs to imag part (M_y)
         MS.ConfigArr[ic][1]+=xyB;
	   else // contribs to real part
         MS.ConfigArr[ic][0]+=xyB;
       //cout<<"chg M to "<<MS.ConfigArr[ic]<<", Sgn="<<Sgn<<", y-exp="<<q<<endl;
       if(wNeg) Sgn=-Sgn; // so Sgn remains (sign w)^q
//	   if(xyB!=0) if(c1==My->WatchedLLGCell) 
//		   cout<<"Nonzero xyB="<<
//		   xyB<<", q="<<q<<", MS="<<MS.ConfigArr[ic]<<", cell"<<*c1<<endl;
     } // end for q
     if(!odd) MS.ConfigArr[ic]*=c1->center[2]; // factor of z
  } // end for ic
  cout<<"Done with MakeSeed; W->MS="<<MS.ConfigArr[My->WatchedLLGCell->SmCellID]<<endl;//"config is:"MS;
  return MS;
} // end MakeSeed

//Now do projection 
void Symmetry::Project_l(Config& M0, Config &MP) // output projected MP 
{  Config ggS, Sq=M0; // see p. 4.745
   for(short iq=1;iq<=2*l_;iq++) // will never hit 2*l though
   { cout<<"In Project_l, iq="<<iq<< // ", Sq="<<endl<<Sq<<endl;
       ", Sq(watched)="<<Sq.ConfigArr[My->WatID]<<endl;
     RotC(Sq,ggS); // now S_q
	 cout<<"ggS(watched)="<<ggS.ConfigArr[My->WatID]<<endl;
	 //cout<<"ggS="<<ggS<<endl;
     if(iq==2*l_) {MP = (1.0/(2.0*l_))*ggS; //cout<<"Returning "<<MP<<endl;
	   cout<<"Returning watched "<<MP.ConfigArr[My->WatID]<<endl;
	   return;}
     Sq = ggS + M0;
   }
}

void Symmetry::ProjectOut(Config& M0, Config &MP) // REMOVE this sym from M0
{  Project_l(M0,MP); // now MP is P M0
   MP=M0-MP; // now (1-P) M0
}

void Symmetry::RotC(Config &M0,Config &MRot) // Rotate a Configuration M0 to MRot by pi/l
{  // also mirror (for pseudovector, rotate M 180 degrees)
  MVector MR; // position rotated, not M
  for (CellIDType ic = 0; ic < LLGcell::NextSmCellID; ic++) // calc CRot at ic
  {  MR.SetToZero();  
     for (short j=0;j<3;j++) // loop nbrs
     {  assert(NbrID[ic][j]<LLGcell::NextSmCellID); 
        assert(NbrID[ic][j]>=0); // sanity check
        MR+= M0.ConfigArr[ NbrID[ic][j] ] * NbrW[ic][j];
     } 
	 if(odd) {MR[0]=-MR[0]; MR[1]=-MR[1];} // mirror*inv
     //cout<<"RotC: cell#"<<ic<<" has MR="<<MR;
	 // now rotate M
	 MRot.ConfigArr[ic][0]=MR[0]*Cos - MR[1]* Sin;
     MRot.ConfigArr[ic][1]=MR[0]*Sin + MR[1]* Cos;
	 MRot.ConfigArr[ic][2]=MR[2];
     //cout<<", rotates to "<<MRot.ConfigArr[ic]<<endl;
  } // for ic

} // end function RotC

