// this is  cell.cpp
#include "Lowest.h" // is in all .cpp, not headers
#include "cell.h"
//#include "mpole.h" // so cell can have moments
//#include "_Groups.h" // 

#include <sstream> // for stringstream LocalOut, below
#include <iomanip> // needed for setw
#include "Systm.h"
// is in system.h: #include "pulse.h"
#include "FMM.h" // so NearPartner is defined, to make lists of it
//#include "Outputs.h" // to average cell properties

extern Systm *My;
#define SUCC(x) (x+1)%3
#ifdef DEBUG1
  bool cell::PD1=false; // controls UpdateStat error messages
#endif  //  
#ifdef DEBUG2
  bool cell::PD2=false; // controls CullPartners error messages
#endif // DEBUG2

CellIDType LLGcell::NextSmCellID = 0; //declare static data members outside of any class
  // if you don't declare (initialize??) you get a linking error


  // Try to do this outside any class: don't want in cell constructor, done too often:

// unsigned short cell::MaxCellID = (unsigned short) (pow((double)2,(int)(sizeof(unsigned short)*8) )-1);
//  cout<<"MaxShort="<<cell::MaxShort<<endl; cin.get(); won't allow cout here!

#ifdef GRAPHICS
  #include "display.h" // for Open Inventor & OpenGL classes
short cell::NumLocalColor=0;
short cell::iLoc=0; // which local display we're working with
short cell::HDisp_Which[]; // which fields to display at each cell
float cell::LocalColorR[]; // this is treated as initialization of pointer?
float cell::LocalColorG[];
float cell::LocalColorB[];// actual color components
float cell::H_max[MaxLocalColor]; 
float cell::H_scale[MaxLocalColor]; 
bool cell::H_scaleAuto[MaxLocalColor];
float cell::H_scaleToDisplay[MaxLocalColor]; 
short cell::*NumLocalColor;
// TextDisplay* cell::HDispText[];
NumberDisplay *cell::HDispNumb[MaxLocalColor];
#endif  //  GRAPHICS

Rectang::Rectang(cell *Whol)
{ // create rectang whose WholeCell is Whol
  My->RectFromID[My->NRects]=this;
  index = My->NRects;
  My->NRects++; 

  if (My->NRects>My->MaxRects) {cout<<My->NRects<<"> MaxRects="<<My->MaxRects<<endl; MYEXIT(53);}
  WholeRect=Whol;
  RectInfoPtr=NULL; // can't set up until know LLG size ('divide' command)

#ifndef H_ONLY
  for(short j=0;j<6;j++) {tJ[j]=0.0; IntPartner[j]=My->MaxRects;}// just must be >NRects
  H_rand_scale=0.;
  //SpinTorque=false; 
  LLGSize.SetToZero();
//  shape = NULL; //initialize to null
  avEasyAxis = MVector(1.0, .0, .0); // defaults so won't crash
  M0 = MVector(.0, .0, .0); 
  Hk = 0.; Ms=0.; // so can tell if uninitialized
  AnisPower=NULL; AnisCubic=NULL; AL=NULL; shape=NULL;
  // AvCoef[0]=NULL; // flag for un-init.
  ExchangeA = 0.; ExchLen2=0.;
  SpinAx=SpinAy=SpinPh=0.; //spin wave amplitude=0
  kVector.SetToZero(); // spin wave vector
  //Mp= MVector(.0, .0, .0); // Shuxia_1120
  sizeToDisplay.SetToZero(); // default to show smallest cells
  ShowM=0; // ShowH=0; // ShowM used to be in cell; now DispInfoPtr tells whether to show M?
  //LevelToDisplay=0; // display lowest by default
  //MToDisplay=0; // also show M of smallest
  // NumFieldDisp=0;
#endif  //  H_ONLY
} // end Rectang constructor

CellInfo::CellInfo(MVector CellSize, short ord) // means HALF size!
{  // constructor calculates AvCoef array
  cout<<"Create CellInfo: AvCoef"<<endl;
  AvCoef = My->MyFMM->MakeAvCoef(CellSize,ord);
} // end constructor CellInfo()
  
cell::cell()
{
  child0=NULL; child1=NULL; rect=NULL; 
  moment=NULL; SmoothField=NULL;
#ifndef H_ONLY
  DispInfoPtr=NULL;
#endif  //  
  // gone 7/09: if(parent1) Level=parent1->Level+1; else Level=0; // Level measured down from root
  parent=NULL;
  if(My->MyFMM) {moment = new mpole(); SmoothField = new mpole();}
} // end cell constructor 

LLGcell::LLGcell(cell* ce) // copy constructor from already-defined cell
{ center=ce->center; half=ce->half; parent=ce->parent; // no children yet
rect=ce->rect; radius2=ce->radius2; // these were set already
  CellInfoPtr=NULL;
  
} // end cell constructor 

LLGcell::LLGcell() { // non-copy constructor
	child0=NULL; child1=NULL; rect=NULL;
	parent=NULL; CellInfoPtr=NULL;
} 

LLGcell::~LLGcell() {
	// delete lots of things
}



/* // unused 06/2013
LLGcell* cell::ConvertToLLG(cell* &ptr) // also creates faces
{  LLGcell* p1 = new LLGcell(ptr);
   delete ptr;
   p1->CreateFaces();
   return p1;
} // end ConvertToLLG
*/
cell::~cell()
{
	// clean up memory leakage
	/*
	delete &center;
	delete &half;
	delete moment;
	delete &Partners;
	delete &radius2;
	// delete ptrs for tree
	delete &child0; 
	delete &child1;
	delete &parent;
	*/
	// everything else is static or a function.

}


bool cell::sane()
{ // note that we can't call check_faces from this, it would be circular call.
  //if(My->Psetup) cout<<"entering cell::sane()"<<endl;
  if (parent)
     assert( (parent->child0==this) || (parent->child1==this) );
   else // no parent
     assert(this==(My->root));
  if (child0) assert(child0->parent==this);
  if (child1) assert(child1->parent==this);
  return 1; 
}

bool cell::Contains(MVector v)
{   for (short i=0;i<3;i++) if(fabs(v[i]-center[i])>half[i]) return false;
    return true;
}

bool cell::AncestorOf(cell* c1)
{  //cout<<"In AncestorOf ("<<*c1<<") with this:"<<*this<<endl;
   if(child0) {if (child0->AncestorOf(c1)) return true;}
   if(child1) return child1->AncestorOf(c1);
   //cout<<"No children, must be same to be anc"<<endl;
   return (this==c1); // at bottom: must BE c1 to be anc
}

LLGcell* cell::SubCellContaining(MVector &v)
   // returns subcell of "this" cell, that v is in,NULL if v not in this
   // does NOT require that v be in the shape, as long as the cell exists.
   // Assumes that all cells have bounding box defined (center+-half).
   // 
   // This only works at LLG level and above (not inside partial cells)
   // before 6/2013, it passed MVector halfsizeToFind, but was always smallest.
   {
#ifdef DEBUG_SUBCELL
     cout<<"Find SubCellContaining "<<v<<", in "
       <<*this<<", ptr c="<<this<<", child0="<<child0<<endl;
#endif  //  
   if(!Contains(v)) return NULL;
   LLGcell *result;
   if(child0) 
   {  result=child0->SubCellContaining(v);
      if(result) return result; 
	  return child1->SubCellContaining(v);
     // note that this can still be NULL, if v in bounding box but not in subcell
   }
   else // no children; had better be LLG cell
   {   assert(typeid(*this)==typeid(LLGcell));
	   return static_cast<LLGcell*>(this); // smallest, LLG, cell
   }
} // end SubCellContaining(v)

/*
LLGcell* cell::SubCellContaining(MVector &v)
   // Like SubCellCOntaining, but returns LLGcell (NULL if not in LLGcell)
{  cell* c1=SubCellContaining(v);
   LLGcell* LLGptr = dynamic_cast<LLGcell*>(c1);
   return LLGptr;
}
*/

ostream& operator<<(ostream &fout, cell c){ // overload <<
  fout<<c.LowerLeft()<<"_to"<<c.UpperRight(); return fout; }

//added by DMAf
//recursively update all multipole moments in all cells, to be called before updateStat
void cell::updateMoment() // recursive, down to LLG level
{  // where M is known
   SmoothField->SetToZero(); // UpdateStat requires this later

  if (child0) //not the smallest cell
  {
    child0->updateMoment(); child1->updateMoment(); 
    moment->SetToZero();
    moment->AddShift(child0->moment, child0->center - center);
    moment->AddShift(child1->moment, child1->center - center);
    // AddShift uses FMM::scratchUpM, which needs to be passed (see updateStat())
    // if we try to parallelize updateMoment
  }
  else //smallest cell, see notes for definition of MomentCoefs=AvCoefs
	    // tho 2014 I don't know where the notes are -- new ones on p. 21.4
  {  assert(typeid(*this)==typeid(LLGcell));
     LLGcell* LL=static_cast<LLGcell*>(this);
     CellInfo* CellInf=LL->GetInfoPtr(); // to access in loop
    //calculate the moment:
	 moment->SetToZero();
	 // Added - sign here: (?? not there 2014)
	for (short i=0;i<VDIMENSION;i++) 
      moment->AddMult(CellInf->AvCoef[i],volume()*LL->M[i]);
   /*
   cout<<"In updateMoment for "<<*this<<", mom="<<*moment;
   cout<<"AvCoef"<<*CellInf->AvCoef[0];
   cout<<"vol="<<volume()<<", LL->M[0]="<<LL->M[0]<<endl;
   cout<<"WatPart:"<<*My->WatLLGPart<<", M="<<My->WatLLGPart->M<<endl;
   */
  }  //end of else for smallest cell
} // end updateMoment()

//added by DMAf
//add magnetostatic field from NearPartner to H_quad
void LLGcell::AddNearField(const NearPartner *source)
{ const MVector &m = source->cel->M;
  H_quad[0] += source->K->Kxx*m[0]+source->K->Kxy*m[1]+ source->K->Kxz*m[2];
  H_quad[1] += source->K->Kxy*m[0]+source->K->Kyy*m[1]+ source->K->Kyz*m[2];
  H_quad[2] += source->K->Kxz*m[0]+source->K->Kyz*m[1]+ source->K->Kzz*m[2];
#ifdef DEBUG1
  if (this==My->WatchedLLGCell) if(PD1) cout<<"  Adding field "<<*source->cel<<
      " with M="<<m<<",  H_quad = "<<H_quad<<endl; 
#endif  //  
}

//added by DMAf, returns 1 if source1 is close to this
bool cell::Near(const DispCell &source1)
{
  real Rad2=0.25*(My->MyFMM->MaxAngle2)*((source1.absCenter()) - center ).Square();
// Rad2 = max allowed squared radius
  return ( (radius2>Rad2) || (source1.cel->radius2>Rad2) );
}

//used to average SmoothField
MVector cell::averageMpole (const mpole *mpoleInput) const
{ // this is in cell, not LLGcell, because we use it for smaller cells than LLG
  MVector averageField;
  CellInfo* CellInf;
  CellInf=rect->RectInfoPtr; // points to AvCoef, etc
  assert(CellInf); // debug
  averageField[0] = *(CellInf->AvCoef[0])*(*mpoleInput);
  averageField[1] = *(CellInf->AvCoef[1])*(*mpoleInput);
  averageField[2] = *(CellInf->AvCoef[2])*(*mpoleInput);
  // PBV introduced - signs 12/13/2011 empirically -- removed 2014!
  // also removed in definition of AvCoef
  return averageField;
}

CellInfo* LLGcell::GetInfoPtr() const
{ // points to AvCoef, etc
  if (CellInfoPtr) return CellInfoPtr; // cell has its own if partial LLG
  else return rect->RectInfoPtr; // otherwise use rectangle's
  // there will be another option if fine cell: rect has array of fine infoPtr 
}

//added by DMAf
void cell::CullPartners() // called for root, before first updateStat call.
{  // on entry, assume Partners list contains all cells near parent
  // i.e. sources whose field is not included in parents SmoothField.
  // On exit, Partners list should contain only sources in this cell's
  // SmoothField.

/* Profiler says approx 47% of time is spent here when run serially (100x100x1 cell grid, 
  timed until COIN graphics visible) 06/25/2013 
*/
  std::list<DispCell>::iterator j;
  std::map<int3, Kernel>::iterator iterMap;//iterator for Kernels
  LLGcell* LL=NULL;
  //cout<<"Starting CullPartners with cell "<<*this<<endl;
//we need to clear the children's lists:
  if (child0)
  {
    child0->Partners.clear();
    child1->Partners.clear();
    //child0->NearPartners.clear(); must do only if child0is LLGcell!
    //child1->NearPartners.clear();
	//if(My->Psetup)cout<<"Enter CullPartners for composite cell "<<this<<","<<*this<<endl;
  }
  else 
  {   assert(typeid(*this)==typeid(LLGcell));
      LL=static_cast<LLGcell*>(this);
	  LL->NearPartners.clear();
	  //if(My->Psetup)cout<<"Enter CullPartners for smallest "<<LL->SmCellID<<","<<*this<<endl;
  }
  j = Partners.begin();
#ifdef DEBUG2
  //cout<<"In CullP., debug:WatchedCell="<<*My->WatchedCell<<", this="<<*this<<endl;
  bool Watch=AncestorOf(My->WatchedCell); // watch ancestors too
  //cout<<"Watch="<<Watch<<endl;
  if (Watch&&PD2) 
  {  cout<<"Looping partners of cell "<<*this<<" "<<this<<", radius2="<<radius2;
     if(typeid(*this)==typeid(LLGcell)) cout<<", SmCellID="<<static_cast<LLGcell*>(this)->SmCellID;
     cout<<endl;
  }
#endif  //  

  while (j!=Partners.end()) // loop partners
  {
#ifdef DEBUG2
    bool Watch_j;
    if (My->WatLLGPart) Watch_j=Watch && j->cel->AncestorOf(My->WatLLGPart); 
      else Watch_j=Watch; // WatLLGPart=NULL means watch all partners
    Watch_j=true; // override for debug
    if (Watch_j&&PD2) 
    {  cout<<"  Partner="<<j->cel<<","<<*j->cel<<", radius2="<<j->cel->radius2<<endl; // PBV3
       if(typeid(j->cel)==typeid(LLGcell)) cout<<"ID="<<static_cast<LLGcell*>(j->cel)->SmCellID<<
           ",d2="<<j->cel->radius2<<endl;
    }
#endif  //  
	  //if((j->cel->radius2)<12) {My->CBMode=My->EXIT; return;} // bail out
	  // Wierd!  Delete this: if((j->cel->SmCellID)>0) if((j->cel->SmCellID)<0) {
	  //  My->CBMode=My->EXIT; return;} // bail out
	  // was probably trying to check if SmCellID had been overwritten by something wierd?

    if (Near(*j)) //very close, not in the crescent region
    {// don't give up yet, children might be far
#ifdef DEBUG2
      if(Watch_j&PD2) cout<<"Partner near, split if big"<<endl;
#endif  //  
      if (j->cel->radius2 > radius2 && j->cel->child0) // split it if big
      { 
#ifdef DEBUG2
      if(Watch_j&PD2) cout<<"Partner big, split it & go on to new partners (including children)"<<endl;
#endif  //  
        Partners.push_back(*(j->GetChild0()));// add to list;
        Partners.push_back(*(j->GetChild1()));
        j = Partners.erase(j);//returns iterator that points to the next node
      }
      else // partner already small, must pass to children, if any
      {
        if (child0)
        {
#ifdef DEBUG2
          if(Watch_j&&PD2) cout<<"Partner small, pass to children"<<endl;
#endif  //  
          child0->Partners.push_back(*j);
          child1->Partners.push_back(*j); // done w/this j, deal w/kids later in loop
          j = Partners.erase(j);//returns iterator for next node, continue while loop
        }
        else // both are smallest; need to put this partner in NearPartners
        {           // note LLGSize is HALF size
 	   int3 key = (j->absCenter() - center).ToInt(rect->LLGSize*2); //key for Kernel
           LLGcell* jLLG=static_cast<LLGcell*>(j->cel);
#ifdef DEBUG2
           assert(!j->cel->child0); // both should be smallest
           if (Watch_j&&PD2) 
           {  cout<<"both smallest; look for kernel";
              // cout<<"rect="<<rect<<", split partner rect="<<j->cel->rect<<endl;
              // cout<<"CullP found partner at "<<j->cel->center<<", cell="<<center<<endl; // PBV3
              cout<<", found sep="<<key<<", look for kernel"<<endl;
           }
#endif  //  
              iterMap = rect->RectKer->find(key);
              if (iterMap==rect->RectKer->end())  //key was not found
				  // odd: rect->RectKer->end() could be just map::end? see PutInArray
                { // you have to calculate it!
				cout<<"Trying to calculate kernel for key="<<key<<": NOT IMPLEMENTED!"<<endl;
                                MYEXIT(21);
                        /*
				Kernel Ker1=CalcKernel(&*j); // calc effect of source j on "this"
				  // bizarre syntax &*j, but seems to work!!
				(*rect->RectKer)[key] = Ker1; // insert it	
					//cout<<"ERROR: Kernel for separation "<<doubleKey<<
					//" was not read in. Have you specified the correct *.ker"<<
					//  " file in script file?"<<endl; MYEXIT(22);
			*/
                }
              // here, have either found or computed kernel.
#ifdef DEBUG2
               if(Watch_j&&PD2) cout<<"Got kernel with key="<<key<<", put in NearPartners"<<  
                  ", Kernel was:"<<iterMap->second<<endl;
#endif  //  
               LL->NearPartners.push_back(NearPartner(jLLG, &iterMap->second));
//************ NEED understand this! ***************

            j=Partners.erase(j);  //erases the object and sets j to the next one
        } //end of else for smallest cell
      }  //end of else if already small
    } // end if Near(j)
    else //it is far
	  j++;// if far, leave source j in Partners list
  } // end while
#ifdef DEBUG2
   if(Watch&&PD2) cout<<"End loop partners of cell "<<*this<<", call CullPartners for kids if any"<<endl;
#endif  //  
  if (child0) {
		//#pragma omp flush ( My )
		if (My->num_threads>=omp_get_num_procs()) {
			child0->CullPartners();
			child1->CullPartners(); // recurse serially if enough threads
		} else { // recurse in parallel, creating threads as needed
		//	#pragma omp atomic // uninteruptable, prevent sharing issues
			My->num_threads++;
		//	#pragma omp flush( My )

			//#pragma omp parallel
		//	#pragma omp sections nowait
			{
			//	#pragma omp section
				child0->CullPartners();
		//		#pragma omp section
				child1->CullPartners();
			} // end parallel sections
		//	#pragma omp atomic
			My->num_threads--;
		} // end else make more threads
	 } // end if child0
} // end CullPartners()


//added by DMAf
void cell::updateStat()
{  // assumes SmoothField was set=0 in UpdateMoments
	// NO: and root->Energy has "correction" on entry,see CalcAllFromM.

#ifdef DEBUG1
	 // PD1=true; // turns on printing when updateStat starts; comment this out.
    stringstream LocalOut; // Write to stream on this core, to avoid interruption
    bool Watch=(this==My->WatchedCell);
    Watch=true; // debug
    if(Watch&&PD1) LocalOut<<"LocalOut:Begin updateStat for cell@"<<this<<", "<<*this<<endl;
    // comment this out if omp: cout<<"Start updateStat for "<<*this<<endl;
#endif  //  
	// Because this function is called recursively on different cores, we need to 
	// define LOCAL scratch arrays that won't be written by other cores.
	// Maybe there's a way to pass them down recursively and create a new one only when
	// a new thread is started, but working that out may not be worthwhile.
	// Though it uses a lot of storage (16 cores*22 levels * 2 mpoles * 50 reals * 4 bytes ~ 150kB)
	// it's negligible on the scale of GB.
	short OrderD = My->MyFMM->UsedOrder+My->MyFMM->ExtraOrderD;
	mpole* scratchMpole1 = new mpole(OrderD); 
	  // only needed for Deriv
	mpole* scratchMpole2 = new mpole(OrderD);
	scratchMpole2->order = My->MyFMM->UsedOrder; // don't need it all yet
    if (parent) {
        //parent is present, need to initialize SmoothField to that of parent
        // with proper shift
        scratchMpole2->SetToPowers(parent->center-center);
		// parent-center, not center-parent, because SmoothField is W(r)=V(-r)
        //to translate the origin of SmoothField from parent, need to be mpole
        SmoothField->AddDownConv(parent->SmoothField, scratchMpole2,0); 
          // 0=ExtraOrder, need only for AddTaylor
    }
#ifndef H_ONLY  // periodicity is not an option in H_ONLY
    else   //no parent, root cell, we need to take into account exterior potential
    if (My->periodic[0] || My->periodic[1] || My->periodic[2])
    { 
        //SmoothField->invert();  //DMAh: we need to invert only DownConv(Vext, moment)
        SmoothField->AddDownConv (My->MyFMM->Vext, moment,0);
        //SmoothField->invert(); // parity operation, changes sign of odd moments
    }
#endif  //  H_ONLY
	scratchMpole2->order = OrderD; // from now on, used for AddTaylor, for which
	  // we need full orderD
    // Now loop partners:
    for (std::list<DispCell>::iterator iterP=Partners.begin(); iterP!=Partners.end(); iterP++)
    { 
#ifdef DEBUG1
        bool WatchP; // this code repeated below in other loop over partners
        if(My->WatLLGPart) WatchP = Watch && (iterP->cel==My->WatLLGPart); 
          else WatchP=Watch; // use all partners
        WatchP = true; // debug

        if (WatchP&&PD1) LocalOut<<"Partner at "<<*iterP->cel<<endl;
        MVector VT=center - iterP->absCenter(); 
        if (VT.Square()<0.001) if(PD1) cout<<"Error, cell "<<this<<
            " at partner "<<iterP->absCenter()<<endl;
#endif  // DEBUG1

#ifdef DBG_ // debug Green fn; this section ONLY for efficiency, shd change nothing:
   // use cout, turn off omp.
   //cout<<"Test whether ancestor: "<<*(iterP->cel)<<"mom="<<*(iterP->cel->moment)<<endl;
   if(My->DBG) // compiler directive could be on, but haven't got command Green yet
   {
     if(!iterP->cel->AncestorOf(My->WatLLGPart)) // if not ancestor of source, check moment=0
	 { //cout<<"Partner "<<*(iterP->cel)<<" not ancestor of source "<<
		    //*My->WatLLGPart<<", don't include"<<endl;
	   if(iterP->cel->moment->IsNonZero()) {cout<<"Non-ancestor moment nonzero!"<<
         *iterP->cel<<endl; iterP->cel->moment->printMP("anc mom:",cout,10); 
         MYEXIT(81);}
	   goto SkipSmoothField; // don't bother adding its field
	 } 
	 else // iterP = ancestor of source -- add to SmoothField
     {   
       //Delete: My->DBG->PlotFields(iterP->cel,this); // pass anc of source & field cells
  	   if (AncestorOf(My->WatchedLLGCell)) //-- both are ancestors
	   { My->DBG->FieldAnc = this;
		  //My->PD4=true; // turn on printing
		  My->DBG->SourceAnc = iterP->cel; // remember, for PlotFields()
        cout<<"Found ancestor "<<*iterP->cel<<" of source in Partner list "<<
          "of ancestor "<<*this<<" of field cell"<<endl;
	     iterP->cel->moment->printMP("partner mom",cout,15);
	   }
	   // SmoothField->AddTaylor(iterP->cel->moment, center-iterP->absCenter(),scratchMpole1,scratchMpole2);
     }  // end else ancestor
   } // end if DBG
#endif // !DBG_
   SmoothField->AddTaylor(iterP->cel->moment, center-iterP->absCenter(),scratchMpole1,scratchMpole2);
SkipSmoothField: // jump to here only if DBG_
   if(My->FirstUpdate) My->FirstUpdate=true; // dummy line
    } // end Partner loop

  if (!child0)
  //smallest cell, base case for recursion, need to add field from NearPartners
  { assert(typeid(*this)==typeid(LLGcell));
    LLGcell* LL=static_cast<LLGcell*>(this);
    LL->H_quad = averageMpole(SmoothField); // was H_av
	    // FALSE 7/2011: already includes uniform part, bias+pulse
        //we do not need Taylor expansion for smallest cells
#ifdef DEBUG1
    if(PD1) LocalOut<<"Smallest cell LL@"<<LL<<", H_quad = "<<LL->H_quad<<
        ", adding the field from NearPartners"<<endl;
#endif  //  
    std::list<NearPartner>::iterator iterN;
    for (iterN=LL->NearPartners.begin(); iterN!=LL->NearPartners.end(); iterN++)
 	{LL->AddNearField(&*iterN); // adds near field to H_quad (&* converts iterator to pointer)
#ifdef DEBUG1
        bool WatchP; // this code repeated above in other loop over partners
        if(My->WatLLGPart) WatchP = Watch && (iterN->cel==My->WatLLGPart); 
          else WatchP=Watch; // use all partners
        WatchP = true; // debug
        if (WatchP&&PD1) LocalOut<<"LL@"<<LL<<", NearPartner@"<<iterN->cel<<" "<<*iterN->cel<<
          ", M="<<iterN->cel->M<<", now H_quad="<<LL->H_quad<<endl;
#endif  //  
	// for face charge code:cout<<iterN->f->faceCenter<<endl;
	} // end NearPartner loop

#ifdef DEBUG1
        if(Watch&&PD1) LocalOut<<"Done near partners of cell@"<<LL<<", "<<*LL<<
          ", final H_quad="<<LL->H_quad<<", watched H="<<My->WatchedLLGCell->H_quad<<endl;
        //if((My->WatchedLLGCell->H_quad[1]) < (-350.0)) LocalOut<<"####WRONG####"<<endl; // debug
#endif  //  
		// Now we have magnetostatic field H_quad

#ifdef DBG_ // debug Green's function
  if(My->DBG) My->DBG->CheckField(LL->H_quad,this);
#endif // DBG_

#ifndef H_ONLY
	LL->H_quad += LL->H_anis()+LL->H_exch(); // now is entire quadratic (in M) field
	LL->H_lin = My->H_ext; // now calc part linear in M, H_ext is uniform
	// Loop list of Terms (of Modes) in rectangle
	if(My->NModes) // if there are Modes
	{
		My->CalcShapeFCell(LL); // gets ShapeF(integral over this cell) for all Modes, puts in ShapeF
		for (short i=1;i<=My->NModes;i++) 
      //if (i==3) if (center[0]==1.5 && center[1]==1.5 && center[2]==1.5) cout<<"Cell 1.5:H_lin="<<H_lin<<endl; 
		LL->H_lin+=My->Modes[i]->ModeField(LL); // should be okay in parallel, 
		 //no other thread will use the same value of LL and only reads from ModeField

		//cout<<"UpdateStat: used HVal=h1="<<My->Modes[1]->HVal<<
		//	" to get "<<"H_lin="<<My->root->H_lin<<endl;
	} //code that was here is cell.cpp.zhupiece

#endif  //   // |H_ONLY

  // update
  } //end if(!child0)
  else //not smallest cell, need to recurse
    {  // pbv inserted two //, and /*..*/
		//#pragma omp flush (My)
		//if((My->num_threads>=omp_get_num_procs())) // can't spawn thread
      {
			child0->updateStat();
			child1->updateStat();
      } 
      /*
      else // spawn thread
      {
			#pragma omp atomic
			My->num_threads++;

			#pragma omp flush( My )

			#pragma omp parallel shared(My)
			{
				#pragma omp sections nowait
				{
					#pragma omp section	
					child0->updateStat();
					#pragma omp section
					child1->updateStat();
				}
			}
			#pragma omp atomic
			My->num_threads--;
		}  // end else spawn thread
      */
  } // end else not smallest cell

  delete scratchMpole1; // local pointer deleted when return, but not array
  delete scratchMpole2; 

#ifdef DEBUG1
  if(PD1)LocalOut<<"Exiting updateStat for cell "<<*this;
  string s = LocalOut.str();
  if(PD1) cout<<s<<" -- writing out now"<<endl<<endl; // write it out
#endif  //  
} // end of updateStat

//fails: // void cell::ListCellsRecursively(ostream &ost)
void cell::ListCellsRecursively()
{  cout<<"Cell @ "<<this<<" is "<<*this<<", ch0="<<child0<<",ch1="<<child1<<endl;
if (child0)
	child0->ListCellsRecursively();
else {
	assert(typeid(this)==typeid(LLGcell));
	cout<<"is smallest, ID="<<((LLGcell*)this)->SmCellID<<endl;
}
if (child1)
	child1->ListCellsRecursively();
}

void cell::SetOrderRec() // fix this if only some cells have moments
{  // need reset to study convergence, moment->order determines accuracy
   moment->order=My->MyFMM->UsedOrder;
   SmoothField->order = My->MyFMM->UsedOrder;
if (child0)	child0->SetOrderRec();
if (child1)	child1->SetOrderRec();

}  // end SetOrderRec

DispCell* DispCell::GetChild0()
{return new DispCell(cel->child0,disp);}
DispCell* DispCell::GetChild1()
{return new DispCell(cel->child1,disp);}


MVector DispCell::absCenter() const // absolute center, adds disp
{
  return My->Displace(cel->center,disp);
}

void DispCell::SetDispFrom(DispCell DC)
{
  disp[0]=DC.disp[0]; disp[1]=DC.disp[1]; disp[2]=DC.disp[2];
  // set disp from another DispCell
}


#ifndef H_ONLY // i.e. below is not needed for magnetostatic field calculation
Kernel cell::CalcKernel(DispCell *source)
{
	cout<<"In CalcKernel for this="<<center<<", source="<<source->cel->center<<endl;
    // set source M to (1,0,0) , (0,1,0) , (0,0,1) , measure H in this cell = kernel
    // guaranteed that this and source cells do not have children

    EXIT1("Not yet implememnted!");
}

extern const string HDispLabel[NDisp+1];
#include "shape.h"
void GenTwoNorVar(real *FirstVariate, real *SecondVariate); // in Random.cpp
  // Try to do this outside any class: don't want in cell constructor, done too often:

inline MVector LLGcell::H_anis() {return MVector(rect->avEasyAxis*(anisCoef*(M*rect->avEasyAxis)));} //anisotropy field

void LLGcell::check_faces()
{ 
  //if(My->Psetup) cout<<"entering check_faces() for cell: "<<this<<endl;
  // debug:
  for(short faceN=0; faceN<6; faceN++)
   {  
	 if(cellfacelist[faceN].myFace) // can be null
     //cellList[0] is in back of face, check that back of face is "this" 
     assert(cellfacelist[faceN].myFace->cellList[1-cellfacelist[faceN].out].cel == this); 
     //check the cell in front
    }
}

MVector LLGcell::H_exch() // changed 7/5/09
{  // previous version UpdateExch recursive, see older archives
  assert(!child0); // should only call for smallest cells
#ifdef DEBUG3
  cout<<" looping over faces to get Exch field "<<endl;
#endif  //  DEBUG3
    MVector H_ex; H_ex.SetToZero();

    for (short faceN=0; faceN<6; faceN++) 
      { //loop faces to get exchange field
       //need to find neighbor cell on the other side of this face
       //recall cellfacelist[faceN].myFace is an undirected face
		 if (!cellfacelist[faceN].myFace) continue;
         cell *nbr;
         nbr = cellfacelist[faceN].myFace->cellList[cellfacelist[faceN].out].cel;
		 assert(typeid(*nbr)==typeid(LLGcell)); // DispCell.cel returns cell, not LLGcell
         LLGcell* LL=static_cast<LLGcell*>(nbr);
         if(nbr)
           H_ex += (LL->M-M)*(cellfacelist[faceN].myFace->Jex);
           //contribute to exchange field, Jex is exchange integral
		   // -M has no effect on dynamics, but keeps energy reasonable
      } //end for
	return H_ex;
} // end H_exch


// gone: MVector cell::RandomField()
// void cell::update() is GONE now. DOne in loop in evolve

/*
void cell::IncAvgs() // needed for both small and large cells; Level no longer exists!!!
{if (My->Avgs[Level])
   {
     real *v = Average::val; //rename for conciseness
     v[0] = (((-M).azimuthal() - M_PI)*180)/M_PI;
     v[1] = M.Magnitude();// /My->Ms;
     MVector MxH = M^H_av;
     MVector MxMxH = M^MxH;
     v[2] = H_av[0];
     v[3] = H_av[1];
     v[4] = H_av[2];
     v[5] = M[0];
     v[6] = M[1];
     v[7] = M[2];
     // to print out single points(usually use), use the following one line of code.
     My->Avgs[Level]->increment();
    }
} // end function IncAvgs
*/

#ifdef GRAPHICS
void cell::CreateHDisp()
{  // recursively creates displays for all [smallest] cells [had: having M displays]
	// now M display is irrelevant, can't display coarse-grained exch. field easily.
	// for now, ALWAYS computational cells, or don't know some of fields!

  if (child0) // not show H, too big, descend recursively
	
  {  
    child0->CreateHDisp(); // recurse
    child1->CreateHDisp();
  }
  else // smallest cell, create several local color arrows
  {   assert(!DispInfoPtr); //not created yet
	  DispInfoPtr=new DispInfo;
	  DispInfoPtr->HDisp=new VectDisp*[NumLocalColor];
   for (short i=0;i<NumLocalColor;i++)
   {
    //cout<<"in CreateHDisp, cell center is "<<center<<endl;
	DispInfoPtr->HDisp[i] = new VectDisp(center,LocalColorR[i],
		LocalColorG[i],LocalColorB[i]);
    // created scaled-down VectorDisplay without label or scale!

    // I have NO idea why this was here: myMTransform->rotation.setValue(1,0,0,0);
                     // dummies, shd change before display?
    //real lengthScale = GlobalMScale*curCell->M.Magnitude(); // in nm
    //curCell->myMTransform->scaleFactor.setValue(lengthScale, lengthScale, lengthScale);
	//this is darned odd!
    // My->rootReal->addChild(HDisp->getNode()); // why do I need ->getNode here???
  } // end for
 }// end else
} // end CreateHDisp()

void LLGcell::InsertHDisp() // put correct field in HDisp
{
	for (short i=0;i<NumLocalColor;i++)
	{  MVector H_; real mag;
		
	switch(HDisp_Which[i])
	{ // these need to agree with labels in Systm::Systm() line~60
	case 0: // nothing; 
        H_.SetToZero();break;  // MYEXIT(19)
	case 1: // linear part (external+modes)
		H_=H_lin; break;
	case 2: // recover magnetostatic field:
		H_=H_quad-H_anis()-H_exch(); break;
	case 3: 
		H_=H_exch(); break;
	case 4:
		H_=H_anis(); break;
	case 5: // total
		H_=H_av(); break; // defined as H_lin+H_quad
	case 6: // random
		H_=H_rand; break;
	case 7: // DelM!
		H_=DelM; break;
	case 8: //  is redundant with mode displays
		// was spin torque, effectively m_p in mxmxm_p
		H_=center; // radial arrows, for fun
		//cout<<"There is no case HDisp=8!"<<endl; // prints for EVERY cell!
		// leave it here for now, re-purpose '8' when add new display
        break;

//Addition by Kam 2013 
	/*case 9: 
		//Display configuration at energy minimum
		if(My->My_norm){
		H_=My->My_norm->Meq.ConfigArr[SmCellID];
		} else{
			 H_.SetToZero();
		}
			 break;
			 */
	case 9: ////Display perturbation at each time step for normal modes
		if(My->My_norm){
		H_=My->My_norm->Pt.ConfigArr[SmCellID]; 
		} else{
			 H_.SetToZero();
		}
		// end addition by Kam 2013
			 break;

	default:
		cout<<"Error in HDisp_Which["<<i<<"] ="<<cell::HDisp_Which[i]<<endl; MYEXIT(29);
	}
  mag = H_.Magnitude(); if(mag>H_max[i]) H_max[i]=mag;
  assert(DispInfoPtr->HDisp[i]); // this warning failed when HDisp[i] wasn't init. to NULL
  DispInfoPtr->HDisp[i]->updateVector(H_*H_scale[i]);
	} // end for i
} // end InsertHDisp()

// gone: void cell::SetDisplayRecurs()

#endif  //   // GRAPHICS

// gone: MVector cell::RandomField()
// void cell::update() is GONE now. DOne in loop in evolve

/*
void cell::IncAvgs() // needed for both small and large cells; Level no longer exists!!!
{if (My->Avgs[Level])
   {
     real *v = Average::val; //rename for conciseness
     v[0] = (((-M).azimuthal() - M_PI)*180)/M_PI;
     v[1] = M.Magnitude();// /My->Ms;
     MVector MxH = M^H_av;
     MVector MxMxH = M^MxH;
     v[2] = H_av[0];
     v[3] = H_av[1];
     v[4] = H_av[2];
     v[5] = M[0];
     v[6] = M[1];
     v[7] = M[2];
     // to print out single points(usually use), use the following one line of code.
     My->Avgs[Level]->increment();
    }
} // end function IncAvgs
*/

//added by DMA
MVector LLGcell::RandomFieldForM()
{ // find vectors N_1, N_2 both perp. to M
  MVector N_1, N_2, HR;

/*MUCH better, but not changed yet: (disadvantage is, this hard-wires quaternions and
  makes it harder to improve efficiency by removing them entirely)
real hx, hy;
GenTwoNorVar (&hx, &hy); // each has variance 1
MVector HR = ((q * MVector(hx,hy,0)) * (~q)).getMVector();
// in frame where H is along z, create random field in xy plane; then rotate to lab frame
HR*=rect->H_rand_scale; // now each component has variance H_rand_scale
assert(  Square((q * M) * (~q)).getMVector() - M)<0.0001*rect->Ms); // debug check M and q match
assert(Square(M^HR)<0.0001*H_rand_scale*rect->Ms); // debug check HR perp. to M
return HR; 
*/

  if (fabs(M[1]) < 0.9*(rect->Ms)) { // M not along y, cross prod. not zero
    MVector temp (0,1,0);
    N_1=(M^temp).normalize();
    N_2=(M^N_1).normalize();
  }
  else {
    MVector temp (0,0,1);
    N_1=(M^temp).normalize();
    N_2=(M^N_1).normalize();
  }

  // assert (N_1*N_2 < 0.05);
  real hx, hy;
  GenTwoNorVar (&hx, &hy);
  HR = N_1*hx + N_2*hy;
  HR *= (rect->H_rand_scale);
  return HR;
}
void LLGcell::CalcDelM() // assumes M, H_rand, H_av (if FMM) are set
{
   /*if(!My->Repeat) //DEB
  {                   //for debug
    cout<<"\ninside CalcDelM(), smallest cell, cell is: "<<this<<
		", center = "<<center<<endl;
    q.print("in CalcDelM(), original q ");
    M.print("in CalcDelM(), original M ");// was M.print(outlog,"..
  }
  */
  // H_rand is treated separately because it does NOT change in 2nd Heun pass
  MVector HxM;
   // cout<<"CalcDelM: H_lin="<<H_lin<<endl;
  // note H_av()=cell::H_lin+H_quad, has no random field (but has mode field,from UpdateStat)
  if (My->Noise) HxM = (H_av()+H_rand)^M; else HxM = H_av()^M; // H_lin,quad were calc by UpdateStat
  //cout<<"H_av="<<H_av()<<", M="<<M<<", HxM="<<HxM<<endl;
  //if(!My->Repeat) cout<<"DEBUG: noise="<<My->Noise<<", H_rand_scale="<<rect->H_rand_scale<<
	  //", H_rand="<<H_rand<<", HxM="<<HxM<<endl; // needed all these couts
      // because release version failed, debug version seemed fine!
  // MVector DelM0 = (H_av+H_rand)^M*My->gdt; // Delta-M without damping
#ifdef LINEARIZE 
  // need fix this! replace DelM0 by HxM, without gdt, fix aJ:
  if(rect->SpinTorque) {
	  DelM0[0]-=(M[1]*rect->Mp[2]-rect->Ms*rect->Mp[1])*rect->bJgdt+ // bJ is field-like term,
		  (M[0]*rect->Mp[2]-rect->Ms*rect->Mp[0])*rect->Ms*rect->aJgdtOverMs; // aJ=Sloncz.
	  DelM0[1]-=(-M[0]*rect->Mp[2]+rect->Ms*rect->Mp[0])*rect->bJgdt+ // bJ is field-like term,
		  (M[1]*rect->Mp[2]-rect->Ms*rect->Mp[1])*rect->Ms*rect->aJgdtOverMs; // aJ=Sloncz.
      }
  //cout<<"DelM0="<<DelM0<<", M="<<M<<endl;
  DelM[0] = (DelM0[0]-DelM0[1]*rect->alpha) * rect->alphaSq1; // solve LLG eqn for DelM
  DelM[1] = (DelM0[1]+DelM0[0]*rect->alpha) * rect->alphaSq1; // solve LLG eqn for DelM
  DelM[2]=0.; // Mz alwys Ms
#else // %LINEARIZE // %%% changed from %LINEARIZE %%% changed from // don't linearize:
   
  /*  DELETE: this is now done in UpdateStat like other modes:
  if(My->STMode) { 
	  My->CalcShapeFCell(this); // gets ShapeF(integral over this cell) for all Modes, puts in ShapeF
	    // inefficient, did already when calc H_lin, but forgot it. 
	    // Use ShapeF for pinned magnetization for spin torque.
      Mode *STM=My->Modes[My->STMode]; 
	  MVector MxMp=M^STM->ShapeF; // bJ is field-like term, aJ=Sloncz:
	  HxM-=MxMp*My->bJ+(M^MxMp)*My->aJ/rect->Ms; // sort of "effective" HxM
      }
*/
   // Or gamma*alpha/Ms/(1+alpha^2)=1!
	// as alpha->infinity, gamma*Ms/alpha=1, so dt is fractional change
   // if damping infinite, dt is not really time
		  
   // if(!My->Repeat) cout<<"DEB: got DelM="<<DelM<<", HxM="<<HxM<<", gadtM="<<rect->gadtM<<endl;
   // Write LLG eqn as (1+Mx )dM/dt=-gamma*MxH; invert (1+Mx) operator to get 
   DelM = M^HxM*rect->gadtM; // gadtM = gamma*alpha*dt/Ms(1+alpha^2) if DampingFInite, 
     // or gadtM=dt/Ms^2 if infinite
   // if(DelM.Magnitude()>100)
     // cout<<"DelM large="<D<DelM<<" at "<<center<<endl;
   
   if (My->DampingFinite) DelM+= HxM*rect->gdt; // precession coeff gdt=gamma*dt/(1+alpha^2)
	  // If damping infinite, gadtM must be finite so gamma->infinity too,
	  // so gdt->0 and we don't need precession term.
      // In that case redefine dt so it's fractional change, gadtM~dt/Ms^2
	// if combine both terms, parens around ^ are NECESSARY!  Has lowest precedence.
     //if(this==My->WatchedLLGCell) cout<<"Watched DelM="<<DelM<<endl;
      if(My->cons) // constraining variables, need to know d??/dh
   {  My->CalcShapeFCell(this); // get F for all modes
	  real Ms2=rect->Ms*rect->Ms;
	  real SMSM;
	  short im,jm; 
	  Mode *IM,*JM;
	  MVector *Si, *Sj;
	  real MSi; 
	  for(im=0;im<My->cons->NC;im++) 
	  {  IM=My->Modes[My->cons->i_[im]];
	     Si=&IM->ShapeF;
		 MSi=M*(*Si); // M dot ShapeF for mode i
		 My->cons->Dm[im]+=DelM**Si; // update moment-change Dm with DelM dot ShapeF

		 for (jm=0;jm<My->cons->NC;jm++) 
		 {  

		    JM=My->Modes[My->cons->i_[jm]];
			Sj = &JM->ShapeF;
		    // pre-calculate SMSM = Si dot (Mx(SixM)), needed almost always
		    SMSM=((*Si*(*Sj))*Ms2) - (MSi*(*Sj*M));// BAC-CAB

			real Temp;
 		    if(JM->NonField) 
             {  
				Temp =((*Si)*(*Sj^M))*(-rect->gadtM*rect->Ms);
                if(My->DampingFinite) Temp+=rect->gdt*SMSM/rect->Ms;
             }
             else // field-like, g and ga reversed except for factors Ms
			{  Temp=rect->gadtM*SMSM;
			   if(My->DampingFinite) Temp+=((*Si)*(*Sj^M))*rect->gdt;	
	   	    }
		    My->cons->dDmdh[im][jm]+=Temp/volume();	 

		 } // end for jm
	  } // end for im
   }  // end if(cons)
#endif  // !LINEARIZE 
   //if(!My->Repeat) cout<<"DEB: Leaving CalcDelM with DelM="<<DelM<<", gdt="<<rect->gdt<<endl;
} //end of CalcDelM

void LLGcell::RotateMby(MVector Angle) // rotates M and q, once was recursive
{   Quaternion qlocal; qlocal.qRot(Angle); // rotates by angle
	cout<<"In RotateMby, qlocal="<<qlocal<<", q = "<<q<<", M="<<M<<endl;
    q = qlocal*q;// mult on left by qlocal, so in viewed frame
	cout<<"In RotateMby, new q="<<q<<endl;
    M = ((q * (rect->M0)) * (~q)).getMVector(); 
	cout<<"In RotateMby, new M="<<M<<endl;
} // end function

//void cell::getlongaxis(short& diameterInt, short& longaxis)
short cell::LongAxis() // returns direction of long axis
{  real max=half[0]; short LA=0;
   for (short i=1;i<3;i++) if(half[i]>max) {max=half[i]; LA=i;}
   return LA;
}

// This supersedes WhichChild() .
cell** cell::PtrToPtrToThis() // if parent->child0 points to this, returns address of parent->child0
{  // otherwise returns &parent->child1
   if (parent){
	   if (parent->child0==this) return &(parent->child0);
	   assert (parent->child1==this);
	   return &(parent->child1);
   } else {
	   // can't go to root's parent to get a pointer to it
	   // also can't go to My->root (not yet intialized).
	   // rect->WholeRect should work.
	   assert(rect->WholeRect==this);
	   return &(rect->WholeRect);
   }
}

void cell::PassTo(cell* child)
{  child->parent=this;
 //we need to pass the pointer to Rectang class down the hierarchy
   child->rect = rect;
   child->half = half;
   child->center = center;
}

void cell::split(short lgaxis, short Len0, short Len1)
{ // creates child cells; check whether you need to split BEFORE calling split()
   //cout<<"entering cell::split(), cell being split is: "<<this<<", radius2="<<radius2<<endl;
  
   bool MaybeLLG=true; // means is LLG size in other 2 directions
   for (short i=0;i<3;i++) 
	   if(i!=lgaxis) 
		   if(!My->almost_same(half[i],rect->LLGSize[i])) MaybeLLG=false;
   //create two child cells of proper type
   if (MaybeLLG && (Len0==1)) child0=new LLGcell(); // new cell is really LLG size
   else child0=new cell();
   assert(child0);
   //if(My->Psetup) cout<<"child0 pointer is: "<<child0<<endl;

   if (MaybeLLG && (Len1==1)) child1=new LLGcell(); 
   else child1=new cell();
   assert(child1);
   //if(My->Psetup) cout<<"child1 pointer is: "<<child1<<endl;
   PassTo(child0); PassTo(child1); // sets parent, rect, half, center
   
   child0->half[lgaxis] = Len0*rect->LLGSize[lgaxis];
   child1->half[lgaxis] -= child0->half[lgaxis];
   //assert( My->almost_same(child1->half[lgaxis],Len1*rect->LLGSize[lgaxis]) );

   child0->center[lgaxis] -= (half[lgaxis] - child0->half[lgaxis]);
   child1->center[lgaxis] += (half[lgaxis] - child1->half[lgaxis]);
   //if (My->Psetup) cout<<"child0->half = "<<child0->half
   //  <<"child1->half = "<<child1->half<<endl;

   child0->SetRadius2(); // can't be done til half known
   child1->SetRadius2();
} //  end split()

bool cell::splitRecursively()
{ 
  //if(My->Psetup)cout<<"Entering splitRecursively for cell: "<<this<<endl;
  //Figure out length of child0 (Len0) in units of LLGsize:
   short lengthInt, dir, Len0; 
   dir = LongAxis();
   Len0 = 1;
   lengthInt = floor((half[dir] / rect->LLGSize[dir]) + 0.5); // integer length of longest direction / LLGSize
   while (2*Len0 < lengthInt) Len0*=2;
   split(dir, Len0, lengthInt-Len0); // creates 2 children
   if (!rect->shape) { child0->splitRecursively(); 
      child1->splitRecursively(); return true;}
   if(rect->shape->outside(child0) || !child0->splitRecursively()) 
		// if clearly outside shape, OR turns up empty when we split it
	{  delete child0;
		if(rect->shape->outside(child1) || !child1->splitRecursively()) 
			// hopefully, this calls splitRecursively only if outside is false!
		{  // child1 empty too
			delete child1;
			return false; // means empty
		}
		else // child1 not empty, but 0 is: promote child1
		{  *PtrToPtrToThis()=child1;
		child1->parent=this->parent;
		delete this; return true;
		}  // deleting this is safe, if just before returning?
	} 
	else // child0 not empty; check child1
	{   if(rect->shape->outside(child1) || !child1->splitRecursively()) 
		// child1 is empty; promote child0
		{  delete child1;
			*PtrToPtrToThis()=child0;
			child0->parent=this->parent;
			delete this; return true;
		}  
		else // child1 is also not empty; usual case
		{ return true; 
		}
	}// end else child0 not empty
}

bool LLGcell::splitRecursively()
    {
		SmCellID=NextSmCellID; NextSmCellID++;  
		if(NextSmCellID==MaxCellID) {cout<<"Too many small cells! NextSmCellID="<<
			NextSmCellID<<", MaxCellID="<<MaxCellID<<endl; MYEXIT(83);}
		return true;
		// refined division here?
} // end smallest cell

void cell::initIDRecursively()
{ // note you can't set this when cell created, don't know size of CellFromID.
  // if(My->Psetup) cout<<"entering initIDRecursively for cell: "<<this<<endl;
	#pragma omp flush (My) // make sure this thread's My->num_threads is up to date and not an old copy in a register etc.
	if (My->num_threads>=omp_get_num_procs()) { // we already have plenty of threads, go deeper serially
		child0->initIDRecursively();
		child1->initIDRecursively();
	} else {
		#pragma omp parallel
		{
			#pragma omp atomic // make next instruction uninteruptable, ensure no sharing issues
			My->num_threads++; 
			#pragma omp flush (My) // make sure other threads have an accurate count
			#pragma omp sections nowait
			{
				#pragma omp section
				child0->initIDRecursively();
				#pragma omp section
				child1->initIDRecursively();
				 // cout<<"in initIDRecursively, not smallest cell, called itself recursively"<<endl;
			}
			#pragma omp atomic
			My->num_threads--;
		}
	}
}
void LLGcell::initIDRecursively() {
	   
	   My->CellFromID[SmCellID]=this; // store pointer in array
     assert(My->almost_same(rect->LLGSize,half)); // should be LLG size
  } // end of initIDRecursively
/*
void cell::initMRecursively() // not clear needs to be recursive
{  // called only within each rectangle.  Uses rect->M0
  //cout<<"initMRec. cell "<<*this<<", ctr="<<center<<endl;
    assert(child0); assert(child1);
	#pragma omp flush (My) // make sure this thread's My->num_threads is up to date and not an old copy in a register etc.
	if (My->num_threads>=omp_get_num_procs()) { // we already have plenty of threads, go deeper serially
		child0->initMRecursively();
		child1->initMRecursively();

	} else {
		#pragma omp parallel
		{
			#pragma omp atomic // make next instruction uninteruptable and done by only one thread
			My->num_threads++; 
			#pragma omp flush (My) // make sure other threads have an accurate count
			#pragma omp sections nowait
			{
				#pragma omp section
				child0->initMRecursively();
				#pragma omp section
				child1->initMRecursively();
			}
			#pragma omp atomic
			My->num_threads--;
		}
	}
	 //if(My->Psetup) cout<<"in initMRecursively, not smallest cell, called itself recursively"<<endl;
}*/

void LLGcell::CreateFaces()
{
  for (short faceN=0; faceN<6; faceN++) //loop faces
  {  // assert(cellfacelist[faceN]);
     MVector JustOutside=center; // see if there's a cell next to it
	 JustOutside[faceN>>1]+=(2*(faceN&1)-1)*1.05*half[faceN>>1];  // faceN>>1 is the normal dir to the face
     LLGcell* Nbr=My->root->SubCellContaining(JustOutside); // could be in a different rect with a different LLGsize, find smallest cell for now
	 if(Nbr==this)
		 EXIT1("Problem in CreateFaces... neighboring cell and this one are the same?");
	 if(Nbr) // need a face between them
	 {  face* F1 = new face;
	    short FaceToRt=faceN&1; // 1 if face to right of this,so CellList[1]=Nbr
		F1->cellList[FaceToRt]=DispCell(Nbr,int3(0,0,0)); 
		F1->cellList[FaceToRt^1]=DispCell(this,int3(0,0,0));  // b^1 is negation of bit b
	    cellfacelist[faceN].myFace=F1;
		cellfacelist[faceN].disp.SetToZero(); // fix for periodic system! 
		cellfacelist[faceN].owned=false; // Nbr was there first 
	    cellfacelist[faceN].out=faceN&1; // true if face on right
		short fNbr=faceN^1; // opposite direction
	    Nbr->cellfacelist[fNbr].myFace=F1;
		cellfacelist[fNbr].disp.SetToZero(); // fix for periodic system! 
		cellfacelist[fNbr].owned=true;  
	    cellfacelist[fNbr].out=(bool)(faceN&1)^1; // true if face on left
	 }  // if no cell next door, no face needed yet
  } // end face loop
} // end CreateFaces

void LLGcell::initM() {

  {   
     /*if(My->Psetup)rect->M0
      {
        cout<<"center is: "<<center<<endl;
        cout<<"rect->kVector is: "<<rect->kVector<<endl;
        cout<<"rect->kVector*center is: "<<rect->kVector*center<<endl;
        cout<<"rect->SpinPh is: "<<rect->SpinPh<<endl;
        cout<<"rect->kVector*center + rect->SpinPh is: "
            <<rect->kVector*center + rect->SpinPh<<endl;
        cout<<"smallest cell, init q, SmCellID is: "<<SmCellID<<endl;
        }
*/
      Quaternion qSpin;
	  real phase = rect->kVector*center + rect->SpinPh;
      qSpin.qRot(MVector(-(rect->SpinAy)*sin(phase),rect->SpinAx*cos(phase), 0));
      q = (rect->q_init)*qSpin;
      //if(My->Psetup) q.print("q is"); // or does this need to be real file, My->outlog?
            //this rotates magnetization away from standard position 001
            //by angle SpinA toward x direction,
            //then precess by k*r(dot product) + phase
      M = ((q * (rect->M0)) * (~q)).getMVector();
#ifdef LINEARIZE
	  M[2]=rect->Ms;
#endif  //  
      //cout<<"rect->M0="<<rect->M0<<", new M is:"<<M<<endl;
	  // Set cell::anisCoef
	  real x=center[0]-rect->WholeRect->LowerLeft()[0]; // assume gradient along x
	  //cout<<"Rect.LL="<<rect->WholeRect->LowerLeft()<<", x="<<x<<endl;
	  // if (rect->AnisPower) anisCoef= rect->AnisPower->value(x);
	  // average over cell:
	  if (rect->AnisPower) anisCoef=(rect->AnisPower->indefInt(x+half[0])-
		                            rect->AnisPower->indefInt(x-half[0]))/half[0]/2;
	  else if (rect->AnisCubic) anisCoef=(rect->AnisCubic->indefInt(x+half[0])-
		                            rect->AnisCubic->indefInt(x-half[0]))/half[0]/2;
	  else if (rect->AL) anisCoef=rect->AL->AnisValue(center[0]);
	  else {anisCoef=0.0;} // no other possibilities, now

	  // rect->LLGSize = half;
	  //if(SmCellID<5) cout<<"Set anisCoef=HK/Ms="<<anisCoef<<" @ "<<center<<endl;
   }
} // end of initMRecursively


void LLGcell::initJex() // also sets display flags
{
  //cout<<"entering initJex for cell: "<<this<<"size/2 is(smallest): "<<half<<endl;
  //sane();
   for(short faceN=0; faceN<6; faceN++)
   { // note 2012: this does each face TWICE! But Jex may be different, need twice.
     //if(My->Psetup) cout<<"setting Jex for face "<<cellfacelist[faceN].myFace->faceCenter<<endl;
     face* fac=cellfacelist[faceN].myFace; // to avoid typing
	 if (!(fac)) continue;
     if(fac->cellList[0].cel && fac->cellList[1].cel) //THIS TEST IS WRONG 2ND TIME THRU!!! .cel changed.
     { // if cells on both sides of face exist (& are in same rect)
       MVector center0 = My->Displace(fac->cellList[0].cel->center, 
                                      fac->cellList[0].disp);
       MVector center1 = My->Displace(fac->cellList[1].cel->center, 
                                      fac->cellList[1].disp);
	   if(My->Layered) for(short j=0;j<3;j++) if(j!=My->Layer_normal) if(j!=My->Layer_offset)
		   if(!My->almost_same(center0[j],center1[j])) 
		   {cout<<"Error: you said Layered with normal~ "<<(char)(My->Layer_normal+'x')<<
		     ", but it's not! Cell centers"<<center0<<center1<<endl; MYEXIT(82);}
       real a = (center0 - center1).Magnitude();
	   if (!a)
		 EXIT1("Error in initJex... attempting to calc interaction when cells are on top of each other?");
       fac->Jex = 2*rect->ExchLen2/(a*a); // added 2@10/08
       // if(My->Psetup) cout<<" a = "<<a<<", Jex = "<<cellfacelist[faceN].myFace->Jex<<endl;
      } // end if both cells exist
     else // one cell missing, am at rectangle boundary: check for cell anyway
     { 
		short nor=faceN>>1; // normal direction, drop last bit  
		bool out=faceN&1; // last bit
		short sign;
		if(out) sign=1; else sign=-1;
	    MVector Nbr=center; // to be shifted
	    Nbr[nor]+=sign*2*half[nor]; // where to look

        real compliance; 
		bool IsListed=rect->IntPartner[faceN]<My->MaxRects;
		//too common: cout<<"Missing nbr of "<<*this<<" at face center"<<fac->faceCenter<<endl;
		if(IsListed) // listed interface in direction faceN
		{ // this gives interface exchange energy tJ, extra compliance
			Rectang* Rect2=My->RectFromID[rect->IntPartner[faceN]];
			assert(Rect2); // shift additionally by gap:
            Nbr[nor]+=Rect2->WholeRect->center[nor]-rect->WholeRect->center[nor]-
				sign*(Rect2->WholeRect->half[nor]+rect->WholeRect->half[nor]);
			compliance = 2.0/rect->tJ[faceN]; // will add compliance within layers below		
#ifdef DEBUG2
			cout<<"Listed Interface between "<<rect->RectName<<
				" and "<<Rect2->RectName<<",interface part of compliance="<<compliance<<endl;
#endif  //  DEBUG2
		} // end if interface
		else compliance = 0.0; // no interface contribution; seamless exchange coupling
		LLGcell* NbrCell=My->root->SubCellContaining(Nbr); // seek smallest cell
		if (NbrCell) // set up exchange
		{  	
#ifdef DEBUG2
			cout<<"extra-rectangular nbr of "<<*this<<" at face center"<<FaceCenter(faceN)<<endl;
#endif  //  
			assert(fac->cellList[out].cel==NULL);
			//assert(NbrCell->rect==Rect2);
			real MRatio=NbrCell->rect->Ms/rect->Ms; // M2/M1
			real gap=0.0;
			if (!IsListed) gap=NbrCell->center[nor]-center[nor]-
				sign*(NbrCell->half[nor]+half[nor]);
			if (IsListed || (abs(gap)<0.02*half[nor]) ) // assume coupling if no gap
			{ // then couple the cells:
	            fac->cellList[out].cel=NbrCell;
			    compliance+=half[nor]*MRatio/(rect->ExchLen2)+ // see page 4.316
				  NbrCell->half[nor]/MRatio/(NbrCell->rect->ExchLen2);
#ifdef DEBUG2
				cout<<"Total compliance is "<<compliance<<endl;
#endif  //  
			    fac->Jex=1.0/(half[nor]*compliance);
			// was:	for(short i=0;i<3;i++) if ((i!=nor) && (!My->Layered || (i!=My->Layer_offset) ) )
				// chgd because present code is WRONG if thicknesses different along nor, too!
				// Need to fix this before using different thicknesses.
				// also there's a bug when this is called again by KeyboardEvent::E:
				for(short i=0;i<3;i++) if ( (!My->Layered || (i!=My->Layer_offset) ) )
				{  if(!My->almost_same(half[i],NbrCell->half[i]))  // sanity check
				     cout<<"sizes differ across interface"<<half[i]<<" vs. "<<NbrCell->half[i]<<endl; 
				   if(!My->almost_same(center[i],NbrCell->center[i])) // need account for size!!
					  cout<<"FIX THIS!:centers differ across interface"<<center[i]<<" vs. "<<NbrCell->center[i]<<endl;
				}
			}  // end if(IsListed...)
#ifdef DEBUG2
			else // note break on "else" line FAILS! Need put break on next line
				cout<<"not listed or adjacent, no coupling"<<endl;
#endif  //  
	     } // end if(NbrCell)
		 //else {cout<<"Failed to find neighbor cell across interface"<<endl; MYEXIT(39);}
			// (used it for debugging; happens too often to print routinely)
		 
		 // fac->Jex = 0; shouldn't need this! no cell pointer.
       //if(My->Psetup) cout<<" no neighbor, Jex = "<<cellfacelist[faceN].myFace->Jex<<endl;
      } // end if at bdry of rectangle
      // done setting Jex and pointer fac->cellList[out].cel
	  //cout<<"Setting exchange at "<<center<<", faceN="<<faceN<<", Jex="<<fac->Jex<<endl; // print a couple
      //set the ownership:  //DMA
      if (this==cellfacelist[faceN].myFace->cellList[0].cel)
           //the cell is to the left of the face; in this case it is owned only if the other
           //cell is missing
        cellfacelist[faceN].owned = (!cellfacelist[faceN].myFace->cellList[1].cel);
      else //the cell is to the right of the face; it always owns the face
        cellfacelist[faceN].owned = 1;
   } // end for(face) loop
   //if(My->Psetup)cout<<"returning from initJex() for cell"<<this<<endl;
} // end function initJex

void LLGcell::AddCellEnergy()
{  MVector mom;
   mom=M*volume();
   My->ZeemanEnergy-=H_lin*mom; //all missing mu_0!
   My->IntEnergy-=0.5*(H_quad*mom);
      // need be sure H_lin,quad is calculated before this is called
}

void LLGcell::Mqsane()
{ 
#ifdef LINEARIZE
	assert(My->almost_same(M[2],rect->Ms)); //Shuxia_Pc 
#else // not linearize:
	assert(My->almost_same(M.Magnitude(),rect->Ms)); //Shuxia_Pc 
    assert(My->almost_same(q.norm(),1.0));
    MVector MfromQ = ((q * (rect->M0)) * (~q)).getMVector();
	if ((MfromQ-M).Magnitude()>0.001*rect->Ms)  
	  {cout<<"**** UpdateDiscrep, qM0="
	   <<(q * (rect->M0)) * (~q)<<", M="<<M<<endl; // M,q fit
	   MYEXIT(19);} 
#endif  //  
}

void cell::readSize() // reads the end of the line with display, size
   // and center information 
{ // though in class cell, reads rect info.
      MVector L1,R1;
	  // bool ShowM1;
      string junk;
	  assert(rect);
      My->paramFile>>junk>>rect->RectName>>rect->ShowM>>L1>>R1;
      // was My->paramFile>>junk>>rect->RectName>>ShowM1..;ignored ShowM1!
      half = (R1 - L1)*0.5;
      center = (R1 + L1)*0.5; // this may be overwritten for rectangles
      radius2 = half.Square(); 
} // end readSize()

void cell::buildRect() // assumes center, half, parent, radius2 are set by readSize()
{
   cout<<" enter buildRect() "<<endl;

/*  this is not used!
  //create RectPeriodic[3], it is set only when the system is periodic and the 
  //System contains only this Rectang
  bool RectPeriodic[3]; 
  for (short i=0; i<3; i++)
   {
    RectPeriodic[i]=(((2.0*half[i]) == My->PeriodVec[i]) && My->periodic[i]);
    // cout<<"RectPeriodic["<<i<<"] is: "<<RectPeriodic[i]<<endl;
    }
*/
  cout<<"In buildRect, cell="<<this<<","<<*this<<", radius2="<<radius2<<endl;
  // SetRadius2(); was done in readSize(); calculated for children in SplitRecursively
  if (half<=rect->LLGSize) { // if only one cell in rect, convert to LLGcell
	  LLGcell* l1 = new LLGcell(this);
	  l1->rect->WholeRect = l1;
	  cout<<"Rect has only one LLG cell,@"<<l1<<" = "<<*l1<<", rad2="<<l1->radius2<<endl;
	  l1->splitRecursively();
	  return;
  }

  if (!splitRecursively()) EXIT1("Very odd, rectangle is empty? Is Shape defined with outside true everywhere?"); // wait to initJexRec. until read A, Ms!
 
  cout<<"about to return from buildRect()"<<endl;
}

//function to get the Kernels. Hav is known at every cell
void cell::getKernelsRecursively() // was const char NormDir)
{
  std::map<int3, Kernel>::iterator iterMap;  //iterator for Kernels
  cout<<"Entering undebugged GetKernelsRecursively, fix level*****"<<endl; assert(1==2);
/*
  MVector coarseFaceCenter; //the center of the 2x2 face
  //centerFaces - pointer to small faces, that constitute coarse grain faces

  if (My->Psetup)
    cout<<"\n\nin getKernelsRecursively for center = "<<center
      <<", NormDir = "<<NormDir<<", half = "<<half<<endl;
// **********if (Level-rect->MaxLevel == 3)   //coarse grain cell, need to find Kernel
	    // this has changed enuf you need to debug it!!! 3 above lowest?
  if(My->Psetup) // ************* dummy if()
  {
    for (short i=0; i<4; i++)
      coarseFaceCenter+=My->MyFMM->centerFaces[NormDir][i]->faceCenter*0.25;
    int3 doubleKey;  //key for the Kernel for this cell
    doubleKey = (coarseFaceCenter - center).ToInt(My->quantum);  //double space
    iterMap = My->MyFMM->newKernels.find (doubleKey);

    if (iterMap!=My->MyFMM->newKernels.end()) //key was found
    {
      iterMap->second.Kx = H_av[0];
      iterMap->second.Ky = H_av[1];
      iterMap->second.Kz = H_av[2];
    }
    else //the key was not found
      if (My->Psetup)//not an error, not all keys are used
        cout<<"the doubleKey = "<<doubleKey
          <<" was not found, Kernel was not calculated"<<endl;
  }  //end of if 3 levels above smallest
  else   //cell is larger than coarse grain size (level 2)
  {
    child0->getKernelsRecursively (NormDir);
    child1->getKernelsRecursively (NormDir);
  }
  	*/
} // end GetKernelsRecursively

//set parameters for every rectangle

void cell::setRectangleParamRecurs() //set parameters for every rectangle
{  //needs to be called after dt ,gamma, alpha, are read, or are changed.
	// sets gdt,gadtM, H_rand_scale
	// usually called from SetEvolParams, which sets gdt.
  if (!rect) //this cell is not in a rectangle,i.e. is bigger
  {
    child0->setRectangleParamRecurs();
    child1->setRectangleParamRecurs();
  }
  else //this is a rectangle (never recurse to smaller cells)
  { // assert(rect->WholeRect==this); // sanity check
	if(My->DampingFinite) 
	{
	  rect->gdt=My->gamma*My->dt/(1+rect->alpha*rect->alpha); // renormalized gamma, *dt
	  rect->gadtM=rect->gdt*rect->alpha/rect->Ms; // is gamma*alpha*dt/(1+alpha^2)*Ms
	}
	else // infinite, no need for gdt but set it just in case
	{   rect->gdt=0.0;
		rect->gadtM=My->dt/rect->Ms/rect->Ms; // so dt dimensionless
	}
    if(My->Noise) 
	{
		if(My->DampingFinite) rect->H_rand_scale = 
		  sqrt(2*rect->alpha*My->Temp/(My->gamma*rect->Ms*My->dt*volume()));
		// YIKES! 7/2012 this was WRONG, had 1/gdt*dt which is too many dt's!
		else rect->H_rand_scale = sqrt(2*My->Temp/(My->dt*volume())); // see p. 9.221
	}
	else // no noise, but some mischievous person might turn it on later:
      rect->H_rand_scale=0.0;
	   // note that Temp is really k_B*T/mu_0!
	// Using renormalized gamma, hope that's right? Won't matter, alpha is small
    // if(My->MyFMM) rect->setAvAndMomentCoef();   //set AvCoef and MomentCoef
   // I have NO idea why this was here -- never need to re-create AvCoef!?!:
	//if(My->MyFMM) rect->RectInfoPtr = new CellInfo(rect->LLGSize,My->MyFMM->UsedOrder); //set AvCoef 
  }
} // end setRectangleParamRecurs()
  
face::face()
{
}

cell* Rectang::setup()
{  // reads from script file My->paramFile; just finished R line with size.
  char form[101];
  string junk,Nam; 
  MVector M1;
  char ch; short i; int3 i3;
  //FieldDisplay *FD; // for Modes
  // Term *T1; 
  //MVector Origin; // temp dummy
  // short idum,jdum; real fudge;

  My->paramFile>>ws; // eatwhite, get rid of tabs before reading line
  //cout<<endl<<"In rect->setup, char="<<endl;
  while ((ch = (char)My->paramFile.peek()) != '!')
    // '!' returns to case R of ParseBuild
  {cout<<ch;
   switch(ch)
   {
    case '#': // comment, just ignore
      break;
	case 'S': // Shape mask definintion
		// MUST come before cell division! RectInfoPtr is set on division, test for it.
		if (RectInfoPtr) EXIT1("Shape mask must come before cell division! Move the Shape section before the Divide line in the script file.");
		getline(My->paramFile,junk,'\n'); // 'Shape' line
		this->shape = ReadShape(My->paramFile);
		break;

	case '/': // skip between '/*' and '*/'
	  My->paramFile.get(ch); // this is '/'
	  My->paramFile.get(ch);
	  if(ch=='*') // we have comment '/*' 
	  {
		  My->paramFile.get(ch);
		  char ch2;
		  do 
		  {  My->paramFile.get(ch2); 
		     if((ch=='*')&&(ch2=='/')) break; // break jumps out of if, not just do
		       else ch=ch2; // & read new ch2
	      } while (My->paramFile); // quit if read error
	  } // end if commment
	  else {cout<<"comment? read / but not * in Rectang::setup"<<endl; MYEXIT(69);}
	  break;

	case 'D': // divide rectangle into cells
      My->paramFile>>junk>>i3[0]>>i3[1]>>i3[2]; 
	  My->CheckScript(junk,"Divide"); assert(My->paramFile.good());
      // OLD: My->paramFile>>junk>>LLGSize; 
      // cout<<"After D, read"<<junk<<" & min cell size = "<<LLGSize<<endl;

	  // if(LLGSize[0]<0.0001) LLGSize=WholeRect->half;// <=0 is code not to divide at all
	  for (short i=0;i<VDIMENSION;i++) 
	  {   if(i3[i]<1) EXIT1("Rectangle divisor must be >=1");
		  if(WholeRect->half[i]<=0) {cout<<"Rect. dimension must be >0 along "
			  <<(char)(120+i)<<endl; MYEXIT(6);}
	      LLGSize[i]=WholeRect->half[i]/i3[i];
		  if(LLGSize[i]<=0) {cout<<"Error: Min cell size must be >0"<<endl; MYEXIT(8);}
		  if(My->Layered) if(i!=My->Layer_normal) //if (LLGSize[i]<3*LLGSize[My->Layer_normal])
		  {
			  if(i3[i]>1) 
			  {cout<<"You said layered, but you're dividing transverse dimension by "
			    <<i3[i]<<endl;  MYEXIT(51);
			  }
			  if(WholeRect->half[i]<3*WholeRect->half[My->Layer_normal]) 
			  {cout<<"You said layered, but transverse dimension "
			  <<WholeRect->half[i]<<" small???????"<<endl; // MYEXIT(51);
			  }
		  }
	  }
      //Origin = WholeRect->center - WholeRect->half; // origin was left; now use center
      // cout<<"Origin="<<Origin<<endl;
      WholeRect->buildRect(); // was buildRect(Origin);
/* MOVED THIS to ReadScript:'G', then moved back, 2/6/2012! */
	  if(My->MyFMM) RectInfoPtr = new CellInfo(LLGSize,My->MyFMM->UsedOrder); 
/* but this remains moved, because we need all geometry to know whether Partners read or CubeKer needed:
	  if (LLGSize.Is111()) RectKer=My->MyFMM->CubeKer; // can use same cubic kernels
	  else RectKer = new std::map<int3, Kernel>; // make new kernel list
	    // later might check if other rect has same LLGSize???
*/       
      // cout<<"rect->setup() built rect, size="<<WholeRect->size<<
		//  ", center="<<WholeRect->center<<endl;
	  // gone 7/09: LevelToDisplay=MaxLevel; MToDisplay=MaxLevel; // defaults, can override w/'l','m'
	  break;

	case 's': //size to display
		My->paramFile>>junk>>sizeToDisplay; sizeToDisplay*=0.501; // really half size now; 
		  // fudge so roundoff won't make it seem too small
		//cout<<"Read lengthToDisplay="<<2*sizeToDisplay<<"(000 means smallest),junk="<<junk<<endl;
		break;
	case 'r': // refinement level
		My->paramFile>>junk>>refLevel;
		My->CheckScript(junk,"refine");
		break;
	case 'M': // M_saturation
	  My->paramFile>>junk>>form>>Ms;
  assert ( (junk[0]=='M') && (junk[3]=='a') );  //should read "M_sat.."
  assert ( form[0]=='c');  //should read "constant", others not supported yet
      // cout<<"Ms = "<<Ms<<" kA/m"<<endl;
	     M0 = MVector(.0, .0, Ms); // wrong place?????????but someday might depend on rectangle?
		 if(Ms==0.0) {cout<<"Error: Ms can not be zero, read "<<junk<<form<<", Ms="<<Ms<<endl;
		 MYEXIT(49);}
      break;
	case 'd': // input demag factors by hand
		{ // bracket creates scope so we can use doubleKey?
		real Nx,Ny,Nz;
        My->paramFile>>junk>>Nx>>Ny>>Nz;
		Kernel Ktemp; // for single cell self-interaction, defaults to zero
	    int3 doubleKey(0,0,0);
	    Ktemp.Kxx=-Nx; Ktemp.Kyy=-Ny; Ktemp.Kzz=-Nz; 
	    (*RectKer)[doubleKey]=Ktemp; // insert in map
	    }
	   break;

	case 'K': // Anisotropy parameters
	  My->paramFile>>junk>>form>>avEasyAxis;
	  if (avEasyAxis.Magnitude()>3.0) {cout<<"easy axis strange:"<<avEasyAxis<<" in script file K_anisotropy line"<<endl; MYEXIT(84);}

	  avEasyAxis.normalize();
	  My->CheckScript(junk,"K_anisotropy");
      switch(form[0]) // may have several forms for anisotropy, now powerlaw,cubic,const
	  {
		case 'c': // constant; treat as special case of powerlaw
		real H_K;
		AnisPower = new PowerLaw;
		My->paramFile>>H_K; // rest of line read later
		AnisPower->exponent=0;
		AnisPower->constant=H_K/Ms;
        AnisPower->coef=0; // coef of power law
 		 // WholeRect is cell corresponding to this rectangle
		// cout<<"Read anisotropy, is constant H_K="<<H_K<<endl;
		break;
		
        case 'l': // layered anisotropy
		AL= new AnisLayers; My->CheckScript(form,"layered");
		AL->ReadLines(My->paramFile,Ms); // reads NLayers, layer thick, layer anis
		AL->Write();
		break;

		case 'p': real HK_soft,HK_hard;
		AnisPower = new PowerLaw;
		My->paramFile>>AnisPower->exponent>>HK_soft>>HK_hard;
		AnisPower->constant=HK_soft/Ms;
        AnisPower->SetCoef((HK_hard-HK_soft)/Ms,
			2*WholeRect->half[0]); // diff,length
 		 // WholeRect is cell corresponding to this rectangle
		// cout<<"Read anisotropy, hard="<<HK_hard<<"PowerLaw="<<
		//	AnisPower->constant<<"+"<<AnisPower->coef<<"*x^"<<
		//	AnisPower->exponent<<endl;
		break;

		case '3': //real HK_soft,HK_hard;
		AnisCubic = new Cubic;
		My->paramFile>>AnisCubic->C0>>AnisCubic->C1>>AnisCubic->C2>>AnisCubic->C3;
		AnisCubic->C0/=Ms; AnisCubic->C1/=Ms; AnisCubic->C2/=Ms; AnisCubic->C3/=Ms;
		// cout<<"Read polynomial anisotropy, c0..c3="<<AnisCubic->C0<<", "<<
		//	AnisCubic->C1<<", "<<AnisCubic->C2<<", "<<AnisCubic->C3<<endl;
		break;

		default: cout<<"rect->setup() found error in script file, reading anisotropy, read "<<form<<endl; 
      MYEXIT(11); 
	  }
  break;
 
  	case 'L': // LL damping 
	  My->paramFile>>junk>>form>>alpha; // if gamma=0, this is ignored, alpha infinite
	  // cout<<"in Rectang::setup, read junk="<<junk<<
	  //	  ", form="<<form<<", alpha="<<alpha<<endl;
	  assert (( junk[0]=='L'));  //should read "LL"
      switch(form[0]) // now only constant and infinite forms
	  {
		case 'c': // constant, usual case; do nothing more
			// OLD: damping*=My->gamma; // 'damping' was really alpha
			// if(My->gamma<1e-2) {cout<<"Are you sure you want gamma so small?"<<endl; MYEXIT(12);}
		break;
		default: cout<<endl<<"error in script file, reading damping, read form="
			<<form<<endl; MYEXIT(14); 
	  } // end switch(form)
  break; // end of case L
 
case 'E': // Exchange constant
	  My->paramFile>>junk>>form>>ExchangeA;

  // cout<<"the following parameters were read in Rectang::setup:\n"
   // <<" A = "<<ExchangeA<<" J/m"<<endl;
  assert ( (junk[0]=='E') && (form[0]=='c') );  //should read "Exchange_A", "constant"

  ExchangeA*=1e18;   //J/m to J'/nm (see description of the units in readme.txt)
  assert (Ms>0);
  CalcExchLen2(); // now in units nm^2 // was field^2/nm??? comment was wrong 
  cout<<"Exchange length ~sqrt(A/M^2) in rectangle "<<RectName<<" = "<<sqrt(ExchLen2)<<" nm"<<endl;
  break;

    case 'W': // Spin wave vector k, 0.5 gives HALF wavelength in system
      real nx, ny, nz;
      My->paramFile>>junk>>nx>>ny>>nz;
      cout<<endl<<"found 1st char in inner loop(W), (nx,ny,nz)=("<<nx<<","
                     <<ny<<","<<nz<<")"<<endl;
	  assert(WholeRect->rect == this); // sanity check, WholeRect is this cell
      assert(WholeRect->half[0]); 
      kVector=MVector(nx/WholeRect->half[0],ny/WholeRect->half[1],
                              nz/WholeRect->half[2]);
      kVector *= M_PI;
      // cout<<" kVector is = ("<<kVector[0]<<","<<kVector[1]<<","
       //       <<kVector[2]<<")"<<endl;
      break;
    case 'A': //amplitude angle and phase angle
      My->paramFile>>junk>>SpinAx>>SpinAy>>SpinPh;
      // cout<<"found 1st char in inner loop(A), SpinAx="<<SpinAx
       //              <<", SpinAy="<<SpinAy<<", SpinPh="<<SpinPh<<endl;
      break;

    case 'Q': //arbitrary rotation q_init (can also set M via 'I')
      real q0, q1, q2, q3;
      My->paramFile>>junk>>q0>>q1>>q2>>q3;
      q_init = Quaternion(q0,q1,q2,q3);
      // cout<<"found 1st char in inner loop(Q), q_init = ("<<q0
      //    <<","<<q1<<","<<q2<<","<<q3<<")"<<endl;
      break;

      case 'X': // exchange coupling to another layer 
		  {
      //#ifdef COUPLING now gone
	  string Nam;
      real EperArea; // read in J/m^2, *1e9 to get J'/nm^2
	  My->paramFile>>junk>>Nam>>EperArea;
	  My->CheckScript(junk,"X");
	  assert(junk[2]=='h'); // should be "Xchange..." but see below for "X1hange"
	  Rectang* Rect2 = My->RectNamed(Nam); // if not read yet, will give error!
	  if(!Rect2) {cout<<"Reading X line:"<<junk<<endl; MYEXIT(72);}
	  MVector Sep=Rect2->WholeRect->center - WholeRect->center; // vector to other layer
	  real biggest=0; // find biggest component
	  short nor=-1; // set normal to interface as direction of separation vector:
	  for (short j=0;j<3;j++) 
		  if (abs(Sep[j])>biggest) {biggest=abs(Sep[j]); nor=j;}
	  if(nor<0) {cout<<"The two rectangles "<<RectName<<" & "<<Rect2->RectName
		  <<" you are trying to link are the same?"<<endl; MYEXIT(39);}
      short NFace=2*nor; // index of face, 
	  short NFace1=NFace; // describes reverse direction; last bit depends on left or right:
      if(Sep[nor]>0) NFace+=1; else NFace1+=1; // partner to right or left
	  IntPartner[NFace]=Rect2->index;
	   tJ[NFace]=1e9*EperArea/(mu_0*Ms*Rect2->Ms); // now units nm
      if(junk[1]!='1')Rect2->IntPartner[NFace1]=index;
	    //junk[1] is special code for 1-way influence, for twist demo! Use "X1change"
	  Rect2->tJ[NFace1]=tJ[NFace];
	  cout<<"Setting exchange between "<<RectName<<" & "<<Rect2->RectName<<
		  ", E/area="<<EperArea<<",coupling length tJ="<<tJ[NFace]<<"nm"<<endl;
		  }
      break;

/* This is broken, PreviousLayer doesn't exist
	  case 'R': // magnetoresistance
#ifdef COUPLING
	    My->paramFile>>junk; // & ??
		assert(junk[1]=='e'); // "Resistance(Magneto)"
		cout<<endl<<"In rect::setup, read char R, line "<<junk<<endl;
		My->MyMR = new Magnetoresistance(PreviousLayer->rect,this);
#else
		  cout<<"Error: Can't calculate MR unless 'COUPLING' flag on"<<endl; MYEXIT(17);
#endif  //  
	    break;
*/

	  case 'I': // initial magnetization of rectangle (sets q_init)
 		My->paramFile>>junk>>M1; // M dummy, to set q_init
		 // since we know how to make a spin wave, just turn this into a spin wave:
		
	    q_init.rotateTo(M1); // now q_init rotates to M
		// unless explicitly set, spin wave amplitude defaults to zero
        // initMRecursively() will be called later, in ParseBuild
		// cout<<"Read I & "<<junk<<"&M= "<<WholeRect->M<<"\n &q="<<q_init<<endl;
        break;

    default:
		cout<<"rect->setup() found error in script file"<<endl; 
      WRITECHLINE; WRITECHLINE; MYEXIT(18); 
     } // end of switch, ready for new line {should be at '!' now????}

	 if(!My->paramFile) {cout<<"Error in script, rectangle setup, near:"
		 <<junk<<endl; MYEXIT(22);}
	 getline(My->paramFile,junk,'\n');
	 //cout<<"Done with line in rect->setup, found junk@end="<<junk<<endl;
   My->paramFile>>ws; // eatwhite, get rid of tabs
   } // while loop terminates when peeks at !end 
   // check that a few mandatory things got set:
   // assert(Ms>0.0); // check that you really read material parameters
   getline(My->paramFile,junk,'\n');
   if (junk[6]!='r'){cout<<"Error in script file,need ! end rect, had "<<junk<<endl;
	   MYEXIT(37);}
    // so you leave this function just after '!'
   //cout<<"just read junk="<<junk<<endl;

   // Now set some things based on what you read:
   real MinLLG=9e9; for (i=0;i<3;i++) { if(LLGSize[i]<MinLLG) MinLLG=LLGSize[i]; }
   FineInfo=new CellInfo*[refLevel+1]; // must be 1 even if not refining
   MVector Cube=MVector(MinLLG,MinLLG,MinLLG); // make CellInfo for this & smaller:
   cout<<"Create FineInfo, "<<refLevel<<"+1 times: ";
   for (i=0;i<=refLevel;i++) {FineInfo[i]=new CellInfo(Cube,My->MyFMM->UsedOrder); Cube*=0.5;}
     // this is not debugged!!

#ifdef GRAPHICS
    //WholeRect->SetDisplayRecurs(); // set "display" flag in this rectangle. gone May 2013
	//set up display somehow?
#endif  //   // GRAPHICS
   //cout<<"Done with rect->setup()"<<endl;
   return WholeRect;
}

void Rectang::RotateEasy(MVector Angle) // rotates easy axis
{   Quaternion qE; qE.qRot(Angle); // rotates by angle
	cout<<"In RotateEasy, qE="<<qE<<", avEasyAxis="<<avEasyAxis<<endl;
    avEasyAxis = ((qE * (avEasyAxis)) * (~qE)).getMVector(); 
	cout<<"In RotateEasy, new easy axis="<<avEasyAxis<<endl;
} // end function

#ifdef GRAPHICS

void cell::ReadLocalColor() // these are static members of cell we're reading
{ real xText,yText,dy; // where to put name of displayed field
//if(NumLocalColor==MaxLocalColor) {cout<<"Too many local Field displays:"
//  <<NumLocalColor+1<<endl; MYEXIT(91);}

		My->paramFile>>xText>>yText>>dy>>NumLocalColor; // units of root cell
		for (short i=0;i<NumLocalColor;i++) 
		{
			My->paramFile>>HDisp_Which[i]>>LocalColorR[i]>>LocalColorG[i]>>LocalColorB[i]; 
            My->CheckGood("--fields line error--"); // warn if error after junk
			assert(cell::HDisp_Which[i]>-1); 
			cout<<"NDisp: "<<NDisp;
			assert(cell::HDisp_Which[i]<NDisp+1);
			H_scale[i]=0.02; // wild guess --if bad, read it in
			H_scaleAuto[i]=true; // default to auto-scale

		}
		// compiler LIED: said HDisp_Which=2072!
		if(NumLocalColor==0) {cout<<"NumLocalColor=0!Press key"<<endl; cin.get();}

	    // call recursive function CreateHDisp() to recursively loop cells and create.
        if(NumLocalColor>0) My->root->CreateHDisp(); 

#ifdef OPENGL
			  // do below for OPENGL:
#endif  //  OPENGL
#ifdef COIN
    for (short i=0;i<NumLocalColor;i++)
	{  cout<<"Set up local field display #"<<HDisp_Which[i]<<"@x="<<xText<<",y="<<yText<<endl;
		
		// Create text display: // don't delete this, may want to go back!
		//HDispText[i] = new TextDisplay(HDispLabel[HDisp_Which[i]],MVector(My->root->Scale(0,xText),My->root->Scale(1,yText),0),
		//	LocalColorR[i],LocalColorG[i],LocalColorB[i]); //
	
		// Try to make it a number display!
		HDispNumb[i] = new NumberDisplay (HDispLabel[HDisp_Which[i]]+"~",1.0,3,
			&H_scaleToDisplay[i],"kA/m",
			My->root->Scale(0,xText),My->root->Scale(1,yText),0,
			LocalColorR[i],LocalColorG[i],LocalColorB[i]);
		yText+=dy;
	} // end for i
#endif  //  COIN
} // end ReadLocalColor

void cell::ReadNewLocalColor() // complains if you call it static here, not if you call it static in header
{       short NewDis;
		My->paramFile>>iLoc>>NewDis;
		// before, compiler LIED: said HDisp_Which=2072!
		if((NewDis<0) || (NewDis>NDisp)) {
			cout<<"Field display index "<<NewDis<<" out of bounds in script file"<<endl;
		    MYEXIT(55);}
		HDisp_Which[iLoc]=NewDis;
		cout<<"changing field display #"<<iLoc<<" to "<<HDispLabel[iLoc]<<endl; 
        ChangeHDisp(); // changes HDisp_Which[iLoc]
}
void cell::ChangeHDisp()
{
  if (HDispNumb[iLoc]) HDispNumb[iLoc]->RepPrefix(HDispLabel[HDisp_Which[iLoc]]+"~");
  else cout<<"To change field display, "<<
	  "script file needs 'fields,local' line"<<endl;
}

void cell::DrawFace(short NFace) // this is separate from ConvertToLLG 
	  // so we could later create LLGcell from scratch, or draw faces of non-LLG cells
{   static real Norms[6][3] = 
  { {-1,0,0},{1,0,0},{0,-1,0},{0,1,0},{0,0,-1},{0,0,1} }; 
    MVector v0,v1,v2,v3;//,cen;
	short iNorm=NFace>>1; // normal direction
	short i1=SUCC(iNorm); short i2=SUCC(i1); // other 2 directions
	// gives strange results when iNorm!=0.
	v0=FaceCenter(NFace);              // will be ++ corner
	//cen=v0;
	v1=v0; v1[i1]+=half[i1]; v1[i2]-=half[i2]; // +- corner
	v2=v0; v2[i1]-=half[i1]; v2[i2]+=half[i2]; // -+ corner
	v3=v0; v3[i1]-=half[i1]; v3[i2]-=half[i2]; // -- corner
	v0[i1]+=half[i1]; v0[i2]+=half[i2]; // ++ corner
	//cout<<"F-dir: "<<iNorm<<" cen:"<<cen<<" v0:"<<v0<<" v1:"<<v1<<" v2:"<<v2<<" v3:"<<v3<<endl;
  // if periodic, worry about displacements?

#ifdef COIN // nested
	//My->FaceNormals->vector.set1Value(My->numFace,Norms[NFace]);
	// bizarre: this was an inconsistent ordering of the 6 directions!!

    My->FaceCoords->point.set1Value(My->numVert++,v0.get_val());
	   // puts in one corner of square
	My->FaceCoords->point.set1Value(My->numVert++,v1.get_val());
    My->FaceCoords->point.set1Value(My->numVert++,v2.get_val());
    My->FaceCoords->point.set1Value(My->numVert++,v3.get_val());
	//if(numFace==17) cout<<"Vertices of face#17:"<<v0<<v1<<v2<<v3<<endl;
    My->FaceStripSet->numVertices.set1Value(My->numFace,4);
	// REALLY BIZARRE error: in above line, NumVertices.set1Value.
	// get wierd address. Added cout line, now no error!!!!!
	My->numFace++;
#endif  //   COIN
#ifdef OPENGL
   My->PutInArray(v0); My->PutInArray(v1); 
   My->PutInArray(v2); My->PutInArray(v3); 
   // puts in map if not there, also puts in IndexArray
#endif  //   // OPENGL
} // end fn DrawFace
#endif  //  GRAPHICS

MVector cell::FaceCenter(short iFace)
{  MVector FC(center); // copy constructor
   short dirn=iFace>>1;
   if(iFace&1) FC[dirn]+=half[dirn]; else FC[dirn]-=half[dirn];
   return FC;
}

#endif  //  H_ONLY

/*
void cell::integrate(void) // this could be sum over smallest cells, not recursive
{    // for modularity, really should be member of class Integrals, in Outputs.cpp.
	if(child0) 
	{child0->integrate(); child1->integrate();}
	else //smallest cell
	{
		My->MyInt->ValInt[0] += volume() * (M[1]*M[1] + M[2]*M[2]); //for mobility
		My->MyInt->ValInt[1] += volume() * M[0] * M[0]; //for debugging only, they should add to total vol*Ms^2
	} //end else
}
*/
