#ifndef CELL_H
#define CELL_H

// #include <ctype.h>
// #include <unistd.h> // for exit?
// for some reason, can't put this here: using namespace std;

#include <list>
#include <string>

//#include "Lowest.h" this is in .cpp, not .h
// gone: #include "edge.h" // need edge object (lower level) in cell class
#include "MVector.h" // need in cell class

#ifndef H_ONLY
#include "Quaternion.h"
#include "display.h"
#endif  //   

#include "utility.h"
#include "FMM.h"
//#include "Groups.h"

#include "mpole.h" // so cell can have moments // class mpole;

//class FMM;
class int3;
//class NearPartner;
class Mode;
class Term;
class FaceOf;
// using namespace std; // for string- not needed for cell.h? Is somewhere else?

#ifdef GRAPHICS
class SoTransform; // so these can be members of cell class
class SoSeparator;
class SoMaterial;
class VectorDisplay;
#endif  //  

// class cell; // is in FMM.h above
class DispCell;
class Rectang;
class LLGcell;

class CellInfo{ // info needed for each cell, sometimes const in rect tho
public:
  mpole **AvCoef;  // used to calculate the average magnetostatic field
  // duplicates AvCoef, removed: mpole *MomentCoefX, *MomentCoefY, *MomentCoefZ; 
  CellInfo(MVector,short ord); // constructor
};

#ifdef GRAPHICS
// DIsplay flags: any cell can have DispInfo, which can cause displays. 
// Although at present it's only implemented for LLGcell;
// for other cells, we would need to calculate the relevant quantities like M or H_lin, etc.
class DispInfo
{   // now HDisp only, not: bool ShowH; //tells whether to display the H vector in certain cells
public:
	VectDisp **HDisp; // was VectorDisplay
#ifdef COIN
    SoTransform *myMTransform; //to display M: translates to the cell center and rescales 
    // SoMaterial *color, *color1;  //not used
    SoSeparator *RealPtr;
#endif  //  COIN

};
#endif  //  GRAPHICS

class DispCell { // displaced so you can distinguish between periodic copies
public:   
  cell *cel;
  int3 disp; //short disp[3];
  
  //constructor
  DispCell(){ cel=NULL; disp[0]=0; disp[1]=0; disp[2]=0; };
  DispCell(cell *c, int3 i){cel=c; disp[0]=i[0]; disp[1]=i[1]; disp[2]=i[2];};
  DispCell(cell *c, short *d){cel=c; disp[0]=d[0]; disp[1]=d[1]; disp[2]=d[2];};
  ~DispCell() {cel = NULL; };

  //member function
  void SetDispToMinus(FaceOf *DF);
  void SetDispFrom(DispCell DC);
  MVector absCenter() const;
  DispCell *GetChild0(); // creates a new DispCell corresponding to child0
    //{return new DispCell(cel->child0,disp);} strange complaint cell undefined!
  DispCell *GetChild1();
    //{return new DispCell(cel->child1,disp);}
  short operator==(const DispCell right)
    {return (cel==right.cel && disp[0]==right.disp[0]
       && disp[0]==right.disp[0] && disp[0]==right.disp[0]);}
}; // end class DispCell

#ifndef H_ONLY
class FaceOf{
  public:
  bool out; //0 or 1, 1 if increasing x points out of cell
                  // 0 if increasing x points into cell
  bool owned; //0 if this FaceOf is not owned by the cell
             //1 if it is owned by the cell.  Was used to compute moment, but now used
			// ONLY to decide when to display (MakeSceneTree).
    // note that these are NOT set during construction, but in InitJexRecursively
     // these are really short integers, but declared bool to save space (??)
  face* myFace;
  int3 disp; //short disp[3]; // displacement in periodic system
  FaceOf(){ myFace=NULL; out=0; disp[0]=0; disp[1]=0; disp[2]=0; owned=0;};
};

//this operator is needed for map operations
short operator<(const int3&, const int3&); //added by DMAf

class face{
  public:
  real Jex; //exchange integral between two cells
//  real magneticCharge;  //magnetic charge density of the face (DMA)
  DispCell cellList[2]; // cellList[0] is to left (low coordinate).
  // Note that "normal" is used in many different senses: 
  // normalize a vector, normal mode, normal to face.
  // Convention: normal mode & normalize are lower case, 
  // Normal means perpendicular (because SoNormal is cap).
  // NormaL is perp. outside graphics context (as face::NormaL)
  face();
  //~face() {face0 = NULL; face1 = NULL; parent = NULL; centerEdge = NULL; };
  
  void sane(); //sanity check function
  //void setFaceCenter();

}; // end class face
#endif  //  H_ONLY

class cell
{
  public:
 cell *child0, *child1, *parent;

  Rectang *rect;   //the pointer to a class that contains material and other param.
  mpole *moment; //multipole moment of this cell, added by DMAf
    // Note it seems to have a MINUS SIGN!!! 2011
  mpole *SmoothField; //taylor expansion of the scalar potential of the smooth field
    // Note that it is expansion of W(r)=V(-r), in that the odd orders are reversed 
  // real Energy;  // energy // NEVER NEED for one cell!
  std::list<DispCell> Partners;  //added by DMAf - list of Partners
  
  MVector center;//center of the cell in System window, used in display function
  MVector half;  // half the size of the cell 
  real radius2; //used for FMM, added by DMAf
#ifdef DEBUG1
  static bool PD1;
#endif  //  
#ifdef DEBUG2
  static bool PD2;
#endif

  // functions:
  cell();  // copy constructor
  virtual ~cell();
  static LLGcell* ConvertToLLG(cell* &);
  void updateMoment(); //update recursively multipole moments, added by DMA
  void updateStat(); //calculate H recursively due to Partners and NearPartners,
  void CullPartners(); //setup Partners and NearPartners lists, added by DMAf
  //CellInfo* GetInfoPtr() const; // 'const' means it doesn't modify cell object
    // most of these functions could have 'const' at the end, didn't bother to do tho
  bool Near(const DispCell &);//is this DispCell near to this, added by DMAf
  friend ostream& operator<< (ostream&, cell) ;
  bool Contains(MVector);
    LLGcell* SubCellContaining(MVector&);
	//LLGcell* SubLLGContaining(MVector &v);
  bool AncestorOf(cell*);
  void ListCellsRecursively();
  void SetOrderRec();
  MVector averageMpole(const mpole*) const;   //returns averaged SmoothField
  bool sane(); //sanity check
  inline MVector LowerLeft() {return center-half;}
  inline MVector UpperRight() {return center+half;}
  inline void SetRadius2() {radius2=half.Square();}
  inline real volume() {return 8*half[0]*half[1]*half[2];} //added by zhuru

#ifndef H_ONLY
  void setRectangleParamRecurs();  //set gdt, gadtM, H_rand_scale for this subtree
  //added by DMAk (used for recursive Kernel calculation):
  void getKernelsRecursively();  //to get Kernels from H_av
  //void setChargeAtCenterFace (short);
   //to initialize charges of all faces to zero except the faces of
   //coarse 2x2x2 center cell (closest to the center of the root cell)

  // delete short LongAxis(const MVector&);
  short LongAxis();
  //void parseBuild(); //parse build cells according to the script input file
  void buildRect(); //build a rectangle cell, called within parseBuild()
  //void print();
  inline real Scale(int axis,real frac) // frac goes 0 to 1 across cell
    {return center[axis]+(2*frac-1.0)*half[axis];}
  void readSize();
  // void SetInterPartners();
  //splitPoint was added by DMAg for modification of geometry
  void split(short, short, short);
  //split a large cell into child cells along the longaxis
  virtual bool splitRecursively();//split the cell recursively untill the bottom of tree
  virtual void initIDRecursively();//set the array CellFromID
  //void initJexRecursively();//set the initial exchange constant  
  //the following member functions are for debugging problem at a specific place
  //short whichChild();
  cell** PtrToPtrToThis();
  void PassTo(cell*);
  //void GetSizeCenter(); // get from children
  //void GetSizeCenterRecursively(); // get from tree
  //void integrate(); // not used recently
  MVector FaceCenter(short);
  Kernel CalcKernel(DispCell *source);  // returns effect of source on "this"
#endif  //  H_ONLY

#ifdef GRAPHICS
    DispInfo *DispInfoPtr;  // info needed for display only; null if this cell not displayed
	// (plays role of old ShowM or DisplayCell)
   	//tells whether to display in real space and System; 
    	//may be true only for some cells
		#define MaxLocalColor 3
// following static vbls must be declared in .cpp where used, also:
	// these are used only if DispInfo, but want to refer to easily from cell class
    static short HDisp_Which[MaxLocalColor]; // which fields to display at each cell
	// these were members of class systm, but avoid My-> if static members of cell?
    static float LocalColorR[MaxLocalColor]; // actual color components
    static float LocalColorG[MaxLocalColor]; 
    static float LocalColorB[MaxLocalColor]; 
	static float H_max[MaxLocalColor]; 
	static float H_scale[MaxLocalColor]; 
	static bool H_scaleAuto[MaxLocalColor];
	static float H_scaleToDisplay[MaxLocalColor]; 
    static short NumLocalColor;
	static short iLoc; // which local display we're working with
	static void ReadLocalColor(); // complains if I put cell:: in front! 
	  // Then "defined at file scope"
	static void ReadNewLocalColor();

	//void SetDisplayRecurs(); // set Display flag (gone as of May 2013?)
	void CreateHDisp(); // InsertHDisp is in LLGcell; 
	  // may later create InsertHDisp for non-LLG cells, but will take time
    static NumberDisplay *HDispNumb[MaxLocalColor];
	static void cell::ChangeHDisp();
	//void DisplayFace(); // not used 06/2013 replaced by DisplayFaces?
    void DisplayFaces();
    void change_color(real *material, real *material1, short flag);  //not used
    void MakeSceneTree();
	void DrawFace(short);
#endif  //  GRAPHICS
}; // end class cell

class LLGcell : public cell   // descends from class cell
{
public:
  /**********data members **********/
  CellIDType SmCellID; // cell ID to identify cells when write into or read from system
  static CellIDType NextSmCellID;   
  //static unsigned short MaxCellID; // too big SmCellID
    //static data member, used to count the total number of cells in the system
  MVector M; //magnetization vector
  std::list<NearPartner> NearPartners;  //added by DMAf - list of NearPartners
  MVector H_quad; // part of avg field in cell that appears 
    // quadratically in energy (m'static, exchange, anis)

  CellInfo *CellInfoPtr; // only if partial cell, has AvCoef, etc. Use as flag for partial.
    // for other cells, use rect->RectInfoPtr
  CellInfo* GetInfoPtr() const; // needs to exist for cells also?
  void AddNearField(const NearPartner *);

  /********** constructor and destructor **********/
  LLGcell(cell*);  // copy constructor
  LLGcell(); //non-copy constructor
  ~LLGcell();

#ifdef GRAPHICS
  void InsertHDisp();
  //void LLGcell::DisplayFace(); // not used 06/2013 replaced by DisplayFaces?
  void LLGcell::DisplayFaces();
#endif  //  GRAPHICS
#ifndef H_ONLY

  // things not needed just to calculate H:
  FaceOf cellfacelist[6]; //facelist for a specific cell (DMA)
     	//we use array. Index for direction: -x=0, +x=1, -y=2, +y=3, 
      //-z=4, +z=5 so last bit is sign, next 2 are axis label
  Quaternion q; //someday we may want to get rid of M and just store q 
  MVector H_lin; // part of avg (external) field in cell that appears linearly in energy
  MVector DelMpre, DelM, H_rand;
  // Material Properties
  real anisCoef;          //=Hk/Ms

  /********** member functions **********/
  bool splitRecursively(); // overrides cell::splitRecursivley for LLGcells
  void initM(); // initialize magnetization and quaternion, no longer recursive aug 2013
  void initIDRecursively(); // overrides cell::initIDRecursively for LLGcells
  void CreateFaces();
// unused  inline real RightSide(short indx) {return center[indx]+half[indx];};
  //MVector RandomField();
  MVector RandomFieldForM();
  //void update_q();   //quaternion-based algorithm is used
  // void update_m();   //usual algorithm is used
  inline MVector H_anis();  //anisotropy field
  void RotateMby(MVector); // rotates M and q
  MVector H_exch();
  // void CalcH_av(); // only used if no fmm
  void CalcDelM(); // called in evolve()
  // void IncAvgs(); // called in evolve()? Need re-debug, no levels
              //functions used for FMM:
  void Mqsane(); // check M,q
  void check_faces();
  void AddCellEnergy();  
  void initJex();
  inline MVector MV() {return M*volume();}
  inline MVector H_av() {return H_lin+H_quad;}
#endif  //  H_ONLY
}; // end class LLGcell


#ifndef H_ONLY // below not needed for plugin
class face;
class Shape;

 // GLobal variables, hard to make static memebers of cell:
#define NDisp 9
const string HDispLabel[NDisp+1]= {"Show:Zeroes","Show:ExtField","Show:M'static","Show:Exch",
"Show:Anis","Show:determ.","Show:random","Show:DeltaM","Show:no#8!","Show:P(t)"};

#endif  //  H_ONLY

class Rectang // was called Rectangle, but conflicts w/Windows function
{
public:
  Rectang(cell*);
  ~Rectang();
  MVector LLGSize;       //the (half) size of the smallest cell in this rectangle
    // should change this globally to LLGHalf!!!
  short refLevel; // how many times to /8 LLG cells
  CellInfo** FineInfo;
  CellInfo *RectInfoPtr; // default info for whole LLG cells 
  short index; // for arrays
  std::map<int3, Kernel> *RectKer; //Kernels for calculation of the potential from
  cell *WholeRect; // cell that corresponds to this

#ifndef H_ONLY
  Shape* shape;

  cell* setup();   //read lines with parameters, changed to cell* 06/03/2013 to return root ptr if changed.
  // void HandleKey(short,short); this is now part of Systm
  void RotateEasy(MVector Angle); // rotate easy axis
  inline void CalcExchLen2(){ExchLen2=ExchangeA/(mu_0*Ms*Ms);}
  inline real RoughExchEn() {return ExchLen2*Ms*Ms*WholeRect->volume()/LLGSize[0]/LLGSize[1];}
     /**************** data members **************/
  real alpha;    // damping constant
  real gdt; // really has renormalized gamma: dt*gamma/(1+alpha^2)
  real gadtM; // dt*gamma*alpha/(1+alpha^2)Ms
  real Ms;     //saturation magnetization of this rectangle
  real H_rand_scale;   //used to calculate the effective thermal field
  MVector avEasyAxis;       //average easy axis of the uniaxial anisotropy
  MVector M0;             //MVector(0,0,Ms)
  real Hk;                  //average effective anisotropy field
  // moved to Systm: #define MaxModes 4, & Modes too; here list TERMS:
  std::list<Term> Terms; 
  std::list<Term>::iterator iTerm; // could be just where used, tho

  AnisLayers *AL; // layer thicknesses and Hk 
  PowerLaw *AnisPower;      //coefficients in powerlaw anisotropy
  Cubic *AnisCubic;      //coefficients in polynomial anisotropy
  real ExchangeA;                  //exchange constant, density A(grad M/M)^2 -- no 1/2!!!!
  real ExchLen2; // square of exch length
  short IntPartner[6]; // interface exchange
  real tJ[6]; // exchange "length"
  MVector sizeToDisplay; // half cell length, internally; replaces levels ; exists even without GRAPHICS?
  bool ShowM; // whether to display M; used to be cell::ShowM, now only rect & small

  string RectName;

  real SpinAx; //spin wave amplitude
  real SpinAy; 
  real SpinPh; //spin wave phase
  MVector kVector; // spin wave vector

  Quaternion q_init;      //used to set initial quaternion for a cell
#endif  //  H_ONLY
};

#endif  //  	

  
