#include "Lowest.h"
#include "config.h"
#include "Systm.h"
#include <iomanip>
#include <sstream> // for stringstream StSt
extern Systm* My;

Config::Config()
{
	ConfigArr = new MVector [LLGcell::NextSmCellID];
}

Config::Config(const Config& config)
{
	ConfigArr = new MVector[LLGcell::NextSmCellID];
	for (CellIDType i = 0; i!= LLGcell::NextSmCellID; i++)
		ConfigArr[i] = config.ConfigArr[i];
}

Config::Config(const MVector& vec) //uniform M
{
	ConfigArr = new MVector[LLGcell::NextSmCellID];
	for (CellIDType i = 0; i!= LLGcell::NextSmCellID; i++)
		ConfigArr[i] = vec;
}

void Config::ProjectOut(Config C1){
	*this=*this-((C1*(*this))/(C1*C1))*C1;
}
void Config::restore(void)
{   //cout<<"   in restore from "<<this<<",t="<<My->t<<",watched M was="<<My->WatchedLLGCell->M<<endl;
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++){
		My->CellFromID[i]->M = ConfigArr[i];
	//cout<<"Changed M at "<<My->CellFromID[i]->center<<" to "<<My->CellFromID[i]->M<<endl;
	}
}

void Config::restoreAnd(void) // restore AND renorm, CalcAllFromM so ready for evolve()
{   //cout<<"   in restore from "<<this<<",t="<<My->t<<",watched M was="<<My->WatchedCell->M<<endl;
	LLGcell* c1;
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++)
	{ 
		c1=My->CellFromID[i];
		c1->M = ConfigArr[i];
		c1->M.normalizeTo(c1->rect->Ms);
	}
	My->CalcAllFromM(1); 
	//cout<<"Changed M to "<<My->WatchedCell->M<<endl;
}

void Config::saveM(void)
{	//cout<<"   in save from "<<this<<" ,t="<<My->t<<",watched m="<<My->WatchedCell->M<<endl;
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++)
		 ConfigArr[i] = My->CellFromID[i]->M;
}

void Config::saveDelM(void)
{
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++)
		 ConfigArr[i] = My->CellFromID[i]->DelM;
}

real Config::norm(void) const
{
	real Norm(0.);
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++)
		 Norm += ConfigArr[i].Square();
	return sqrt(Norm);
}

void Config::ExporttoFile(ofstream& configfile)
{
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++)
		configfile<<"i:"<<i<<" "<<ConfigArr[i]<<endl;	 
}

void Config::print()
{
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++)
		cout<<"i:"<<i<<" "<<ConfigArr[i]<<endl;	 
}

void Config::writeBinary(ofstream& fileOut)
{   stringstream StSt; string St;
	// no header, starts with '0:...'
	char char12[12];
	//for(int i=0;i<12;i++) {cout<<char12[i]; char12[i]=0;} cout<<endl;
    // to write in a more accurate representation of data
    for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++) 
	{  //char12="abcdefghijkl"; screws us up?
#ifdef WIN32
		sprintf_s(char12,11,"\n%i:",i); // create c-string
		// oddly, sets unused chars to -3=FD, last to 0
#else
		sprintf(char12,"\n%i:",i); // create c-string
#endif
	   //cout<<"writing char12="<<char12<<"."<<endl;
	   //for(int j=0;j<12;j++) {cout<<(int)char12[j]<<",";} cout<<endl;

	   fileOut.write(char12,12); // write all chars rather than figure out length
	   	     // note this may not be seen 


	/*
	   StSt=""; StSt<<"\n"<<i<<":"; // newline before number:
	   St = StSt.str(); // now standard string, no way to access data in StSt?
	   cout<<"St:"<<*St.data()<<", length="<<St.length()<<endl; 
	   fileOut.write(St.data(),St.length()); // data=pointer to chars
*/
           //My->writeBase64Obj(fileOut,&ConfigArr[i], sizeof(ConfigArr[i]));
       fileOut.write((char*)&ConfigArr[i], sizeof(ConfigArr[i]));
    } // end for i
    fileOut.write("\n!end Config",12);
}
void Config::readBinary(ifstream& inFile) // leaves at 1st line after !
{   char char12[12];
    CellIDType in,id=0;
    for(id=0;id<LLGcell::NextSmCellID;id++) 
	{   inFile.read(char12,12); char12[0]=' '; // may not like \n!
	    //cout<<"read char12="<<char12<<endl;
		//for(int i=0;i<12;i++) cout<<(int)char12[i]<<","; cout<<endl;
		in=atoi(char12); // seems not to mind ':' at end
		// or could use StSt>>in to extract
		if(in!=id) EXIT1("Wrong cell index in movie file:"<<in);
		inFile.read((char*)&ConfigArr[id], sizeof(ConfigArr[id]));
        //My->readBase64Obj(inFile,&ConfigArr[id], sizeof(ConfigArr[id]));// leaves on NEXT line?
    } // end for id
    if(!inFile.good()) EXIT1("Error reading movie file, last read cell with id: "<<id<<endl);
    //getline(inFile,junk,'\n'); // get rid of rest of ! line
	inFile.read(char12,12); My->CheckScript(char12,"\n!end Config");
}

ostream& operator<< (ostream& fout, const Config& c) 
{  for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++)
		fout<<"i: "<<i<<" "<<c.ConfigArr[i]<<endl;	 
   return fout;
}

istream& operator>> (istream& fin, Config& c) 
{  string junk;CellIDType i,i1; 
   for (i = 0; i < LLGcell::NextSmCellID; i++)
   {   fin>>junk>>i1>>c.ConfigArr[i];
       if(i!=i1) EXIT1("Cell ID wrong in config");
   }
   getline(fin,junk); // finish off line, should be empty though
   if(!fin.good()) EXIT1("Error reading Config");
   getline(fin,junk); // could be at EOF now
   if(junk!="!") EXIT1("config should end with !");
   return fin;
}

void Config::ImportfromFile(ifstream& configfile)
{
	string junk; char ch1;
	int i1;
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++)
	{	 configfile>>i1>>ch1>>ConfigArr[i]; getline(configfile,junk,'\n');
		assert(ch1==':'); assert(i1==i);
	}
}

Config& Config::operator += (const Config& config)
{
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++)
			 ConfigArr[i] += config.ConfigArr[i];
	return *this;
}

Config& Config::operator -= (const Config& config)
{
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++)
			 ConfigArr[i] -= config.ConfigArr[i];
	return *this;
}

Config& Config::operator *= (const real r)
{
	for (CellIDType i = 0; i < LLGcell::NextSmCellID; i++)
			 ConfigArr[i] *= r;
	return *this;
}

MVector& Config::operator [] (int i)
{
	return ConfigArr[i];
}

Config operator + (const Config lhs, const Config rhs)
{
	Config ret(lhs);
	ret += rhs;
	return ret;
}

Config operator - (const Config lhs,const Config rhs)
{
	Config ret(lhs);
	ret -= rhs;
	return ret;
}

Config operator * (const real r, const Config& rhs)
{   Config ret;
    for (CellIDType i=0;i<LLGcell::NextSmCellID;i++)
	   ret.ConfigArr[i]=r*rhs.ConfigArr[i];
	// Before, this just called *=: Config ret(rhs); ret*=r;
	return ret;
}

real operator * (const Config& lhs, const Config& rhs)
{ // this is the dot product, weighted by volume
	real ret = 0.;
	for (CellIDType i = 0; i != LLGcell::NextSmCellID; i++)
			ret += lhs.ConfigArr[i] * rhs.ConfigArr[i]*My->CellFromID[i]->volume();
	return ret;
}
