// AlaMag -- Copyright(C) 1999-2009 Pieter B. Visscher, see main.cpp for notices
// This is file display.cpp
#include "Lowest.h" // has iostream,math,assert,defines GRAPHICS
#ifdef GRAPHICS // skip rest of file if not; COIN &OPENGL are nested
#include "cell.h"
#include "display.h"
#include "Systm.h"
#include <sstream>

// is in system.h:#include "pulse.h"

extern Systm *My;

#ifdef COIN // all in display.cpp are nested in #ifdef GRAPHICS
  // this is so COIN-dependent it has to be completely rewritten
void myKeyPressCB(void *userData, SoEventCallback *eventCB){
  // Mourad's was is adapted from Ex. 10-2 of Mentor, PBV changed it
  // based on 10.3.4.Gate.c++
  // This function is NOT in a class. Must be in same file it is USED in
  // (above seems strange 2011: is NOT in file it is used in!!)
  userData; // just to avoid "unreferenced param" warning
  const SoEvent *event = eventCB->getEvent();
  if (!event->isOfType(SoKeyboardEvent::getClassTypeId())) cout<<"wrong type event:"<<
    endl<<flush; // not needed, Viewer intercepts mouse events
  const SoKeyboardEvent *e = (SoKeyboardEvent *) event; // type cast to keyboard event
  //if(e->isKeyPressEvent(e,SoKeyboardEvent::ANY)) cout<<"KeyPressEvent"<<endl;
  //if(e->isKeyReleaseEvent(e,SoKeyboardEvent::ANY)) cout<<"KeyReleaseEvent"<<endl;
  
  if( e->isKeyReleaseEvent(e,SoKeyboardEvent::LEFT_SHIFT) || 
	  e->isKeyReleaseEvent(e,SoKeyboardEvent::RIGHT_SHIFT) )
  {  cout<<"Shift Released"<<endl; if(!My->ShiftIsDown) cout<<"VERY ODD: it wasn't down!!"<<endl; 
     My->ShiftIsDown = 0;
  }
  if( e->isKeyReleaseEvent(e,SoKeyboardEvent::LEFT_CONTROL) ||
	  e->isKeyReleaseEvent(e,SoKeyboardEvent::RIGHT_CONTROL) )
  {  cout<<"Control released"<<endl; if(!My->ControlIsDown) cout<<"VERY ODD: it wasn't down!!"<<endl;
       My->ControlIsDown = 0;
  }
  // removed 06/2013 problems with Alt
  if( e->isKeyReleaseEvent(e,SoKeyboardEvent::LEFT_ALT) || 
	  e->isKeyReleaseEvent(e,SoKeyboardEvent::RIGHT_ALT) )
  {  cout<<"Alt Released"<<endl; if(!My->AltIsDown) cout<<"VERY odd: it wasn't down!!"<<endl;
     My->AltIsDown = 0; // ?? ALT key seems not to work in Open inventor?
  }
  if (!e->isKeyPressEvent(e,SoKeyboardEvent::ANY)) return;
  //(don't respond to key releases!)
  //cout<<"Entering myKeyPressCB"<<endl;
  SoKeyboardEvent::Key e1=e->getKey();
  //cout<<"KeyPressEvent #"<<e1; 
  // if(((short)e1<256) && ((short)e1>=0)) cout<<" may be "<<(char)e1<<endl; else cout<<", unprintable"<<endl; //DEBUG
  switch (e1){
	case SoKeyboardEvent::F9:
		real Rq,Gq,Bq;
		assert(My->surfaceMaterial);

		if(My->ShiftIsDown)
			if(My->AltIsDown) // both down ?? ALT key seems not to work in Open inventor?
			{cout<<"Enter diffuseColor in percent"<<endl;
		      cin>>Rq>>Gq>>Bq; Rq/=100; Gq/=100;Bq/=100;
              My->surfaceMaterial->diffuseColor.setValue(Rq,Gq,Bq);
			}
			else // only shift 
		    { cout<<"Enter ambientColor in percent"<<endl;
		      cin>>Rq>>Gq>>Bq; Rq/=100; Gq/=100;Bq/=100;
              My->surfaceMaterial->ambientColor.setValue(Rq,Gq,Bq);
		}
		else // not shift
		if(My->AltIsDown)//?? ALT key seems not to work in Open inventor?
		{  cout<<"Enter emissiveColor in percent"<<endl;
		   cin>>Rq>>Gq>>Bq; Rq/=100; Gq/=100;Bq/=100;
           My->surfaceMaterial->emissiveColor.setValue(Rq,Gq,Bq);
		}
		else // not shift or alt
		{
			if(My->ControlIsDown)
			{  cout<<"Enter BackgroundColor in percent"<<endl;
		       cin>>Rq>>Gq>>Bq; Rq/=100; Gq/=100;Bq/=100;
               My->myViewer->setBackgroundColor(SbColor(Rq,Gq,Bq)); 
		    }
			else // not shift, alt, or ctrl
		{  cout<<"Enter specularColor in percent"<<endl;
		   cin>>Rq>>Gq>>Bq; Rq/=100; Gq/=100;Bq/=100;
           My->surfaceMaterial->specularColor.setValue(Rq,Gq,Bq);
		}
		}
		cout<<"Changed it to "<<Rq<<","<<Gq<<","<<Bq<<endl;
        break;

	case SoKeyboardEvent::LEFT_SHIFT:
	case SoKeyboardEvent::RIGHT_SHIFT: // for either shift key:
	  // assert(!My->ShiftIsDown); // sometimes triggers
	  My->ShiftIsDown = 1;	  //cout<<"Shift pressed"<<endl;
	  break;
	case SoKeyboardEvent::LEFT_CONTROL: // this codes to zero out variable
	case SoKeyboardEvent::RIGHT_CONTROL: //Tried backspace, triggered multiply?
		// may add option so when shift AND Control down, set vbl to 1?
	  // assert(!My->ControlIsDown); creaes problems too
		// Note that if you put a BREAK anywhere, it MISSES these keyboard events
		// and ControIsDown=true even after you release key!! Makes a mess.
	  My->ControlIsDown = 1; //cout<<"Control pressed"<<endl;
	  break;
	case SoKeyboardEvent::LEFT_ALT: // this sets local field mode??
	case SoKeyboardEvent::RIGHT_ALT:
	  cout<<"Alt pressed"<<endl;
	  My->AltIsDown = 1;
	  break;
    case SoKeyboardEvent::NUMBER_1: // these decrement mode fields or moments
		My->HandleKey(1,-1); break; // arguments are mode # and sign
    case SoKeyboardEvent::NUMBER_2: 
		My->HandleKey(2,-1); break;
    case SoKeyboardEvent::NUMBER_3:
		My->HandleKey(3,-1); break;
    case SoKeyboardEvent::NUMBER_4:
		My->HandleKey(4,-1); break;
    case SoKeyboardEvent::NUMBER_5:
		My->HandleKey(5,-1); break;
    case SoKeyboardEvent::NUMBER_6:
		My->HandleKey(6,-1); break;
    case SoKeyboardEvent::NUMBER_7:
		My->HandleKey(7,-1); break;
 //   case SoKeyboardEvent::NUMBER_8: Alas, 8 conflicts with up arrow!
//		My->HandleKey(8,-1); break;

	case SoKeyboardEvent::F1: // these increment mode fields or moments
		My->HandleKey(1,+1); break; 
	case SoKeyboardEvent::F2: 
		My->HandleKey(2,+1); break; 
	case SoKeyboardEvent::F3: 
		My->HandleKey(3,+1); break; 
	case SoKeyboardEvent::F4: 
		My->HandleKey(4,+1); break;  
	case SoKeyboardEvent::F5:  
		My->HandleKey(5,+1); break;  
	case SoKeyboardEvent::F6:  
		My->HandleKey(6,+1); break;  
	case SoKeyboardEvent::F7:  
		My->HandleKey(7,+1); break;  
//	case SoKeyboardEvent::F8:  
//		My->HandleKey(8,+1); break;  

    case SoKeyboardEvent::RIGHT_ARROW:
	  //cout<<"Right Arrow pressed, 'Pausing' is ";
	  //if (!My->Pausing) cout<<"already"; cout<<" false"<<endl;
	  My->Pausing=false;
      if (My->Movname != "") 
          My->CBMode = My->FORWARD;
	   /* ?? why did I turn this off?
     if (My->Movname[0] != NULL)  //we are reading movie file
       My->CBMode=My->FORWARD;
     else
     {
		 switch(My->CBMode)
		 {
		 case My->STOP: My->CBMode=My->COMPUTE(now RUN?);	break;
		 //case My->DO_ORBITS_PAUSE: My->CBMode=My->DO_ORBITS; break;	
		 // note you will never get here if ORBITS compiler option is not on
		 }
       // cout<<"right_arrow is pressed, continue"<<endl;
     }
	 */
     break;
   case SoKeyboardEvent::LEFT_ARROW:
     if (My->Movname != "") {
         My->CBMode=My->BACKWARD; // only if movie read
         My->Pausing=false;
     } else cout<<"Left arrow illegal now"<<endl;
     break;
   case SoKeyboardEvent::DOWN_ARROW:
     cout<<"program is paused, press right_arrow to continue"<<endl;
	 My->Pausing=true;
	 /*
     switch(My->CBMode)
		 {
		 case My->COMPUTE(now RUN?): My->CBMode=My->STOP;	break;
		 case My->DO_ORBITS: My->CBMode=My->DO_ORBITS_PAUSE; break;	
		 }
	 */
     break;

   case SoKeyboardEvent::A: // increase damping alpha in watched cell rectangle
	   assert(My->WatchedCell);
	 if(My->DampingFinite)
	   {
	   My->FixReal(My->WatchedCell->rect->alpha,0.01); //does something to alpha,&writes what
	   cout<<" damping alpha within watched rect "<<My->WatchedCell->rect->RectName<<",to "<<My->WatchedCell->rect->alpha<<endl;
	   }
	 else  {cout<<"Alpha was infinite, setting finite"<<endl;
	       My->WatchedCell->rect->alpha=0.01; My->DampingFinite=TRUE;}
	 My->SetEvolParams();
	 break;

   case SoKeyboardEvent::D: // increase d (or NextDisp if iterating)
	   if(My->ShiftIsDown) {cout<<"In"; My->d*=2; My->NextDisp=1;} 
	   else {cout<<"De"; My->d/=2;}
	   cout<<"crease display interval d to "<<My->d<<endl;
	   if(My->d==0) 
	   {cout<<"and back to 1, zero not allowed"<<endl;My->d=1;}
	 break;

     case SoKeyboardEvent::H: // fix noise Temperature
	   My->FixReal(My->Temp,0.01); //does something to dt,&writes what
	   cout<<" Temperature, to "<<My->Temp<<endl;
	   My->Noise = (My->Temp>0.0);
	   My->SetEvolParams();
	 break;

   case SoKeyboardEvent::T: // increase dt
	   My->FixReal(My->dt,0.00001); //does something to dt,&writes what
	   cout<<" dt, to "<<My->dt<<endl;
	   My->SetEvolParams();
	 break;

  case SoKeyboardEvent::E: // inc/decrease exchange in watched rectangle
	  {  // brackets to allow declaring Watrec
	   Rectang* WatRec=NULL;
	   if(My->WatchedCell)
		WatRec=My->WatchedCell->rect; // for convenience
	   else if (My->WatchedLLGCell)
		WatRec=My->WatchedLLGCell->rect;
	   assert(WatRec);
	   My->FixReal(WatRec->ExchangeA,1e7); //does something to ExchangeA,&writes what
	   // isn't this missing factor mu_0? May be irrelevant now.  
	   cout<<" exchange within watched rect "<<My->WatchedCell->rect->RectName<<",to "<<WatRec->ExchangeA*1e-7<<"e-11 j/m??"<<endl;
	   WatRec->CalcExchLen2(); // from new ExchangeA 
	   //what to do if no watched LLG cell?
	   for (CellIDType i=0;i<My->WatchedLLGCell->NextSmCellID;i++) {My->CellFromID[i]->initJex();}// note A may affect exch field at NBR rectangle! Slightly, ignore here?
	  }
     break;

	 // Now arrow keys ("PAD"); simulate for laptop with 8,U,O,K keys
   case SoKeyboardEvent::K: // if K or pad2, decrease selected vector(y)
   case SoKeyboardEvent::PAD_2:
     My->H_bias[1]-=My->Increment; cout<<"-y"; 
	 // cout<<"Decreased y-field by "<<My->Increment<<endl;
	 if (My->gameMode) My->LimitField();
	 My->updateScene(); // so you see it right away, not after next step
	 break; 
    case SoKeyboardEvent::PAD_4: // decrease x
    case SoKeyboardEvent::U:
     My->H_bias[0]-=My->Increment; cout<<"-x"; 
	 if (My->gameMode) My->LimitField();
     My->updateScene(); 
	 break;
    case SoKeyboardEvent::PAD_6:
    case SoKeyboardEvent::O:
     My->H_bias[0]+=My->Increment; cout<<"+x"; 
	 if (My->gameMode) My->LimitField();
     My->updateScene(); 
    break;
   case SoKeyboardEvent::PAD_8:
   case SoKeyboardEvent::NUMBER_8:
     My->H_bias[1]+=My->Increment; cout<<"+y"; 
	 if (My->gameMode) My->LimitField();
     My->updateScene(); 
     break;
   case SoKeyboardEvent::PAD_ADD: // increase selected vector(z)
   //case SoKeyboardEvent::SLASH: why slash here??????
     My->H_bias[2]+=My->Increment; cout<<"+z"; 
	 if (My->gameMode) My->LimitField();
     My->updateScene(); 
     break;
   case SoKeyboardEvent::PAD_SUBTRACT:
   case SoKeyboardEvent::SEMICOLON:
     My->H_bias[2]-=My->Increment; cout<<"-z"; 
	 if (My->gameMode) My->LimitField();
     My->updateScene(); 
     break;

   case SoKeyboardEvent::BRACKETLEFT: // decrease xyz increment
	   ChangeInc(My->Increment,0); // 0 means decrease
	   // My->Increment/=2; 
	   cout<<"Decreased field increment to "<<My->Increment<<endl;
	 break;
   case SoKeyboardEvent::BRACKETRIGHT: // increase xyz increment
     // My->Increment*=2; 
	   ChangeInc(My->Increment,1); // 1 means increase
	   cout<<"increased field increment to "<<My->Increment<<endl;
	 // irrelevant:My->updateOIScene(); // so you see it right away, not after next step
     break;

   case SoKeyboardEvent::C: // cycle field display
	 if (My->ControlIsDown) 
	 {   cout<<"Pressed C, NDisp="<<NDisp<<endl;
		 if(My->ShiftIsDown)
	 {cell::HDisp_Which[cell::iLoc]--; if(cell::HDisp_Which[cell::iLoc]<0) cell::HDisp_Which[cell::iLoc]=NDisp;}
		else // no shift key
    	 {cell::HDisp_Which[cell::iLoc]++; if(cell::HDisp_Which[cell::iLoc]>NDisp) cell::HDisp_Which[cell::iLoc]=0;}
        cell::ChangeHDisp();
	 }	
	 cout<<"Local display now # "<<cell::HDisp_Which[cell::iLoc]<<endl;
	 break;
		 
	case SoKeyboardEvent::V: // cycle iLoc
	 if (My->ControlIsDown) 
	 {  if(cell::NumLocalColor<2) {cout<<"Can't switch local display, there's only one"<<endl; break;}
	    if(My->ShiftIsDown)
	    {cell::iLoc--; if(cell::iLoc<0) cell::iLoc=cell::NumLocalColor-1;}
		else // no shift key // don't really need reverse cycling, will be few iLoc's
		{cell::iLoc++; if(cell::iLoc==cell::NumLocalColor) cell::iLoc=0;}
	 cout<<"Switching which local color display to adjust: now "<<cell::iLoc<<
		 ", RGB="<<cell::LocalColorR[cell::iLoc]<<","<<cell::LocalColorG[cell::iLoc]<<","<<cell::LocalColorB[cell::iLoc]<<endl;
	 }		 
	 break;
 
  case SoKeyboardEvent::COMMA: // or <
	  if (My->ControlIsDown) // assume user wants to auto-scale
	  { cell::H_scaleAuto[cell::iLoc]=true;
	    cout<<"Pressed >, turning on auto-scale"<<endl;
// old, before H_scaleAuto:
//	    VectDisp::multFactor*=0.5; // note this is static! 
//	    cout<<"decrease field scale to "<<VectDisp::multFactor<<endl;
	  }
     else
	 {  if (My->numberSkipFrames>=2) My->numberSkipFrames/=2;
	    cout<<"number of frames to skip:"<<My->numberSkipFrames<<endl;}
     break;
  case SoKeyboardEvent::PERIOD: // or >
	  if (My->ControlIsDown) 
	  { if(cell::H_scaleAuto[cell::iLoc]) 
	    {cout<<"Turning off auto-scale"<<endl;
         cell::H_scaleAuto[cell::iLoc]=false;
	    }	    
        cell::H_scale[cell::iLoc]*=2.; 
		// alas, must fix display scale each time H_scale changes:
		if(!My->WatchedLLGCell) {cout<<"No watchedLLG cell to use for scaling"<<endl; break;}
	    cell::H_scaleToDisplay[cell::iLoc]=2*My->WatchedLLGCell->half[0]/cell::H_scale[cell::iLoc]; // would display as cell size
		cout<<"Doubling H scale to "<<cell::H_scaleToDisplay[cell::iLoc]<<endl;
		  // old, pre-H_scaleAuto
		  //VectDisp::multFactor*=2.;  // there is no 8 yet!
	  	  //  cout<<"increase field scale to "<<VectDisp::multFactor<<endl;
	  }
	 else {My->numberSkipFrames*=2;
	   cout<<"number of frames to skip:"<<My->numberSkipFrames<<endl;}
     break;
// removed some events to comments.cpp

// delete  case SoKeyboardEvent::SLASH: // or ?
//     if (My->ControlIsDown) My->ChangeHDisp(0);  // sets to zero
//     break;
//
   case SoKeyboardEvent::M:
       if (My->ShiftIsDown) {
           // switch to RUN mode from reading movie
          if (!(My->movIn)) break;
          My->CBMode=My->RUN;
          My->Movname="";
          My->movIn.close();
       } else {
           My->WritingMovie = !(My->WritingMovie);
           if(My->WritingMovie) cout<<"Now writing movie frames each "<<My->MovInt<<" timesteps."<<endl;
           else cout<<"Movie write suspended."<<endl;
       }
       break;
   case SoKeyboardEvent::Q:
	   if(My->ShiftIsDown) 
	   {   cout<<"Q: user quit program"<<endl;
		   My->CBMode=My->EXIT; // quit entirely if Q
	   }
	   else 
	   {  cout<<"q: user terminated Run"<<endl;
	      My->CBMode=My->READ_COMMAND;
	   }
	   My->Pausing=false; // or won't ever exit?
     break;
   case SoKeyboardEvent::P:
     My->Repeat = !My->Repeat;
	 cout<<"P pressed, turned pause ";
	 if(My->Repeat) cout<<"off"; else cout<<"on";
	 cout<<endl;
	 My->Pausing=false; // will turn on later
     break;
   case SoKeyboardEvent::MINUS:
     if (My->surfaceMaterial->transparency[0]>0.02)
       My->surfaceMaterial->transparency=My->surfaceMaterial->transparency[0]-0.02;
	 else My->surfaceMaterial->transparency=0.0;
     cout<<"decreasing the transparency, it is now "
       <<My->surfaceMaterial->transparency[0]<<endl;
     break;
   case SoKeyboardEvent::EQUAL: // oddly, this is plus sign
     if (My->surfaceMaterial->transparency[0]<0.98)
     My->surfaceMaterial->transparency=My->surfaceMaterial->transparency[0]+0.02;
	 else My->surfaceMaterial->transparency=1.0;
     cout<<"increasing the transparency, it is now "
       <<My->surfaceMaterial->transparency[0]<<endl;
     break;
   case SoKeyboardEvent::R:
	 if (My->WatchedLLGCell)  // later put angle & direction in script file? now 0.3
		 My->WatchedLLGCell->RotateMby(MVector(0.3,0,0.0)); //rotate about x, in screen plane
	     else cout<<"Can't rotate, watched cell isn't LLG cell"<<endl;
	     // change this to My->WatchedCell->rect->RotateEasy???????
     break;
   case SoKeyboardEvent::S: //Strobe mode toggle (was spin torque)
	   My->Strobe=!My->Strobe;
	   cout<<"Toggled strobe flag, now "<<My->Strobe<<endl;
     break;
   case SoKeyboardEvent::HOME:
      if (My->gameMode) // restart
	    { My->t=0.0; My->tDisp=0.0; My->nt=0; My->H_bias.SetToZero(); My->TermCrit=My->MX;	
		 if(My->WatchedLLGCell) My->WatchedLLGCell->M = MVector(-1,0.0,0.0); // hard-wire, for now
	     else EXIT1("watched cell isn't LLG cell");
		} // end if game
      if (My->movIn)
	    {
		My->CBMode = My->FORWARD; My->Pausing=true;
        My->this_frame = 0;
	    cout<<"starting from the beginning of the movie"<<endl;
	    }
      break;

     //My->CBMode=My->HOME; // no path!
     //break;
   case SoKeyboardEvent::W:
     My->writeSnapshot();
     break;
   case SoKeyboardEvent::X:
     My->ExportData(true,true);
     break;

   // case SoKeyboardEvent::F:// was spinb torque, now gone

  default:
    cout<<"incorrect key was pressed"<<endl;
    break;
  }  // end switch(GetKey)
 } // end function myKeyPressCB

 // This fn has COIN even in header, make separate MakeUpVectorGL() too
SoSeparator* Systm::MakeUpVector(real HeadLength,real HeadRad)
  // UpVector is arrow for fields & magnetization. Uses line, not cylinder.
   { // Cannot call this until COIN initialized!  (can't create SoSeparator)
  SoSeparator *UpVec=new SoSeparator; //initialize data member
  assert(UpVec); // will be unit vector in z direction, with cone point.
  SoCoordinate3 *myUpCoords;
  myUpCoords = new SoCoordinate3;
  assert(myUpCoords);
  SoLineSet *myLineSet;
  myLineSet = new SoLineSet;
  assert(myLineSet); 
  //width of line set by [most recent DrawStyle]->lineWidth.setValue(1.0);
  myUpCoords->point.set1Value(0,0,0,0);
  myUpCoords->point.set1Value(1,0,0.0,1.0-HeadLength);
  myLineSet->numVertices.set1Value(0,2);
  //set up arrow line coords and LineSet 
  SoCone *myCone = new SoCone;
  myCone->height = HeadLength;
  myCone->bottomRadius = HeadRad;
  SoTransform *myConeTransform = new SoTransform;
  assert (myConeTransform); // defaults to centered at origin?
  myConeTransform->translation.setValue(0,0,1.0-0.5*HeadLength); 
  myConeTransform->rotation.setValue(1/sqrt(2.0),0,0,1/sqrt(2.0));
  UpVec->addChild(myUpCoords);
  UpVec->addChild(myLineSet);
  UpVec->addChild(myConeTransform);
  UpVec->addChild(myCone);
  return UpVec;
} // end MakeUpVector()
#endif --COIN

#ifdef OPENGL
   gl_Vertex::gl_Vertex() {} // needed for new?
   gl_Vertex::gl_Vertex(MVector v) {vert[0]=v[0];vert[1]=v[1];vert[2]=v[2];
     vert[4]=1.0; color[0]=1.0; color[1]=0.5; color[2]=0.0; color[3]=0.4;} 
     // random guess for color, orange=Rg0
   vertindex::vertindex(){} // fails w/o trivial constructor, I DON'T know why.

   vertindex::vertindex(MVector MV, CellIDType Ind)
   {  // obvious constructor
      vertex1=MV; Index1=Ind;
   }
#endif // OPENGL

// now start fns which are used for both OPENGL and COIN
// but have #ifdef inside them for each
TextDisplay::TextDisplay(const string labelIn, const MVector &TextTranslation, 	 
	 // tried const char* labelIn, failed before, but it works for VectorDisplay so 
	 // it MUST be possible to make it work.
  float colorR, float colorG, float colorB)
{  // Sets up scene tree as follows: Node
	// Matl  TextTransform Text
	// (TextTransform  relative to origin.)
#ifdef COIN
  Node = new SoSeparator; //node that contains all the tree structure
  assert (Node);
  SoMaterial *TextMaterial = new SoMaterial; //material for the vector
  assert (TextMaterial);
  assert ((colorR<=1.) && (colorG<=1.) && (colorB<=1.0)); //sanity check
  TextMaterial->diffuseColor.setValue(colorR, colorG, colorB);
  Node->addChild(TextMaterial);

//to translate the text
  TextTransform = new SoTransform;
  assert (TextTransform);
  TextTransform->translation.setValue(TextTranslation.get_val());
  Node->addChild(TextTransform);

// we assume SoFont node was already added in the tree. If not, add here.
//put the label
  Text = new SoText2;   assert (Text);
  //strcpy(Text->string,labelIn); // copy to string
  //strcpy(label,labelIn); // copy to class member so it will be available later
  // ??SbString MySbString; MySbString.sprintf("%s",labelIn.c_str());
  // Text->string.sprintf("%s",labelIn.c_str());
  Text->string=labelIn.c_str();
  Node->addChild(Text);
  My->rootBase->addChild(Node); // add all in
#endif --COIN
} // end constructor TextDisplay

void TextDisplay::RepText(const string Text1)
{
#ifdef OPENGL
	cout<<" NumberDisplay is not implemented in OpenGL yet"<<endl;
#endif --OPENGL
#ifdef COIN
	Text->string=Text1.c_str();
#endif --COIN
}

NumberDisplay::NumberDisplay(const string PrefixIn, real DispFacIn, short precIn,
	real *NumIn, const string SuffixIn,
	//const char *formatIn, real Pow10In,
	//const string labelIn, const string unitsIn, 
   real xt, real yt, real zt, float colorR, float colorG, float colorB)
{  // Sets up scene tree as follows: Node
	// Matl  TextTransform Text
	// (TextTransform  relative to origin.)
  NumPtr=NumIn; DispFac=DispFacIn; 
  Prefix=PrefixIn; Suffix=SuffixIn; prec=precIn; 
  //cout<<"creating number display at "<<xt<<","<<yt<<":"<<Prefix<<
    // *NumIn<<" * "<<DispFac<<",prec="<<prec<<Suffix<<endl;
#ifdef OPENGL
	//cout<<" NumberDisplay is not implemented in OpenGL yet"<<endl;
#endif --OPENGL

#ifdef COIN
  Node = new SoSeparator; //node that contains all the tree structure
  assert (Node);
  SoMaterial *NDMaterial = new SoMaterial; //material for the number
  assert (NDMaterial);
  assert ((colorR<=1.) && (colorG<=1.) && (colorB<=1.0)); //sanity check
  NDMaterial->diffuseColor.setValue(colorR, colorG, colorB);
  Node->addChild(NDMaterial);

//to translate the ND
  SoTransform* NDTransform = new SoTransform;
  assert (NDTransform);
  NDTransform->translation.setValue(xt,yt,zt);
  Node->addChild(NDTransform);

// we assume SoFont node was already added in the tree. If not, add here.
//put the label
  Text = NULL; // so we can tell if it is created correctly
  Text = new SoText2;   assert (Text);
  //Text->string=labelIn; // just to initialize? unnecessary, don't display until changed
  Node->addChild(Text);
  My->rootBase->addChild(Node); // add all in
#endif --COIN
} // end constructor NumberDisplay

void NumberDisplay::UpdateND()
{   stringstream StSt;
    StSt.precision(prec); // or can put in << line?
    StSt<<Prefix<<fixed<<*NumPtr*DispFac<<Suffix;
	//StSt<<"Make this simple";
	//cout<<"In UpdateND, NumPtr="<<NumPtr<<", wrote "<<*NumPtr*DispFac<<endl;
	//DEBUG if(Prefix[1]=='1') cout<<"In UPdateND, this="<<this<<", Prefix="<<Prefix<<", StSt="<<StSt.str()<<endl;

#ifdef COIN
	//string MyString; // format
	//MyString=label; MyString+="=%7.5f"; MyString+=units;
	//stg=label; stg+="%7.5f "; stg+=units; // do we need this?????
    //stg.sprintf(Format,*NumPtr*1e-9);
	// Nov 2012: SbString MySbString;
	//MySbString.sprintf(Nformat,*NumPtr*Pow10); 	
	//Text->string=MySbString;
	// but now just
	// Text->string.sprintf(Nformat,*NumPtr*Pow10); // somehow thought sprintf in SoMFString?
	 // confused SoMFString with SbString??
	Text->string=StSt.str().c_str();
	//if(Prefix[1]=='1') cout<<"NumberDisplay's TextString="<<StSt.str()<<endl;
#endif --COIN
#ifdef OPENGL
	//cout<<"updateND is not implemented in OpenGL yet"<<endl;
#endif --OPENGL
} // end function updateND()

void NumberDisplay::RepPrefix(const string Text1)
{
#ifdef OPENGL
	cout<<" NumberDisplay is not implemented in OpenGL yet"<<endl;
#endif --OPENGL
#ifdef COIN
	Prefix=Text1;
#endif --COIN
}

VectorDisplay::VectorDisplay(const char *labelIn, short WhichCompsIn, short WhereLabel1,
   const MVector &VectorTranslation, real multFactorIn, 
  float colorR, float colorG, float colorB)
{  // Sets up scene tree as follows: VectorNode
	// Matl  OriginTr  TextSep             VectTrans  UpVector
	//                 TextTr VectorText
	// so VectorTextTransform and VectorTranform are relative to origin.
  multFactor = multFactorIn;
  WhichComps=WhichCompsIn;
  WhereLabel=WhereLabel1;
  strcpy_s(label,labelIn); // copy to class member so it will be available later
#ifdef OPENGL
  cout<<"VectorDisplay is not implemented in OpenGL yet"<<endl;
#endif --OPENGL

#ifdef COIN
  VectorNode = new SoSeparator; //node that contains all the tree structure
  assert (VectorNode);
  SoMaterial *VectorMaterial = new SoMaterial; //material for the vector
  assert (VectorMaterial);
  assert ((colorR<=1.) && (colorG<=1.) && (colorB<=1.0)); //sanity check
  VectorMaterial->diffuseColor.setValue(colorR, colorG, colorB);
  VectorNode->addChild(VectorMaterial);

//to translate the origin of the Vector and the label
  originTransform = new SoTransform;
  assert (originTransform);
  originTransform->translation.setValue(VectorTranslation.get_val());
  //cout<<"In VectorDisplay constructor, V..Transl="<<VectorTranslation<<
	//", try getValue:"<<", hit key" delete these 4 lines
	//<<",MV="<<MVector(originTransform->translation.getValue())<<endl;
  // originTransform->translation.getValue()
  //cin.get();

  VectorNode->addChild(originTransform);

// we assume SoFont node was already added in the tree. If not, add here.
//put the label
  SoSeparator *VectorTextSep = new SoSeparator;
  assert (VectorTextSep);
  VectorTextTransform = new SoTransform; // defined this, never set it <2008?
  assert (VectorTextTransform); // value set in UpdateVector
  VectorText = new SoText2;
  assert (VectorText);
  VectorTextSep->addChild(VectorTextTransform);
  VectorTextSep->addChild(VectorText);
  VectorNode->addChild(VectorTextSep);

//transformation of the UpVector to get the Vect
  VectorTransform = new SoTransform;
  assert (VectorTransform);
  VectorNode->addChild(VectorTransform);
  VectorNode->addChild(My->UpVector); //THIS LINE PRODUCES ERROR!!!!
    // which mysteriously went away.
#endif --COIN
} // end constructor VectorDisplay


//this function updates the value of the vector but does not change the origin
//it should be called anytime the vector changes (example in cell::update function)
void VectorDisplay::updateVector(const MVector &newVector) //newVector has units kA/m
{    
  //cout<<"newVector="<<newVector<<", multFactor="<<multFactor<<endl;
  real scaleFact = newVector.Magnitude()*multFactor; 
  real a, x, y, z; // OI gives SbMatrix error (det=0) if pass 0 to Transform->ScaleFactor:
  if(scaleFact<1e-15) // e-20 gives error, e-15 does not
  { scaleFact=1e-15; x=y=z=0; a=1;} // identity quaternion
  else
  {    Quaternion qRotation;  
       qRotation.rotateTo(newVector); // this also crashes if newVector=0
       qRotation.CopyTo(a,x,y,z);
  }

#ifdef OPENGL
  //cout<<"updateVector is not implemented in OpenGL yet"<<endl;
#endif --OPENGL

#ifdef COIN
  VectorTransform->rotation.setValue(x,y,z,a);
  VectorTransform->scaleFactor.setValue(scaleFact, scaleFact, scaleFact);

  //char VecText[30]; // plenty long
  SbString VecText;
  switch(WhichComps) { // which components to write
	  case -1: VecText.sprintf("%s",label);  // sprintf(VecText,"%s",label);  // just label
		break;
	  case 0: case 1: case 2:// only one comp, write label+newVector[WhichComps]
        //sprintf(VecText,"%s=%1.0f",label,newVector[WhichComps]); 
        VecText.sprintf("%s=%1.0f",label,newVector[WhichComps]); 
		break;
	  case 3: // all 3 comps
        VecText.sprintf("%s=(%1.0f,%1.0f,%1.0f)",label,
			newVector[0],newVector[1],newVector[2]); 
		break;
	  default: cout<<"error in WhichComps"<<endl; MYEXIT(26);
  } // end switch
  VectorText->string=VecText; // this is here rather than VectorDisplay constructor
  MVector TextTrans;          // in case I want posn conditional on value (!!)
  if (WhereLabel) // 1 means tip
	  TextTrans = newVector*multFactor+MVector (0.1, 0.1, 0.0);
  else // tail
	  TextTrans = MVector(0.4, 0.4, 0.0); // same extra disp 0.05, for now!
  VectorTextTransform->translation.setValue(TextTrans.get_val());
#endif --COIN
} // end fn. updateVector

MVector VectorDisplay::whereis()
{
#ifdef COIN
	return MVector(originTransform->translation.getValue());
#endif --COIN
#ifdef OPENGL
  cout<<"whereis is not implemented in OpenGL yet"<<endl;
  return MVector(6,6,6);
#endif --OPENGL
#ifndef GRAPHICS
  cout<<"Shouldn't call whereis() if no graphics"<<endl; cin.get();
#endif
  // return  MVector(6,6,6); // willnever get here
} // end whereis()

#ifdef COIN // these 2 functions are specifically COIN
  //need write another if needed with OPENGL
//this function returns the node to insert it into a tree
SoSeparator *VectorDisplay::getNode()
{
  return VectorNode;
}
  
#endif --COIN

// Define VectDisp as a scaled-down VectorDisplay with no label.
// Logically, VectorDisplay should descend from this? but lots diffs
// real VectDisp::multFactor=1.0; // Static vbl needs init outside!

VectDisp::VectDisp(const MVector &VectorTranslation,// real multFactorIn, 
  float colorR, float colorG, float colorB)
{  // Sets up scene tree as follows: VectorNode
	// Matl  OriginTr              VectTrans  UpVector
	// so VectorTranform is relative to origin.
  //multFactor = multFactorIn;
#ifdef COIN
  VectorNode = new SoSeparator; //node that contains all the tree structure
  assert (VectorNode);
  SoMaterial *VectorMaterial = new SoMaterial; //material for the vector
  assert (VectorMaterial);
  assert ((colorR<=1.) && (colorG<=1.) && (colorB<=1.0)); //sanity check
  VectorMaterial->diffuseColor.setValue(colorR, colorG, colorB);
  VectorNode->addChild(VectorMaterial);

//to translate the origin of the Vector
  originTransform = new SoTransform;
  assert (originTransform);
  originTransform->translation.setValue(VectorTranslation.get_val());
  VectorNode->addChild(originTransform);

//transformation of the UpVector to get the Vect
  VectorTransform = new SoTransform;
  assert (VectorTransform);
  VectorNode->addChild(VectorTransform);
  VectorNode->addChild(My->UpVector); 

  My->rootBase->addChild(VectorNode);
#endif --COIN
} // end constructor VectDisp

//this function updates the value of the vector but does not change the origin
//it should be called anytime the vector changes (example in cell::update function)
void VectDisp::updateVector(const MVector &newVector) //newVector has units nm
{    
  //cout<<"in VectDisp, newVector="<<newVector<<endl;
  real scaleFact = newVector.Magnitude(); // *multFactor gone now, mult before calling this; 
  real a, x, y, z; // OI gives SbMatrix error (det=0) if pass 0 to Transform->ScaleFactor:
  if(scaleFact<1e-15) // e-20 gives error, e-15 does not
  { scaleFact=1e-15; x=y=z=0; a=1;} // identity quaternion
  else
  {    Quaternion qRotation;  
       qRotation.rotateTo(newVector); // this also crashes if newVector=0
       qRotation.CopyTo(a,x,y,z);
  }
#ifdef OPENGL
  //cout<<"updateVector is not implemented in OpenGL yet"<<endl;
#endif --OPENGL

#ifdef COIN
  VectorTransform->rotation.setValue(x,y,z,a);
  VectorTransform->scaleFactor.setValue(scaleFact, scaleFact, scaleFact);
#endif --COIN
} // end fn. updateVector

 void cell::DisplayFaces() // this is more efficient than cell::DisplayFaces because 
	 // an LLGcell has cellfacelist, can do each face only once
 {  for (short i=0; i<6; i++)   
	  if(typeid(*this)==typeid(LLGcell)) // may require #include <typeinfo.h>?
	  {  LLGcell* ptr = static_cast<LLGcell*>(this);
	     if(ptr->cellfacelist[i].myFace){   // there is a face there 
			 if (ptr->cellfacelist[i].owned) DrawFace(i);
	     }
	     else DrawFace(i);// no face, is external
         // note each face owned by one & only one cell
	  }
	  else DrawFace(i); // if not LLGcell
 }

void cell::MakeSceneTree() // now called for each rectangle
   //display this cell, and its children recursively, in the real-space window.
{   //cout<<"Starting MakeSceneTree"<<endl;
	// Will eventually want to insert multiple COPIES of this cell, if periodic!
	// Put the below in loop over displacements?
	// Maybe multiple M arrows, NOT multiple cell faces, except use all faces
	// of the boundary cells.
	if((half<=rect->sizeToDisplay) || (!child0)) // small enough to display
	{
		if(!DispInfoPtr)
			DispInfoPtr= new DispInfo; // create place for display info
			//the following part deals with the magnetization vector magVector
#ifdef COIN
      SoSeparator *magVector;
      magVector = new SoSeparator;
      assert(magVector);

      //set the color for magVector
      SoMaterial *mymagMaterial; // color is local variable
      mymagMaterial = new SoMaterial;
      assert(mymagMaterial);
	  mymagMaterial->diffuseColor.setValue(1.0,0.0,0.0); //red
//	  if (child0) need fix this!! no showm
//		  if (child0->ShowM) // child is shown too; overwrite with distinct color (pink)
//		  mymagMaterial->diffuseColor.setValue(1.0,0.5,0.5); //pink for larger?
   
      //set linewidth of magVector? See DrawStyle.
      //SoDrawStyle *mymagDrawStyle=new SoDrawStyle;
      //assert(mymagDrawStyle);
      //mymagDrawStyle->lineWidth.setValue(1.0);

      //set magVector to the center of the cell
      DispInfoPtr->myMTransform = new SoTransform;
      assert (DispInfoPtr->myMTransform);
      DispInfoPtr->myMTransform->translation.setValue(this->center.get_val());

      magVector->addChild(mymagMaterial);
      //magVector->addChild(mymagDrawStyle);
      magVector->addChild(DispInfoPtr->myMTransform);
      magVector->addChild(My->MUpVector); // special size for M

      //now actually put the magVector into the real-space graph
     // cout<<"About to add M vector"<<endl;
      My->rootBase->addChild(magVector);	
#endif --COIN
	} //end of if(small enuf)

	else // don't display; check children
	{
		// what did this mean?: BEFORE displaying this cell, because we may want to USE children in display of this cell
      // recurse to children to see if they should be displayed
      if (child0) // has children
	    {child0->MakeSceneTree(); child1->MakeSceneTree();} 
	}

    if(DispInfoPtr)
	  // actually display cell, as "surface", a set of square faces.
      DisplayFaces(); // this is implemented efficiently for LLGcell, awkwardly for cell
      //loop faces, put in face set
        // ??? -- DON"T recurse to children, never display 2 levels
     
}//end of MakeSceneTree()

void Systm::LimitField() // for game mode, limit to FieldLimit
{
	real fac = H_bias.Magnitude()/FieldLimit;
	if (fac>1.0) H_bias/=fac;
}
#endif //GRAPHICS
