#ifndef DISPLAY_H
#define DISPLAY_H
#ifdef COIN 
#ifdef WIN32 // these are COIN versions of names
  #include <Inventor/Win/SoWin.h>
  #include <Inventor/Win/SoWinRenderArea.h>
  #include <Inventor/Win/viewers/SoWinPlaneViewer.h>
  #include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#else
  #include <Inventor/Xt/SoXt.h>
  #include <Inventor/Xt/SoXtRenderArea.h>
  #include <Inventor/Xt/viewers/SoXtPlaneViewer.h>
  #include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#endif --WIN32
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodekits/SoShapeKit.h>
#include <Inventor/actions/SoLineHighlightRenderAction.h>
#include <Inventor/nodes/SoSelection.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/SbColor.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoCallback.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoTriangleStripSet.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoIndexedLineSet.h>

#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoOrthographicCamera.h>

#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoNurbsCurve.h> // for quaternionDisplay
#endif COIN

//#include "Lowest.h"
class MVector;

#ifdef OPENGL
   class vertindex
   {   public:
       vertindex(MVector, CellIDType);
	   vertindex();
	   MVector vertex1; CellIDType Index1;
   };
   class gl_Vertex 
   {  public: 
	  real vert[4]; real color[4];
	  gl_Vertex();
	  gl_Vertex(MVector v);
   };
#endif // OPENGL

#ifdef GRAPHICS // these headers compile for both OPENGL and COIN
  // but right now don't do anything in OPENGL
class TextDisplay
{
public:  // TextTranslation is in units of 
  TextDisplay (const string labelIn, const MVector &TextTranslation,
         float colorR=1., float colorG=1., float colorB=1.);
  // static void initVectorSep();  //initialize static VectorSep
  //SoSeparator *getNode(); //returns Node, to addChild?
  void RepText(const string Text1); 
private:
  string label;//char label[20]; // text label, like "H", generous length limit
#ifdef COIN
  SoTransform *TextTransform; //transformation 
  SoText2 *Text;
  SoSeparator *Node; //node that contains all the tree structure
#endif --COIN
 }; // end class TextDisplay

class NumberDisplay 
{  // This creates a display of a single number.  These are specified
   // in script file in order, and kept in a list NumbDisps .
   // A subset of these are the "Xtra displays", starting at xND,yND
	// (see Systm.cpp case 'X') and continuing UPWARD.
	// yND increments with carriage return, xND stays same (left margin).
	// These can be in the same column, or you can start a new column (NCol=0).
public:
  //NumberDisplay (const string labelIn, string unitsIn,real *NumIn, 
  //NumberDisplay (const char *formatIn, real Pow10In, real *NumIn, 
  NumberDisplay (const string PrefixIn, real DispFacIn, short precIn,
    real *NumIn, const string SuffixIn,
    real xt, real yt, real zt, float colorR=0., float colorG=0., float colorB=0.);
  void UpdateND(); 
  void RepPrefix(const string);
//#define NformatLENGTH 31
//  char Nformat[NformatLENGTH]; // need specific length, for C string in sprintf
// DELETE  static short TimePrecision, Precision5, Precision3; // or could be in System?
  string Prefix, Suffix; // like "t=",8.7," ns"
  short prec;
private:
  real *NumPtr;
  real DispFac; // this must be 1e-3 to be consistent with "e3" in format
  //string label, units; // not here, loca;l?? 
#ifdef COIN
  SoTransform *NumberTransform; //transformation 
  SoText2 *Text;
  SoSeparator *Node; //node that contains all the tree structure
#endif --COIN
};
 
class VectorDisplay
{
public:
  VectorDisplay (const char *labelIn, short WhichCompsIn, short WhereLabel1,
	     const MVector &VectorTranslation, real multFactorIn,
         float colorR=1., float colorG=1., float colorB=1.);
  static void initVectorSep();  //initialize static VectorSep
    // this function is never defined or used! Compiler doesn't complain!
  void updateVector(const MVector &newOrigin, const MVector &newVector);
  void updateVector(const MVector &newVector);//no change in the origin
  MVector whereis();
#ifdef COIN
  SoSeparator *getNode(); //returns VectorNode
  SoTransform *VectorTransform; //rotate and scale VectorSep
  SoTransform *VectorTextTransform; //transformation of the label
  SoText2 *VectorText;
  SoTransform *originTransform; //origin of the vector
  SoSeparator *VectorNode; //node that contains all the tree structure
#endif ---COIN
private:
  char label[13]; // text label, like "H", generous length limit
  real multFactor;  //unit for displaying this vector
  short WhichComps; // -1=none, 012=xy or z, 3=write all 3
  short WhereLabel; // 0 for tail, 1 for tip
}; // end class VectorDisplay

class VectDisp // short version of VectorDisplay, for local field
{
public:  // need ; after these?? none in MVector.h!
  VectDisp (const MVector &VectorTranslation, // real multFactorIn,
         float colorR=1., float colorG=1., float colorB=1.);
  void updateVector(const MVector &newVector);//no change in the origin
#ifdef COIN
  SoSeparator *getNode(); //returns VectorNode
  SoTransform *VectorTransform; //rotate and scale VectorSep
  SoTransform *originTransform; //origin of the vector
  SoSeparator *VectorNode; //node that contains all the tree structure
#endif --COIN
    // was private:
  // static real multFactor; now gone //ONLY ONE! Unlike VectorDisplay
  //void MultMultFactor(float fac){multFactor*=fac;};
  //void SetMultFactor(float fac){float x=fac;}; 
}; // end class VectDisp
#endif --GRAPHICS
#endif // ifndef DISPLAY_H