// this is main.C file of AlaMag micromagnetics simulator v. >= 1.9
// (for actual version number see Systm::Systm(){Version=...} in Systm.cpp)
// Copyright(C) 1999-2009 Pieter B. Visscher, U of Alabama, AlaMag@mint.ua.edu
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include "Lowest.h"
#include "Systm.h"
#include "FMM.h"
#include "cell.h"
#include <omp.h>
#ifdef GRAPHICS 
#include "display.h"
#endif

Systm *My = new Systm(); 

// There are 2 versions of main(), one for AlaMag and one for SeaPlug.
#ifdef H_ONLY 
   // this is the SeaPlug version
int main(short argc,char **argv){
	cout<<"This is SeaPlug main()"<<endl;
} // end main(SeaPlug)
#else 
   // this is the regular version
int main(short argc,char **argv){
#ifdef _DEBUG
    ofstream Errorlog; // for startup errors
    Errorlog.open("error.log"); 
	if(Errorlog) cout<<"opened"; else cout<<"Failed to open"; 
	cout<<" error log"<<endl;
	Errorlog<<"This file logs errors in startup."<<endl;
	Errorlog<<"Starting with "<<argc<<
		" arguments (2 means script file specified)"<<endl;
#ifdef _OPENMP
	cout<<"Compiled with Open MP -- detected "<<omp_get_num_procs()<<" cores."<<endl;
	/* File reading etc. must be done serially -- start off parallelizing the easy things first. 06/2013
	TODO: get a single "#pragma omp parallel" to cover as much code as possible to minimize thread creation/destruction overhead. */
	omp_set_nested(1); 
   omp_set_num_threads(1); cout<<"Turn off omp"<<endl;
   //omp_set_num_threads(2); cout<<"Turn on omp"<<endl;
   // these settings necessary for parallel recursion, will adjust to have same number threads as logical cores
#endif _OPENMP
#endif _DEBUG

   cout<<"Starting micromagnetics program AlaMag, Copyright(2009) P. B. Visscher, v. "
	  <<My->Version<<endl;
   cout<<"This is free software -- see GPLicense.txt for details."<<endl;
   cout.precision(DIGITS);

   cout<<"My->ExeDir="<<My->ExeDir<<", argc="<<argc<<", argv[0]="<<argv[0]<<endl;
   My->ExeDir.append(argv[0]); // is path to .exe;remove filename to get dir
   My->ExeDir.erase(My->ExeDir.find_last_of('\\')+1); //remove \alamag.exe
   // cout<<"Will look for kernel in .exe Directory:"<<endl;
   // cout<<My->ExeDir<<endl;
   string ParFileString; // empty string
   if (argc>1) // then had filename on command line, turn into string:
   {for(short argi=1;;argi++) 
      {ParFileString.append(argv[argi]); 
       // cout<<"argi="<<argi<<",ParFileString="<<ParFileString<<endl;
	   if ((argi+1)==argc) break; // out of for loop
	   cout<<"Too many command line arguments, hit key"<<endl;
	   cin.get(); // I have no idea (7/09) what this was for, but won't be used:
	   ParFileString+=' '; // in case there are blanks in file or dir names!
	   } // end for argi
	  cout<<"From command line, read script file name:"<<endl;
	  My->ScriptDir.append(argv[1]); // should be same; use for writing
     // remove filename to get directory path:
     My->ScriptDir.erase(My->ScriptDir.find_last_of('\\')+1);
	} // end if argc>1
   else // no filelname, use default
   {ParFileString=string("default.script"); // in working directory
   //My->ScriptDir.append(argv[0]); // used directory of .exe file?
   My->ScriptDir=string("");// now use working directory for outputs too
   cout<<"Using default input:";
   } // end else no script filename

  cout<<ParFileString<<endl;
#ifdef _DEBUG
  Errorlog<<"will read "<<ParFileString<<endl;
#endif

  cout<<"Will write to ";
  if(ParFileString=="") cout<<"working directory"<<endl; 
    else cout<<"script file's Directory "<<My->ScriptDir<<endl;

  // open parameter file on command line, or default:  
   My->ParFileString = ParFileString;
  My->paramFile.open(ParFileString.c_str()); // open requires c_string
#ifdef _DEBUG
  if(My->paramFile) Errorlog<<"Opened";
  else {Errorlog<<"Cannot open";cout<<"Cannot open";}
  Errorlog<<" script file "<<ParFileString<<endl; 
#endif
  if(!My->paramFile) EXIT1("Cannot open script file "+ParFileString);
  cout<<"Opened script file "<<ParFileString<<endl; 

  // now parse ParFileString to get runID; erase up to last backslash
  size_t posn=ParFileString.find_last_of("\\",ParFileString.length());
  if (posn<ParFileString.length()) ParFileString.erase(0,posn+1);

  string Suffix = ParFileString.substr(ParFileString.length()-7,7);
  if (Suffix.compare(".script")) // compare=0 if same, else exit
     EXIT1("Input file "+ParFileString+", suffix should be .script");
  My->runID = ParFileString.erase(ParFileString.length()-7,7); // removes .script from end
  //cout<<"Set runid = "<<My->runID<<endl;
#ifdef _DEBUG
  Errorlog<<"Set runid = "<<My->runID<<", about to call Systm::init()"<<endl;
#endif
  // create and initialize some global things
  // My->MVector0.SetToZero();

  real r3,r4;
  short pow2=-10; My->MachineAcc=1.0/1024.0;
  do
  {  My->MachineAcc/=2.0; pow2-=1;
	 r3=1.0+My->MachineAcc;
     r4=r3 - 1.0;
  } while (r4>0.0);
  My->MachineAcc*=2.0; pow2+=1;// was just too small
  cout<<"MachineAcc="<<My->MachineAcc<<" =2^"<<pow2<<endl;
  
  My->ReadScript(); // ReadScript() is in file Systm.cpp; reads paramFile
  My->prepareExit(); //close files and output final information Shuxia
  cout<<"Program terminated"<<endl;
  return 0;  //successful program completion
} // end program main
#endif !H_ONLY

