//mpole.C file

#include "Lowest.h"
#include "FMM.h"
#include "mpole.h"
#include "Systm.h"  //DMAfs

//  extern Systm *My;  //DMAfs

//overloaded operator to output mpole
ostream &operator<<(ostream &out, const mpole &right)
{
  out<<" maximum order is "<<right.order<<"\nvalues:\n";
  for (short orderTemp = 0; orderTemp <= right.order; orderTemp++)
  {
    out<<"  order "<<orderTemp<<" ( ";
  //start from i=0 if orderTemp==0
    for (short i = (orderTemp>0 ? My->MyFMM->NumBelow[orderTemp-1] : 0); 
      i<My->MyFMM->NumBelow[orderTemp]; i++)
      out<<right.values[i]<<"   ";
    out<<") \n";
  }
  return out;
}

//overloaded operator to input mpole
istream& operator>>(istream& input, mpole& right)
{
  char junk[20]; // perhaps use string for consistency with other files
  short orderTemp;
  input>>junk>>junk>>junk;
  assert ((junk[0] == 'i') && (junk[1] == 's'));  //must be "is"
  input>>right.order;
  input.getline (junk, '\n');
  input.getline (junk, '\n');
  assert (junk[0] == 'v');  //must be "values"

  for (short orderCur = 0; orderCur <= right.order; orderCur++)
  {
    input>>junk>>orderTemp;
    assert (orderTemp == orderCur);
    assert (junk[0] == 'o'); //must be "order"
    input>>junk;

    for (short i = (orderTemp>0 ? My->MyFMM->NumBelow[orderTemp-1] : 0); 
        i<My->MyFMM->NumBelow[orderTemp]; i++)
      input>>right.values[i];
    input.getline (junk, '\n'); 
  }
  return input;
}

mpole::mpole(short orderIn)//default constructor
{
  if(orderIn<0) // -1 is flag for UsedOrder(<=MaxOrder)
	  order = My->MyFMM->UsedOrder; // default value
  else order = orderIn;    
  values = new real[My->MyFMM->NumBelow[order]];
  assert(values);
  for (short i = 0; i < My->MyFMM->NumBelow[order]; i++)//initialize all values to zero
    values[i] = 0.0;
    // cout<<"created mpole @"<<(short)this<<", values@"<<values<<", order="<<order<<endl;
}

/*
mpole::mpole (const mpole &right) // copy constructor
{
  order = right.order;
  values = new real[My->MyFMM->NumBelow[order]];
  assert (values);

  for (short i=0; i < My->MyFMM->NumBelow[order]; i++)
    values[i] = right.values[i];
  // cout<<"Copy-created mpole @"<<(short)this<<", values@"<<values<<", order="<<order<<endl;
}
*/
mpole::~mpole()//destructor
{
  // cout<<"Destroyed mpole @"<<(short)this<<", values@"<<values<<", order="<<order<<endl;
  delete [] values;//need to deallocate memory
  //values = NULL; // needed? isn't this automatically deleted?
}


real& mpole::GetValue(short i, short j, short k)   //returns value[i][j][k]
{
  assert ((i+j+k)<=order);
  return values[My->MyFMM->index[i][j][k]];
}


const real& mpole::GetValue(short i, short j, short k) const  //constant version
{
  assert ((i+j+k)<=order);
  return values[My->MyFMM->index[i][j][k]];
}


void mpole::SetToZero()//set all values to zero
{
  for (short i = 0; i<My->MyFMM->NumBelow[order]; i++)
    values[i] = 0.0;
}

bool mpole::IsNonZero()
{  for (short i = 0; i<My->MyFMM->NumBelow[order]; i++)
    if(values[i]!=0.0) return true;
   return false;
};

real mpole::compare(const mpole *right)//compare mpole right with this
{
  real error = 0.0, errorCurrent;
  assert (order==right->order);  //we can compare mpoles with the same order
  for (short i = 0; i<My->MyFMM->NumBelow[order]; i++)
  {
    errorCurrent = fabs(values[i] - right->values[i]);
    if (errorCurrent > error)
      error = errorCurrent;
  }
  return error;  //returns the maximum difference between the two mpoles
}


void mpole::rescale (real factor) //divide values by factor^(n+1)
{
  assert (factor!=0.);
  short curOrder=0;
  real term = 1./factor;

  for (short i=0; i < My->MyFMM->NumBelow [order]; i++)
  {
    if (My->MyFMM->IndArr[i].sum() > curOrder)    //order is increased
    {
      term/=factor;
      curOrder++;         
    }
    values[i]*=term;
  }
  assert (curOrder == order);
}

void mpole::InsertVector(const MVector &c)
//regard vector as a special case of mpole
{
  values[1] = c[0];
  values[2] = c[1];
  values[3] = c[2];
}


void mpole::SetToSimplePowers(const MVector &c)
//this = cx^nx*cy^ny*cz^nz
{
  SetToZero();
  values[0]=1.;
  values[1]=c[0];
  values[2]=c[1];
  values[3]=c[2];

  for (short i=4; i < My->MyFMM->NumBelow[order]; i++)
    values[i]=values[My->MyFMM->RecurseMpole[i]]*c[My->MyFMM->RecurseAxis[i]];
}

void mpole::SetToPowers(const MVector &c)
{  // this[n] = cx^nx*cy^ny*cz^nz / nx!ny!nz!
  SetToZero();
  values[0]=1.;
  values[1]=c[0];
  values[2]=c[1];
  values[3]=c[2];

  for (short i=4; i < My->MyFMM->NumBelow[order]; i++)
    values[i]=values[My->MyFMM->RecurseMpole[i]]*c[My->MyFMM->RecurseAxis[i]]
             *My->MyFMM->RecurseFactor[i];
}

void mpole::AddCarefully(const int3 pVec, const real term1)
{  short i=My->MyFMM->index[pVec[0]][pVec[1]][pVec[2]];
	values[i]+=term1; // check for exact cancellation: 
	if(abs(values[i])<0.001*abs(term1)) values[i]=0.0; // remove roundoff error
}

void mpole::SetToDeriv(const MVector &c, mpole* scratch2)
{   //this = D_n(r) in equation for derivatives
   // Order requirements: this->order controls output; scratch2->order >= this->order
  SetToZero();
  scratch2->SetToSimplePowers(c); // should have order MaxOrder+ExtraOrderD
#ifdef DEBUG4
  if(My->PD4) scratch2->printMP("In SetToDeriv, c & Powers",cout);
#endif // DEBUG4
  assert(c.Square());//to check that c is non-zero
  real c2 = 1.0/c.Square();
  real factor = sqrt(c2); // =1/c
  short iEnd = 0, iBegin = 0;// why need set iBegin??

  for (short ord1 = 0; ord1 <=order; ord1++)//loop order
  {//such initialization is needed for speeding up the performance
    iBegin = iEnd; iEnd = My->MyFMM->NumBelow[ord1];
    for (short i = iBegin; i < iEnd; i++)
    {  
#ifdef DEBUG4
      if(My->PD4) cout<<"Calculate D["<<i<<" = "<<My->MyFMM->IndArr[i]<<"]"<<endl;
#endif // DEBUG4
      real D = 0.0;
      for (short j = 0; j < My->MyFMM->DerivTerms[i]; j++)
	   { assert(My->MyFMM->DerivPower[i][j]<My->MyFMM->NumBelow[scratch2->order]);
#ifdef DEBUG4
		if(My->MyFMM->DerivPower[i][j]>=My->MyFMM->NumBelow[scratch2->order])
			{cout<<"Error, DerivPower["<<i<<"]["<<j<<"] >= NumBelow["<<scratch2->order<<
		    	"]"<<endl; MYEXIT(71);}
	    if(My->PD4)
        if(abs((My->MyFMM->DerivCoef[i][j])*scratch2->values[My->MyFMM->DerivPower[i][j]])>1e-30) 
          cout<<"Add to D["<<i<<"] ="<<D<<" term # "<<j<<", Coef="<<My->MyFMM->DerivCoef[i][j]<<
              ", *r^"<<My->MyFMM->IndArr[My->MyFMM->DerivPower[i][j]]<<" = "
              <<scratch2->values[My->MyFMM->DerivPower[i][j]]<<endl;
           // should write D[i]=?, term # j, coef=, r^(1,0,0) = ?
#endif // DEBUG4
       D += (My->MyFMM->DerivCoef[i][j])*scratch2->values[My->MyFMM->DerivPower[i][j]];
	   }
      values[i] = D*factor;
#ifdef DEBUG4
	  if(My->PD4) cout<<"Got D="<<D<<" *1/c^"<<2*ord1+1<<" (="<<factor<<") gives Der= "<<values[i]<<endl;
#endif // DEBUG4
    }
    factor *= c2; // so factor = 1/c^(2*ord1+1)
  }
}

void mpole::AddTaylor(const mpole *Q, const MVector &c, mpole* scratch1, mpole* scratch2)
{ // order requirements: Q controls order; this->order >= Q->order; s2>=s1>=Q+ExtraOrderD
  //cout<<"AddTaylor, mpole[1]="<<Q->values[1]<<", dist="<<c<<endl;
#ifdef _DEBUG
   if(order < Q->order) MYEXIT(31);
   if(scratch1->order < Q->order+My->MyFMM->ExtraOrderD) 
      MYEXIT(32);
   if(scratch2->order < Q->order+My->MyFMM->ExtraOrderD) MYEXIT(33);
#endif //_DEBUG
   scratch1->SetToDeriv(c,scratch2); // need scrat2 >= scrat1 >= Q+ExtraOrderD
#ifdef DEBUG4
  if(My->PD4) scratch1->printMP("After SetToDeriv",cout);
#endif // DEBUG4
  AddDownConv(scratch1, Q, My->MyFMM->ExtraOrderD); // needs scrat1 >= Q + ExtraOrderD
#ifdef DEBUG4
  if(My->PD4) printMP("After AddTaylor",cout);
#endif // DEBUG4
}

void mpole::AddUpConv(const mpole *m1, const mpole *m2)
{ // add up-convolution of m1 and m2 (see Eq. (9) of paper) to this mpole.
  assert(m2->order>=order); // check enuf room in m2
  short im,ip;
  for (im = 0; im < My->MyFMM->NumBelow[order]; im++) //? im was changed to start from 1
  {                                             // (we don't have monopoles)
    for (ip = 0; ip < My->MyFMM->NumBelow[order - My->MyFMM->IndArr[im].sum()]; ip++)
      values[My->MyFMM->ConvSum[im][ip]]+=m1->values[ip]*m2->values[im];
  }//end for im-loop
}

void mpole::AddDownConv(const mpole *m1, const mpole *m2, short Extra1)
{ // orders: output order same as m2; output mpole can be bigger, just leave blanks.
   // Requirements: this->order >= m2->order, m1->order >= m2->order + Extra1
#ifdef _DEBUG // like assert, but can insert breakpoint:
   if(m1->order < m2->order+Extra1)
       cout<<"Error, too small m1 order "<<m1->order<<endl; 
    if(order < m2->order)
       cout<<"Error, too small order "<<order<<endl; 
#endif //_DEBUG
   short im, ip; // index im goes with "this", ip with m2, 
   // m1 must be at least as big as m2; if it's bigger, that's a flag that we're
   // calculating D (use ExtraOrderD); if same size, it's just moment translation
   // delete: if(m1->order>m2->order) ordermp=m2->order+My->MyFMM->ExtraOrderD; else ordermp=m2->order;
   short orderOK, ordermp = m2->order + Extra1;
   // ordermp is order allowed for m1
   assert(m1->order >= ordermp); // m1 can be arbitrarily large, but at least this

 for (im = 0; im < My->MyFMM->NumBelow[m2->order]; im++)  //im was changed to start from 1
  {                                  // (we don't have monopoles)
	orderOK=ordermp - My->MyFMM->IndArr[im].sum(); 
	if (orderOK > m2->order) orderOK=m2->order;
	// someday after we're sure whether to use ExtraOrderD, we can simplify this stuff!
    for (ip = 0; ip < My->MyFMM->NumBelow[orderOK]; ip++)
	{ assert(My->MyFMM->ConvSum[im][ip] < My->MyFMM->NumBelow[m1->order]);
	  assert(My->MyFMM->ConvSum[im][ip]>=0);
	  assert(ip < My->MyFMM->NumBelow[m2->order]); // must fit in m2

#ifdef DEBUG4 
	if(My->PD4) 
        if(abs(m1->values[My->MyFMM->ConvSum[im][ip]]*m2->values[ip])>1e-30) 
          cout<<"AddDownConv"<<My->MyFMM->IndArr[im]<<" ="<<values[im]<<
		    ", add m1"<<My->MyFMM->IndArr[My->MyFMM->ConvSum[im][ip]]<<"="<<m1->values[My->MyFMM->ConvSum[im][ip]]
		     <<"*m2"<<My->MyFMM->IndArr[ip]<<"="<<m2->values[ip]<<endl;
#endif DEBUG4
      values[im] += m1->values[My->MyFMM->ConvSum[im][ip]]*m2->values[ip];
	}
  }//end for im-loop
}//end of AddDownConv

void mpole::AddMult(const mpole *m1, const real &r) // adds a multiple of another multipole
{
  assert(order <= m1->order); //sanity; extra order of m1 is ignored
  for (short i = 0; i < My->MyFMM->NumBelow[order]; i++) 
  {values[i] += m1->values[i]*r;
  }
}//end of AddMult

void mpole::AddShift(const mpole *Q, const MVector &c)
{
  My->MyFMM->scratchUpM->SetToPowers(c);
  AddUpConv(My->MyFMM->scratchUpM, Q);
}

/* now there's better print
void mpole::print0(char * title, ostream &out)//output an array
{      // was ofstream, to file; not used??
  //we want to restore the initial settings
  long originalPrecision = out.precision(); 

  out.precision(4);
  out<<title<<" is:";
// zero order:
  out<<"\n\torder = 0: "<<values[0]<<endl;
//first order:
  out<<"\n\torder = 1:  "<<values[1]<<"  "<<values[2]<<"  "<<values[3]<<endl;
//second order: output as a matrix:  
  out<<"\n\torder = 2:  "<<values[4]<<"  "<<values[5]<<"  "<<values[7]
     <<"\n\t            "<<values[5]<<"  "<<values[6]<<"  "<<values[8]
     <<"\n\t            "<<values[7]<<"  "<<values[8]<<"  "<<values[9]<<endl;
  for (short orderTemp = 3; orderTemp <= order; orderTemp++)
  {
    out<<"\n\torder = "<<orderTemp<<":\n";
    for (short i = My->MyFMM->NumBelow[orderTemp-1]; i < My->MyFMM->NumBelow[orderTemp]; i++)
      out<<"\t\tn = "<<My->MyFMM->IndArr[i][0]<<" "<<My->MyFMM->IndArr[i][1]<<" "
        <<My->MyFMM->IndArr[i][2]<<"\t"<<values[i]<<endl;
  }
  out.precision(originalPrecision);
} // end fn print()
*/
void mpole::printMP(char * title, ostream &out, unsigned short Num)//output an array
{            // was ofstream, to file
  out<<title<<" = "<<endl;
  real AbsV, Max=0.0; 
  short NumPrinted=0;
  for (short i=0; i<My->MyFMM->NumBelow[order]; i++)
  { AbsV=abs(values[i]);
    if(AbsV>My->MachineAcc*Max) // omit insignificant ones
    { 
      if(AbsV>Max) Max=AbsV; 
      NumPrinted++;
      if (NumPrinted>Num) {out<<"..."<<endl; return;}
      out<<setw(2)<<i;
      My->MyFMM->print("", My->MyFMM->IndArr[i]); // FMM::print
      out<<"-> "<<setw(10)<<values[i]<<endl;
    }
  }
}  // end printMP

//dot operator for mpoles
real mpole::operator* (const mpole& right) const
{
  short smallestOrder = order;
  if (smallestOrder > right.order)
    smallestOrder = right.order;

  real dotProduct = 0.0;

  for (short i=0; i<My->MyFMM->NumBelow[smallestOrder]; i++)
    dotProduct += values[i]*right.values[i];

  return dotProduct;
}
/*
//Plan B: dot operator for mpole*MVector
real operator* (const mpole& m, const MVector& right)
{
  assert(m->order>0); // need vector part of mpole
  real dotProduct = 0.0;
  for (short i=0; i<VDIMENSION; i++)
    dotProduct += m->values[i]*right[i];
  return dotProduct;
}
*/

// Plan A for dot operator for mpole*MVector
real mpole::operator* (const MVector& right) const // call as m*v?
{
  assert(order>0); // need vector part of mpole
  real dotProduct = 0.0;
  for (short i=0; i<VDIMENSION; i++)
    dotProduct += values[i+1]*right[i];
  return dotProduct;
}


//multiplication by a real number: // Used in calcVshell, as (*m)*r
mpole operator* (const mpole& m, const real &r)
{
  mpole result (m.order);
  for (short i=0; i < My->MyFMM->NumBelow [result.order]; i++)
    result.values[i] = m.values[i] * r;
  return result;
}

//division by a real number:
mpole mpole::operator/ (const real &r) const
{
  mpole result (order);

  for (short i=0; i < My->MyFMM->NumBelow [order]; i++)
    result.values[i] = values[i] / r;

  return result;
}

mpole mpole::operator+ (const mpole& m) const
{
  assert (order == m.order);
  mpole result (order);

  for (short i=0; i<My->MyFMM->NumBelow[order]; i++)
    result.values[i] = values[i] + m.values[i];

  return result;
}

const mpole& mpole::operator+= (const mpole &m)
{
  assert (order==m.order);

  for (short i=0; i<My->MyFMM->NumBelow[order]; i++)
    values[i] += m.values[i];

  return *this;
}

const mpole& mpole::operator*= (const real& r)
{
  for (short i=0; i<My->MyFMM->NumBelow[order]; i++)
    values[i] *= r;

  return *this;
}

const mpole& mpole::operator= (const mpole& m)
{
  assert (m.order==order);  //to catch errors

  for (short i=0; i<My->MyFMM->NumBelow[order]; i++)
    values[i] = m.values[i];

  return *this;
}

#ifndef H_ONLY
//used for periodic FMM (DMApf)
void mpole::SetToPowerSym (const bool* periodicity)
{
//  cout<<"\n\nStarting Sym"<<endl;
  mpole *tempMpole = new mpole();
  int3 t;
  SetToZero ();

  //sum over all t's:
  for (t.values [0]=(periodicity[0] ? -1 : 0);
       t.values [0]<= (periodicity[0] ? 1 :0); t.values[0]++)
    for (t.values [1]=(periodicity[1] ? -1 : 0);
         t.values [1]<= (periodicity[1] ? 1 :0); t.values[1]++)
      for (t.values [2]=(periodicity[2] ? -1 : 0);
           t.values [2]<= (periodicity[2] ? 1 :0); t.values[2]++)

      {
        tempMpole->SetToPowers (MVector (t[0]*My->MyFMM->reducedSize[0],
          t[1]*My->MyFMM->reducedSize[1], t[2]*My->MyFMM->reducedSize[2])*(-.5));
        *this += (*tempMpole)*My->MyFMM->calculateSymFactor(t);
      }
  delete tempMpole;
}
#endif  //  H_ONLY
