
#include "Lowest.h" // has iostream, math,..
#include "cell.h"
#include "pulse.h" // tho is inside Systm.h

#include "Systm.h" // for My
#include <sstream>
extern Systm *My;

real PulsedField::RiseFunction(real t)
{
   real x;
   switch (PulseType)
   {
      case 1: x=(t-StartTime)/RiseTime;
		break;
      case 2: x=(exp(t-StartTime)-1)/(exp(RiseTime)-1);
		break;
      default: cout<<"error in the choice of pulse type at t="<<t<<"\n";
	       MYEXIT(52);
   }
   return x;
}

real PulsedField::FallFunction(real t)
{
   real x;
   switch (PulseType)
   {
      case 1: x=(StartTime+TotalTime-t)/FallTime;
		break;
      case 2: x=(exp(-t+StartTime+RiseTime+PeakTime)-exp(-FallTime))/(1-exp(-FallTime));
		break;
      default: cout<<"error in the choice of pulse type%%%\n";
	       MYEXIT(53);
   }
   return x;
}

void PulsedField::Read(ifstream *File)
{
   real Hx,Hy,Hz;
   *File>>PulseType>>Hx>>Hy>>Hz>>StartTime>>RiseTime>>FallTime>>PeakTime;
   //Hx *= KOe_to_kA_per_meter;
   //Hy *= KOe_to_kA_per_meter;
   //Hz *= KOe_to_kA_per_meter;
   TotalTime = RiseTime+PeakTime+FallTime;
   Hmax = MVector(Hx, Hy, Hz);
}

MVector PulsedField::GetValue(real time)
{
	   switch (PulseType)
   {
      case 1: //trapezoidal, same as 2= expo
      case 2:
    real ampl; // normalized so max is 1
   if ((time > StartTime) && (time < StartTime+TotalTime))
   {
      if (time < StartTime+RiseTime)
         ampl = RiseFunction(time);
      else if (time < StartTime+RiseTime+PeakTime)
         ampl = 1.0;
      else
         ampl = FallFunction(time);
   }
   else
      ampl = 0;
   // cout<<"about to return from PulsedField:GetValue, ampl="
        // <<ampl<<", Hmax="<<Hmax<<endl;
   return (Hmax*ampl);
   break;
	 
   default: cout<<"error in the choice of pulse type";MYEXIT(54);
}
} // end GetValue

// term functions:
Term::Term(Mode* M1){ItsMode=M1;} // constructor

void Term::ReadTerm(ifstream& File,Rectang* rec) 
{
	// have already read index:
	File>>Sinus[0]>>Sinus[1]>>Sinus[2]>>kVectorTerm>>TermAmp>>Phase; // someday overload >> for int3!
	//cout<<"in ReadTerm, read Sinus="<<Sinus<<" and kVectorTerm="<<kVectorTerm<<endl;
	//cout<<"   and TermAmp="<<TermAmp<<", Phase="<<Phase<<endl;
	Phase=2*M_PI*Phase; // was in units of 2pi
	numOfSinusDir=0; // initialize to count
 	for (short i=0;i<3;i++) 
	{   if(Sinus[i]) {numOfSinusDir++;
	      kVectorTerm[i]*=(M_PI/rec->WholeRect->half[i]); // multiply short by 2pi/L
          }
	else if(kVectorTerm[i]!=floor(kVectorTerm[i])) 
	{cout<<"Exponent must be integer! "<<kVectorTerm<<endl; MYEXIT(59);}
	} // end for short
}

void Term::DebugWrite()
{
cout<<"New Term, kVectorTerm="<<kVectorTerm<<", TermAmp="<<TermAmp<<endl;
cout<<"  Phase="<<Phase<<endl;
}

// void Term::AddTermTo(MVector &V,cell* ce) {V+=TermAmp*IntOverCell(ce);}

MVector Term::EvalAtPoint(cell *WRect,MVector &r)
{ // WRect (Whole Rect) points to rectangle containing r
   real Monomial=1.;
   real DistFromCenter;
   assert(ItsMode);
   real Angle=Phase; // Term::Phase; someday add omega*t for AC?
   for (short dir=0;dir<3;dir++) // loop directions
      {
	  DistFromCenter=r[dir]-WRect->center[dir]; // from ctr of rect
	  DistFromCenter/=WRect->half[dir]; //in units of half rect size
      if (Sinus[dir]) // kx*x in angle, 
         Angle+=kVectorTerm[dir]*DistFromCenter*M_PI;
      else // not sinusoidal
	  {
		 int kInt=(int)kVectorTerm[dir]; // treat as integer exponent
         Monomial*=pow(DistFromCenter,kInt); // power of coordinate
	  }
	} // end for dir
   return TermAmp*cos(Angle)*Monomial;
} // end EvalAtPoint

MVector Term::IntOverCell(cell* ce) 
{  // cell must be in rectangle for which this is a term!
 // simpleminded temporary substitute for IntOverCell
 return EvalAtPoint(ce->rect->WholeRect,ce->center)*ce->volume(); // wasteful because later divide by volume, 
     // but in real function below, not a problem.

/* comment out real function, replace above by simple eval at center
{ // complicated logic, see notes p. 6.61-6.614
   real Int=0;
   real TermVal;
   short NumMinus=0;
   cell* rec=ce->rect->WholeRect; // for ease of reference
   for (short bits=0;bits<8;bits++) // have two terms per axis, bit=1&0, from left & right edges
      {
	  real Angle=0;
      if (Sinus[0]) // 
	  {	 // kx*x in angle, there are 2 terms; 0.5-bits&4=+ or - 0.5 is disp. from center
         Angle+=kVectorTerm[0]*(ce->center[0]+(0.5-(bits&4))*ce->size[0]-rec->center[0]); //upper right or LL
		   // Note that distance is relative to CENTER of rectangle, for grads
            // there is a function LowerLeft, but it calculates all components so is inefficient here.
         NumMinus=+bits&4; // sign changes with each LL term;
       }
         else // not sinusoidal
            if(bits&4) goto NextBits; // only count term in Int if this bit=0; 1/2 as many terms.
             // "break" here breaks out of for loop, not what we want!

// copy above for y & z, replace 0 by 1 or 2, bits&4 by 2 or 1 too.
	     if (Sinus[1])
         { // sinusoidal in x, there are 2 terms; 0.5-bits&4=+ or - 0.5 is disp. from center
         Angle+=kVectorTerm[1]*(ce->center[1]+(0.5-(bits&2))*ce->size[1]-rec->center[1]); //upper right or LL
            // there is a function LowerLeft, but it calculates all components so is inefficient here.
         NumMinus+=bits&2; // sign changes with each LL term; 
         }
         else // not sinusoidal
            if(bits&2) goto NextBits; // only count term in Int if this bit=0; 1/2 as many terms.
     if (Sinus[2])
         { // sinusoidal in x, there are 2 terms; 0.5-bits&4=+ or - 0.5 is disp. from center
         Angle+=kVectorTerm[2]*(ce->center[2]+(0.5-(bits&1))*ce->size[2]-rec->center[2]); //upper right or LL
            // there is a function LowerLeft, but it calculates all components so is inefficient here.
         NumMinus+=bits&1; // sign changes with each LL term;
         }
         else // not sinusoidal
            if(bits&1) goto NextBits; // only count term in Int if this bit=0; 1/2 as many terms.
               // "break" here breaks out of for loop, not what we want!
	// Now have Angle for one term of sinusoidal-direction factor
	 if((numOfSinusDir)&1) TermVal=cos(Angle); else TermVal=sin(Angle);
	 // if(num is odd) // this had +index, don't know why
	 //reverse term in needed: see p. 6.16
		switch(numOfSinusDir)
		{
			case 1:
				// no break, chg sign;
			case 2: 
				TermVal=-TermVal;// in cases 0&3, no change sign; is i^num, really
		} // end switch
		Int+=TermVal;
NextBits:
		Angle=0; // no effect, but need a statement here?
	} // end for(bits); now have Int=sum over sinusoidal directions, except:
	if(numOfSinusDir==0) Int=1.0;

	for (short i=0;i<3;i++) // loop axes
	{
		if(Sinus[i]) Int/=(ce->size[i]*kVectorTerm[i]); // 1/ak in integrals
		else 
		{short n1=(short)kVectorTerm[i]+1; // treat as integer exponent
		real disp=ce->center[i]-rec->center[i]; // displacement from rect. center
		Int*=pow(disp+0.5*ce->size[i],n1)-pow(disp-0.5*ce->size[i],n1)/(real)n1;
		} // Is integral of disp^n
	}
	return Int;
	*/
} // fn. IntOverCell

// mode functions
Mode::Mode()
{ HVal=0.0; MVal=0.0; MMax=0.0;
#ifdef GRAPHICS
  MomDisp=NULL;  
#endif !GRAPHICS
  ConstrainIndex=-1; } // default unconstrained

// short Mode::NConstrained=0.0; // static, needs init outside fn

void Mode::ReadMode()// reading mode from script file
{  short iM, i, idum, jdum; string junk,Nam; char ch,chLast; 
   real fudge; MVector Origin;
#ifdef GRAPHICS
   bool DispH, DispM;
   NumberDisplay *ND;
   ostringstream StSt;
#endif
   Rectang* rect;
   cell* c1;
   iM=My->NModes;
	My->paramFile>>junk>>ModeName>>Nam>>HVal>>HInc>>MInc; 
	  // iM=mode index,Nam=type(field or non)
	// here MInc is fraction, made absolute when 'Q' read in ReadScript()
	//if (iM!=My->NModes) 
	//{cout<<"Problem with mode order: line 'mode' has "<<iM<<
	//", but reading "<<My->NModes<<"th mode."<<endl; cin.get(); }
	My->CheckScript(junk,"mode");
	switch(Nam[0])
	{  case 'n': NonField=true; break;
	   case 'f': NonField=false; break;
	   default: EXIT1("Need non or field-like in mode script line");
	}

	//My->paramFile.getline(junk); done at end of switch{}!
	cout<<endl<<"In ReadMode, index="<<iM<<", name="<<ModeName;
	cout<<", HVal="<<HVal<<", HInc="<<HInc<<", MIncFrac="<<MInc<<"; ";
	if(NonField) cout<<"Non-"; cout<<"field-like"<<endl;

	// Now read terms:
    getline(My->paramFile,junk); My->paramFile>>ws; // skip white space
	while((ch = (char)My->paramFile.peek()) != '!')  //inner nested while loop
      {  cout<<ch; // debug
         switch(ch)
          {
            case '#':
              break;
            case 'T': //term line // stole from cell.cpp, rect setup
  	          My->paramFile>>junk>>Nam; // rect name
			  assert (junk[1]=='e'); // shd be Term
			  if(Nam=="ALL") 
				  rect=My->RectFromID[0]; // start with first rect
			  else
			  {
				  // cout<<"Found Term line in mode, rect name="<<Nam<<",junk="<<junk<<endl;
				  rect = My->RectNamed(Nam);
			  }
	          // if (iRect>=My->NRects) {cout<<"index "<<iRect<<" exceeds highest index="<<My->NRects-1<<endl; MYEXIT(15);}
			  // rect = My->RectFromID[iRect];
			  Term *T1; 
              T1 = new Term(this); // was Term(My->Modes[iM])
	          T1->ReadTerm(My->paramFile,rect); 
              rect->Terms.push_back(*T1); // again may leave 2 copies?
			  if(Nam=="ALL") // copy it to other rectangles
			  {   assert(T1->numOfSinusDir==0); 
			      //assert(T1->kVectorTerm.IsZero()); shd be uniform??
				  for(short i=1;i<My->NRects;i++) {assert(i<My->NRects); 
				    My->RectFromID[i]->Terms.push_back(*T1);}
			  }
              break;
				
       	 case 'F': // 'mode' Field_display (H at points)- shows only ONE mode, near ONE rectangle
			 // this is DIFFERENT from line beginning NumberDisplay, which has no arrow.
       	    // reads a line like this for each vector to display
			 // note you can have a field display even if iMode>7, can't change with keys
       		My->paramFile>>junk>>Nam>>Origin>>idum>>jdum>>fudge>>chLast;
			  // Origin=pos'n,rect index,i,jdum=WhichComps,WhereLabel
			if(!My->paramFile) cout<<"Error reading Field_Display line"<<endl;
       		My->CheckScript(junk,"Field");
			//assert(iRect<My->NRects);
			rect=My->RectNamed(Nam);//My->RectFromID[iRect];
			c1=rect->WholeRect; // rescale position to rectangle size
            for (i=0;i<3;i++) Origin[i]=Origin[i]*c1->half[i]*2+c1->center[i];
       		if (chLast!=EOD) {cout<<"End of data character "<<EOD<<" missing after "<<junk<<endl; MYEXIT(12);}
       		// if(kdum>My->NModes) {cout<<"Field_display -- no mode "<<kdum<<endl; MYEXIT(13);}
#ifdef GRAPHICS // this MUST come after setting up graphics, above
            // DumStr = "h."; DumStr+=My->Modes[iM]->ModeName.c_str(); //DumStr.addIntString(iM); // gives "H2", e.g.
			StSt<<"h."<<ModeName; // puts into StringStream
			FieldDisplay *FD; // for Modes
       		FD = new FieldDisplay(StSt.str().c_str(),idum,jdum, // label,whichcomps,wherelabel
               Origin, rect->LLGSize[0]*fudge/rect->Ms, 0, 0.5, 0, // these are from VectorDisplay,next two new:
	       	this,rect); // create display for mode iM
       		My->FieldDisplays.push_back(*FD); // add pointer to list
       		// Does this leave us w/ two copies of *FD??
       		// cout<<"about to add Field display for mode "<<iM;
               assert (FD);
#ifdef OPENGL
			   cout<<"Field display not implemented in OpenGL"<<endl;
#endif --OPENGL
#ifdef COIN
               My->rootBase->addChild(FD->getNode()); 
#endif --COIN

#else 
		    cout<<"Found "<<junk<<" line, ignoring because graphics off"<<endl;
#endif 
	        // cout<<"Done"<<endl;
       		break;
		 case 'L': // locate mode display
#ifdef GRAPHICS
			if(My->NModes!=1) {cout<<"LocateModeDisplay must be specified for Mode 1 only"
				 <<endl; MYEXIT(29);}
       		My->paramFile>>junk>>My->xMd>>My->yMd>>My->dxMd>>My->dyMd;
			My->xMd=My->root->Scale(0,My->xMd); // rescale & shift to root cell size
			My->yMd=My->root->Scale(1,My->yMd);
			My->dxMd*=2*My->root->half[0]; // rescale w/o shift
			My->dyMd*=2*My->root->half[1];
			My->CheckScript(junk,"LocateModeDisplay");
            // cout<<"Will start display of mode values at "<<My->xMd<<","<<My->yMd<<
			//	", increment "<<My->dxMd<<","<<My->dyMd<<endl;
#else
			 cout<<"Found 'L', (LocateNumberDisplay?) -- ignore, no graphics"<<endl;
#endif // GRAPHICS
			break;
		 case 'N': // number display of this mode; ignored if iM>7
#ifdef GRAPHICS
			My->paramFile>>junk>>DispH>>DispM;
			My->CheckScript(junk,"NumberDisplay");
			if(My->NModes>7) // can't display
			{  if(DispH || DispM) cout<<"Sorry, modes>7 can't be displayed: mode #="<<iM<<endl; 
			   DispH = DispM = false;
			}
			else // can display
			if(My->NModes!=1) {My->yMd+=My->dyMd; // cout<<"Increment yMd to "<<My->yMd<<endl;
			  }
            //cout<<"Display mode vals at "<<My->xMd<<","<<My->yMd<<endl;

	        if(DispM) 
			{  StSt.str(""); //cout<<"reset StSt="<<StSt.str()<<endl;
			   StSt<<" "<<My->NModes<<":"<<ModeName<<",m= ";
			     // first blank leaves room for ! later
			   //DumStr.addIntString(iM); DumStr+=":m."; // now 1:m.
			   //DumStr+=My->Modes[iM]->ModeName.c_str(); //DumStr.addIntString(iM); 
			   //DumStr+="= %7.2f"; //DumStr+="= %7.2f e3 muA nm^2";
			   // cout<<"Display moment: string="<<StSt.str()<<endl;
       		   ND = new NumberDisplay(StSt.str(),1e-3,2,&MVal,"",
				   My->xMd,My->yMd,0.0, 0.5,0.0,0.0); // dark red
       		   My->NumbDisps.push_back(*ND); // add pointer to list
			   //My->Modes[iM]->MomFormat=ND->Nformat; // save address to add'!'
			   // Unfortunately, Nformat seems to be moved AFTER this! Save iterator to display instead?
			   // Save pointer so we can change '!' later:
			   MomDisp=&My->NumbDisps.back(); // pointer to back one;
               //cout<<My->Modes[iM]->MomDisp->Nformat<<endl;
			   //My->Modes[iM]->MomDisp->Nformat[0]='!';
			   //cout<<My->Modes[iM]->MomDisp->Nformat<<endl;
         	}

	        if(DispH) 
	        {  // ODD thing about stringstream: StSt.str("h.") initializes,
		   	   // cout<<"Initialized StSt="<<StSt.str()<<endl;
				// but then contents are LOST when you append to it; 
       		   ND = new NumberDisplay("h=",1.0,2,&HVal,"",
				   My->xMd+My->dxMd,My->yMd,0.0,0.0,0.0,0.5); // dark blue=00,0.5
       		   My->NumbDisps.push_back(*ND); // add pointer to list
         	}
#else
cout<<"Found 'N'umber display, ignoring -- no graphics"<<endl;
#endif // GRAPHICS
            break; // end case 'N'

		 default: 
			 cout<<"Unrecognized 1st character in mode block:"<<ch<<endl;
			 MYEXIT(99);
           } //end of switch(ch)
		 if(!My->paramFile) {cout<<"Error in script, modes, near:"
		   <<junk<<endl; MYEXIT(22);}

          getline(My->paramFile,junk,'\n');
	      My->paramFile>>ws; // skip white space at beginning of next line
        } //end of while(ch)
} // end of function ReadMode

short Mode::ModeNamed(string Nam)
{   for (short i=1;i<=My->NModes;i++)
      if(Nam==My->Modes[i]->ModeName) return i;
    cout<<"Mode name not found:"<<Nam<<endl; return 0;
}

void Mode::ToggleConstraint() // constr. or unconstrains 1 mode
{  short NCons=0;
   if(My->cons) NCons=My->cons->NC;
   if(ConstrainIndex>-1)  // means constrained 
   { assert(My->cons);
	 NCons-=1; // decreases by one
     ConstrainIndex=-1; // now unconstrained
#ifdef GRAPHICS
	  if(MomDisp) MomDisp->Prefix[0]=' '; 
	    // needn't reset this when constrain, done in constructor
	  cout<<"ToggleCOnstraint; MomDisp="<<MomDisp<<", Prefix="<<MomDisp->Prefix<<endl; //DEBUG
      cout<<"un";
#endif
   }
   else // not constrained, constrain it
     {
		 NCons+=1;
         ConstrainIndex=0; // don't know exact value yet, but >=0
     }
  cout<<"constraining mode "<<ModeName<<endl;
  if(My->cons) My->cons->~Constrain(); //destructor!
  if(NCons>0) // something constrained
  {
    My->cons = new Constrain(NCons); // construct 
	assert(My->cons);
	//cout<<"Constructed new Constrain object with "<<NCons<<" constrained modes"<<endl;
	// only does this one! My->cons->mWant[ConstrainIndex]=MVal; // constrain to present value (hope MVal is current!)
	My->cons->SetmWantToPresent(); // other arrays should be ok
  }
  else My->cons=NULL; // nothing constrained
}

void Mode::ChangeModeInc(bool Up)
{  if(ConstrainIndex>-1)  // means constrained 
   { assert(My->cons);
     cout<<", chg MInc["<<ConstrainIndex<<"]:";
     ChangeInc(MInc,Up); 
   }
   else // not constrained, change Hinc instead:
   {cout<<", chg HInc:"; ChangeInc(HInc,Up); } 
}

void Mode::IncMode(short sgn)
{  if(ConstrainIndex>-1)  // means constrained 
   { assert(My->cons);
     My->cons->mWant[ConstrainIndex]+=MInc*sgn; My->ConvSteps=0; 
     My->cons->Prorating=false; // do this whenever reset mWant
     cout<<", MWant["<<ConstrainIndex<<"] changed to "<<My->cons->mWant[ConstrainIndex]<<endl;
   }
   else // not constrained, change H instead:
     {HVal+=HInc*sgn; cout<<", HVal changed to "<<HVal<<endl;} 
}

real Mode::B_mu0() {return HVal+MVal/My->root->volume();}

real* Mode::WhatToInc()
  {if(ConstrainIndex>-1) return &(My->cons->mWant[ConstrainIndex]);
     else return &HVal;
  }

#ifdef GRAPHICS
// FieldDisplay functions
FieldDisplay::FieldDisplay(const char *labelIn, short WhichCompsIn, short WhereLabel1,
	     const MVector &VectorTranslation, real multFactorIn,
		 float colorR, float colorG, float colorB, Mode* M1, Rectang *R1):
		VectorDisplay(labelIn, WhichCompsIn, WhereLabel1, VectorTranslation, 
			multFactorIn, colorR, colorG, colorB)
{
	ItsMode=M1; ItsRect=R1;
}
#endif // GRAPHICS
// MedField functions:
	void MedField::AddMedField(MVector& H) // add field of medium grain to H, at time Systm::t
{
       real Pos1=StartPos+Vel*My->t + WidthMed*0.5; // x+g/2 in downtrack direction
       real Pos2=Pos1-WidthMed; // x-g/2
	   real BotHt=FlyHt+ThickMed; // distance from field point (cell center) to media bottom
	   if (My->DisplayTime) cout<<"Grain x-positions "<<Pos1<<" & "<<Pos2<<endl;

	   H[perp]+=MsMed4pi*2.0* // field of uniform rectangle, from Bill's class notes for top surface
		   (atan(Pos1/FlyHt)-atan(Pos2/FlyHt)-atan(Pos1/BotHt)+atan(Pos2/BotHt)); 
	   H[down]-=MsMed4pi*
		 log((Pos1*Pos1+FlyHt*FlyHt)*(Pos2*Pos2+BotHt*BotHt)/((Pos2*Pos2+FlyHt*FlyHt)*(Pos1*Pos1+BotHt*BotHt)));
// FOrmerly H[perp]+=MsMed4pi*2.0*(atan(Pos1/FlyHt)-atan(Pos2/FlyHt)); // For one rect. charge
//& H[down]-=MsMed4pi*log((Pos1*Pos1+FlyHt*FlyHt)/(Pos2*Pos2+FlyHt*FlyHt));


	   //cout<<"New H = "<<H<<endl;
	} // end GetMedField

	void MedField::ReadMedField(ifstream *File)// member data FlyHt,WidthMed,MsMed4pi, StartPos, Vel, perp,down;
	// down=0 means downtrack direction is x, perp=y means perp to media is y
	{  *File>>MsMed4pi>>FlyHt>>ThickMed>>WidthMed>>StartPos>>Vel>>perp>>down;
	   MsMed4pi/=(4*M_PI); // to avoid continually multiplying
       cout<<"Leaving ReadMedField, MsMed4pi="<<MsMed4pi<<endl;
	}

#ifdef WINDOW2
	Window2::Window2()
	{
	// doesn't like this, says class already exists: HWND myWindow2=SoWin::init("Plots");
	SoWinRenderArea myViewer2=SoWinRenderArea(); // if(myWindow), puts in same window
	// was SoWinExaminerViewer; area has no decoration, also won't resize?
	myViewer2.setTitle("Plots");
	NextPlotx=0.0; NextPloty=0.0;
	PlotMM = Plot(-1,1,-1,1,"m2","xm2",NextPlotx,NextPloty);
	// gives x1,x2,y1,y2,labels, upper-left position
	// nmay add more plots here
	plotArea->setSceneGraph(plotRoot);
//	plotArea->setBackgroundColor(SbColor(1., 1., 1.));
	plotArea->show();
	}

	Plot::Plot(float x1, float x2,  float y1, float y2, char * labelX, char * labelY)
{
	xRange1 =x1;
	xRange2 =x2;
	assert(xRange2-xRange1>0);

	yRange1 =y1;
	yRange2 =y2;
	assert(yRange2-yRange1>0);

	border =0.1;
	graphWidth=2-2*border;
	//graphHeight=graphWidth*((yRange2-yRange1)/(xRange2-xRange1));
	graphHeight=graphWidth;
        
		
	//get the scale of the graph
	xScale = (graphWidth)/(xRange2-xRange1);
	yScale = (graphHeight)/(yRange2-yRange1);
	
	plotRoot = new SoSeparator;
	plotRoot->ref();
	plotLineset = new SoLineSet;
	plotArea = new SoXtRenderArea;
	
	SoFont * myFont;
	assert(myFont = new SoFont);
	myFont->name.setValue("Times-Roman");
	myFont->size.setValue(18); //in pixels

	//set up the x axes label
	SoSeparator * Xtext = new SoSeparator;
	SoTransform * xtextTrans = new SoTransform;
	xtextTrans ->translation.setValue(graphWidth/2+0.01, -0.01, 0);
	SoText2 * Xlabel = new SoText2;
	Xlabel->string= labelX;
	Xtext->addChild(xtextTrans);
	Xtext->addChild(Xlabel);

	//set up the y axes label
	SoSeparator * Ytext = new SoSeparator;
	SoTransform * ytextTrans = new SoTransform;
	ytextTrans->translation.setValue(-0.01,graphHeight/2+0.01,0);
	SoText2 * Ylabel = new SoText2;
	Ylabel->string= labelY;
	Ytext->addChild(ytextTrans);
	Ytext->addChild(Ylabel);
        
	coors = new SoCoordinate3;

	
	//X axes and Y axes coordinate
	coors->point.set1Value(0,-graphWidth/2 ,0,0);//-graphHeight/2,0);
	coors->point.set1Value(1, graphWidth/2,0,0);//-graphHeight/2,0);

	coors->point.set1Value(2,0, -graphHeight/2,0);
	coors->point.set1Value(3,0,graphHeight/2, 0);
	
	//Tic Mark coordinates

	coors->point.set1Value(4,-graphWidth/2 ,0,0);
	coors->point.set1Value(5,-graphWidth/2 ,-0.04,0);
	coors->point.set1Value(6,graphWidth/2,0,0);
	coors->point.set1Value(7,graphWidth/2,-0.04,0);

        SoSeparator *XvalueText1 = new SoSeparator;
        SoText2 *Xvalue1 = new SoText2;
        Xvalue1->string = "30";
        SoTransform * xvalueTrans1= new SoTransform;
        xvalueTrans1 ->translation.setValue(graphWidth/2-0.04, -0.10, 0);
        XvalueText1->addChild(xvalueTrans1);
        XvalueText1->addChild(Xvalue1);
        
        SoSeparator *XvalueText2 = new SoSeparator;
        SoText2 *Xvalue2 = new SoText2;
        Xvalue2->string = "-30";
        SoTransform * xvalueTrans2 = new SoTransform;
        xvalueTrans2 ->translation.setValue(-graphWidth/2+0.01, -0.10, 0);
        XvalueText2->addChild(xvalueTrans2);
        XvalueText2->addChild(Xvalue2);

	coors->point.set1Value(8,0 ,-graphHeight/2,0);
	coors->point.set1Value(9,-0.04,-graphHeight/2,0);
	coors->point.set1Value(10,0,graphHeight/2,0);
	coors->point.set1Value(11,-0.04,graphHeight/2,0);

        SoSeparator *YvalueText1 = new SoSeparator;
        SoText2 *Yvalue1 = new SoText2;
        Yvalue1->string = "15";
        SoTransform * yvalueTrans1= new SoTransform;
        yvalueTrans1 ->translation.setValue(-0.15, graphHeight/2-0.04, 0);
        YvalueText1->addChild(yvalueTrans1);
        YvalueText1->addChild(Yvalue1);
        
        SoSeparator *YvalueText2 = new SoSeparator;
        SoText2 *Yvalue2 = new SoText2;
        Yvalue2->string = "-15";
        SoTransform * yvalueTrans2 = new SoTransform;
        yvalueTrans2 ->translation.setValue(-0.20,-graphHeight/2+0.02, 0);
        YvalueText2->addChild(yvalueTrans2);
        YvalueText2->addChild(Yvalue2);

	//draw X and Y axes 
	plotLineset->numVertices.set1Value(0,2);
	plotLineset->numVertices.set1Value(1,2);

	//draw tic mark
	plotLineset->numVertices.set1Value(3,2);
	plotLineset->numVertices.set1Value(4,2);
	plotLineset->numVertices.set1Value(5,2);
	plotLineset->numVertices.set1Value(6,2);

	plotRoot->addChild(coors);
	plotRoot->addChild(plotLineset);
	plotRoot->addChild(myFont);
	plotRoot->addChild(Xtext);
	plotRoot->addChild(Ytext);
        plotRoot->addChild(XvalueText1);
        plotRoot->addChild(XvalueText2);
        plotRoot->addChild(YvalueText1);
        plotRoot->addChild(YvalueText2);
}

Window2::FixWin2() // updates each time 
{
plotMM.AddPoint(My->Modes
}

#endif // WINDOW2
