#ifndef PULSE_H
#define PULSE_H
//#include "Lowest.h"
#include "MVector.h"
#include <fstream>
#include "display.h" // so VectorDisplay can be used as base
//class cell;
//class Rectang;
#include "cell.h"
class Mode;
// #include <unistd.h> // for exit
class PulsedField
{ // 2/2012: Probably want to replace this by scripting
	// (it seems to have been written before scripting allowed
	// multiple "Run" commands?)
	// TUrn field on, keep the ability to change field continuously 
	// but only during a single interval, not rise&fall together.
 public:
    short PulseType;  // 1:Trapezoidal, 2:Exponential
    real StartTime;
    real RiseTime;
    real FallTime;
    real PeakTime;
    real TotalTime;
    MVector Hmax;

    // member functions 
    void Read(ifstream *File);
    real RiseFunction(real t);
    real FallFunction(real t);
    MVector GetValue(real time);
};

class Term // term in a (normal) mode
{ // each direction can be sinusoidal OR a moment
	//There is more stuff in Ru's version (e.g. March24,2010)
public:
Term(Mode* M1); // was short iM! // not (Mode*)
MVector kVectorTerm, TermAmp; // if moment, kVectorTerm is exponents!!
real Phase; 
int3 Sinus; // whether field sinusoidal (0 if moment, 1 if sinus)
Mode* ItsMode; // which mode this is a term of?
short numOfSinusDir; // sinusoidal directions
void ReadTerm(ifstream&,Rectang*);
MVector EvalAtPoint(cell *WRect, MVector &r);
MVector IntOverCell(cell*); // integral
// void AddTermTo(MVector &V,cell* ce);
void DebugWrite();
};

class Mode  // (normal) mode: field to excite, also moment calc
{  // may have many terms, Fourier components
public:
		//delete: There is more stuff in Ru's version (e.g. March24,2010)
	Mode();
	// Mode does NOT know which terms are in it, each rectangle has a list of terms
	// Term *Terms[4]; 	short NTerms;
	bool NonField; // true if spin torque
	real HVal, MVal; // value of field and moment
	  // once calle HVal, HVal0 -- is a static background -- energy contours assume this value ("head field")
	  // HVal includes changes to move around landscape
	  // but now HVal never has these, just cons->dh does
	real HInc, MInc; // increment either, very different amounts
	// NO! Mode can spread over several rects! Rectang *ItsRect; 
    real MMax; // max (absolute) value of moment
	real Tol; // error tolerance
	string ModeName;
#ifdef GRAPHICS
	// failed: char* MomFormat; // pointer to where to put ! to indicate constrained
	NumberDisplay* MomDisp; // need this only as handle to change '!'.
#endif // GRAPHICS
	MVector ShapeF; //someplace to store the value temporarily (while working on cell?)
	  //  needed because need put stuff in several modes while looping terms 
	  //  This may be bad idea for parallelization! Thread needs its own scratch
	real ddEdh; // d(Delta E)/dh
	short ConstrainIndex; // index in list of constrained moments
	// static short NConstrained; // # of constrained modes
	  // this is NOT necessary!! =cons->NC, & Local where created
    //inline void AddModeToH(MVector &H,real Vol){H+=ShapeF*HVal/Vol;} // used to calc H_lin AND for FieldDisp
    inline MVector ModeField(LLGcell* c1){MVector H1=ShapeF*HVal/c1->volume();
		if(NonField) H1=(c1->M^H1)/c1->rect->Ms; return H1;} // used to calc H_lin AND for FieldDisp

	real* WhatToInc();
    //inline void CorrectH(MVector &H,real Vol){H+=ShapeF*(HVal0-HVal)/Vol;} // replaces HVal by HVal0
	// MVector EvalFAtPoint(MVector &r);
	// MVector IntFOverCell(cell* ce);	
	// real CellMoment(cell* ce); // this mode's moment over a cell, recursive.
	  // total moment is MVal=CellMoment
	void ReadMode();
	static short ModeNamed(string);
	void ToggleConstraint();
	void IncMode(short);
	real B_mu0();
	inline void IncMMax(real M1) {MMax+=ShapeF.Magnitude()*M1;} // M1=Ms; ShapeF has vol
	  // cout<<ModeName<<":MMax="<<MMax<<endl;
	void ChangeModeInc(bool Up);
	// void DebugWrite();
};

#ifdef GRAPHICS
class FieldDisplay: public VectorDisplay // which has lots of arguments
{  // this is a derived class, from base VectorDisplay
	public:
	Rectang *ItsRect;
	Mode* ItsMode;
    FieldDisplay(const char *labelIn, short WhichCompsIn, short WhereLabel1,
	     const MVector &VectorTranslation, real multFactorIn,
         float colorR, float colorG, float colorB, Mode* iM, Rectang *R1);
  // constructor; all but iM, R1 are members of VectorDisplay
};
#endif // GRAPHICS

class MedField  // taken from field of ring head (exact for planar gap)
{  // has perp (=y?) and downtrack (=x?) components
public:
	real FlyHt,ThickMed,WidthMed,MsMed4pi, StartPos, Vel;
	short perp, down; // down=0 means downtrack direction is x, perp=y means perp to media is y
	// member functions
	void AddMedField(MVector&); // at time Systm::t
	void ReadMedField(ifstream *);
};

#ifdef WINDOW2
class Window2
{
	SoWinRenderArea myViewer2;

	// was SoWinExaminerViewer; area has no decoration, also won't resize?
	// maybe SoXtFullViewer better, has decoration(?)
    real NextPlotx,NextPloty;
	Window2(); // constructor
	FixWin2();// updates at each time

};

class Plot { // written by Xiaoguang Deng circa 2001
//This class is used to setup a 2-D graphic plot window.
//It will initialized with X axes Y axes and 4 tic marks
//The class constructor take six arguments, the first two arguments
//are the range of X component, the third and fouth arguments are the
//range of Y component. The fifth and sixth arguments are the label of
//X and Y axes. Last are the origin (top left).

public:
	
	Plot( float x1, float x2, float y1, float y2, char* lableX, char * lableY,xUL,yUL);
	short ModeX, ModeY; // indices of modes to plot; may have later option to plot
	  /// combinations of modes, quadratics
	SoSeparator * plotRoot;
	SoLineSet * plotLineset;
	//plotLineset is used to draw X Y and initial 4 tic mark.
	SoCoordinate3 * coors;
	//coors is used to store the coordinate of X Y and tic marks
	float xRange1, xRange2, yRange1, yRange2,xScale,yScale, border;
	void addPoint(float x1, float y1);
    //SoXtRenderArea * plotArea; does not have its own window, goes in Window2
	float graphWidth, graphHeight;

};
#endif // WINDOW2

#endif // PULSE_H
