
#include "shape.h"


//extern real rand1();

unsigned char Shape::VertIn(const MVector& center, const MVector& half){
	//set a bit high if vert is inside shape
	//cell does not store verts, but center and size
	// can be optimized further if needed
	unsigned char vertin = 0;
	char vert=0;
	for (char i=-1;i<2;i+=2){
		for (char j=-1;j<2;j+=2){
			for (char k=-1;k<2;k+=2){
				vertin |= this->InsideIs(center + MVector(half[0]*i,half[1]*j,half[2]*k))<<vert++;
			}
		}
	}

	return vertin;

}

Intersection::Intersection(Shape *shape1, Shape *shape2){
		this->shape1=shape1;
		this->shape2=shape2;
	}
bool Intersection::InsideIs(const MVector& r){
		return shape1->InsideIs(r) && shape2->InsideIs(r);
	}
bool Intersection::outside(cell* c1){
		return shape1->outside(c1) || shape2->outside(c1);
}
bool Intersection::outside(const MVector& center, const MVector& half){
	return shape1->outside(center,half) || shape2->outside(center,half);
	/*// this loop below will not return false if cell is just outside the shape, as the above expression can
	// take a lot of random points inside given cell, ensure none of them are inside the shape.
	for (int lucky_num=0;lucky_num<10000;lucky_num++){
		// test a random point within the cell
		MVector randpt = center + MVector(half[0]*2*(rand1()-0.5),half[1]*2*(rand1()-0.5),half[2]*2*(rand1()-0.5));
		if (InsideIs(randpt)) return false;
	}
	return true;*/
}

Union::Union(Shape *shape1, Shape *shape2){
	this->shape1=shape1;
	this->shape2=shape2;
}

bool Union::InsideIs(const MVector& r) {
	return shape1->InsideIs(r) || shape2->InsideIs(r);
}

bool Union::outside(cell* c1){
	return shape1->outside(c1) && shape2->outside(c1);
}

bool Union::outside(const MVector& center, const MVector& half){
	return shape1->outside(center,half) && shape2->outside(center,half);
}

TranslatedShape::TranslatedShape(Shape* shape, MVector* disp){
	this->shape = shape;
	this->disp = disp;
}
TranslatedShape::TranslatedShape(Shape* shape, real x, real y, real z){
	this->shape = shape;
	this->disp = new MVector(x, y, z);
}

bool TranslatedShape::InsideIs(const MVector& r){
	return shape->InsideIs(r-*disp);
}

bool TranslatedShape::outside(cell* c1) {
	return VertIn(c1->center,c1->half)==0;
}

bool TranslatedShape::outside(const MVector& center, const MVector& half) {
	return shape->outside(center-*disp,half);
}

InverseShape::InverseShape(Shape* shape){
	this->shape = shape;
}

bool InverseShape::InsideIs(const MVector& r){
	return !shape->InsideIs(r);
}

bool InverseShape::outside(cell* c1){
	return VertIn(c1->center,c1->half)==0;
}

bool InverseShape::outside(const MVector& center, const MVector& half){
	return VertIn(center,half)==0;
}


/*bool TransformedShape::InsideIs(const MVector& r){

		// matrix multiply xform_matrix and r
		return shape->InsideIs(MVector((*xform_matrix[0])*r, (*xform_matrix[1])*r, (*xform_matrix[2])*r));
}*/


// primitives

Ellipsoid::Ellipsoid(real& rx, real& ry, real& rz) {
	this->rx = rx;
	this->ry = ry;
	this->rz = rz;
}

Ellipsoid::Ellipsoid(MVector& r, MVector& c, bool convex) {
	this->rx = (r[0] > 0) ? r[0] : numeric_limits<real>::infinity();
	this->ry = (r[1] > 0) ? r[1] : numeric_limits<real>::infinity();
	this->rz = (r[2] > 0) ? r[2] : numeric_limits<real>::infinity();
	cx = c[0]; cy = c[1]; cz = c[2];
	this->r = (rx,ry,rz);
	this->c = (cx,cy,cz);
	this->convex = convex;
}

Ellipsoid::Ellipsoid(real& rx, real& ry, real& rz, real& cx, real& cy, real& cz, bool& convex) {
	this->rx = (rx > 0) ? rx : numeric_limits<real>::infinity();
	this->ry = (ry > 0) ? ry : numeric_limits<real>::infinity();
	this->rz = (rz > 0) ? rz : numeric_limits<real>::infinity();
	this->cx = cx;
	this->cy = cy;
	this->cz = cz;
	this->r = (rx,ry,rz);
	this->c = (cx,cy,cz);
	this->convex = convex;
}

Ellipsoid::Ellipsoid(){
	// must define a default (no arguments) constructor to make subclasses of ellipsoid
	this->rx = 1.0;
	this->ry = 1.0;
	this->rz = 1.0;
}
Ellipsoid::Ellipsoid(real a){
	// centered sphere
		rx = ry = rz = a;
		cx = cy = cz = 0;
}

bool Ellipsoid::InsideIs(const MVector& r){
	//return pow(r->x()/rx, 2) + pow(r->y()/ry, 2) + pow(r->z()/rz, 2) <= (real)1.0;
	return convex == (pow((r[0]-cx)/rx,2) + pow((r[1]-cy)/ry,2) + pow((r[2]-cz)/rz,2) <= 1.0);
	}
bool Ellipsoid::outside(cell* c1){
	return Ellipsoid::outside(c1->center,c1->half);
}
bool Ellipsoid::outside(const MVector& center, const MVector& half){
	if (!convex) return VertIn(center,half)==0; // not debugged, swim at your own risk
	MVector LL=center-half; // corrected for center displacement in InsideIs function
	MVector UR=center+half;
	MVector closest;
	for (short i=0;i<3;i++) {
		if (LL[i]>0) closest[i]=LL[i];
		else if (UR[i]<0) closest[i]=UR[i];
		else closest[i]=0;
	}
	return !InsideIs(closest);
}

Ellipsoid::~Ellipsoid(){
	delete &rx;
	delete &ry;
	delete &rz;
	delete &r;
	delete &c;
}


Cuboid::Cuboid(real x, real y, real z){
	this->x = x;
	this->y = y;
	this->z = z;
}
Cuboid::Cuboid(real r){
	this->x = r;
	this->y = r;
	this->z = r;
}

bool Cuboid::InsideIs(const MVector& r){
	return r[0] <= x && r[0] >= -x
		&& r[1] <= y && r[1] >= -y
		&& r[2] <= z && r[2] >= -z;
	// note: cuboid runs from -x to x, -y to y, etc. on a side.
}

bool Cuboid::outside(cell* c1){
	return VertIn(c1->center, c1->half)==0;
}


bool Cuboid::outside(const MVector& center, const MVector& half) {
	return VertIn(center,half)==0;
}


Cuboid::~Cuboid(){
		delete &x;
		delete &y;
		delete &z;
}


SemiSpace::SemiSpace(MVector *n, real b){
		// region of space with n.r <= b;
		this->n = n;
		this->b = b;
}

SemiSpace::SemiSpace(MVector* v0, MVector* v1, MVector* v2){
	// calculate n and b from 3 verts on semispace boundary.
	// note: ^ is MVector cross product
	this->n = new MVector((*v0-*v1) ^ (*v0-*v2));
	this->b = *n * *v0;
}

bool SemiSpace::InsideIs(const MVector& r){
	// multiplication is dot product for MVectors
	return *n * r < b;
}

bool SemiSpace::outside(cell* c1){
	/*
	real sum = 0.0;
	for (short i=0;i<2;i++)
		sum+=c1->center[i] * (*n)[i] +c1->half[i]*abs((*n)[i]);
	return sum > b;
	*/
	return VertIn(c1->center,c1->half)==0;
}
bool SemiSpace::outside(const MVector& center, const MVector& half){
	
	/*real sum = 0.0;
	for (short i=0;i<2;i++)
		sum+=center[i] * (*n)[i] + half[i]*abs((*n)[i]);
	return sum > b;
	*/
	return VertIn(center,half)==0;
}

SemiSpace::~SemiSpace(){
	delete &b;
	delete n;
}



Tetrahedron::Tetrahedron(MVector* v0, MVector* v1, MVector* v2, MVector* v3) {
		// given 4 vertices, create necessary SemiSpace objects
		// 06/2013 : might need some tweaking to ensure inside of tetrahedron is always correct.

		this->semispaces[0] = new SemiSpace(v1,v2,v3); // face 0
		this->semispaces[1] = new SemiSpace(v0,v3,v2); // face 1
		this->semispaces[2] = new SemiSpace(v0,v1,v3); // face 2
		this->semispaces[3] = new SemiSpace(v0,v2,v1); // face 3

	}


Tetrahedron::Tetrahedron(SemiSpace* s0, SemiSpace* s1, SemiSpace* s2, SemiSpace* s3){
		this->semispaces[0] = s0;
		this->semispaces[1] = s1;
		this->semispaces[2] = s2;
		this->semispaces[3] = s3;
	}

Tetrahedron::Tetrahedron(SemiSpace** semispaces){
	// don't exceed bounds!
		for (int i=0;i<4;i++){
			this->semispaces[i] = semispaces[i];
		}
	}

bool Tetrahedron::InsideIs(const MVector& r){
		for (char i=0; i<4; i++){
			if (!semispaces[i]->InsideIs(r)){
				return false;
			}
		}
		return true;

		//return semispaces[0]->InsideIs(r) && semispaces[1]->InsideIs(r) && ...
	}

bool Tetrahedron::outside(cell* c1){
	return Tetrahedron::outside(c1->center,c1->half);
}
bool Tetrahedron::outside(const MVector& center, const MVector& half){
	/*if (semispaces[0]->outside(center,half) || semispaces[1]->outside(center,half) || semispaces[2]->outside(center,half) || semispaces[3]->outside(center,half)) return true;

	// this loop below will not return false if cell is just outside the shape, as the above expression can
	// take a lot of random points inside given cell, ensure none of them are inside the shape.
	for (int lucky_num=0;lucky_num<10000;lucky_num++){
		// test a random point within the cell
		MVector randpt = center + MVector(half[0]*2*(rand1()-0.5),half[1]*2*(rand1()-0.5),half[2]*2*(rand1()-0.5));
		if (InsideIs(randpt)) return false;
	}
	return true;
	*/
	return semispaces[0]->outside(center,half) || semispaces[1]->outside(center,half) || semispaces[2]->outside(center,half) || semispaces[3]->outside(center,half);
}

Tetrahedron::~Tetrahedron(){
	for (int i=0;i<4;i++)
		delete semispaces[i];
}

// helper functions

Shape * ReadShape(ifstream &InFile) {
	char ch;
	string line;
	string junk;
	MVector v0,v1,v2,v3;
	MVector r0,rs;
	string convex; // "convex" or "non-convex"
	InFile>>ws; // eat leading whitespace
	ch = (char)InFile.peek();
	switch(ch){
	case '#':
		// skip commented line
		getline(InFile,junk,'\n');
		return ReadShape(InFile);
		break;
	case 'T':
		// read tetrahedron and return
		InFile>>junk>>v0>>v1>>v2>>v3;
		return new Tetrahedron(&v0,&v1,&v2,&v3);
		break;
	case 'E':
		// read ellipse and return
		// read origin and radii from script
		InFile>>junk>>r0>>rs>>convex;
		if (convex[0]=='c')
			return new Ellipsoid(r0,rs,true);
		else if (convex[0]=='n')
			return new Ellipsoid(r0,rs,false);
		else
			EXIT1("Unrecognized option in ellipse: "<<convex<<". Accepted values are convex and non-convex.");
		break;
	case 'H':
		//HalfSpace
		real bh;
		InFile>>junk>>r0>>bh;
		return new SemiSpace(&r0,bh);
	case 'I':
		// recursively read shapes that will be intersected together.
		Shape *s2;
		// read first shape
		getline(InFile,junk,'\n');
		s2 = ReadShape(InFile);
		// recursively intersect subsequent shapes together.
		while ((ch = (char)InFile.peek()) != '!') {
			s2 = new Intersection(s2,ReadShape(InFile));
			InFile>>ws;
		}
        getline(InFile,junk,'\n');
		return s2;
		break;
	case 'U':
		// recursively read shapes that will be unioned together.
		Shape *s3;
		// read first shape in union
		getline(InFile,junk,'\n');
		s3 = ReadShape(InFile);
		// recursively union subsequent shapes together.
		while (InFile.peek() != (int)'!') {
			s3 = new Union(s3,ReadShape(InFile));
			InFile>>ws;
		}
        getline(InFile,junk,'\n');
		return s3;
		break;
	default:
		cout<<ch<<endl;
		EXIT1("Unrecognized 1st letter in script file, ReadShape");
		break;
	}
	//return NULL;

}
