#ifndef SHAPE_H
#define SHAPE_H

#include <limits>
#include "cell.h"

class Shape {
public:
	virtual bool InsideIs(const MVector&){ return true; }// virtual, will be overridden in daughter classes
	virtual bool outside(cell*){return true;}
	virtual bool outside(const MVector&, const MVector&){return true;}
	unsigned char VertIn(const MVector&, const MVector&);
};


class Intersection : public Shape {
private:
	Shape *shape1, *shape2;
public:
	Intersection(Shape *, Shape *);
	virtual bool InsideIs(const MVector&);
	virtual bool outside(cell*);
	virtual bool outside(const MVector&, const MVector&);
};

class Union : public Shape {
private:
	Shape *shape1, *shape2;

public:
	Union(Shape*, Shape*);
	virtual bool InsideIs(const MVector&);
	virtual bool outside(cell*);
	virtual bool outside(const MVector&, const MVector&);
};


class TranslatedShape : public Shape {
private:
	Shape* shape;
	MVector* disp;
public:
	TranslatedShape(Shape*, MVector*);
	TranslatedShape(Shape*, real, real, real);
	virtual bool InsideIs(const MVector&);
	virtual bool outside(cell*);
	virtual bool outside(const MVector&, const MVector&);

};

class InverseShape : public Shape {
private:
	Shape* shape;
public:
	InverseShape(Shape*);
	virtual bool InsideIs(const MVector&);
	virtual bool outside(cell*);
	virtual bool outside(const MVector&, const MVector&);
};

/*class TransformedShape : Shape {
private:
	Shape* shape;
	MVector* xform_matrix[3];
public:
	virtual bool InsideIs(const MVector&);
	virtual bool outside(cell*);
	virtual bool outside(const MVector&, const MVector&);
};*/

// primitives

class Ellipsoid : public Shape {
	real rx, ry, rz, cx, cy, cz;
	MVector r;
	MVector c;
	bool convex;
public:
	Ellipsoid(real&, real&, real&);
	Ellipsoid(real&, real&, real&, real&, real&, real&, bool&);
	Ellipsoid(MVector&, MVector&, bool);

	Ellipsoid();
	Ellipsoid(real);

	virtual bool InsideIs(const MVector&);
	virtual bool outside(cell*);
	virtual bool outside(const MVector&, const MVector&);

	~Ellipsoid();

};

class Cylinder : public Ellipsoid {
public:
	Cylinder();
	Cylinder(real&);
	Cylinder(real&, real&);
};


class Cuboid : public Shape {
private:
	real x,y,z;
public:

	Cuboid(real, real, real);
	Cuboid(real);

	virtual bool InsideIs(const MVector&);
	virtual bool outside(cell*);
	virtual bool outside(const MVector&, const MVector&);

	~Cuboid();

};


class SemiSpace : public Shape {
private:
	real b;
	MVector* n;
public:
	SemiSpace(MVector*, real);
	SemiSpace(MVector*, MVector*, MVector*);
	virtual bool InsideIs(const MVector&);
	virtual bool outside(cell*);
	virtual bool outside(const MVector&, const MVector&);
	~SemiSpace();
};


class Tetrahedron : public Shape {
// tet is composed of 4 semispaces
private:
	SemiSpace* semispaces[4];
public:
	Tetrahedron(MVector*, MVector*, MVector*, MVector*);
	Tetrahedron(SemiSpace*, SemiSpace*, SemiSpace*, SemiSpace*);
	Tetrahedron(SemiSpace**);
	virtual bool InsideIs(const MVector&);
	virtual bool outside(cell*);
	virtual bool outside(const MVector&, const MVector&);
	~Tetrahedron();
};

//helper functions
Shape* ReadShape(ifstream&);
#endif !SHAPE_H
